@@ -107,6 +107,7 @@ SQVM::SQVM(SQSharedState *ss)
 	_errorhandler = _null_;
 	_debughook = _null_;
 	_can_suspend = false;
+	_in_stackoverflow = false;
 	_ops_till_suspend = 0;
 	ci = NULL;
 	INIT_CHAIN();ADD_TO_CHAIN(&_ss(this)->_gc_chain,this);
@@ -180,7 +181,7 @@ bool SQVM::NEG_OP(SQObjectPtr &trg,const SQObjectPtr &o)
 bool SQVM::ObjCmp(const SQObjectPtr &o1,const SQObjectPtr &o2,SQInteger &result)
 {
 	if(type(o1)==type(o2)){
-		if(_userpointer(o1)==_userpointer(o2))_RET_SUCCEED(0);
+		if(_rawval(o1)==_rawval(o2))_RET_SUCCEED(0);
 		SQObjectPtr res;
 		switch(type(o1)){
 		case OT_STRING:
@@ -195,14 +196,19 @@ bool SQVM::ObjCmp(const SQObjectPtr &o1,const SQObjectPtr &o2,SQInteger &result)
 		case OT_INSTANCE:
 			if(_delegable(o1)->_delegate) {
 				Push(o1);Push(o2);
-				if(CallMetaMethod(_delegable(o1),MT_CMP,2,res)) break;
+				if(CallMetaMethod(_delegable(o1),MT_CMP,2,res)) {
+					if(type(res) != OT_INTEGER) {
+						Raise_Error(_SC("_cmp must return an integer"));
+						return false;
+					}
+					_RET_SUCCEED(_integer(res))
+				}
 			}
 			//continues through (no break needed)
 		default:
 			_RET_SUCCEED( _userpointer(o1) < _userpointer(o2)?-1:1 );
 		}
-		if(type(res)!=OT_INTEGER) { Raise_CompareError(o1,o2); return false; }
-			_RET_SUCCEED(_integer(res));
+		assert(0);
 
 	}
 	else{
@@ -459,7 +465,7 @@ bool SQVM::DerefInc(SQInteger op,SQObjectPtr &target, SQObjectPtr &self, SQObjec
 #define sarg1 (*(const_cast<SQInt32 *>(&_i_._arg1)))
 #define arg2 (_i_._arg2)
 #define arg3 (_i_._arg3)
-#define sarg3 ((SQInteger)*((signed char *)&_i_._arg3))
+#define sarg3 ((SQInteger)*((const signed char *)&_i_._arg3))
 
 SQRESULT SQVM::Suspend()
 {
@@ -638,7 +644,7 @@ bool SQVM::CLASS_OP(SQObjectPtr &target,SQInteger baseclass,SQInteger attributes
 bool SQVM::IsEqual(SQObjectPtr &o1,SQObjectPtr &o2,bool &res)
 {
 	if(type(o1) == type(o2)) {
-		res = ((_userpointer(o1) == _userpointer(o2)?true:false));
+		res = ((_rawval(o1) == _rawval(o2)?true:false));
 	}
 	else {
 		if(sq_isnumeric(o1) && sq_isnumeric(o2)) {
@@ -742,7 +748,7 @@ exception_restore:
 				continue;
 			case _OP_LOAD: TARGET = ci->_literals[arg1]; continue;
 			case _OP_LOADINT: TARGET = (SQInteger)arg1; continue;
-			case _OP_LOADFLOAT: TARGET = *((SQFloat *)&arg1); continue;
+			case _OP_LOADFLOAT: TARGET = *((const SQFloat *)&arg1); continue;
 			case _OP_DLOAD: TARGET = ci->_literals[arg1]; STK(arg2) = ci->_literals[arg3];continue;
 			case _OP_TAILCALL:
 				temp_reg = STK(arg1);
@@ -1017,7 +1023,8 @@ common_call:
 					if(type(_class(STK(arg1))->_metamethods[MT_NEWMEMBER]) != OT_NULL ) {
 						Push(STK(arg1)); Push(STK(arg2)); Push(STK(arg3));
 						Push((arg0&NEW_SLOT_ATTRIBUTES_FLAG) ? STK(arg2-1) : _null_);
-						int nparams = 4;
+						Push(bstatic);
+						int nparams = 5;
 						if(Call(_class(STK(arg1))->_metamethods[MT_NEWMEMBER], nparams, _top - nparams, temp_reg,SQFalse,SQFalse)) {
 							Pop(nparams);
 							continue;
@@ -1184,7 +1191,7 @@ bool SQVM::CallNative(SQNativeClosure *nclosure,SQInteger nargs,SQInteger stackb
 		throw;
 	}
 
-	assert(cstksize == _callsstacksize);
+	_callsstacksize = cstksize;
 
 	_nnativecalls--;
 	suspend = false;
@@ -1434,7 +1441,7 @@ bool SQVM::DeleteSlot(const SQObjectPtr &self,const SQObjectPtr &key,SQObjectPtr
 					_table(self)->Remove(key);
 				}
 				else {
-					Raise_IdxError((SQObject &)key);
+					Raise_IdxError((const SQObject &)key);
 					return false;
 				}
 			}
