@@ -1,4 +1,4 @@
-/* $Id: animated_tile.cpp 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: animated_tile.cpp 22017 2011-02-07 22:29:47Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,7 +11,8 @@
 
 #include "stdafx.h"
 #include "core/alloc_func.hpp"
-#include "functions.h"
+#include "tile_cmd.h"
+#include "viewport_func.h"
 
 /** The table/list with animated tiles. */
 TileIndex *_animated_tile_list = NULL;
