@@ -1,4 +1,4 @@
-/* $Id: articulated_vehicles.cpp 22224 2011-03-07 19:18:38Z rubidium $ */
+/* $Id: articulated_vehicles.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,9 +15,9 @@
 #include "vehicle_func.h"
 #include "engine_func.h"
 #include "company_func.h"
+#include "newgrf.h"
 
 #include "table/strings.h"
-#include "table/sprites.h"
 
 static const uint MAX_ARTICULATED_PARTS = 100; ///< Maximum of articulated parts per vehicle, i.e. when to abort calling the articulated vehicle callback.
 
@@ -33,13 +33,43 @@ static EngineID GetNextArticulatedPart(uint index, EngineID front_type, Vehicle
 {
 	assert(front == NULL || front->engine_type == front_type);
 
-	uint16 callback = GetVehicleCallback(CBID_VEHICLE_ARTIC_ENGINE, index, 0, front_type, front);
-	if (callback == CALLBACK_FAILED || GB(callback, 0, 8) == 0xFF) return INVALID_ENGINE;
+	const Engine *front_engine = Engine::Get(front_type);
 
-	if (mirrored != NULL) *mirrored = HasBit(callback, 7);
-	return GetNewEngineID(GetEngineGRF(front_type), Engine::Get(front_type)->type, GB(callback, 0, 7));
+	uint16 callback = GetVehicleCallback(CBID_VEHICLE_ARTIC_ENGINE, index, 0, front_type, front);
+	if (callback == CALLBACK_FAILED) return INVALID_ENGINE;
+
+	if (front_engine->GetGRF()->grf_version < 8) {
+		/* 8 bits, bit 7 for mirroring */
+		callback = GB(callback, 0, 8);
+		if (callback == 0xFF) return INVALID_ENGINE;
+		if (mirrored != NULL) *mirrored = HasBit(callback, 7);
+		callback = GB(callback, 0, 7);
+	} else {
+		/* 15 bits, bit 14 for mirroring */
+		if (callback == 0x7FFF) return INVALID_ENGINE;
+		if (mirrored != NULL) *mirrored = HasBit(callback, 14);
+		callback = GB(callback, 0, 14);
+	}
+
+	return GetNewEngineID(front_engine->GetGRF(), front_engine->type, callback);
 }
 
+/**
+ * Does a NewGRF report that this should be an articulated vehicle?
+ * @param engine_type The engine to check.
+ * @return True iff the articulated engine callback flag is set.
+ */
+bool IsArticulatedEngine(EngineID engine_type)
+{
+	return HasBit(EngInfo(engine_type)->callback_mask, CBM_VEHICLE_ARTIC_ENGINE);
+}
+
+/**
+ * Count the number of articulated parts of an engine.
+ * @param engine_type The engine to get the number of parts of.
+ * @param purchase_window Whether we are in the scope of the purchase window or not, i.e. whether we cannot allocate vehicles.
+ * @return The number of parts.
+ */
 uint CountArticulatedParts(EngineID engine_type, bool purchase_window)
 {
 	if (!HasBit(EngInfo(engine_type)->callback_mask, CBM_VEHICLE_ARTIC_ENGINE)) return 0;
@@ -68,7 +98,7 @@ uint CountArticulatedParts(EngineID engine_type, bool purchase_window)
 
 /**
  * Returns the default (non-refitted) capacity of a specific EngineID.
- * @param engine the EngineID of iterest
+ * @param engine the EngineID of interest
  * @param cargo_type returns the default cargo type, if needed
  * @return capacity
  */
@@ -82,25 +112,30 @@ static inline uint16 GetVehicleDefaultCapacity(EngineID engine, CargoID *cargo_t
 }
 
 /**
- * Returns all cargos a vehicle can carry.
- * @param engine the EngineID of iterest
+ * Returns all cargoes a vehicle can carry.
+ * @param engine the EngineID of interest
  * @param include_initial_cargo_type if true the default cargo type of the vehicle is included; if false only the refit_mask
  * @return bit set of CargoIDs
  */
 static inline uint32 GetAvailableVehicleCargoTypes(EngineID engine, bool include_initial_cargo_type)
 {
-	uint32 cargos = 0;
-	CargoID initial_cargo_type;
+	const Engine *e = Engine::Get(engine);
+	if (!e->CanCarryCargo()) return 0;
 
-	if (GetVehicleDefaultCapacity(engine, &initial_cargo_type) > 0) {
-		const EngineInfo *ei = EngInfo(engine);
-		cargos = ei->refit_mask;
-		if (include_initial_cargo_type && initial_cargo_type < NUM_CARGO) SetBit(cargos, initial_cargo_type);
+	uint32 cargoes = e->info.refit_mask;
+
+	if (include_initial_cargo_type) {
+		SetBit(cargoes, e->GetDefaultCargoType());
 	}
 
-	return cargos;
+	return cargoes;
 }
 
+/**
+ * Get the capacity of the parts of a given engine.
+ * @param engine The engine to get the capacities from.
+ * @return The cargo capacities.
+ */
 CargoArray GetCapacityOfArticulatedParts(EngineID engine)
 {
 	CargoArray capacity;
@@ -159,9 +194,9 @@ bool IsArticulatedVehicleRefittable(EngineID engine)
 void GetArticulatedRefitMasks(EngineID engine, bool include_initial_cargo_type, uint32 *union_mask, uint32 *intersection_mask)
 {
 	const Engine *e = Engine::Get(engine);
-	uint32 veh_cargos = GetAvailableVehicleCargoTypes(engine, include_initial_cargo_type);
-	*union_mask = veh_cargos;
-	*intersection_mask = (veh_cargos != 0) ? veh_cargos : UINT32_MAX;
+	uint32 veh_cargoes = GetAvailableVehicleCargoTypes(engine, include_initial_cargo_type);
+	*union_mask = veh_cargoes;
+	*intersection_mask = (veh_cargoes != 0) ? veh_cargoes : UINT32_MAX;
 
 	if (!e->IsGroundVehicle()) return;
 	if (!HasBit(e->info.callback_mask, CBM_VEHICLE_ARTIC_ENGINE)) return;
@@ -170,9 +205,9 @@ void GetArticulatedRefitMasks(EngineID engine, bool include_initial_cargo_type,
 		EngineID artic_engine = GetNextArticulatedPart(i, engine);
 		if (artic_engine == INVALID_ENGINE) break;
 
-		veh_cargos = GetAvailableVehicleCargoTypes(artic_engine, include_initial_cargo_type);
-		*union_mask |= veh_cargos;
-		if (veh_cargos != 0) *intersection_mask &= veh_cargos;
+		veh_cargoes = GetAvailableVehicleCargoTypes(artic_engine, include_initial_cargo_type);
+		*union_mask |= veh_cargoes;
+		if (veh_cargoes != 0) *intersection_mask &= veh_cargoes;
 	}
 }
 
@@ -208,14 +243,14 @@ uint32 GetIntersectionOfArticulatedRefitMasks(EngineID engine, bool include_init
  * Note: Vehicles not carrying anything are ignored
  * @param v the first vehicle in the chain
  * @param cargo_type returns the common CargoID if needed. (CT_INVALID if no part is carrying something or they are carrying different things)
- * @return true if some parts are carrying different cargos, false if all parts are carrying the same (nothing is also the same)
+ * @return true if some parts are carrying different cargoes, false if all parts are carrying the same (nothing is also the same)
  */
-bool IsArticulatedVehicleCarryingDifferentCargos(const Vehicle *v, CargoID *cargo_type)
+bool IsArticulatedVehicleCarryingDifferentCargoes(const Vehicle *v, CargoID *cargo_type)
 {
 	CargoID first_cargo = CT_INVALID;
 
 	do {
-		if (v->cargo_cap > 0 && v->cargo_type != CT_INVALID) {
+		if (v->cargo_type != CT_INVALID && v->GetEngine()->CanCarryCargo()) {
 			if (first_cargo == CT_INVALID) first_cargo = v->cargo_type;
 			if (first_cargo != v->cargo_type) {
 				if (cargo_type != NULL) *cargo_type = CT_INVALID;
@@ -240,7 +275,7 @@ bool IsArticulatedVehicleCarryingDifferentCargos(const Vehicle *v, CargoID *carg
  */
 void CheckConsistencyOfArticulatedVehicle(const Vehicle *v)
 {
-	const Engine *engine = Engine::Get(v->engine_type);
+	const Engine *engine = v->GetEngine();
 
 	uint32 purchase_refit_union, purchase_refit_intersection;
 	GetArticulatedRefitMasks(v->engine_type, true, &purchase_refit_union, &purchase_refit_intersection);
@@ -261,7 +296,7 @@ void CheckConsistencyOfArticulatedVehicle(const Vehicle *v)
 		v = v->HasArticulatedPart() ? v->GetNextArticulatedPart() : NULL;
 	} while (v != NULL);
 
-	/* Check whether the vehicle carries more cargos than expected */
+	/* Check whether the vehicle carries more cargoes than expected */
 	bool carries_more = false;
 	for (CargoID cid = 0; cid < NUM_CARGO; cid++) {
 		if (real_default_capacity[cid] != 0 && purchase_default_capacity[cid] == 0) {
@@ -276,6 +311,10 @@ void CheckConsistencyOfArticulatedVehicle(const Vehicle *v)
 	}
 }
 
+/**
+ * Add the remaining articulated parts to the given vehicle.
+ * @param first The head of the articulated bit.
+ */
 void AddArticulatedParts(Vehicle *first)
 {
 	VehicleType type = first->type;
@@ -367,6 +406,6 @@ void AddArticulatedParts(Vehicle *first)
 
 		if (flip_image) v->spritenum++;
 
-		VehicleMove(v, false);
+		VehicleUpdatePosition(v);
 	}
 }
