@@ -1,4 +1,4 @@
-/* $Id: base_media_func.h 20608 2010-08-24 00:03:26Z rubidium $ */
+/* $Id: base_media_func.h 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -7,7 +7,10 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file base_media_func.h Generic function implementations for base data (graphics, sounds). */
+/**
+ * @file base_media_func.h Generic function implementations for base data (graphics, sounds).
+ * @note You should _never_ include this file due to the SET_TYPE define.
+ */
 
 #include "base_media_base.h"
 #include "debug.h"
@@ -31,8 +34,16 @@ template <class Tbase_set> /* static */ Tbase_set *BaseMedia<Tbase_set>::duplica
 		return false; \
 	}
 
-template <class T, size_t Tnum_files, Subdirectory Tsubdir>
-bool BaseSet<T, Tnum_files, Tsubdir>::FillSetDetails(IniFile *ini, const char *path, const char *full_filename, bool allow_empty_filename)
+/**
+ * Read the set information from a loaded ini.
+ * @param ini      the ini to read from
+ * @param path     the path to this ini file (for filenames)
+ * @param full_filename the full filename of the loaded file (for error reporting purposes)
+ * @param allow_empty_filename empty filenames are valid
+ * @return true if loading was successful.
+ */
+template <class T, size_t Tnum_files, bool Tsearch_in_tars>
+bool BaseSet<T, Tnum_files, Tsearch_in_tars>::FillSetDetails(IniFile *ini, const char *path, const char *full_filename, bool allow_empty_filename)
 {
 	memset(this, 0, sizeof(*this));
 
@@ -70,9 +81,9 @@ bool BaseSet<T, Tnum_files, Tsubdir>::FillSetDetails(IniFile *ini, const char *p
 	for (uint i = 0; i < Tnum_files; i++) {
 		MD5File *file = &this->files[i];
 		/* Find the filename first. */
-		item = files->GetItem(BaseSet<T, Tnum_files, Tsubdir>::file_names[i], false);
+		item = files->GetItem(BaseSet<T, Tnum_files, Tsearch_in_tars>::file_names[i], false);
 		if (item == NULL || (item->value == NULL && !allow_empty_filename)) {
-			DEBUG(grf, 0, "No " SET_TYPE " file for: %s (in %s)", BaseSet<T, Tnum_files, Tsubdir>::file_names[i], full_filename);
+			DEBUG(grf, 0, "No " SET_TYPE " file for: %s (in %s)", BaseSet<T, Tnum_files, Tsearch_in_tars>::file_names[i], full_filename);
 			return false;
 		}
 
@@ -123,15 +134,19 @@ bool BaseSet<T, Tnum_files, Tsubdir>::FillSetDetails(IniFile *ini, const char *p
 			file->missing_warning = strdup(item->value);
 		}
 
-		switch (file->CheckMD5(Tsubdir)) {
+		switch (T::CheckMD5(file, BASESET_DIR)) {
 			case MD5File::CR_MATCH:
 				this->valid_files++;
-				/* FALL THROUGH */
+				this->found_files++;
+				break;
+
 			case MD5File::CR_MISMATCH:
+				DEBUG(grf, 1, "MD5 checksum mismatch for: %s (in %s)", filename, full_filename);
 				this->found_files++;
 				break;
 
 			case MD5File::CR_NO_FILE:
+				DEBUG(grf, 1, "The file %s specified in %s is missing", filename, full_filename);
 				break;
 		}
 	}
@@ -140,14 +155,14 @@ bool BaseSet<T, Tnum_files, Tsubdir>::FillSetDetails(IniFile *ini, const char *p
 }
 
 template <class Tbase_set>
-bool BaseMedia<Tbase_set>::AddFile(const char *filename, size_t basepath_length)
+bool BaseMedia<Tbase_set>::AddFile(const char *filename, size_t basepath_length, const char *tar_filename)
 {
 	bool ret = false;
 	DEBUG(grf, 1, "Checking %s for base " SET_TYPE " set", filename);
 
 	Tbase_set *set = new Tbase_set();
 	IniFile *ini = new IniFile();
-	ini->LoadFromDisk(filename);
+	ini->LoadFromDisk(filename, BASESET_DIR);
 
 	char *path = strdup(filename + basepath_length);
 	char *psep = strrchr(path, PATHSEPCHAR);
@@ -169,7 +184,8 @@ bool BaseMedia<Tbase_set>::AddFile(const char *filename, size_t basepath_length)
 			/* The more complete set takes precedence over the version number. */
 			if ((duplicate->valid_files == set->valid_files && duplicate->version >= set->version) ||
 					duplicate->valid_files > set->valid_files) {
-				DEBUG(grf, 1, "Not adding %s (%i) as base " SET_TYPE " set (duplicate)", set->name, set->version);
+				DEBUG(grf, 1, "Not adding %s (%i) as base " SET_TYPE " set (duplicate, %s)", set->name, set->version,
+						duplicate->valid_files > set->valid_files ? "less valid files" : "lower version");
 				set->next = BaseMedia<Tbase_set>::duplicate_sets;
 				BaseMedia<Tbase_set>::duplicate_sets = set;
 			} else {
@@ -184,7 +200,8 @@ bool BaseMedia<Tbase_set>::AddFile(const char *filename, size_t basepath_length)
 				 * version number until a new game is started which isn't a big problem */
 				if (BaseMedia<Tbase_set>::used_set == duplicate) BaseMedia<Tbase_set>::used_set = set;
 
-				DEBUG(grf, 1, "Removing %s (%i) as base " SET_TYPE " set (duplicate)", duplicate->name, duplicate->version);
+				DEBUG(grf, 1, "Removing %s (%i) as base " SET_TYPE " set (duplicate, %s)", duplicate->name, duplicate->version,
+						duplicate->valid_files < set->valid_files ? "less valid files" : "lower version");
 				duplicate->next = BaseMedia<Tbase_set>::duplicate_sets;
 				BaseMedia<Tbase_set>::duplicate_sets = duplicate;
 				ret = true;
@@ -208,6 +225,11 @@ bool BaseMedia<Tbase_set>::AddFile(const char *filename, size_t basepath_length)
 	return ret;
 }
 
+/**
+ * Set the set to be used.
+ * @param name of the set to use
+ * @return true if it could be loaded
+ */
 template <class Tbase_set>
 /* static */ bool BaseMedia<Tbase_set>::SetSet(const char *name)
 {
@@ -229,6 +251,12 @@ template <class Tbase_set>
 	return false;
 }
 
+/**
+ * Returns a list with the sets.
+ * @param p    where to print to
+ * @param last the last character to print to
+ * @return the last printed character
+ */
 template <class Tbase_set>
 /* static */ char *BaseMedia<Tbase_set>::GetSetsList(char *p, const char *last)
 {
@@ -241,7 +269,7 @@ template <class Tbase_set>
 			if (missing == 0) {
 				p += seprintf(p, last, " (%i corrupt file%s)\n", invalid, invalid == 1 ? "" : "s");
 			} else {
-				p += seprintf(p, last, " (unuseable: %i missing file%s)\n", missing, missing == 1 ? "" : "s");
+				p += seprintf(p, last, " (unusable: %i missing file%s)\n", missing, missing == 1 ? "" : "s");
 			}
 		} else {
 			p += seprintf(p, last, "\n");
@@ -255,13 +283,13 @@ template <class Tbase_set>
 #if defined(ENABLE_NETWORK)
 #include "network/network_content.h"
 
-template <class Tbase_set> bool HasBaseSet(const ContentInfo *ci, bool md5sum, const Tbase_set *s)
+template <class Tbase_set> const char *TryGetBaseSetFile(const ContentInfo *ci, bool md5sum, const Tbase_set *s)
 {
 	for (; s != NULL; s = s->next) {
 		if (s->GetNumMissing() != 0) continue;
 
 		if (s->shortname != ci->unique_id) continue;
-		if (!md5sum) return true;
+		if (!md5sum) return  s->files[0].filename;
 
 		byte md5[16];
 		memset(md5, 0, sizeof(md5));
@@ -270,21 +298,26 @@ template <class Tbase_set> bool HasBaseSet(const ContentInfo *ci, bool md5sum, c
 				md5[j] ^= s->files[i].hash[j];
 			}
 		}
-		if (memcmp(md5, ci->md5sum, sizeof(md5)) == 0) return true;
+		if (memcmp(md5, ci->md5sum, sizeof(md5)) == 0) return s->files[0].filename;
 	}
-
-	return false;
+	return NULL;
 }
 
 template <class Tbase_set>
 /* static */ bool BaseMedia<Tbase_set>::HasSet(const ContentInfo *ci, bool md5sum)
 {
-	return HasBaseSet(ci, md5sum, BaseMedia<Tbase_set>::available_sets) ||
-			HasBaseSet(ci, md5sum, BaseMedia<Tbase_set>::duplicate_sets);
+	return (TryGetBaseSetFile(ci, md5sum, BaseMedia<Tbase_set>::available_sets) != NULL) ||
+			(TryGetBaseSetFile(ci, md5sum, BaseMedia<Tbase_set>::duplicate_sets) != NULL);
 }
 
 #else
 
+template <class Tbase_set>
+const char *TryGetBaseSetFile(const ContentInfo *ci, bool md5sum, const Tbase_set *s)
+{
+	return NULL;
+}
+
 template <class Tbase_set>
 /* static */ bool BaseMedia<Tbase_set>::HasSet(const ContentInfo *ci, bool md5sum)
 {
@@ -293,6 +326,10 @@ template <class Tbase_set>
 
 #endif /* ENABLE_NETWORK */
 
+/**
+ * Count the number of available graphics sets.
+ * @return the number of sets
+ */
 template <class Tbase_set>
 /* static */ int BaseMedia<Tbase_set>::GetNumSets()
 {
@@ -304,6 +341,10 @@ template <class Tbase_set>
 	return n;
 }
 
+/**
+ * Get the index of the currently active graphics set
+ * @return the current set's index
+ */
 template <class Tbase_set>
 /* static */ int BaseMedia<Tbase_set>::GetIndexOfUsedSet()
 {
@@ -316,6 +357,10 @@ template <class Tbase_set>
 	return -1;
 }
 
+/**
+ * Get the name of the graphics set at the specified index
+ * @return the name of the set
+ */
 template <class Tbase_set>
 /* static */ const Tbase_set *BaseMedia<Tbase_set>::GetSet(int index)
 {
@@ -327,12 +372,26 @@ template <class Tbase_set>
 	error("Base" SET_TYPE "::GetSet(): index %d out of range", index);
 }
 
+/**
+ * Return the used set.
+ * @return the used set.
+ */
 template <class Tbase_set>
 /* static */ const Tbase_set *BaseMedia<Tbase_set>::GetUsedSet()
 {
 	return BaseMedia<Tbase_set>::used_set;
 }
 
+/**
+ * Return the available sets.
+ * @return The available sets.
+ */
+template <class Tbase_set>
+/* static */ Tbase_set *BaseMedia<Tbase_set>::GetAvailableSets()
+{
+	return BaseMedia<Tbase_set>::available_sets;
+}
+
 /**
  * Force instantiation of methods so we don't get linker errors.
  * @param repl_type the type of the BaseMedia to instantiate
@@ -341,7 +400,7 @@ template <class Tbase_set>
 #define INSTANTIATE_BASE_MEDIA_METHODS(repl_type, set_type) \
 	template const char *repl_type::ini_set; \
 	template const char *repl_type::GetExtension(); \
-	template bool repl_type::AddFile(const char *filename, size_t pathlength); \
+	template bool repl_type::AddFile(const char *filename, size_t pathlength, const char *tar_filename); \
 	template bool repl_type::HasSet(const struct ContentInfo *ci, bool md5sum); \
 	template bool repl_type::SetSet(const char *name); \
 	template char *repl_type::GetSetsList(char *p, const char *last); \
@@ -349,5 +408,7 @@ template <class Tbase_set>
 	template int repl_type::GetIndexOfUsedSet(); \
 	template const set_type *repl_type::GetSet(int index); \
 	template const set_type *repl_type::GetUsedSet(); \
-	template bool repl_type::DetermineBestSet();
+	template bool repl_type::DetermineBestSet(); \
+	template set_type *repl_type::GetAvailableSets(); \
+	template const char *TryGetBaseSetFile(const ContentInfo *ci, bool md5sum, const set_type *s);
 
