@@ -1,4 +1,4 @@
-/* $Id: 32bpp_optimized.hpp 21890 2011-01-22 14:52:20Z rubidium $ */
+/* $Id: 32bpp_optimized.hpp 25987 2013-11-13 21:53:40Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,21 +14,24 @@
 
 #include "32bpp_simple.hpp"
 
+/** The optimised 32 bpp blitter (without palette animation). */
 class Blitter_32bppOptimized : public Blitter_32bppSimple {
 public:
+	/** Data stored about a (single) sprite. */
 	struct SpriteData {
-		uint32 offset[ZOOM_LVL_COUNT][2];
-		byte data[];
+		uint32 offset[ZOOM_LVL_COUNT][2]; ///< Offsets (from .data) to streams for different zoom levels, and the normal and remap image information.
+		byte data[];                      ///< Data, all zoomlevels.
 	};
 
 	/* virtual */ void Draw(Blitter::BlitterParams *bp, BlitterMode mode, ZoomLevel zoom);
-	/* virtual */ Sprite *Encode(SpriteLoader::Sprite *sprite, AllocatorProc *allocator);
+	/* virtual */ Sprite *Encode(const SpriteLoader::Sprite *sprite, AllocatorProc *allocator);
 
 	/* virtual */ const char *GetName() { return "32bpp-optimized"; }
 
 	template <BlitterMode mode> void Draw(const Blitter::BlitterParams *bp, ZoomLevel zoom);
 };
 
+/** Factory for the optimised 32 bpp blitter (without palette animation). */
 class FBlitter_32bppOptimized: public BlitterFactory<FBlitter_32bppOptimized> {
 public:
 	/* virtual */ const char *GetName() { return "32bpp-optimized"; }
