@@ -1,4 +1,4 @@
-/* $Id: 8bpp_simple.hpp 21782 2011-01-14 15:34:33Z frosch $ */
+/* $Id: 8bpp_simple.hpp 25987 2013-11-13 21:53:40Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,14 +15,16 @@
 #include "8bpp_base.hpp"
 #include "factory.hpp"
 
-class Blitter_8bppSimple : public Blitter_8bppBase {
+/** Most trivial 8bpp blitter. */
+class Blitter_8bppSimple FINAL : public Blitter_8bppBase {
 public:
 	/* virtual */ void Draw(Blitter::BlitterParams *bp, BlitterMode mode, ZoomLevel zoom);
-	/* virtual */ Sprite *Encode(SpriteLoader::Sprite *sprite, AllocatorProc *allocator);
+	/* virtual */ Sprite *Encode(const SpriteLoader::Sprite *sprite, AllocatorProc *allocator);
 
 	/* virtual */ const char *GetName() { return "8bpp-simple"; }
 };
 
+/** Factory for the most trivial 8bpp blitter. */
 class FBlitter_8bppSimple: public BlitterFactory<FBlitter_8bppSimple> {
 public:
 	/* virtual */ const char *GetName() { return "8bpp-simple"; }
