@@ -1,4 +1,4 @@
-/* $Id: base.hpp 21782 2011-01-14 15:34:33Z frosch $ */
+/* $Id: base.hpp 25987 2013-11-13 21:53:40Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,10 +15,11 @@
 #include "../spritecache.h"
 #include "../spriteloader/spriteloader.hpp"
 
+/** The modes of blitting we can do. */
 enum BlitterMode {
-	BM_NORMAL,
-	BM_COLOUR_REMAP,
-	BM_TRANSPARENT,
+	BM_NORMAL,       ///< Perform the simple blitting.
+	BM_COLOUR_REMAP, ///< Perform a colour remapping.
+	BM_TRANSPARENT,  ///< Perform transparency colour remapping.
 };
 
 /**
@@ -26,20 +27,25 @@ enum BlitterMode {
  */
 class Blitter {
 public:
+	/** Parameters related to blitting. */
 	struct BlitterParams {
-		const void *sprite;      ///< Pointer to the sprite how ever the encoder stored it
-		const byte *remap;       ///< XXX -- Temporary storage for remap array
+		const void *sprite; ///< Pointer to the sprite how ever the encoder stored it
+		const byte *remap;  ///< XXX -- Temporary storage for remap array
 
-		int skip_left, skip_top; ///< How much pixels of the source to skip on the left and top (based on zoom of dst)
-		int width, height;       ///< The width and height in pixels that needs to be drawn to dst
-		int sprite_width;        ///< Real width of the sprite
-		int sprite_height;       ///< Real height of the sprite
-		int left, top;           ///< The offset in the 'dst' in pixels to start drawing
+		int skip_left;      ///< How much pixels of the source to skip on the left (based on zoom of dst)
+		int skip_top;       ///< How much pixels of the source to skip on the top (based on zoom of dst)
+		int width;          ///< The width in pixels that needs to be drawn to dst
+		int height;         ///< The height in pixels that needs to be drawn to dst
+		int sprite_width;   ///< Real width of the sprite
+		int sprite_height;  ///< Real height of the sprite
+		int left;           ///< The left offset in the 'dst' in pixels to start drawing
+		int top;            ///< The top offset in the 'dst' in pixels to start drawing
 
-		void *dst;               ///< Destination buffer
-		int pitch;               ///< The pitch of the destination buffer
+		void *dst;          ///< Destination buffer
+		int pitch;          ///< The pitch of the destination buffer
 	};
 
+	/** Types of palette animation. */
 	enum PaletteAnimation {
 		PALETTE_ANIMATION_NONE,           ///< No palette animation
 		PALETTE_ANIMATION_VIDEO_BACKEND,  ///< Palette animation should be done by video backend (8bpp only!)
@@ -71,7 +77,7 @@ public:
 	/**
 	 * Convert a sprite from the loader to our own format.
 	 */
-	virtual Sprite *Encode(SpriteLoader::Sprite *sprite, AllocatorProc *allocator) = 0;
+	virtual Sprite *Encode(const SpriteLoader::Sprite *sprite, AllocatorProc *allocator) = 0;
 
 	/**
 	 * Move the destination pointer the requested amount x and y, keeping in mind
@@ -81,7 +87,7 @@ public:
 	 * @param y How much you want to scroll to the bottom.
 	 * @return A new destination pointer moved the the requested place.
 	 */
-	virtual void *MoveTo(const void *video, int x, int y) = 0;
+	virtual void *MoveTo(void *video, int x, int y) = 0;
 
 	/**
 	 * Draw a pixel with a given colour on the video-buffer.
@@ -111,12 +117,13 @@ public:
 	 * @param screen_width The width of the screen you are drawing in (to avoid buffer-overflows).
 	 * @param screen_height The height of the screen you are drawing in (to avoid buffer-overflows).
 	 * @param colour A 8bpp mapping colour.
+	 * @param width Line width.
 	 */
-	virtual void DrawLine(void *video, int x, int y, int x2, int y2, int screen_width, int screen_height, uint8 colour) = 0;
+	virtual void DrawLine(void *video, int x, int y, int x2, int y2, int screen_width, int screen_height, uint8 colour, int width);
 
 	/**
 	 * Copy from a buffer to the screen.
-	 * @param video The destionation pointer (video-buffer).
+	 * @param video The destination pointer (video-buffer).
 	 * @param src The buffer from which the data will be read.
 	 * @param width The width of the buffer.
 	 * @param height The height of the buffer.
@@ -166,11 +173,10 @@ public:
 
 	/**
 	 * Called when the 8bpp palette is changed; you should redraw all pixels on the screen that
-	 *  are equal to the 8bpp palette indexes 'start' to 'start + count'.
-	 * @param start The start index in the 8bpp palette.
-	 * @param count The amount of indexes that are (possible) changed.
+	 *  are equal to the 8bpp palette indexes 'first_dirty' to 'first_dirty + count_dirty'.
+	 * @param palette The new palette.
 	 */
-	virtual void PaletteAnimate(uint start, uint count) = 0;
+	virtual void PaletteAnimate(const Palette &palette) = 0;
 
 	/**
 	 * Check if the blitter uses palette animation at all.
