@@ -1,4 +1,4 @@
-/* $Id: command_type.h 22654 2011-07-10 20:35:31Z rubidium $ */
+/* $Id: command_type.h 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -58,7 +58,7 @@ public:
 	 * Adds the given cost to the cost of the command.
 	 * @param cost the cost to add
 	 */
-	FORCEINLINE void AddCost(const Money &cost)
+	inline void AddCost(const Money &cost)
 	{
 		this->cost += cost;
 	}
@@ -69,7 +69,7 @@ public:
 	 * Multiplies the cost of the command by the given factor.
 	 * @param factor factor to multiply the costs with
 	 */
-	FORCEINLINE void MultiplyCost(int factor)
+	inline void MultiplyCost(int factor)
 	{
 		this->cost *= factor;
 	}
@@ -78,7 +78,7 @@ public:
 	 * The costs as made up to this moment
 	 * @return the costs
 	 */
-	FORCEINLINE Money GetCost() const
+	inline Money GetCost() const
 	{
 		return this->cost;
 	}
@@ -87,7 +87,7 @@ public:
 	 * The expense type of the cost
 	 * @return the expense type
 	 */
-	FORCEINLINE ExpensesType GetExpensesType() const
+	inline ExpensesType GetExpensesType() const
 	{
 		return this->expense_type;
 	}
@@ -137,7 +137,7 @@ public:
 	 * Did this command succeed?
 	 * @return true if and only if it succeeded
 	 */
-	FORCEINLINE bool Succeeded() const
+	inline bool Succeeded() const
 	{
 		return this->success;
 	}
@@ -146,7 +146,7 @@ public:
 	 * Did this command fail?
 	 * @return true if and only if it failed
 	 */
-	FORCEINLINE bool Failed() const
+	inline bool Failed() const
 	{
 		return !this->success;
 	}
@@ -248,17 +248,26 @@ enum Commands {
 	CMD_FOUND_TOWN,                   ///< found a town
 	CMD_RENAME_TOWN,                  ///< rename a town
 	CMD_DO_TOWN_ACTION,               ///< do a action from the town detail window (like advertises or bribe)
+	CMD_TOWN_CARGO_GOAL,              ///< set the goal of a cargo for a town
+	CMD_TOWN_GROWTH_RATE,             ///< set the town growth rate
+	CMD_TOWN_SET_TEXT,                ///< set the custom text of a town
 	CMD_EXPAND_TOWN,                  ///< expand a town
 	CMD_DELETE_TOWN,                  ///< delete a town
 
-	CMD_ORDER_REFIT,                  ///< change the refit informaction of an order (for "goto depot" )
+	CMD_ORDER_REFIT,                  ///< change the refit information of an order (for "goto depot" )
 	CMD_CLONE_ORDER,                  ///< clone (and share) an order
 	CMD_CLEAR_AREA,                   ///< clear an area
 
 	CMD_MONEY_CHEAT,                  ///< do the money cheat
 	CMD_BUILD_CANAL,                  ///< build a canal
 
+	CMD_CREATE_SUBSIDY,               ///< create a new subsidy
 	CMD_COMPANY_CTRL,                 ///< used in multiplayer to create a new companies etc.
+	CMD_CUSTOM_NEWS_ITEM,             ///< create a custom news message
+	CMD_CREATE_GOAL,                  ///< create a new goal
+	CMD_REMOVE_GOAL,                  ///< remove a goal
+	CMD_GOAL_QUESTION,                ///< ask a goal related question
+	CMD_GOAL_QUESTION_ANSWER,         ///< answer(s) to CMD_GOAL_QUESTION
 	CMD_LEVEL_LAND,                   ///< level land
 
 	CMD_BUILD_LOCK,                   ///< build a lock
@@ -268,7 +277,7 @@ enum Commands {
 
 	CMD_GIVE_MONEY,                   ///< give money to another company
 	CMD_CHANGE_SETTING,               ///< change a setting
-	CMD_CHANGE_COMPANY_SETTING,       ///< change a company etting
+	CMD_CHANGE_COMPANY_SETTING,       ///< change a company setting
 
 	CMD_SET_AUTOREPLACE,              ///< set an autoreplace entry
 
@@ -293,7 +302,9 @@ enum Commands {
 	CMD_AUTOFILL_TIMETABLE,           ///< autofill the timetable
 	CMD_SET_TIMETABLE_START,          ///< set the date that a timetable should start
 
-	CMD_END                           ///< Must ALWAYS be on the end of this list!! (period)
+	CMD_OPEN_CLOSE_AIRPORT,           ///< open/close an airport to incoming aircraft
+
+	CMD_END,                          ///< Must ALWAYS be on the end of this list!! (period)
 };
 
 /**
@@ -311,9 +322,10 @@ enum DoCommandFlag {
 	DC_NO_TEST_TOWN_RATING   = 0x020, ///< town rating does not disallow you from building
 	DC_BANKRUPT              = 0x040, ///< company bankrupts, skip money check, skip vehicle on tile check in some cases
 	DC_AUTOREPLACE           = 0x080, ///< autoreplace/autorenew is in progress, this shall disable vehicle limits when building, and ignore certain restrictions when undoing things (like vehicle attach callback)
-	DC_ALL_TILES             = 0x100, ///< allow this command also on MP_VOID tiles
-	DC_NO_MODIFY_TOWN_RATING = 0x200, ///< do not change town rating
-	DC_FORCE_CLEAR_TILE      = 0x400, ///< do not only remove the object on the tile, but also clear any water left on it
+	DC_NO_CARGO_CAP_CHECK    = 0x100, ///< when autoreplace/autorenew is in progress, this shall prevent truncating the amount of cargo in the vehicle to prevent testing the command to remove cargo
+	DC_ALL_TILES             = 0x200, ///< allow this command also on MP_VOID tiles
+	DC_NO_MODIFY_TOWN_RATING = 0x400, ///< do not change town rating
+	DC_FORCE_CLEAR_TILE      = 0x800, ///< do not only remove the object on the tile, but also clear any water left on it
 };
 DECLARE_ENUM_AS_BIT_SET(DoCommandFlag)
 
@@ -335,7 +347,6 @@ DECLARE_ENUM_AS_BIT_SET(DoCommandFlag)
  */
 enum FlaggedCommands {
 	CMD_NETWORK_COMMAND       = 0x0100, ///< execute the command without sending it on the network
-	CMD_NO_TEST_IF_IN_NETWORK = 0x0200, ///< When enabled, the command will bypass the no-DC_EXEC round if in network
 	CMD_FLAGS_MASK            = 0xFF00, ///< mask for all command flags
 	CMD_ID_MASK               = 0x00FF, ///< mask for the command ID
 };
@@ -346,15 +357,18 @@ enum FlaggedCommands {
  * This enumeration defines flags for the _command_proc_table.
  */
 enum CommandFlags {
-	CMD_SERVER    = 0x01, ///< the command can only be initiated by the server
-	CMD_SPECTATOR = 0x02, ///< the command may be initiated by a spectator
-	CMD_OFFLINE   = 0x04, ///< the command cannot be executed in a multiplayer game; single-player only
-	CMD_AUTO      = 0x08, ///< set the DC_AUTO flag on this command
-	CMD_ALL_TILES = 0x10, ///< allow this command also on MP_VOID tiles
-	CMD_NO_TEST   = 0x20, ///< the command's output may differ between test and execute due to town rating changes etc.
-	CMD_NO_WATER  = 0x40, ///< set the DC_NO_WATER flag on this command
-	CMD_CLIENT_ID = 0x80, ///< set p2 with the ClientID of the sending client.
+	CMD_SERVER    = 0x001, ///< the command can only be initiated by the server
+	CMD_SPECTATOR = 0x002, ///< the command may be initiated by a spectator
+	CMD_OFFLINE   = 0x004, ///< the command cannot be executed in a multiplayer game; single-player only
+	CMD_AUTO      = 0x008, ///< set the DC_AUTO flag on this command
+	CMD_ALL_TILES = 0x010, ///< allow this command also on MP_VOID tiles
+	CMD_NO_TEST   = 0x020, ///< the command's output may differ between test and execute due to town rating changes etc.
+	CMD_NO_WATER  = 0x040, ///< set the DC_NO_WATER flag on this command
+	CMD_CLIENT_ID = 0x080, ///< set p2 with the ClientID of the sending client.
+	CMD_DEITY     = 0x100, ///< the command may be executed by COMPANY_DEITY
+	CMD_STR_CTRL  = 0x200, ///< the command's string may contain control strings
 };
+DECLARE_ENUM_AS_BIT_SET(CommandFlags)
 
 /** Types of commands we have. */
 enum CommandType {
@@ -406,10 +420,10 @@ typedef CommandCost CommandProc(TileIndex tile, DoCommandFlag flags, uint32 p1,
  * the #CMD_AUTO, #CMD_OFFLINE and #CMD_SERVER values.
  */
 struct Command {
-	CommandProc *proc; ///< The procedure to actually executing
-	const char *name;  ///< A human readable name for the procedure
-	byte flags;        ///< The (command) flags to that apply to this command
-	CommandType type;  ///< The type of command.
+	CommandProc *proc;  ///< The procedure to actually executing
+	const char *name;   ///< A human readable name for the procedure
+	CommandFlags flags; ///< The (command) flags to that apply to this command
+	CommandType type;   ///< The type of command.
 };
 
 /**
