@@ -1,4 +1,4 @@
-/* $Id: company_cmd.cpp 20431 2010-08-09 21:53:44Z rubidium $ */
+/* $Id: company_cmd.cpp 25500 2013-06-28 19:29:08Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -10,7 +10,7 @@
 /** @file company_cmd.cpp Handling of companies. */
 
 #include "stdafx.h"
-#include "engine_base.h"
+#include "company_base.h"
 #include "company_func.h"
 #include "company_gui.h"
 #include "town.h"
@@ -20,51 +20,54 @@
 #include "network/network.h"
 #include "network/network_func.h"
 #include "network/network_base.h"
+#include "network/network_admin.h"
 #include "ai/ai.hpp"
 #include "company_manager_face.h"
-#include "group.h"
 #include "window_func.h"
 #include "strings_func.h"
-#include "gfx_func.h"
 #include "date_func.h"
 #include "sound_func.h"
-#include "autoreplace_func.h"
-#include "autoreplace_gui.h"
 #include "rail.h"
 #include "core/pool_func.hpp"
 #include "settings_func.h"
 #include "vehicle_base.h"
 #include "vehicle_func.h"
-#include "sprite.h"
+#include "smallmap_gui.h"
+#include "game/game.hpp"
 
 #include "table/strings.h"
 
-CompanyByte _local_company;
-CompanyByte _current_company;
-/* NOSAVE: can be determined from company structs */
-Colours _company_colours[MAX_COMPANIES];
+CompanyByte _local_company;   ///< Company controlled by the human player at this client. Can also be #COMPANY_SPECTATOR.
+CompanyByte _current_company; ///< Company currently doing an action.
+Colours _company_colours[MAX_COMPANIES];  ///< NOSAVE: can be determined from company structs.
 CompanyManagerFace _company_manager_face; ///< for company manager face storage in openttd.cfg
 uint _next_competitor_start;              ///< the number of ticks before the next AI is started
 uint _cur_company_tick_index;             ///< used to generate a name for one company that doesn't have a name yet per tick
 
-CompanyPool _company_pool("Company");
+CompanyPool _company_pool("Company"); ///< Pool of companies.
 INSTANTIATE_POOL_METHODS(Company)
 
-Company::Company(uint16 name_1, bool is_ai) :
-	name_1(name_1),
-	location_of_HQ(INVALID_TILE),
-	is_ai(is_ai)
+/**
+ * Constructor.
+ * @param name_1 Name of the company.
+ * @param is_ai  A computer program is running for this company.
+ */
+Company::Company(uint16 name_1, bool is_ai)
 {
+	this->name_1 = name_1;
+	this->location_of_HQ = INVALID_TILE;
+	this->is_ai = is_ai;
+	this->terraform_limit = _settings_game.construction.terraform_frame_burst << 16;
+	this->clear_limit     = _settings_game.construction.clear_frame_burst << 16;
+	this->tree_limit      = _settings_game.construction.tree_frame_burst << 16;
+
 	for (uint j = 0; j < 4; j++) this->share_owners[j] = COMPANY_SPECTATOR;
 	InvalidateWindowData(WC_PERFORMANCE_DETAIL, 0, INVALID_COMPANY);
 }
 
+/** Destructor. */
 Company::~Company()
 {
-	free(this->name);
-	free(this->president_name);
-	free(this->num_engines);
-
 	if (CleaningPool()) return;
 
 	DeleteCompanyWindows(this->index);
@@ -79,7 +82,7 @@ void Company::PostDestructor(size_t index)
 	InvalidateWindowData(WC_GRAPH_LEGEND, 0, (int)index);
 	InvalidateWindowData(WC_PERFORMANCE_DETAIL, 0, (int)index);
 	InvalidateWindowData(WC_COMPANY_LEAGUE, 0, 0);
-	/* If the currently shown error message has this company in it, the close it. */
+	/* If the currently shown error message has this company in it, then close it. */
 	InvalidateWindowData(WC_ERRMSG, 0);
 }
 
@@ -99,23 +102,35 @@ void SetLocalCompany(CompanyID new_company)
 	InvalidateWindowData(WC_SEND_NETWORK_MSG, DESTTYPE_TEAM, _local_company);
 #endif
 
-	_local_company = new_company;
+	assert(IsLocalCompany());
+
+	_current_company = _local_company = new_company;
 
 	/* Delete any construction windows... */
 	DeleteConstructionWindows();
 
 	/* ... and redraw the whole screen. */
 	MarkWholeScreenDirty();
+	InvalidateWindowClassesData(WC_SIGN_LIST, -1);
 }
 
-uint16 GetDrawStringCompanyColour(CompanyID company)
+/**
+ * Get the colour for DrawString-subroutines which matches the colour of the company.
+ * @param company Company to get the colour of.
+ * @return Colour of \a company.
+ */
+TextColour GetDrawStringCompanyColour(CompanyID company)
 {
-	/* Get the colour for DrawString-subroutines which matches the colour
-	 * of the company */
-	if (!Company::IsValidID(company)) return _colour_gradient[COLOUR_WHITE][4] | IS_PALETTE_COLOUR;
-	return (_colour_gradient[_company_colours[company]][4]) | IS_PALETTE_COLOUR;
+	if (!Company::IsValidID(company)) return (TextColour)_colour_gradient[COLOUR_WHITE][4] | TC_IS_PALETTE_COLOUR;
+	return (TextColour)_colour_gradient[_company_colours[company]][4] | TC_IS_PALETTE_COLOUR;
 }
 
+/**
+ * Draw the icon of a company.
+ * @param c Company that needs its icon drawn.
+ * @param x Horizontal coordinate of the icon.
+ * @param y Vertical coordinate of the icon.
+ */
 void DrawCompanyIcon(CompanyID c, int x, int y)
 {
 	DrawSprite(SPR_COMPANY_ICON, COMPANY_SPRITE_COLOUR(c), x, y);
@@ -127,7 +142,7 @@ void DrawCompanyIcon(CompanyID c, int x, int y)
  * @param cmf the fact to check
  * @return true if and only if the face is valid
  */
-bool IsValidCompanyManagerFace(CompanyManagerFace cmf)
+static bool IsValidCompanyManagerFace(CompanyManagerFace cmf)
 {
 	if (!AreCompanyManagerFaceBitsValid(cmf, CMFV_GEN_ETHN, GE_WM)) return false;
 
@@ -152,6 +167,10 @@ bool IsValidCompanyManagerFace(CompanyManagerFace cmf)
 	return true;
 }
 
+/**
+ * Refresh all windows owned by a company.
+ * @param company Company that changed, and needs its windows refreshed.
+ */
 void InvalidateCompanyWindows(const Company *company)
 {
 	CompanyID cid = company->index;
@@ -160,6 +179,11 @@ void InvalidateCompanyWindows(const Company *company)
 	SetWindowDirty(WC_FINANCES, cid);
 }
 
+/**
+ * Verify whether the company can pay the bill.
+ * @param cost [inout] Money to pay, is changed to an error if the company does not have enough money.
+ * @return Function returns \c true if the company has enough money, else it returns \c false.
+ */
 bool CheckCompanyHasMoney(CommandCost &cost)
 {
 	if (cost.GetCost() > 0) {
@@ -173,6 +197,11 @@ bool CheckCompanyHasMoney(CommandCost &cost)
 	return true;
 }
 
+/**
+ * Deduct costs of a command from the money of a company.
+ * @param c Company to pay the bill.
+ * @param cost Money to pay.
+ */
 static void SubtractMoneyFromAnyCompany(Company *c, CommandCost cost)
 {
 	if (cost.GetCost() == 0) return;
@@ -198,12 +227,21 @@ static void SubtractMoneyFromAnyCompany(Company *c, CommandCost cost)
 	InvalidateCompanyWindows(c);
 }
 
+/**
+ * Subtract money from the #_current_company, if the company is valid.
+ * @param cost Money to pay.
+ */
 void SubtractMoneyFromCompany(CommandCost cost)
 {
 	Company *c = Company::GetIfValid(_current_company);
 	if (c != NULL) SubtractMoneyFromAnyCompany(c, cost);
 }
 
+/**
+ * Subtract money from a company, including the money fraction.
+ * @param company Company paying the bill.
+ * @param cst     Cost of a command.
+ */
 void SubtractMoneyFromCompanyFract(CompanyID company, CommandCost cst)
 {
 	Company *c = Company::Get(company);
@@ -216,6 +254,17 @@ void SubtractMoneyFromCompanyFract(CompanyID company, CommandCost cst)
 	if (cost != 0) SubtractMoneyFromAnyCompany(c, CommandCost(cst.GetExpensesType(), cost));
 }
 
+/** Update the landscaping limits per company. */
+void UpdateLandscapingLimits()
+{
+	Company *c;
+	FOR_ALL_COMPANIES(c) {
+		c->terraform_limit = min(c->terraform_limit + _settings_game.construction.terraform_per_64k_frames, (uint32)_settings_game.construction.terraform_frame_burst << 16);
+		c->clear_limit     = min(c->clear_limit     + _settings_game.construction.clear_per_64k_frames,     (uint32)_settings_game.construction.clear_frame_burst << 16);
+		c->tree_limit      = min(c->tree_limit      + _settings_game.construction.tree_per_64k_frames,      (uint32)_settings_game.construction.tree_frame_burst << 16);
+	}
+}
+
 /**
  * Set the right DParams to get the name of an owner.
  * @param owner the owner to get the name of.
@@ -249,17 +298,17 @@ void GetNameOfOwner(Owner owner, TileIndex tile)
  * @param owner the owner of the thing to check.
  * @param tile  optional tile to get the right town.
  * @pre if tile == 0 then the owner can't be OWNER_TOWN.
- * @return true iff it's owned by the current company.
+ * @return A succeeded command iff it's owned by the current company, else a failed command.
  */
-bool CheckOwnership(Owner owner, TileIndex tile)
+CommandCost CheckOwnership(Owner owner, TileIndex tile)
 {
 	assert(owner < OWNER_END);
 	assert(owner != OWNER_TOWN || tile != 0);
 
-	if (owner == _current_company) return true;
-	_error_message = STR_ERROR_OWNED_BY;
+	if (owner == _current_company) return CommandCost();
+
 	GetNameOfOwner(owner, tile);
-	return false;
+	return_cmd_error(STR_ERROR_OWNED_BY);
 }
 
 /**
@@ -267,52 +316,51 @@ bool CheckOwnership(Owner owner, TileIndex tile)
  * the given tile.  If that isn't the case an
  * appropriate error will be given.
  * @param tile the tile to check.
- * @return true iff it's owned by the current company.
+ * @return A succeeded command iff it's owned by the current company, else a failed command.
  */
-bool CheckTileOwnership(TileIndex tile)
+CommandCost CheckTileOwnership(TileIndex tile)
 {
 	Owner owner = GetTileOwner(tile);
 
 	assert(owner < OWNER_END);
 
-	if (owner == _current_company) return true;
-	_error_message = STR_ERROR_OWNED_BY;
+	if (owner == _current_company) return CommandCost();
 
 	/* no need to get the name of the owner unless we're the local company (saves some time) */
 	if (IsLocalCompany()) GetNameOfOwner(owner, tile);
-	return false;
+	return_cmd_error(STR_ERROR_OWNED_BY);
 }
 
+/**
+ * Generate the name of a company from the last build coordinate.
+ * @param c Company to give a name.
+ */
 static void GenerateCompanyName(Company *c)
 {
-	TileIndex tile;
-	Town *t;
-	StringID str;
-	Company *cc;
-	uint32 strp;
 	/* Reserve space for extra unicode character. We need to do this to be able
 	 * to detect too long company name. */
-	char buffer[MAX_LENGTH_COMPANY_NAME_BYTES + MAX_CHAR_LENGTH];
+	char buffer[(MAX_LENGTH_COMPANY_NAME_CHARS + 1) * MAX_CHAR_LENGTH];
 
 	if (c->name_1 != STR_SV_UNNAMED) return;
+	if (c->last_build_coordinate == 0) return;
 
-	tile = c->last_build_coordinate;
-	if (tile == 0) return;
-
-	t = ClosestTownFromTile(tile, UINT_MAX);
+	Town *t = ClosestTownFromTile(c->last_build_coordinate, UINT_MAX);
 
+	StringID str;
+	uint32 strp;
 	if (t->name == NULL && IsInsideMM(t->townnametype, SPECSTR_TOWNNAME_START, SPECSTR_TOWNNAME_LAST + 1)) {
-		str = t->townnametype - SPECSTR_TOWNNAME_START + SPECSTR_PLAYERNAME_START;
+		str = t->townnametype - SPECSTR_TOWNNAME_START + SPECSTR_COMPANY_NAME_START;
 		strp = t->townnameparts;
 
 verify_name:;
 		/* No companies must have this name already */
+		Company *cc;
 		FOR_ALL_COMPANIES(cc) {
 			if (cc->name_1 == str && cc->name_2 == strp) goto bad_town_name;
 		}
 
 		GetString(buffer, str, lastof(buffer));
-		if (strlen(buffer) >= MAX_LENGTH_COMPANY_NAME_BYTES) goto bad_town_name;
+		if (Utf8StringLength(buffer) >= MAX_LENGTH_COMPANY_NAME_CHARS) goto bad_town_name;
 
 set_name:;
 		c->name_1 = str;
@@ -327,9 +375,8 @@ set_name:;
 			SetDParam(1, STR_NEWS_COMPANY_LAUNCH_DESCRIPTION);
 			SetDParamStr(2, cni->company_name);
 			SetDParam(3, t->index);
-			AddNewsItem(STR_MESSAGE_NEWS_FORMAT, NS_COMPANY_NEW, NR_TILE, c->last_build_coordinate, NR_NONE, UINT32_MAX, cni);
+			AddNewsItem(STR_MESSAGE_NEWS_FORMAT, NT_COMPANY_INFO, NF_COMPANY, NR_TILE, c->last_build_coordinate, NR_NONE, UINT32_MAX, cni);
 		}
-		AI::BroadcastNewEvent(new AIEventCompanyNew(c->index), c->index);
 		return;
 	}
 bad_town_name:;
@@ -345,7 +392,9 @@ bad_town_name:;
 	}
 }
 
+/** Sorting weights for the company colours. */
 static const byte _colour_sort[COLOUR_END] = {2, 2, 3, 2, 3, 2, 3, 2, 3, 2, 2, 2, 3, 1, 1, 1};
+/** Similar colours, so we can try to prevent same coloured companies. */
 static const Colours _similar_colour[COLOUR_END][2] = {
 	{ COLOUR_BLUE,       COLOUR_LIGHT_BLUE }, // COLOUR_DARK_BLUE
 	{ COLOUR_GREEN,      COLOUR_DARK_GREEN }, // COLOUR_PALE_GREEN
@@ -365,6 +414,10 @@ static const Colours _similar_colour[COLOUR_END][2] = {
 	{ COLOUR_GREY,       INVALID_COLOUR    }, // COLOUR_WHITE
 };
 
+/**
+ * Generate a company colour.
+ * @return Generated company colour.
+ */
 static Colours GenerateCompanyColour()
 {
 	Colours colours[COLOUR_END];
@@ -385,7 +438,7 @@ static Colours GenerateCompanyColour()
 				Swap(colours[j - 1], colours[j]);
 			}
 		}
-	};
+	}
 
 	/* Move the colours that look similar to each company's colour to the side */
 	Company *c;
@@ -417,6 +470,10 @@ static Colours GenerateCompanyColour()
 	NOT_REACHED();
 }
 
+/**
+ * Generate a random president name of a company.
+ * @param c Company that needs a new president name.
+ */
 static void GeneratePresidentName(Company *c)
 {
 	for (;;) {
@@ -426,16 +483,16 @@ restart:;
 
 		/* Reserve space for extra unicode character. We need to do this to be able
 		 * to detect too long president name. */
-		char buffer[MAX_LENGTH_PRESIDENT_NAME_BYTES + MAX_CHAR_LENGTH];
+		char buffer[(MAX_LENGTH_PRESIDENT_NAME_CHARS + 1) * MAX_CHAR_LENGTH];
 		SetDParam(0, c->index);
 		GetString(buffer, STR_PRESIDENT_NAME, lastof(buffer));
-		if (strlen(buffer) >= MAX_LENGTH_PRESIDENT_NAME_BYTES) continue;
+		if (Utf8StringLength(buffer) >= MAX_LENGTH_PRESIDENT_NAME_CHARS) continue;
 
 		Company *cc;
 		FOR_ALL_COMPANIES(cc) {
 			if (c != cc) {
 				/* Reserve extra space so even overlength president names can be compared. */
-				char buffer2[MAX_LENGTH_PRESIDENT_NAME_BYTES + MAX_CHAR_LENGTH];
+				char buffer2[(MAX_LENGTH_PRESIDENT_NAME_CHARS + 1) * MAX_CHAR_LENGTH];
 				SetDParam(0, cc->index);
 				GetString(buffer2, STR_PRESIDENT_NAME, lastof(buffer2));
 				if (strcmp(buffer2, buffer) == 0) goto restart;
@@ -445,6 +502,11 @@ restart:;
 	}
 }
 
+/**
+ * Reset the livery schemes to the company's primary colour.
+ * This is used on loading games without livery information and on new company start up.
+ * @param c Company to reset.
+ */
 void ResetCompanyLivery(Company *c)
 {
 	for (LiveryScheme scheme = LS_BEGIN; scheme < LS_END; scheme++) {
@@ -457,7 +519,7 @@ void ResetCompanyLivery(Company *c)
 /**
  * Create a new company and sets all company variables default values
  *
- * @param is_ai is a ai company?
+ * @param is_ai is an AI company?
  * @param company CompanyID to use for the new company
  * @return the company struct
  */
@@ -481,36 +543,40 @@ Company *DoStartupNewCompany(bool is_ai, CompanyID company = INVALID_COMPANY)
 	ResetCompanyLivery(c);
 	_company_colours[c->index] = (Colours)c->colour;
 
-	c->money = c->current_loan = 100000;
+	c->money = c->current_loan = (100000ll * _economy.inflation_prices >> 16) / 50000 * 50000;
 
 	c->share_owners[0] = c->share_owners[1] = c->share_owners[2] = c->share_owners[3] = INVALID_OWNER;
 
 	c->avail_railtypes = GetCompanyRailtypes(c->index);
 	c->avail_roadtypes = GetCompanyRoadtypes(c->index);
 	c->inaugurated_year = _cur_year;
-	RandomCompanyManagerFaceBits(c->face, (GenderEthnicity)Random(), false); // create a random company manager face
+	RandomCompanyManagerFaceBits(c->face, (GenderEthnicity)Random(), false, false); // create a random company manager face
 
 	SetDefaultCompanySettings(c->index);
 
 	GeneratePresidentName(c);
 
 	SetWindowDirty(WC_GRAPH_LEGEND, 0);
-	SetWindowDirty(WC_TOOLBAR_MENU, 0);
+	SetWindowClassesDirty(WC_CLIENT_LIST_POPUP);
 	SetWindowDirty(WC_CLIENT_LIST, 0);
+	BuildOwnerLegend();
+	InvalidateWindowData(WC_SMALLMAP, 0, 1);
 
 	if (is_ai && (!_networking || _network_server)) AI::StartNew(c->index);
 
-	c->num_engines = CallocT<uint16>(Engine::GetPoolSize());
+	AI::BroadcastNewEvent(new ScriptEventCompanyNew(c->index), c->index);
+	Game::NewEvent(new ScriptEventCompanyNew(c->index));
 
 	return c;
 }
 
+/** Start the next competitor now. */
 void StartupCompanies()
 {
 	_next_competitor_start = 0;
 }
 
-#ifdef ENABLE_AI
+/** Start a new competitor company if possible. */
 static void MaybeStartNewCompany()
 {
 #ifdef ENABLE_NETWORK
@@ -528,14 +594,13 @@ static void MaybeStartNewCompany()
 	if (n < (uint)_settings_game.difficulty.max_no_competitors) {
 		/* Send a command to all clients to start up a new AI.
 		 * Works fine for Multiplayer and Singleplayer */
-		DoCommandP(0, 1, INVALID_COMPANY, CMD_COMPANY_CTRL);
+		DoCommandP(0, 1 | INVALID_COMPANY << 16, 0, CMD_COMPANY_CTRL);
 	}
 }
-#endif /* ENABLE_AI */
 
+/** Initialize the pool of companies. */
 void InitializeCompanies()
 {
-	_company_pool.CleanPool();
 	_cur_company_tick_index = 0;
 }
 
@@ -547,15 +612,14 @@ void InitializeCompanies()
  */
 bool MayCompanyTakeOver(CompanyID cbig, CompanyID csmall)
 {
-	uint big_counts[4], small_counts[4];
-	CountCompanyVehicles(cbig,   big_counts);
-	CountCompanyVehicles(csmall, small_counts);
+	const Company *c1 = Company::Get(cbig);
+	const Company *c2 = Company::Get(csmall);
 
 	/* Do the combined vehicle counts stay within the limits? */
-	return big_counts[VEH_TRAIN]     + small_counts[VEH_TRAIN]    <= _settings_game.vehicle.max_trains &&
-		big_counts[VEH_ROAD]     + small_counts[VEH_ROAD]     <= _settings_game.vehicle.max_roadveh &&
-		big_counts[VEH_SHIP]     + small_counts[VEH_SHIP]     <= _settings_game.vehicle.max_ships &&
-		big_counts[VEH_AIRCRAFT] + small_counts[VEH_AIRCRAFT] <= _settings_game.vehicle.max_aircraft;
+	return c1->group_all[VEH_TRAIN].num_vehicle + c2->group_all[VEH_TRAIN].num_vehicle <= _settings_game.vehicle.max_trains &&
+		c1->group_all[VEH_ROAD].num_vehicle     + c2->group_all[VEH_ROAD].num_vehicle     <= _settings_game.vehicle.max_roadveh &&
+		c1->group_all[VEH_SHIP].num_vehicle     + c2->group_all[VEH_SHIP].num_vehicle     <= _settings_game.vehicle.max_ships &&
+		c1->group_all[VEH_AIRCRAFT].num_vehicle + c2->group_all[VEH_AIRCRAFT].num_vehicle <= _settings_game.vehicle.max_aircraft;
 }
 
 /**
@@ -614,12 +678,13 @@ static void HandleBankruptcyTakeover(Company *c)
 
 	c->bankrupt_timeout = TAKE_OVER_TIMEOUT;
 	if (best->is_ai) {
-		AI::NewEvent(best->index, new AIEventCompanyAskMerger(c->index, ClampToI32(c->bankrupt_value)));
+		AI::NewEvent(best->index, new ScriptEventCompanyAskMerger(c->index, ClampToI32(c->bankrupt_value)));
 	} else if (IsInteractiveCompany(best->index)) {
 		ShowBuyCompanyDialog(c->index);
 	}
 }
 
+/** Called every tick for updating some company info. */
 void OnTick_Companies()
 {
 	if (_game_mode == GM_EDITOR) return;
@@ -630,7 +695,6 @@ void OnTick_Companies()
 		if (c->bankrupt_asked != 0) HandleBankruptcyTakeover(c);
 	}
 
-#ifdef ENABLE_AI
 	if (_next_competitor_start == 0) {
 		_next_competitor_start = AI::GetStartNextTime() * DAY_TICKS;
 	}
@@ -638,11 +702,14 @@ void OnTick_Companies()
 	if (AI::CanStartNew() && _game_mode != GM_MENU && --_next_competitor_start == 0) {
 		MaybeStartNewCompany();
 	}
-#endif /* ENABLE_AI */
 
 	_cur_company_tick_index = (_cur_company_tick_index + 1) % MAX_COMPANIES;
 }
 
+/**
+ * A year has passed, update the economic data of all companies, and perhaps show the
+ * financial overview window of the local company.
+ */
 void CompaniesYearlyLoop()
 {
 	Company *c;
@@ -658,55 +725,17 @@ void CompaniesYearlyLoop()
 		ShowCompanyFinances(_local_company);
 		c = Company::Get(_local_company);
 		if (c->num_valid_stat_ent > 5 && c->old_economy[0].performance_history < c->old_economy[4].performance_history) {
-			SndPlayFx(SND_01_BAD_YEAR);
+			if (_settings_client.sound.new_year) SndPlayFx(SND_01_BAD_YEAR);
 		} else {
-			SndPlayFx(SND_00_GOOD_YEAR);
+			if (_settings_client.sound.new_year) SndPlayFx(SND_00_GOOD_YEAR);
 		}
 	}
 }
 
-/** Change engine renewal parameters
- * @param tile unused
- * @param flags operation to perform
- * @param p1 packed data
- *   - bits 16-31 = engine group
- * @param p2 packed data
- *   - bits  0-15 = old engine type
- *   - bits 16-31 = new engine type
- * @param text unused
- * @return the cost of this operation or an error
- */
-CommandCost CmdSetAutoReplace(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
-{
-	Company *c = Company::GetIfValid(_current_company);
-	if (c == NULL) return CMD_ERROR;
-
-	EngineID old_engine_type = GB(p2, 0, 16);
-	EngineID new_engine_type = GB(p2, 16, 16);
-	GroupID id_g = GB(p1, 16, 16);
-	CommandCost cost;
-
-	if (!Group::IsValidID(id_g) && !IsAllGroupID(id_g) && !IsDefaultGroupID(id_g)) return CMD_ERROR;
-	if (!Engine::IsValidID(old_engine_type)) return CMD_ERROR;
-
-	if (new_engine_type != INVALID_ENGINE) {
-		if (!Engine::IsValidID(new_engine_type)) return CMD_ERROR;
-		if (!CheckAutoreplaceValidity(old_engine_type, new_engine_type, _current_company)) return CMD_ERROR;
-
-		cost = AddEngineReplacementForCompany(c, old_engine_type, new_engine_type, id_g, flags);
-	} else {
-		cost = RemoveEngineReplacementForCompany(c, old_engine_type, id_g, flags);
-	}
-
-	if ((flags & DC_EXEC) && IsLocalCompany()) InvalidateAutoreplaceWindow(old_engine_type, id_g);
-
-	return cost;
-}
-
 /**
  * Fill the CompanyNewsInformation struct with the required data.
  * @param c the current company.
- * @param other the other company.
+ * @param other the other company (use \c NULL if not relevant).
  */
 void CompanyNewsInformation::FillData(const Company *c, const Company *other)
 {
@@ -729,56 +758,70 @@ void CompanyNewsInformation::FillData(const Company *c, const Company *other)
 
 }
 
-/** Control the companies: add, delete, etc.
+/**
+ * Called whenever company related information changes in order to notify admins.
+ * @param company The company data changed of.
+ */
+void CompanyAdminUpdate(const Company *company)
+{
+#ifdef ENABLE_NETWORK
+	if (_network_server) NetworkAdminCompanyUpdate(company);
+#endif /* ENABLE_NETWORK */
+}
+
+/**
+ * Called whenever a company is removed in order to notify admins.
+ * @param company_id The company that was removed.
+ * @param reason     The reason the company was removed.
+ */
+void CompanyAdminRemove(CompanyID company_id, CompanyRemoveReason reason)
+{
+#ifdef ENABLE_NETWORK
+	if (_network_server) NetworkAdminCompanyRemove(company_id, (AdminCompanyRemoveReason)reason);
+#endif /* ENABLE_NETWORK */
+}
+
+/**
+ * Control the companies: add, delete, etc.
  * @param tile unused
  * @param flags operation to perform
  * @param p1 various functionality
- * - p1 = 0 - create a new company, Which company (network) it will be is in p2
- * - p1 = 1 - create a new AI company
- * - p1 = 2 - delete a company. Company is identified by p2
- * @param p2 various functionality, dictated by p1
- * - p1 = 0 - ClientID of the newly created client
- * - p1 = 1 - CompanyID to start AI (INVALID_COMPANY for first available)
- * - p1 = 2 - CompanyID of the that is getting deleted
+ * - bits 0..15:
+ *        = 0 - create a new company
+ *        = 1 - create a new AI company
+ *        = 2 - delete a company
+ * - bits 16..24: CompanyID
+ * @param p2 ClientID
  * @param text unused
  * @return the cost of this operation or an error
- *
- * @todo In the case of p1=0, create new company, the clientID of the new client is in parameter
- * p2. This parameter is passed in at function DEF_SERVER_RECEIVE_COMMAND(PACKET_CLIENT_COMMAND)
- * on the server itself. First of all this is unbelievably ugly; second of all, well,
- * it IS ugly! <b>Someone fix this up :)</b> So where to fix?@n
- * @arg - network_server.c:838 DEF_SERVER_RECEIVE_COMMAND(PACKET_CLIENT_COMMAND)@n
- * @arg - network_client.c:536 DEF_CLIENT_RECEIVE_COMMAND(PACKET_SERVER_MAP) from where the map has been received
  */
 CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	if (flags & DC_EXEC) _current_company = OWNER_NONE;
-
 	InvalidateWindowData(WC_COMPANY_LEAGUE, 0, 0);
+	CompanyID company_id = (CompanyID)GB(p1, 16, 8);
+#ifdef ENABLE_NETWORK
+	ClientID client_id = (ClientID)p2;
+#endif /* ENABLE_NETWORK */
 
-	switch (p1) {
+	switch (GB(p1, 0, 16)) {
 		case 0: { // Create a new company
 			/* This command is only executed in a multiplayer game */
 			if (!_networking) return CMD_ERROR;
 
 #ifdef ENABLE_NETWORK
-
-			/* Joining Client:
-			 * _local_company: COMPANY_SPECTATOR
-			 * cid = clientid
-			 *
-			 * Other client(s)/server:
-			 * _local_company: what they play as
-			 * cid = requested company/company of joining client */
-			ClientID cid = (ClientID)p2;
-
 			/* Has the network client a correct ClientIndex? */
 			if (!(flags & DC_EXEC)) return CommandCost();
-			NetworkClientInfo *ci = NetworkFindClientInfoFromClientID(cid);
+			NetworkClientInfo *ci = NetworkClientInfo::GetByClientID(client_id);
+#ifndef DEBUG_DUMP_COMMANDS
+			/* When replaying the client ID is not a valid client; there
+			 * are actually no clients at all. However, the company has to
+			 * be created, otherwise we cannot rerun the game properly.
+			 * So only allow a NULL client info in that case. */
 			if (ci == NULL) return CommandCost();
+#endif /* NOT DEBUG_DUMP_COMMANDS */
 
 			/* Delete multiplayer progress bar */
-			DeleteWindowById(WC_NETWORK_STATUS_WINDOW, 0);
+			DeleteWindowById(WC_NETWORK_STATUS_WINDOW, WN_NETWORK_STATUS_WINDOW_JOIN);
 
 			Company *c = DoStartupNewCompany(false);
 
@@ -792,15 +835,13 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 			}
 
 			/* This is the client (or non-dedicated server) who wants a new company */
-			if (cid == _network_own_client_id) {
+			if (client_id == _network_own_client_id) {
 				assert(_local_company == COMPANY_SPECTATOR);
 				SetLocalCompany(c->index);
 				if (!StrEmpty(_settings_client.network.default_company_pass)) {
-					NetworkChangeCompanyPassword(_settings_client.network.default_company_pass);
+					NetworkChangeCompanyPassword(_local_company, _settings_client.network.default_company_pass);
 				}
 
-				_current_company = _local_company;
-
 				/* Now that we have a new company, broadcast our company settings to
 				 * all clients so everything is in sync */
 				SyncCompanySettings();
@@ -809,17 +850,17 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 			}
 
 			if (_network_server) {
-				/* XXX - UGLY! p2 (pid) is mis-used to fetch the client-id, done at
-				 * server side in network_server.c:838, function
-				 * DEF_SERVER_RECEIVE_COMMAND(PACKET_CLIENT_COMMAND) */
-				CompanyID old_playas = ci->client_playas;
-				ci->client_playas = c->index;
-				NetworkUpdateClientInfo(ci->client_id);
+				if (ci != NULL) {
+					/* ci is only NULL when replaying.
+					 * When replaying no client is actually in need of an update. */
+					ci->client_playas = c->index;
+					NetworkUpdateClientInfo(ci->client_id);
+				}
 
-				if (Company::IsValidID(ci->client_playas)) {
+				if (Company::IsValidID(c->index)) {
 					_network_company_states[c->index].months_empty = 0;
 					_network_company_states[c->index].password[0] = '\0';
-					NetworkServerUpdateCompanyPassworded(ci->client_playas, false);
+					NetworkServerUpdateCompanyPassworded(c->index, false);
 
 					/* XXX - When a client joins, we automatically set its name to the
 					 * client's name (for some reason). As it stands now only the server
@@ -832,26 +873,41 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 					 * TODO: Perhaps this could be improved by when the client is ready
 					 * with joining to let it send itself the command, and not the server?
 					 * For example in network_client.c:534? */
-					NetworkSend_Command(0, 0, 0, CMD_RENAME_PRESIDENT, NULL, ci->client_name, ci->client_playas);
+					if (ci != NULL) {
+						/* ci is only NULL when replaying.
+						 * When replaying, the command to rename the president will
+						 * automatically be ran, so this is not even needed to get
+						 * the exact same state. */
+						NetworkSendCommand(0, 0, 0, CMD_RENAME_PRESIDENT, NULL, ci->client_name, c->index);
+					}
 				}
 
-				/* Announce new company on network, if the client was a SPECTATOR before */
-				if (old_playas == COMPANY_SPECTATOR) {
-					NetworkServerSendChat(NETWORK_ACTION_COMPANY_NEW, DESTTYPE_BROADCAST, 0, "", ci->client_id, ci->client_playas + 1);
+				/* Announce new company on network. */
+				NetworkAdminCompanyInfo(c, true);
+
+				if (ci != NULL) {
+					/* ci is only NULL when replaying.
+					 * When replaying, the message that someone started a new company
+					 * is not interesting at all. */
+					NetworkServerSendChat(NETWORK_ACTION_COMPANY_NEW, DESTTYPE_BROADCAST, 0, "", ci->client_id, c->index + 1);
 				}
 			}
 #endif /* ENABLE_NETWORK */
-		} break;
+			break;
+		}
 
 		case 1: // Make a new AI company
 			if (!(flags & DC_EXEC)) return CommandCost();
 
-			if (p2 != INVALID_COMPANY && (p2 >= MAX_COMPANIES || Company::IsValidID(p2))) return CMD_ERROR;
-			DoStartupNewCompany(true, (CompanyID)p2);
+			if (company_id != INVALID_COMPANY && (company_id >= MAX_COMPANIES || Company::IsValidID(company_id))) return CMD_ERROR;
+			DoStartupNewCompany(true, company_id);
 			break;
 
 		case 2: { // Delete a company
-			Company *c = Company::GetIfValid(p2);
+			CompanyRemoveReason reason = (CompanyRemoveReason)GB(p2, 0, 2);
+			if (reason >= CRR_END) return CMD_ERROR;
+
+			Company *c = Company::GetIfValid(company_id);
 			if (c == NULL) return CMD_ERROR;
 
 			if (!(flags & DC_EXEC)) return CommandCost();
@@ -865,7 +921,7 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 			SetDParam(0, STR_NEWS_COMPANY_BANKRUPT_TITLE);
 			SetDParam(1, STR_NEWS_COMPANY_BANKRUPT_DESCRIPTION);
 			SetDParamStr(2, cni->company_name);
-			AddCompanyNewsItem(STR_MESSAGE_NEWS_FORMAT, NS_COMPANY_BANKRUPT, cni);
+			AddCompanyNewsItem(STR_MESSAGE_NEWS_FORMAT, cni);
 
 			/* Remove the company */
 			ChangeOwnershipOfCompanyItems(c->index, INVALID_OWNER);
@@ -873,16 +929,24 @@ CommandCost CmdCompanyCtrl(TileIndex tile, DoCommandFlag flags, uint32 p1, uint3
 
 			CompanyID c_index = c->index;
 			delete c;
-			AI::BroadcastNewEvent(new AIEventCompanyBankrupt(c_index));
-		} break;
+			AI::BroadcastNewEvent(new ScriptEventCompanyBankrupt(c_index));
+			Game::NewEvent(new ScriptEventCompanyBankrupt(c_index));
+			CompanyAdminRemove(c_index, (CompanyRemoveReason)reason);
+			break;
+		}
 
 		default: return CMD_ERROR;
 	}
 
+	InvalidateWindowClassesData(WC_GAME_OPTIONS);
+	InvalidateWindowClassesData(WC_AI_SETTINGS);
+	InvalidateWindowClassesData(WC_AI_LIST);
+
 	return CommandCost();
 }
 
-/** Change the company manager's face.
+/**
+ * Change the company manager's face.
  * @param tile unused
  * @param flags operation to perform
  * @param p1 unused
@@ -903,7 +967,8 @@ CommandCost CmdSetCompanyManagerFace(TileIndex tile, DoCommandFlag flags, uint32
 	return CommandCost();
 }
 
-/** Change the company's company-colour
+/**
+ * Change the company's company-colour
  * @param tile unused
  * @param flags operation to perform
  * @param p1 bitstuffed:
@@ -941,6 +1006,7 @@ CommandCost CmdSetCompanyColour(TileIndex tile, DoCommandFlag flags, uint32 p1,
 				if (scheme == LS_DEFAULT) {
 					_company_colours[_current_company] = colour;
 					c->colour = colour;
+					CompanyAdminUpdate(c);
 				}
 				break;
 
@@ -986,16 +1052,27 @@ CommandCost CmdSetCompanyColour(TileIndex tile, DoCommandFlag flags, uint32 p1,
 		InvalidateWindowData(WC_DELIVERED_CARGO, 0);
 		InvalidateWindowData(WC_PERFORMANCE_HISTORY, 0);
 		InvalidateWindowData(WC_COMPANY_VALUE, 0);
+		/* The smallmap owner view also stores the company colours. */
+		BuildOwnerLegend();
+		InvalidateWindowData(WC_SMALLMAP, 0, 1);
 
 		/* Company colour data is indirectly cached. */
 		Vehicle *v;
 		FOR_ALL_VEHICLES(v) {
 			if (v->owner == _current_company) v->InvalidateNewGRFCache();
 		}
+
+		extern void UpdateObjectColours(const Company *c);
+		UpdateObjectColours(c);
 	}
 	return CommandCost();
 }
 
+/**
+ * Is the given name in use as name of a company?
+ * @param name Name to search.
+ * @return \c true if the name us unique (that is, not in use), else \c false.
+ */
 static bool IsUniqueCompanyName(const char *name)
 {
 	const Company *c;
@@ -1007,7 +1084,8 @@ static bool IsUniqueCompanyName(const char *name)
 	return true;
 }
 
-/** Change the name of the company.
+/**
+ * Change the name of the company.
  * @param tile unused
  * @param flags operation to perform
  * @param p1 unused
@@ -1020,7 +1098,7 @@ CommandCost CmdRenameCompany(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 	bool reset = StrEmpty(text);
 
 	if (!reset) {
-		if (strlen(text) >= MAX_LENGTH_COMPANY_NAME_BYTES) return CMD_ERROR;
+		if (Utf8StringLength(text) >= MAX_LENGTH_COMPANY_NAME_CHARS) return CMD_ERROR;
 		if (!IsUniqueCompanyName(text)) return_cmd_error(STR_ERROR_NAME_MUST_BE_UNIQUE);
 	}
 
@@ -1029,11 +1107,17 @@ CommandCost CmdRenameCompany(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 		free(c->name);
 		c->name = reset ? NULL : strdup(text);
 		MarkWholeScreenDirty();
+		CompanyAdminUpdate(c);
 	}
 
 	return CommandCost();
 }
 
+/**
+ * Is the given name in use as president name of a company?
+ * @param name Name to search.
+ * @return \c true if the name us unique (that is, not in use), else \c false.
+ */
 static bool IsUniquePresidentName(const char *name)
 {
 	const Company *c;
@@ -1045,7 +1129,8 @@ static bool IsUniquePresidentName(const char *name)
 	return true;
 }
 
-/** Change the name of the president.
+/**
+ * Change the name of the president.
  * @param tile unused
  * @param flags operation to perform
  * @param p1 unused
@@ -1058,7 +1143,7 @@ CommandCost CmdRenamePresident(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 	bool reset = StrEmpty(text);
 
 	if (!reset) {
-		if (strlen(text) >= MAX_LENGTH_PRESIDENT_NAME_BYTES) return CMD_ERROR;
+		if (Utf8StringLength(text) >= MAX_LENGTH_PRESIDENT_NAME_CHARS) return CMD_ERROR;
 		if (!IsUniquePresidentName(text)) return_cmd_error(STR_ERROR_NAME_MUST_BE_UNIQUE);
 	}
 
@@ -1080,7 +1165,26 @@ CommandCost CmdRenamePresident(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 		}
 
 		MarkWholeScreenDirty();
+		CompanyAdminUpdate(c);
 	}
 
 	return CommandCost();
 }
+
+/**
+ * Get the service interval for the given company and vehicle type.
+ * @param c The company, or NULL for client-default settings.
+ * @param type The vehicle type to get the interval for.
+ * @return The service interval.
+ */
+int CompanyServiceInterval(const Company *c, VehicleType type)
+{
+	const VehicleDefaultSettings *vds = (c == NULL) ? &_settings_client.company.vehicle : &c->settings.vehicle;
+	switch (type) {
+		default: NOT_REACHED();
+		case VEH_TRAIN:    return vds->servint_trains;
+		case VEH_ROAD:     return vds->servint_roadveh;
+		case VEH_AIRCRAFT: return vds->servint_aircraft;
+		case VEH_SHIP:     return vds->servint_ships;
+	}
+}
