@@ -1,4 +1,4 @@
-/* $Id: console_gui.cpp 21982 2011-02-05 17:54:36Z frosch $ */
+/* $Id: console_gui.cpp 26024 2013-11-17 13:35:48Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -10,7 +10,7 @@
 /** @file console_gui.cpp Handling the GUI of the in-game console. */
 
 #include "stdafx.h"
-#include "textbuf_gui.h"
+#include "textbuf_type.h"
 #include "window_gui.h"
 #include "console_gui.h"
 #include "console_internal.h"
@@ -21,6 +21,9 @@
 #include "settings_type.h"
 #include "console_func.h"
 #include "rev.h"
+#include "video/video_driver.hpp"
+
+#include "widgets/console_widget.h"
 
 #include "table/strings.h"
 
@@ -124,9 +127,9 @@ struct IConsoleLine {
 
 
 /* ** main console cmd buffer ** */
-static Textbuf _iconsole_cmdline;
+static Textbuf _iconsole_cmdline(ICON_CMDLN_SIZE);
 static char *_iconsole_history[ICON_HISTORY_SIZE];
-static byte _iconsole_historypos;
+static int _iconsole_historypos;
 IConsoleModes _iconsole_mode;
 
 /* *************** *
@@ -145,20 +148,15 @@ static void IConsoleClearCommand()
 
 static inline void IConsoleResetHistoryPos()
 {
-	_iconsole_historypos = ICON_HISTORY_SIZE - 1;
+	_iconsole_historypos = -1;
 }
 
 
 static const char *IConsoleHistoryAdd(const char *cmd);
 static void IConsoleHistoryNavigate(int direction);
 
-/** Widgets of the console window. */
-enum ConsoleWidgets {
-	CW_BACKGROUND, ///< Background of the console
-};
-
 static const struct NWidgetPart _nested_console_window_widgets[] = {
-	NWidget(WWT_EMPTY, INVALID_COLOUR, CW_BACKGROUND), SetResize(1, 1),
+	NWidget(WWT_EMPTY, INVALID_COLOUR, WID_C_BACKGROUND), SetResize(1, 1),
 };
 
 static const WindowDesc _console_window_desc(
@@ -187,6 +185,7 @@ struct IConsoleWindow : Window
 	~IConsoleWindow()
 	{
 		_iconsole_mode = ICONSOLE_CLOSED;
+		_video_driver->EditBoxLostFocus();
 	}
 
 	/**
@@ -204,7 +203,7 @@ struct IConsoleWindow : Window
 	{
 		const int right = this->width - 5;
 
-		GfxFillRect(0, 0, this->width - 1, this->height - 1, 0);
+		GfxFillRect(0, 0, this->width - 1, this->height - 1, PC_BLACK);
 		int ypos = this->height - this->line_height;
 		for (const IConsoleLine *print = IConsoleLine::Get(IConsoleWindow::scroll); print != NULL; print = print->previous) {
 			SetDParamStr(0, print->buffer);
@@ -218,6 +217,9 @@ struct IConsoleWindow : Window
 			delta = 0;
 		}
 
+		/* If we have a marked area, draw a background highlight. */
+		if (_iconsole_cmdline.marklength != 0) GfxFillRect(this->line_offset + delta + _iconsole_cmdline.markxoffs, this->height - this->line_height, this->line_offset + delta + _iconsole_cmdline.markxoffs + _iconsole_cmdline.marklength, this->height - 1, PC_DARK_RED);
+
 		DrawString(this->line_offset + delta, right, this->height - this->line_height, _iconsole_cmdline.buf, (TextColour)CC_COMMAND, SA_LEFT | SA_FORCE);
 
 		if (_focused_window == this && _iconsole_cmdline.caret) {
@@ -236,10 +238,10 @@ struct IConsoleWindow : Window
 
 	virtual void OnMouseLoop()
 	{
-		if (HandleCaret(&_iconsole_cmdline)) this->SetDirty();
+		if (_iconsole_cmdline.HandleCaret()) this->SetDirty();
 	}
 
-	virtual EventState OnKeyPress(uint16 key, uint16 keycode)
+	virtual EventState OnKeyPress(WChar key, uint16 keycode)
 	{
 		if (_focused_window != this) return ES_NOT_HANDLED;
 
@@ -293,46 +295,13 @@ struct IConsoleWindow : Window
 				MarkWholeScreenDirty();
 				break;
 
-#ifdef WITH_COCOA
-			case (WKC_META | 'V'):
-#endif
-			case (WKC_CTRL | 'V'):
-				if (InsertTextBufferClipboard(&_iconsole_cmdline)) {
-					IConsoleResetHistoryPos();
-					this->SetDirty();
-				}
-				break;
-
 			case (WKC_CTRL | 'L'):
 				IConsoleCmdExec("clear");
 				break;
 
-#ifdef WITH_COCOA
-			case (WKC_META | 'U'):
-#endif
-			case (WKC_CTRL | 'U'):
-				DeleteTextBufferAll(&_iconsole_cmdline);
-				this->SetDirty();
-				break;
-
-			case WKC_BACKSPACE: case WKC_DELETE:
-				if (DeleteTextBufferChar(&_iconsole_cmdline, keycode)) {
-					IConsoleResetHistoryPos();
-					this->SetDirty();
-				}
-				break;
-
-			case WKC_LEFT: case WKC_RIGHT: case WKC_END: case WKC_HOME:
-				if (MoveTextBufferPos(&_iconsole_cmdline, keycode)) {
-					IConsoleResetHistoryPos();
-					this->SetDirty();
-				}
-				break;
-
 			default:
-				if (IsValidChar(key, CS_ALPHANUMERAL)) {
+				if (_iconsole_cmdline.HandleKeyPress(key, keycode) != HKPR_NOT_HANDLED) {
 					IConsoleWindow::scroll = 0;
-					InsertTextBufferChar(&_iconsole_cmdline, key);
 					IConsoleResetHistoryPos();
 					this->SetDirty();
 				} else {
@@ -343,26 +312,82 @@ struct IConsoleWindow : Window
 		return ES_HANDLED;
 	}
 
+	virtual void InsertTextString(int wid, const char *str, bool marked, const char *caret, const char *insert_location, const char *replacement_end)
+	{
+		if (_iconsole_cmdline.InsertString(str, marked, caret, insert_location, replacement_end)) {
+			IConsoleWindow::scroll = 0;
+			IConsoleResetHistoryPos();
+			this->SetDirty();
+		}
+	}
+
+	virtual const char *GetFocusedText() const
+	{
+		return _iconsole_cmdline.buf;
+	}
+
+	virtual const char *GetCaret() const
+	{
+		return _iconsole_cmdline.buf + _iconsole_cmdline.caretpos;
+	}
+
+	virtual const char *GetMarkedText(size_t *length) const
+	{
+		if (_iconsole_cmdline.markend == 0) return NULL;
+
+		*length = _iconsole_cmdline.markend - _iconsole_cmdline.markpos;
+		return _iconsole_cmdline.buf + _iconsole_cmdline.markpos;
+	}
+
+	virtual Point GetCaretPosition() const
+	{
+		int delta = min(this->width - this->line_offset - _iconsole_cmdline.pixels - ICON_RIGHT_BORDERWIDTH, 0);
+		Point pt = {this->line_offset + delta + _iconsole_cmdline.caretxoffs, this->height - this->line_height};
+
+		return pt;
+	}
+
+	virtual Rect GetTextBoundingRect(const char *from, const char *to) const
+	{
+		int delta = min(this->width - this->line_offset - _iconsole_cmdline.pixels - ICON_RIGHT_BORDERWIDTH, 0);
+
+		Point p1 = GetCharPosInString(_iconsole_cmdline.buf, from, FS_NORMAL);
+		Point p2 = from != to ? GetCharPosInString(_iconsole_cmdline.buf, from) : p1;
+
+		Rect r = {this->line_offset + delta + p1.x, this->height - this->line_height, this->line_offset + delta + p2.x, this->height};
+		return r;
+	}
+
+	virtual const char *GetTextCharacterAtPosition(const Point &pt) const
+	{
+		int delta = min(this->width - this->line_offset - _iconsole_cmdline.pixels - ICON_RIGHT_BORDERWIDTH, 0);
+
+		if (!IsInsideMM(pt.y, this->height - this->line_height, this->height)) return NULL;
+
+		return GetCharAtPosition(_iconsole_cmdline.buf, pt.x - delta);
+	}
+
 	virtual void OnMouseWheel(int wheel)
 	{
 		this->Scroll(-wheel);
 	}
+
+	virtual void OnFocusLost()
+	{
+		_video_driver->EditBoxLostFocus();
+	}
 };
 
 int IConsoleWindow::scroll = 0;
 
 void IConsoleGUIInit()
 {
-	_iconsole_historypos = ICON_HISTORY_SIZE - 1;
+	IConsoleResetHistoryPos();
 	_iconsole_mode = ICONSOLE_CLOSED;
 
 	IConsoleLine::Reset();
 	memset(_iconsole_history, 0, sizeof(_iconsole_history));
 
-	_iconsole_cmdline.buf = CallocT<char>(ICON_CMDLN_SIZE); // create buffer and zero it
-	_iconsole_cmdline.max_bytes = ICON_CMDLN_SIZE;
-	_iconsole_cmdline.max_chars = ICON_CMDLN_SIZE;
-
 	IConsolePrintF(CC_WARNING, "OpenTTD Game Console Revision 7 - %s", _openttd_revision);
 	IConsolePrint(CC_WHITE,  "------------------------------------");
 	IConsolePrint(CC_WHITE,  "use \"help\" for more information");
@@ -377,7 +402,6 @@ void IConsoleClearBuffer()
 
 void IConsoleGUIFree()
 {
-	free(_iconsole_cmdline.buf);
 	IConsoleClearBuffer();
 }
 
@@ -454,26 +478,15 @@ static const char *IConsoleHistoryAdd(const char *cmd)
 static void IConsoleHistoryNavigate(int direction)
 {
 	if (_iconsole_history[0] == NULL) return; // Empty history
-	int i = _iconsole_historypos + direction;
+	_iconsole_historypos = Clamp(_iconsole_historypos + direction, -1, ICON_HISTORY_SIZE - 1);
 
-	/* watch out for overflows, just wrap around */
-	if (i < 0) i = ICON_HISTORY_SIZE - 1;
-	if ((uint)i >= ICON_HISTORY_SIZE) i = 0;
+	if (direction > 0 && _iconsole_history[_iconsole_historypos] == NULL) _iconsole_historypos--;
 
-	if (direction > 0) {
-		if (_iconsole_history[i] == NULL) i = 0;
+	if (_iconsole_historypos == -1) {
+		_iconsole_cmdline.DeleteAll();
+	} else {
+		_iconsole_cmdline.Assign(_iconsole_history[_iconsole_historypos]);
 	}
-
-	if (direction < 0) {
-		while (i > 0 && _iconsole_history[i] == NULL) i--;
-	}
-
-	_iconsole_historypos = i;
-	IConsoleClearCommand();
-	/* copy history to 'command prompt / bash' */
-	assert(_iconsole_history[i] != NULL && IsInsideMM(i, 0, ICON_HISTORY_SIZE));
-	ttd_strlcpy(_iconsole_cmdline.buf, _iconsole_history[i], _iconsole_cmdline.max_bytes);
-	UpdateTextBufferSize(&_iconsole_cmdline);
 }
 
 /**
