@@ -1,4 +1,4 @@
-/* $Id: bitmath_func.hpp 19019 2010-02-05 17:05:58Z smatz $ */
+/* $Id: bitmath_func.hpp 26024 2013-11-17 13:35:48Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -13,28 +13,31 @@
 #define BITMATH_FUNC_HPP
 
 /**
- * Fetch n bits from x, started at bit s.
+ * Fetch \a n bits from \a x, started at bit \a s.
  *
- * This function can be used to fetch n bits from the value x. The
- * s value set the startposition to read. The startposition is
- * count from the LSB and starts at 0. The result starts at a
+ * This function can be used to fetch \a n bits from the value \a x. The
+ * \a s value set the start position to read. The start position is
+ * count from the LSB and starts at \c 0. The result starts at a
  * LSB, as this isn't just an and-bitmask but also some
  * bit-shifting operations. GB(0xFF, 2, 1) will so
  * return 0x01 (0000 0001) instead of
  * 0x04 (0000 0100).
  *
  * @param x The value to read some bits.
- * @param s The startposition to read some bits.
+ * @param s The start position to read some bits.
  * @param n The number of bits to read.
+ * @pre n < sizeof(T) * 8
+ * @pre s + n <= sizeof(T) * 8
  * @return The selected bits, aligned to a LSB.
  */
 template <typename T>
-static FORCEINLINE uint GB(const T x, const uint8 s, const uint8 n)
+static inline uint GB(const T x, const uint8 s, const uint8 n)
 {
-	return (x >> s) & ((1U << n) - 1);
+	return (x >> s) & (((T)1U << n) - 1);
 }
 
-/** Set \a n bits in \a x starting at bit \a s to \a d
+/**
+ * Set \a n bits in \a x starting at bit \a s to \a d
  *
  * This function sets \a n bits from \a x which started as bit \a s to the value of
  * \a d. The parameters \a x, \a s and \a n works the same as the parameters of
@@ -43,40 +46,45 @@ static FORCEINLINE uint GB(const T x, const uint8 s, const uint8 n)
  * This is not a bug, its a feature.
  *
  * @note Parameter \a x must be a variable as the result is saved there.
- * @note To avoid unexpecting results the value of \a d should not use more
+ * @note To avoid unexpected results the value of \a d should not use more
  *       space as the provided space of \a n bits (log2)
  * @param x The variable to change some bits
- * @param s The startposition for the new bits
+ * @param s The start position for the new bits
  * @param n The size/window for the new bits
  * @param d The actually new bits to save in the defined position.
+ * @pre n < sizeof(T) * 8
+ * @pre s + n <= sizeof(T) * 8
  * @return The new value of \a x
  */
 template <typename T, typename U>
-static FORCEINLINE T SB(T &x, const uint8 s, const uint8 n, const U d)
+static inline T SB(T &x, const uint8 s, const uint8 n, const U d)
 {
-	x &= (T)(~(((1U << n) - 1) << s));
+	x &= (T)(~((((T)1U << n) - 1) << s));
 	x |= (T)(d << s);
 	return x;
 }
 
-/** Add i to n bits of x starting at bit s.
+/**
+ * Add \a i to \a n bits of \a x starting at bit \a s.
  *
- * This add the value of i on n bits of x starting at bit s. The parameters x,
- * s, i are similar to #GB besides x must be a variable as the result are
+ * This adds the value of \a i on \a n bits of \a x starting at bit \a s. The parameters \a x,
+ * \a s, \a i are similar to #GB. Besides, \ a x must be a variable as the result are
  * saved there. An overflow does not affect the following bits of the given
  * bit window and is simply ignored.
  *
  * @note Parameter x must be a variable as the result is saved there.
  * @param x The variable to add some bits at some position
- * @param s The startposition of the addition
+ * @param s The start position of the addition
  * @param n The size/window for the addition
- * @param i The value to add at the given startposition in the given window.
- * @return The new value of x
+ * @pre n < sizeof(T) * 8
+ * @pre s + n <= sizeof(T) * 8
+ * @param i The value to add at the given start position in the given window.
+ * @return The new value of \a x
  */
 template <typename T, typename U>
-static FORCEINLINE T AB(T &x, const uint8 s, const uint8 n, const U i)
+static inline T AB(T &x, const uint8 s, const uint8 n, const U i)
 {
-	const T mask = (T)(((1U << n) - 1) << s);
+	const T mask = ((((T)1U << n) - 1) << s);
 	x = (T)((x & ~mask) | ((x + (i << s)) & mask));
 	return x;
 }
@@ -85,15 +93,16 @@ static FORCEINLINE T AB(T &x, const uint8 s, const uint8 n, const U i)
  * Checks if a bit in a value is set.
  *
  * This function checks if a bit inside a value is set or not.
- * The y value specific the position of the bit, started at the
- * LSB and count from 0.
+ * The \a y value specific the position of the bit, started at the
+ * LSB and count from \c 0.
  *
  * @param x The value to check
  * @param y The position of the bit to check, started from the LSB
+ * @pre y < sizeof(T) * 8
  * @return True if the bit is set, false else.
  */
 template <typename T>
-static FORCEINLINE bool HasBit(const T x, const uint8 y)
+static inline bool HasBit(const T x, const uint8 y)
 {
 	return (x & ((T)1U << y)) != 0;
 }
@@ -107,12 +116,13 @@ static FORCEINLINE bool HasBit(const T x, const uint8 y)
  *
  * @param x The variable to set a bit
  * @param y The bit position to set
+ * @pre y < sizeof(T) * 8
  * @return The new value of the old value with the bit set
  */
 template <typename T>
-static FORCEINLINE T SetBit(T &x, const uint8 y)
+static inline T SetBit(T &x, const uint8 y)
 {
-	return x = (T)(x | (T)(1U << y));
+	return x = (T)(x | ((T)1U << y));
 }
 
 /**
@@ -136,10 +146,11 @@ static FORCEINLINE T SetBit(T &x, const uint8 y)
  *
  * @param x The variable to clear the bit
  * @param y The bit position to clear
+ * @pre y < sizeof(T) * 8
  * @return The new value of the old value with the bit cleared
  */
 template <typename T>
-static FORCEINLINE T ClrBit(T &x, const uint8 y)
+static inline T ClrBit(T &x, const uint8 y)
 {
 	return x = (T)(x & ~((T)1U << y));
 }
@@ -163,14 +174,15 @@ static FORCEINLINE T ClrBit(T &x, const uint8 y)
  * changed and the value is also returned. Parameter y defines the bit
  * to toggle and starts at the LSB with 0.
  *
- * @param x The varliable to toggle the bit
+ * @param x The variable to toggle the bit
  * @param y The bit position to toggle
+ * @pre y < sizeof(T) * 8
  * @return The new value of the old value with the bit toggled
  */
 template <typename T>
-static FORCEINLINE T ToggleBit(T &x, const uint8 y)
+static inline T ToggleBit(T &x, const uint8 y)
 {
-	return x = (T)(x ^ (T)(1U << y));
+	return x = (T)(x ^ ((T)1U << y));
 }
 
 
@@ -178,7 +190,7 @@ static FORCEINLINE T ToggleBit(T &x, const uint8 y)
 extern const uint8 _ffb_64[64];
 
 /**
- * Returns the first occure of a bit in a 6-bit value (from right).
+ * Returns the first non-zero bit in a 6-bit value (from right).
  *
  * Returns the position of the first bit that is not zero, counted from the
  * LSB. Ie, 110100 returns 2, 000001 returns 0, etc. When x == 0 returns
@@ -190,7 +202,7 @@ extern const uint8 _ffb_64[64];
 #define FIND_FIRST_BIT(x) _ffb_64[(x)]
 
 /**
- * Finds the position of the first bit in an integer.
+ * Finds the position of the first non-zero bit in an integer.
  *
  * This function returns the position of the first bit set in the
  * integer. It does only check the bits of the bitmask
@@ -203,7 +215,7 @@ extern const uint8 _ffb_64[64];
  * @return The position of the first bit which is set
  * @see FIND_FIRST_BIT
  */
-static FORCEINLINE uint8 FindFirstBit2x64(const int value)
+static inline uint8 FindFirstBit2x64(const int value)
 {
 	if ((value & 0xFF) == 0) {
 		return FIND_FIRST_BIT((value >> 8) & 0x3F) + 8;
@@ -226,7 +238,7 @@ uint8 FindLastBit(uint64 x);
  * @return The new value with the first bit cleared
  */
 template <typename T>
-static FORCEINLINE T KillFirstBit(T value)
+static inline T KillFirstBit(T value)
 {
 	return value &= (T)(value - 1);
 }
@@ -261,7 +273,7 @@ static inline uint CountBits(T value)
  * @return does \a value have exactly 1 bit set?
  */
 template <typename T>
-static FORCEINLINE bool HasExactlyOneBit(T value)
+static inline bool HasExactlyOneBit(T value)
 {
 	return value != 0 && (value & (value - 1)) == 0;
 }
@@ -273,69 +285,99 @@ static FORCEINLINE bool HasExactlyOneBit(T value)
  * @return does \a value have at most 1 bit set?
  */
 template <typename T>
-static FORCEINLINE bool HasAtMostOneBit(T value)
+static inline bool HasAtMostOneBit(T value)
 {
 	return (value & (value - 1)) == 0;
 }
 
 /**
- * ROtate x Left by n
+ * ROtate \a x Left by \a n
  *
  * @note Assumes a byte has 8 bits
  * @param x The value which we want to rotate
- * @param n The number how many we waht to rotate
+ * @param n The number how many we want to rotate
+ * @pre n < sizeof(T) * 8
  * @return A bit rotated number
  */
 template <typename T>
-static FORCEINLINE T ROL(const T x, const uint8 n)
+static inline T ROL(const T x, const uint8 n)
 {
 	return (T)(x << n | x >> (sizeof(x) * 8 - n));
 }
 
 /**
- * ROtate x Right by n
+ * ROtate \a x Right by \a n
  *
  * @note Assumes a byte has 8 bits
  * @param x The value which we want to rotate
- * @param n The number how many we waht to rotate
+ * @param n The number how many we want to rotate
+ * @pre n < sizeof(T) * 8
  * @return A bit rotated number
  */
 template <typename T>
-static FORCEINLINE T ROR(const T x, const uint8 n)
+static inline T ROR(const T x, const uint8 n)
 {
 	return (T)(x >> n | x << (sizeof(x) * 8 - n));
 }
 
+/**
+ * Do an operation for each set bit in a value.
+ *
+ * This macros is used to do an operation for each set
+ * bit in a variable. The second parameter is a
+ * variable that is used as the bit position counter.
+ * The fourth parameter is an expression of the bits
+ * we need to iterate over. This expression will be
+ * evaluated once.
+ *
+ * @param Tbitpos_type Type of the position counter variable.
+ * @param bitpos_var   The position counter variable.
+ * @param Tbitset_type Type of the bitset value.
+ * @param bitset_value The bitset value which we check for bits.
+ *
+ * @see FOR_EACH_SET_BIT
+ */
+#define FOR_EACH_SET_BIT_EX(Tbitpos_type, bitpos_var, Tbitset_type, bitset_value) \
+	for (                                                                           \
+		Tbitset_type ___FESBE_bits = (bitpos_var = (Tbitpos_type)0, bitset_value);    \
+		___FESBE_bits != (Tbitset_type)0;                                             \
+		___FESBE_bits = (Tbitset_type)(___FESBE_bits >> 1), bitpos_var++              \
+	)                                                                               \
+		if ((___FESBE_bits & 1) != 0)
+
 /**
  * Do an operation for each set set bit in a value.
  *
  * This macros is used to do an operation for each set
- * bit in a variable. The first variable can be reused
- * in the operation due to it's the bit position counter.
- * The second variable will be cleared during the usage
+ * bit in a variable. The first parameter is a variable
+ * that is used as the bit position counter.
+ * The second parameter is an expression of the bits
+ * we need to iterate over. This expression will be
+ * evaluated once.
  *
- * @param i The position counter
- * @param b The value which we check for set bits
+ * @param bitpos_var   The position counter variable.
+ * @param bitset_value The value which we check for set bits.
  */
-#define FOR_EACH_SET_BIT(i, b)      \
-	for (i = 0; b != 0; i++, b >>= 1) \
-		if (b & 1)
-
+#define FOR_EACH_SET_BIT(bitpos_var, bitset_value) FOR_EACH_SET_BIT_EX(uint, bitpos_var, uint, bitset_value)
 
 #if defined(__APPLE__)
 	/* Make endian swapping use Apple's macros to increase speed
 	 * (since it will use hardware swapping if available).
 	 * Even though they should return uint16 and uint32, we get
 	 * warnings if we don't cast those (why?) */
-	#define BSWAP32(x) ((uint32)Endian32_Swap(x))
-	#define BSWAP16(x) ((uint16)Endian16_Swap(x))
+	#define BSWAP32(x) ((uint32)CFSwapInt32(x))
+	#define BSWAP16(x) ((uint16)CFSwapInt16(x))
+#elif defined(_MSC_VER)
+	/* MSVC has intrinsics for swapping, resulting in faster code */
+	#define BSWAP32(x) (_byteswap_ulong(x))
+	#define BSWAP16(x) (_byteswap_ushort(x))
 #else
 	/**
 	 * Perform a 32 bits endianness bitswap on x.
 	 * @param x the variable to bitswap
 	 * @return the bitswapped value.
 	 */
-	static FORCEINLINE uint32 BSWAP32(uint32 x)
+	static inline uint32 BSWAP32(uint32 x)
 	{
 #if !defined(__ICC) && defined(__GNUC__) && ((__GNUC__ > 4) || ((__GNUC__ == 4)  && __GNUC_MINOR__ >= 3))
 		/* GCC >= 4.3 provides a builtin, resulting in faster code */
@@ -350,7 +392,7 @@ static FORCEINLINE T ROR(const T x, const uint8 n)
 	 * @param x the variable to bitswap
 	 * @return the bitswapped value.
 	 */
-	static FORCEINLINE uint16 BSWAP16(uint16 x)
+	static inline uint16 BSWAP16(uint16 x)
 	{
 		return (x >> 8) | (x << 8);
 	}
