@@ -1,4 +1,4 @@
-/* $Id: pool_type.hpp 22088 2011-02-15 22:04:57Z rubidium $ */
+/* $Id: pool_type.hpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,18 +12,68 @@
 #ifndef POOL_TYPE_HPP
 #define POOL_TYPE_HPP
 
+#include "smallvec_type.hpp"
+#include "enum_type.hpp"
+
+/** Various types of a pool. */
+enum PoolType {
+	PT_NONE    = 0x00, ///< No pool is selected.
+	PT_NORMAL  = 0x01, ///< Normal pool containing game objects.
+	PT_NCLIENT = 0x02, ///< Network client pools.
+	PT_NADMIN  = 0x04, ///< Network admin pool.
+	PT_DATA    = 0x08, ///< NewGRF or other data, that is not reset together with normal pools.
+	PT_ALL     = 0x0F, ///< All pool types.
+};
+DECLARE_ENUM_AS_BIT_SET(PoolType)
+
+typedef SmallVector<struct PoolBase *, 4> PoolVector; ///< Vector of pointers to PoolBase
+
+/** Base class for base of all pools. */
+struct PoolBase {
+	const PoolType type; ///< Type of this pool.
+
+	/**
+	 * Function used to access the vector of all pools.
+	 * @return pointer to vector of all pools
+	 */
+	static PoolVector *GetPools()
+	{
+		static PoolVector *pools = new PoolVector();
+		return pools;
+	}
+
+	static void Clean(PoolType);
+
+	/**
+	 * Constructor registers this object in the pool vector.
+	 * @param pt type of this pool.
+	 */
+	PoolBase(PoolType pt) : type(pt)
+	{
+		*PoolBase::GetPools()->Append() = this;
+	}
+
+	virtual ~PoolBase();
+
+	/**
+	 * Virtual method that deletes all items in the pool.
+	 */
+	virtual void CleanPool() = 0;
+};
+
 /**
  * Base class for all pools.
  * @tparam Titem        Type of the class/struct that is going to be pooled
  * @tparam Tindex       Type of the index for this pool
  * @tparam Tgrowth_step Size of growths; if the pool is full increase the size by this amount
  * @tparam Tmax_size    Maximum size of the pool
+ * @tparam Tpool_type   Type of this pool
  * @tparam Tcache       Whether to perform 'alloc' caching, i.e. don't actually free/malloc just reuse the memory
  * @tparam Tzero        Whether to zero the memory
  * @warning when Tcache is enabled *all* instances of this pool's item must be of the same size.
  */
-template <class Titem, typename Tindex, size_t Tgrowth_step, size_t Tmax_size, bool Tcache = false, bool Tzero = true>
-struct Pool {
+template <class Titem, typename Tindex, size_t Tgrowth_step, size_t Tmax_size, PoolType Tpool_type = PT_NORMAL, bool Tcache = false, bool Tzero = true>
+struct Pool : PoolBase {
 	static const size_t MAX_SIZE = Tmax_size; ///< Make template parameter accessible from outside
 
 	const char * const name; ///< Name of this pool
@@ -32,12 +82,15 @@ struct Pool {
 	size_t first_free;   ///< No item with index lower than this is free (doesn't say anything about this one!)
 	size_t first_unused; ///< This and all higher indexes are free (doesn't say anything about first_unused-1 !)
 	size_t items;        ///< Number of used indexes (non-NULL)
+#ifdef OTTD_ASSERT
+	size_t checked;      ///< Number of items we checked for
+#endif /* OTTD_ASSERT */
 	bool cleaning;       ///< True if cleaning pool (deleting all items)
 
 	Titem **data;        ///< Pointer to array of pointers to Titem
 
 	Pool(const char *name);
-	void CleanPool();
+	virtual void CleanPool();
 
 	/**
 	 * Returs Titem with given index
@@ -45,7 +98,7 @@ struct Pool {
 	 * @return pointer to Titem
 	 * @pre index < this->first_unused
 	 */
-	FORCEINLINE Titem *Get(size_t index)
+	inline Titem *Get(size_t index)
 	{
 		assert(index < this->first_unused);
 		return this->data[index];
@@ -56,7 +109,7 @@ struct Pool {
 	 * @param index index to examine
 	 * @return true if PoolItem::Get(index) will return non-NULL pointer
 	 */
-	FORCEINLINE bool IsValidID(size_t index)
+	inline bool IsValidID(size_t index)
 	{
 		return index < this->first_unused && this->Get(index) != NULL;
 	}
@@ -66,16 +119,20 @@ struct Pool {
 	 * @param n number of items we want to allocate
 	 * @return true if 'n' items can be allocated
 	 */
-	FORCEINLINE bool CanAllocate(size_t n = 1)
+	inline bool CanAllocate(size_t n = 1)
 	{
-		return this->items <= Tmax_size - n;
+		bool ret = this->items <= Tmax_size - n;
+#ifdef OTTD_ASSERT
+		this->checked = ret ? n : 0;
+#endif /* OTTD_ASSERT */
+		return ret;
 	}
 
 	/**
 	 * Base class for all PoolItems
 	 * @tparam Tpool The pool this item is going to be part of
 	 */
-	template <struct Pool<Titem, Tindex, Tgrowth_step, Tmax_size, Tcache, Tzero> *Tpool>
+	template <struct Pool<Titem, Tindex, Tgrowth_step, Tmax_size, Tpool_type, Tcache, Tzero> *Tpool>
 	struct PoolItem {
 		Tindex index; ///< Index of this pool item
 
@@ -85,7 +142,7 @@ struct Pool {
 		 * @return pointer to allocated memory
 		 * @note can never fail (return NULL), use CanAllocate() to check first!
 		 */
-		FORCEINLINE void *operator new(size_t size)
+		inline void *operator new(size_t size)
 		{
 			return Tpool->GetNew(size);
 		}
@@ -95,7 +152,7 @@ struct Pool {
 		 * @param p memory to free
 		 * @note the item has to be allocated in the pool!
 		 */
-		FORCEINLINE void operator delete(void *p)
+		inline void operator delete(void *p)
 		{
 			Titem *pn = (Titem *)p;
 			assert(pn == Tpool->Get(pn->index));
@@ -110,7 +167,7 @@ struct Pool {
 		 * @note can never fail (return NULL), use CanAllocate() to check first!
 		 * @pre index has to be unused! Else it will crash
 		 */
-		FORCEINLINE void *operator new(size_t size, size_t index)
+		inline void *operator new(size_t size, size_t index)
 		{
 			return Tpool->GetNew(size, index);
 		}
@@ -123,7 +180,7 @@ struct Pool {
 		 * @note use of this is strongly discouraged
 		 * @pre the memory must not be allocated in the Pool!
 		 */
-		FORCEINLINE void *operator new(size_t size, void *ptr)
+		inline void *operator new(size_t size, void *ptr)
 		{
 			for (size_t i = 0; i < Tpool->first_unused; i++) {
 				/* Don't allow creating new objects over existing.
@@ -145,7 +202,7 @@ struct Pool {
 		 * @param n number of items we want to allocate
 		 * @return true if 'n' items can be allocated
 		 */
-		static FORCEINLINE bool CanAllocateItem(size_t n = 1)
+		static inline bool CanAllocateItem(size_t n = 1)
 		{
 			return Tpool->CanAllocate(n);
 		}
@@ -154,7 +211,7 @@ struct Pool {
 		 * Returns current state of pool cleaning - yes or no
 		 * @return true iff we are cleaning the pool now
 		 */
-		static FORCEINLINE bool CleaningPool()
+		static inline bool CleaningPool()
 		{
 			return Tpool->cleaning;
 		}
@@ -164,7 +221,7 @@ struct Pool {
 		 * @param index index to examine
 		 * @return true if PoolItem::Get(index) will return non-NULL pointer
 		 */
-		static FORCEINLINE bool IsValidID(size_t index)
+		static inline bool IsValidID(size_t index)
 		{
 			return Tpool->IsValidID(index);
 		}
@@ -175,7 +232,7 @@ struct Pool {
 		 * @return pointer to Titem
 		 * @pre index < this->first_unused
 		 */
-		static FORCEINLINE Titem *Get(size_t index)
+		static inline Titem *Get(size_t index)
 		{
 			return Tpool->Get(index);
 		}
@@ -186,7 +243,7 @@ struct Pool {
 		 * @return pointer to Titem
 		 * @note returns NULL for invalid index
 		 */
-		static FORCEINLINE Titem *GetIfValid(size_t index)
+		static inline Titem *GetIfValid(size_t index)
 		{
 			return index < Tpool->first_unused ? Tpool->Get(index) : NULL;
 		}
@@ -196,7 +253,7 @@ struct Pool {
 		 * all pool items.
 		 * @return first unused index
 		 */
-		static FORCEINLINE size_t GetPoolSize()
+		static inline size_t GetPoolSize()
 		{
 			return Tpool->first_unused;
 		}
@@ -205,7 +262,7 @@ struct Pool {
 		 * Returns number of valid items in the pool
 		 * @return number of valid items in the pool
 		 */
-		static FORCEINLINE size_t GetNumItems()
+		static inline size_t GetNumItems()
 		{
 			return Tpool->items;
 		}
@@ -217,11 +274,11 @@ struct Pool {
 		 * @note when this function is called, PoolItem::Get(index) == NULL.
 		 * @note it's called only when !CleaningPool()
 		 */
-		static FORCEINLINE void PostDestructor(size_t index) { }
+		static inline void PostDestructor(size_t index) { }
 	};
 
 private:
-	static const size_t NO_FREE_ITEM = MAX_UVALUE(size_t); ///< Contant to indicate we can't allocate any more items
+	static const size_t NO_FREE_ITEM = MAX_UVALUE(size_t); ///< Constant to indicate we can't allocate any more items
 
 	/**
 	 * Helper struct to cache 'freed' PoolItems so we
