@@ -1,4 +1,4 @@
-/* $Id: random_func.hpp 19653 2010-04-17 12:00:59Z rubidium $ */
+/* $Id: random_func.hpp 24776 2012-12-01 13:12:39Z alberth $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,19 +17,6 @@
 	#define Random OTTD_Random
 #endif /* __APPLE__ */
 
-/**************
- * Warning: DO NOT enable this unless you understand what it does
- *
- * If enabled, in a network game all randoms will be dumped to the
- *  stdout if the first client joins (or if you are a client). This
- *  is to help finding desync problems.
- *
- * Warning: DO NOT enable this unless you understand what it does
- **************/
-
-//#define RANDOM_DEBUG
-
-
 /**
  * Structure to encapsulate the pseudo random number generators.
  */
@@ -37,23 +24,8 @@ struct Randomizer {
 	/** The state of the randomizer */
 	uint32 state[2];
 
-	/**
-	 * Generate the next pseudo random number
-	 * @return the random number
-	 */
 	uint32 Next();
-
-	/**
-	 * Generate the next pseudo random number scaled to max
-	 * @param max the maximum value of the returned random number
-	 * @return the random number
-	 */
 	uint32 Next(uint32 max);
-
-	/**
-	 * (Re)set the state of the random number generator.
-	 * @param seed the new state
-	 */
 	void SetSeed(uint32 seed);
 };
 extern Randomizer _random; ///< Random used in the game state calculations
@@ -65,7 +37,8 @@ struct SavedRandomSeeds {
 	Randomizer interactive_random;
 };
 
-/** Saves the current seeds
+/**
+ * Saves the current seeds
  * @param storage Storage for saving
  */
 static inline void SaveRandomSeeds(SavedRandomSeeds *storage)
@@ -74,7 +47,8 @@ static inline void SaveRandomSeeds(SavedRandomSeeds *storage)
 	storage->interactive_random = _interactive_random;
 }
 
-/** Restores previously saved seeds
+/**
+ * Restores previously saved seeds
  * @param storage Storage where SaveRandomSeeds() stored th seeds
  */
 static inline void RestoreRandomSeeds(const SavedRandomSeeds &storage)
@@ -94,23 +68,23 @@ void SetRandomSeed(uint32 seed);
 	#define RandomRange(max) DoRandomRange(max, __LINE__, __FILE__)
 	uint32 DoRandomRange(uint32 max, int line, const char *file);
 #else
-	static FORCEINLINE uint32 Random()
+	static inline uint32 Random()
 	{
 		return _random.Next();
 	}
 
-	static FORCEINLINE uint32 RandomRange(uint32 max)
+	static inline uint32 RandomRange(uint32 max)
 	{
 		return _random.Next(max);
 	}
 #endif
 
-static FORCEINLINE uint32 InteractiveRandom()
+static inline uint32 InteractiveRandom()
 {
 	return _interactive_random.Next();
 }
 
-static FORCEINLINE uint32 InteractiveRandomRange(uint32 max)
+static inline uint32 InteractiveRandomRange(uint32 max)
 {
 	return _interactive_random.Next(max);
 }
@@ -130,7 +104,7 @@ static FORCEINLINE uint32 InteractiveRandomRange(uint32 max)
  * @param r The given randomize-number
  * @return True if the probability given by r is less or equal to (a/b)
  */
-static FORCEINLINE bool Chance16I(const uint a, const uint b, const uint32 r)
+static inline bool Chance16I(const uint a, const uint b, const uint32 r)
 {
 	assert(b != 0);
 	return (((uint16)r * b + b / 2) >> 16) < a;
@@ -149,7 +123,7 @@ static FORCEINLINE bool Chance16I(const uint a, const uint b, const uint32 r)
 #ifdef RANDOM_DEBUG
 	#define Chance16(a, b) Chance16I(a, b, DoRandom(__LINE__, __FILE__))
 #else
-static FORCEINLINE bool Chance16(const uint a, const uint b)
+static inline bool Chance16(const uint a, const uint b)
 {
 	return Chance16I(a, b, Random());
 }
@@ -161,7 +135,7 @@ static FORCEINLINE bool Chance16(const uint a, const uint b)
  * This function uses the same parameters as Chance16. The third parameter
  * must be a variable the randomize-number from Random() is saved in.
  *
- * The low 16 bits of r will already be used and can therefor not be passed to
+ * The low 16 bits of r will already be used and can therefore not be passed to
  * Chance16I. One can only send the high 16 bits to Chance16I.
  *
  * @see Chance16I()
@@ -173,7 +147,7 @@ static FORCEINLINE bool Chance16(const uint a, const uint b)
 #ifdef RANDOM_DEBUG
 	#define Chance16R(a, b, r) (r = DoRandom(__LINE__, __FILE__), Chance16I(a, b, r))
 #else
-static FORCEINLINE bool Chance16R(const uint a, const uint b, uint32 &r)
+static inline bool Chance16R(const uint a, const uint b, uint32 &r)
 {
 	r = Random();
 	return Chance16I(a, b, r);
