@@ -1,4 +1,4 @@
-/* $Id: direction_func.h 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: direction_func.h 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -35,7 +35,9 @@ static inline Direction ReverseDir(Direction d)
  */
 static inline DirDiff DirDifference(Direction d0, Direction d1)
 {
-	return (DirDiff)((d0 + 8 - d1) % 8);
+	/* Cast to uint so compiler can use bitmask. If the difference is negative
+	 * and we used int instead of uint, further "+ 8" would have to be added. */
+	return (DirDiff)((uint)(d0 - d1) % 8);
 }
 
 /**
@@ -51,7 +53,8 @@ static inline DirDiff DirDifference(Direction d0, Direction d1)
  */
 static inline DirDiff ChangeDirDiff(DirDiff d, DirDiff delta)
 {
-	return (DirDiff)((d + delta) % 8);
+	/* Cast to uint so compiler can use bitmask. Result can never be negative. */
+	return (DirDiff)((uint)(d + delta) % 8);
 }
 
 /**
@@ -66,7 +69,8 @@ static inline DirDiff ChangeDirDiff(DirDiff d, DirDiff delta)
  */
 static inline Direction ChangeDir(Direction d, DirDiff delta)
 {
-	return (Direction)((d + delta) % 8);
+	/* Cast to uint so compiler can use bitmask. Result can never be negative. */
+	return (Direction)((uint)(d + delta) % 8);
 }
 
 
@@ -89,12 +93,13 @@ static inline DiagDirection ReverseDiagDir(DiagDirection d)
  * the new DiagDirection.
  *
  * @param d The DiagDirection
- * @param delta The difference to applie on
+ * @param delta The difference to apply on
  * @return The new direction which was calculated
  */
 static inline DiagDirection ChangeDiagDir(DiagDirection d, DiagDirDiff delta)
 {
-	return (DiagDirection)((d + delta) % 4);
+	/* Cast to uint so compiler can use bitmask. Result can never be negative. */
+	return (DiagDirection)((uint)(d + delta) % 4);
 }
 
 /**
@@ -202,7 +207,7 @@ static inline DiagDirection XYNSToDiagDir(Axis xy, uint ns)
 }
 
 /**
- * Checks if an interger value is a valid DiagDirection
+ * Checks if an integer value is a valid DiagDirection
  *
  * @param d The value to check
  * @return True if the value belongs to a DiagDirection, else false
@@ -234,4 +239,15 @@ static inline bool IsValidAxis(Axis d)
 	return d < AXIS_END;
 }
 
-#endif /* DIRECTION_H */
+/**
+ * Checks if a given Direction is diagonal.
+ *
+ * @param dir The given direction.
+ * @return True if the direction is diagonal.
+ */
+static inline bool IsDiagonalDirection(Direction dir)
+{
+	return (dir & 1) != 0;
+}
+
+#endif /* DIRECTION_FUNC_H */
