@@ -1,4 +1,4 @@
-/* $Id: disaster_cmd.cpp 22017 2011-02-07 22:29:47Z rubidium $ */
+/* $Id: disaster_cmd.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -40,6 +40,7 @@
 #include "effectvehicle_func.h"
 #include "roadveh.h"
 #include "ai/ai.hpp"
+#include "game/game.hpp"
 #include "company_base.h"
 #include "core/random_func.hpp"
 #include "core/backup_type.hpp"
@@ -75,7 +76,7 @@ static void DisasterClearSquare(TileIndex tile)
 
 	switch (GetTileType(tile)) {
 		case MP_RAILWAY:
-			if (Company::IsHumanID(GetTileOwner(tile))) {
+			if (Company::IsHumanID(GetTileOwner(tile)) && !IsRailDepot(tile)) {
 				Backup<CompanyByte> cur_company(_current_company, OWNER_WATER, FILE_LINE);
 				DoCommand(tile, 0, 0, DC_EXEC, CMD_LANDSCAPE_CLEAR);
 				cur_company.Restore();
@@ -133,7 +134,7 @@ static void DisasterVehicleUpdateImage(DisasterVehicle *v)
  * Initialize a disaster vehicle. These vehicles are of type VEH_DISASTER, are unclickable
  * and owned by nobody
  */
-static void InitializeDisasterVehicle(DisasterVehicle *v, int x, int y, byte z, Direction direction, byte subtype)
+static void InitializeDisasterVehicle(DisasterVehicle *v, int x, int y, int z, Direction direction, byte subtype)
 {
 	v->x_pos = x;
 	v->y_pos = y;
@@ -148,11 +149,10 @@ static void InitializeDisasterVehicle(DisasterVehicle *v, int x, int y, byte z,
 	v->current_order.Free();
 
 	DisasterVehicleUpdateImage(v);
-	VehicleMove(v, false);
-	MarkSingleVehicleDirty(v);
+	VehicleUpdatePositionAndViewport(v);
 }
 
-static void SetDisasterVehiclePos(DisasterVehicle *v, int x, int y, byte z)
+static void SetDisasterVehiclePos(DisasterVehicle *v, int x, int y, int z)
 {
 	v->x_pos = x;
 	v->y_pos = y;
@@ -160,7 +160,7 @@ static void SetDisasterVehiclePos(DisasterVehicle *v, int x, int y, byte z)
 	v->tile = TileVirtXY(x, y);
 
 	DisasterVehicleUpdateImage(v);
-	VehicleMove(v, true);
+	VehicleUpdatePositionAndViewport(v);
 
 	DisasterVehicle *u = v->Next();
 	if (u != NULL) {
@@ -168,19 +168,19 @@ static void SetDisasterVehiclePos(DisasterVehicle *v, int x, int y, byte z)
 		int safe_y = Clamp(y - 1, 0, MapMaxY() * TILE_SIZE);
 
 		u->x_pos = x;
-		u->y_pos = y - 1 - (max(z - GetSlopeZ(safe_x, safe_y), 0U) >> 3);
+		u->y_pos = y - 1 - (max(z - GetSlopePixelZ(safe_x, safe_y), 0) >> 3);
 		safe_y = Clamp(u->y_pos, 0, MapMaxY() * TILE_SIZE);
-		u->z_pos = GetSlopeZ(safe_x, safe_y);
+		u->z_pos = GetSlopePixelZ(safe_x, safe_y);
 		u->direction = v->direction;
 
 		DisasterVehicleUpdateImage(u);
-		VehicleMove(u, true);
+		VehicleUpdatePositionAndViewport(u);
 
 		if ((u = u->Next()) != NULL) {
 			u->x_pos = x;
 			u->y_pos = y;
 			u->z_pos = z + 5;
-			VehicleMove(u, true);
+			VehicleUpdatePositionAndViewport(u);
 		}
 	}
 }
@@ -210,7 +210,7 @@ static bool DisasterTick_Zeppeliner(DisasterVehicle *v)
 				v->age = 0;
 			}
 
-			if (GB(v->tick_counter, 0, 3) == 0) CreateEffectVehicleRel(v, 0, -17, 2, EV_SMOKE);
+			if (GB(v->tick_counter, 0, 3) == 0) CreateEffectVehicleRel(v, 0, -17, 2, EV_CRASH_SMOKE);
 
 		} else if (v->current_order.GetDestination() == 0) {
 			if (IsValidTile(v->tile) && IsAirportTile(v->tile)) {
@@ -218,10 +218,8 @@ static bool DisasterTick_Zeppeliner(DisasterVehicle *v)
 				v->age = 0;
 
 				SetDParam(0, GetStationIndex(v->tile));
-				AddVehicleNewsItem(STR_NEWS_DISASTER_ZEPPELIN,
-					NS_ACCIDENT,
-					v->index); // Delete the news, when the zeppelin is gone
-				AI::NewEvent(GetTileOwner(v->tile), new AIEventDisasterZeppelinerCrashed(GetStationIndex(v->tile)));
+				AddVehicleNewsItem(STR_NEWS_DISASTER_ZEPPELIN, NT_ACCIDENT, v->index); // Delete the news, when the zeppelin is gone
+				AI::NewEvent(GetTileOwner(v->tile), new ScriptEventDisasterZeppelinerCrashed(GetStationIndex(v->tile)));
 			}
 		}
 
@@ -239,7 +237,7 @@ static bool DisasterTick_Zeppeliner(DisasterVehicle *v)
 		if (IsValidTile(v->tile) && IsAirportTile(v->tile)) {
 			Station *st = Station::GetByTile(v->tile);
 			CLRBITS(st->airport.flags, RUNWAY_IN_block);
-			AI::NewEvent(GetTileOwner(v->tile), new AIEventDisasterZeppelinerCleared(st->index));
+			AI::NewEvent(GetTileOwner(v->tile), new ScriptEventDisasterZeppelinerCleared(st->index));
 		}
 
 		SetDisasterVehiclePos(v, v->x_pos, v->y_pos, v->z_pos);
@@ -249,13 +247,13 @@ static bool DisasterTick_Zeppeliner(DisasterVehicle *v)
 
 	int x = v->x_pos;
 	int y = v->y_pos;
-	byte z = GetSlopeZ(x, y);
+	int z = GetSlopePixelZ(x, y);
 	if (z < v->z_pos) z = v->z_pos - 1;
 	SetDisasterVehiclePos(v, x, y, z);
 
 	if (++v->age == 1) {
 		CreateEffectVehicleRel(v, 0, 7, 8, EV_EXPLOSION_LARGE);
-		SndPlayVehicleFx(SND_12_EXPLOSION, v);
+		if (_settings_client.sound.disaster) SndPlayVehicleFx(SND_12_EXPLOSION, v);
 		v->image_override = SPR_BLIMP_CRASHING;
 	} else if (v->age == 70) {
 		v->image_override = SPR_BLIMP_CRASHED;
@@ -344,7 +342,7 @@ static bool DisasterTick_Ufo(DisasterVehicle *v)
 		v->direction = GetDirectionTowards(v, u->x_pos, u->y_pos);
 		GetNewVehiclePosResult gp = GetNewVehiclePos(v);
 
-		byte z = v->z_pos;
+		int z = v->z_pos;
 		if (dist <= TILE_SIZE && z > u->z_pos) z--;
 		SetDisasterVehiclePos(v, gp.x, gp.y, z);
 
@@ -353,18 +351,17 @@ static bool DisasterTick_Ufo(DisasterVehicle *v)
 			if (u->crashed_ctr == 0) {
 				u->Crash();
 
-				AddVehicleNewsItem(STR_NEWS_DISASTER_SMALL_UFO,
-					NS_ACCIDENT,
-					u->index); // delete the news, when the roadvehicle is gone
+				AddVehicleNewsItem(STR_NEWS_DISASTER_SMALL_UFO, NT_ACCIDENT, u->index); // delete the news, when the roadvehicle is gone
 
-				AI::NewEvent(u->owner, new AIEventVehicleCrashed(u->index, u->tile, AIEventVehicleCrashed::CRASH_RV_UFO));
+				AI::NewEvent(u->owner, new ScriptEventVehicleCrashed(u->index, u->tile, ScriptEventVehicleCrashed::CRASH_RV_UFO));
+				Game::NewEvent(new ScriptEventVehicleCrashed(u->index, u->tile, ScriptEventVehicleCrashed::CRASH_RV_UFO));
 			}
 		}
 
 		/* Destroy? */
 		if (v->age > 50) {
 			CreateEffectVehicleRel(v, 0, 7, 8, EV_EXPLOSION_LARGE);
-			SndPlayVehicleFx(SND_12_EXPLOSION, v);
+			if (_settings_client.sound.disaster) SndPlayVehicleFx(SND_12_EXPLOSION, v);
 			delete v;
 			return false;
 		}
@@ -376,7 +373,7 @@ static bool DisasterTick_Ufo(DisasterVehicle *v)
 static void DestructIndustry(Industry *i)
 {
 	for (TileIndex tile = 0; tile != MapSize(); tile++) {
-		if (IsTileType(tile, MP_INDUSTRY) && GetIndustryIndex(tile) == i->index) {
+		if (i->TileBelongsToIndustry(tile)) {
 			ResetIndustryConstructionStage(tile);
 			MarkTileDirtyByTile(tile);
 		}
@@ -385,7 +382,7 @@ static void DestructIndustry(Industry *i)
 
 /**
  * Aircraft handling, v->current_order.dest states:
- * 0: Fly towards the targetted industry
+ * 0: Fly towards the targeted industry
  * 1: If within 15 tiles, fire away rockets and destroy industry
  * 2: Industry explosions
  * 3: Fly out of the map
@@ -433,8 +430,8 @@ static bool DisasterTick_Aircraft(DisasterVehicle *v, uint16 image_override, boo
 			DestructIndustry(i);
 
 			SetDParam(0, i->town->index);
-			AddIndustryNewsItem(news_message, NS_ACCIDENT, i->index); // delete the news, when the industry closes
-			SndPlayTileFx(SND_12_EXPLOSION, i->location.tile);
+			AddIndustryNewsItem(news_message, NT_ACCIDENT, i->index); // delete the news, when the industry closes
+			if (_settings_client.sound.disaster) SndPlayTileFx(SND_12_EXPLOSION, i->location.tile);
 		}
 	} else if (v->current_order.GetDestination() == 0) {
 		int x = v->x_pos + ((leave_at_top ? -15 : 15) * TILE_SIZE);
@@ -477,7 +474,7 @@ static bool DisasterTick_Helicopter_Rotors(DisasterVehicle *v)
 
 	if (++v->cur_image > SPR_ROTOR_MOVING_3) v->cur_image = SPR_ROTOR_MOVING_1;
 
-	VehicleMove(v, true);
+	VehicleUpdatePositionAndViewport(v);
 
 	return true;
 }
@@ -509,7 +506,7 @@ static bool DisasterTick_Big_Ufo(DisasterVehicle *v)
 			return false;
 		}
 
-		byte z = GetSlopeZ(v->x_pos, v->y_pos);
+		int z = GetSlopePixelZ(v->x_pos, v->y_pos);
 		if (z < v->z_pos) {
 			SetDisasterVehiclePos(v, v->x_pos, v->y_pos, v->z_pos - 1);
 			return true;
@@ -529,10 +526,7 @@ static bool DisasterTick_Big_Ufo(DisasterVehicle *v)
 
 		Town *t = ClosestTownFromTile(v->dest_tile, UINT_MAX);
 		SetDParam(0, t->index);
-		AddNewsItem(STR_NEWS_DISASTER_BIG_UFO,
-			NS_ACCIDENT,
-			NR_TILE,
-			v->tile);
+		AddTileNewsItem(STR_NEWS_DISASTER_BIG_UFO, NT_ACCIDENT, v->tile);
 
 		if (!Vehicle::CanAllocateItem(2)) {
 			delete v;
@@ -602,7 +596,7 @@ static bool DisasterTick_Big_Ufo_Destroyer(DisasterVehicle *v)
 		v->current_order.SetDestination(1);
 
 		CreateEffectVehicleRel(u, 0, 7, 8, EV_EXPLOSION_LARGE);
-		SndPlayVehicleFx(SND_12_EXPLOSION, u);
+		if (_settings_client.sound.disaster) SndPlayVehicleFx(SND_12_EXPLOSION, u);
 
 		delete u;
 
@@ -877,8 +871,7 @@ static void Disaster_CoalMine_Init()
 		FOR_ALL_INDUSTRIES(i) {
 			if ((GetIndustrySpec(i->type)->behaviour & INDUSTRYBEH_CAN_SUBSIDENCE) && --index < 0) {
 				SetDParam(0, i->town->index);
-				AddNewsItem(STR_NEWS_DISASTER_COAL_MINE_SUBSIDENCE,
-					NS_ACCIDENT, NR_TILE, i->location.tile + TileDiffXY(1, 1)); // keep the news, even when the mine closes
+				AddTileNewsItem(STR_NEWS_DISASTER_COAL_MINE_SUBSIDENCE, NT_ACCIDENT, i->location.tile + TileDiffXY(1, 1)); // keep the news, even when the mine closes
 
 				{
 					TileIndex tile = i->location.tile;
