@@ -1,4 +1,4 @@
-/* $Id: engine_func.h 18809 2010-01-15 16:41:15Z rubidium $ */
+/* $Id: engine_func.h 25560 2013-07-04 21:20:05Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -9,8 +9,8 @@
 
 /** @file engine_func.h Functions related to engines. */
 
-#ifndef ENGINE_H
-#define ENGINE_H
+#ifndef ENGINE_FUNC_H
+#define ENGINE_FUNC_H
 
 #include "engine_type.h"
 #include "vehicle_type.h"
@@ -18,6 +18,7 @@
 
 void SetupEngines();
 void StartupEngines();
+void CheckEngines();
 
 /* Original engine data counts and offsets */
 extern const uint8 _engine_counts[4];
@@ -25,10 +26,9 @@ extern const uint8 _engine_offsets[4];
 
 bool IsEngineBuildable(EngineID engine, VehicleType type, CompanyID company);
 bool IsEngineRefittable(EngineID engine);
-void SetCachedEngineCounts();
 void SetYearEngineAgingStops();
 void StartupOneEngine(Engine *e, Date aging_date);
 
 uint GetTotalCapacityOfArticulatedParts(EngineID engine);
 
-#endif /* ENGINE_H */
+#endif /* ENGINE_FUNC_H */
