@@ -1,4 +1,4 @@
-/* $Id: fileio.cpp 22841 2011-08-25 13:41:06Z rubidium $ */
+/* $Id: fileio.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,6 +17,7 @@
 #include "tar_type.h"
 #ifdef WIN32
 #include <windows.h>
+# define access _taccess
 #elif defined(__HAIKU__)
 #include <Path.h>
 #include <storage/FindDirectory.h>
@@ -27,12 +28,10 @@
 #include <sys/stat.h>
 #include <algorithm>
 
-/*************************************************/
-/* FILE IO ROUTINES ******************************/
-/*************************************************/
-
+/** Size of the #Fio data buffer. */
 #define FIO_BUFFER_SIZE 512
 
+/** Structure for keeping several open files with just one data buffer. */
 struct Fio {
 	byte *buffer, *buffer_end;             ///< position pointer in local buffer and last valid byte of buffer
 	size_t pos;                            ///< current (system) position in file
@@ -48,7 +47,7 @@ struct Fio {
 #endif /* LIMITED_FDS */
 };
 
-static Fio _fio;
+static Fio _fio; ///< #Fio instance.
 
 /** Whether the working directory should be scanned. */
 static bool _do_scan_working_directory = true;
@@ -56,17 +55,30 @@ static bool _do_scan_working_directory = true;
 extern char *_config_file;
 extern char *_highscore_file;
 
-/* Get current position in file */
+/**
+ * Get position in the current file.
+ * @return Position in the file.
+ */
 size_t FioGetPos()
 {
 	return _fio.pos + (_fio.buffer - _fio.buffer_end);
 }
 
+/**
+ * Get the filename associated with a slot.
+ * @param slot Index of queried file.
+ * @return Name of the file.
+ */
 const char *FioGetFilename(uint8 slot)
 {
 	return _fio.shortnames[slot];
 }
 
+/**
+ * Seek in the current file.
+ * @param pos New position.
+ * @param mode Type of seek (\c SEEK_CUR means \a pos is relative to current position, \c SEEK_SET means \a pos is absolute).
+ */
 void FioSeekTo(size_t pos, int mode)
 {
 	if (mode == SEEK_CUR) pos += FioGetPos();
@@ -87,7 +99,11 @@ static void FioRestoreFile(int slot)
 }
 #endif /* LIMITED_FDS */
 
-/* Seek to a file and a position */
+/**
+ * Switch to a different file and seek to a position.
+ * @param slot Slot number of the new file.
+ * @param pos New absolute position in the new file.
+ */
 void FioSeekToFile(uint8 slot, size_t pos)
 {
 	FILE *f;
@@ -102,6 +118,10 @@ void FioSeekToFile(uint8 slot, size_t pos)
 	FioSeekTo(pos, SEEK_SET);
 }
 
+/**
+ * Read a byte from the file.
+ * @return Read byte.
+ */
 byte FioReadByte()
 {
 	if (_fio.buffer == _fio.buffer_end) {
@@ -115,6 +135,10 @@ byte FioReadByte()
 	return *_fio.buffer++;
 }
 
+/**
+ * Skip \a n bytes ahead in the file.
+ * @param n Number of bytes to skip reading.
+ */
 void FioSkipBytes(int n)
 {
 	for (;;) {
@@ -127,24 +151,41 @@ void FioSkipBytes(int n)
 	}
 }
 
+/**
+ * Read a word (16 bits) from the file (in low endian format).
+ * @return Read word.
+ */
 uint16 FioReadWord()
 {
 	byte b = FioReadByte();
 	return (FioReadByte() << 8) | b;
 }
 
+/**
+ * Read a double word (32 bits) from the file (in low endian format).
+ * @return Read word.
+ */
 uint32 FioReadDword()
 {
 	uint b = FioReadWord();
 	return (FioReadWord() << 16) | b;
 }
 
+/**
+ * Read a block.
+ * @param ptr Destination buffer.
+ * @param size Number of bytes to read.
+ */
 void FioReadBlock(void *ptr, size_t size)
 {
 	FioSeekTo(FioGetPos(), SEEK_SET);
 	_fio.pos += fread(ptr, 1, size, _fio.cur_fh);
 }
 
+/**
+ * Close the file at the given slot number.
+ * @param slot File index to close.
+ */
 static inline void FioCloseFile(int slot)
 {
 	if (_fio.handles[slot] != NULL) {
@@ -160,6 +201,7 @@ static inline void FioCloseFile(int slot)
 	}
 }
 
+/** Close all slotted open files. */
 void FioCloseAll()
 {
 	for (int i = 0; i != lengthof(_fio.handles); i++) {
@@ -191,14 +233,20 @@ static void FioFreeHandle()
 }
 #endif /* LIMITED_FDS */
 
-void FioOpenFile(int slot, const char *filename)
+/**
+ * Open a slotted file.
+ * @param slot Index to assign.
+ * @param filename Name of the file at the disk.
+ * @param subdir The sub directory to search this file in.
+ */
+void FioOpenFile(int slot, const char *filename, Subdirectory subdir)
 {
 	FILE *f;
 
 #if defined(LIMITED_FDS)
 	FioFreeHandle();
 #endif /* LIMITED_FDS */
-	f = FioFOpenFile(filename);
+	f = FioFOpenFile(filename, "rb", subdir);
 	if (f == NULL) usererror("Cannot open file '%s'", filename);
 	uint32 pos = ftell(f);
 
@@ -220,7 +268,7 @@ void FioOpenFile(int slot, const char *filename)
 	FioSeekToFile(slot, pos);
 }
 
-static const char * const _subdirs[NUM_SUBDIRS] = {
+static const char * const _subdirs[] = {
 	"",
 	"save" PATHSEP,
 	"save" PATHSEP "autosave" PATHSEP,
@@ -228,21 +276,27 @@ static const char * const _subdirs[NUM_SUBDIRS] = {
 	"scenario" PATHSEP "heightmap" PATHSEP,
 	"gm" PATHSEP,
 	"data" PATHSEP,
+	"baseset" PATHSEP,
+	"newgrf" PATHSEP,
 	"lang" PATHSEP,
 	"ai" PATHSEP,
 	"ai" PATHSEP "library" PATHSEP,
+	"game" PATHSEP,
+	"game" PATHSEP "library" PATHSEP,
+	"screenshot" PATHSEP,
 };
+assert_compile(lengthof(_subdirs) == NUM_SUBDIRS);
 
 const char *_searchpaths[NUM_SEARCHPATHS];
-TarList _tar_list;
-TarFileList _tar_filelist;
+TarList _tar_list[NUM_SUBDIRS];
+TarFileList _tar_filelist[NUM_SUBDIRS];
 
 typedef std::map<std::string, std::string> TarLinkList;
-static TarLinkList _tar_linklist; ///< List of directory links
+static TarLinkList _tar_linklist[NUM_SUBDIRS]; ///< List of directory links
 
 /**
  * Check whether the given file exists
- * @param filename the file to try for existance
+ * @param filename the file to try for existence.
  * @param subdir the subdirectory to look in
  * @return true if and only if the file can be opened
  */
@@ -255,6 +309,24 @@ bool FioCheckFileExists(const char *filename, Subdirectory subdir)
 	return true;
 }
 
+/**
+ * Test whether the given filename exists.
+ * @param filename the file to test.
+ * @return true if and only if the file exists.
+ */
+bool FileExists(const char *filename)
+{
+#if defined(WINCE)
+	/* There is always one platform that doesn't support basic commands... */
+	HANDLE hand = CreateFile(OTTD2FS(filename), 0, 0, NULL, OPEN_EXISTING, 0, NULL);
+	if (hand == INVALID_HANDLE_VALUE) return 1;
+	CloseHandle(hand);
+	return 0;
+#else
+	return access(OTTD2FS(filename), 0) == 0;
+#endif
+}
+
 /**
  * Close a file in a safe way.
  */
@@ -272,6 +344,14 @@ char *FioGetFullPath(char *buf, size_t buflen, Searchpath sp, Subdirectory subdi
 	return buf;
 }
 
+/**
+ * Find a path to the filename in one of the search directories.
+ * @param buf [out] Destination buffer for the path.
+ * @param buflen Length of the destination buffer.
+ * @param subdir Subdirectory to try.
+ * @param filename Filename to look for.
+ * @return \a buf containing the path if the path was found, else \c NULL.
+ */
 char *FioFindFullPath(char *buf, size_t buflen, Subdirectory subdir, const char *filename)
 {
 	Searchpath sp;
@@ -279,17 +359,16 @@ char *FioFindFullPath(char *buf, size_t buflen, Subdirectory subdir, const char
 
 	FOR_ALL_SEARCHPATHS(sp) {
 		FioGetFullPath(buf, buflen, sp, subdir, filename);
-		if (FileExists(buf)) break;
+		if (FileExists(buf)) return buf;
 #if !defined(WIN32)
 		/* Be, as opening files, aware that sometimes the filename
 		 * might be in uppercase when it is in lowercase on the
-		 * disk. Ofcourse Windows doesn't care about casing. */
-		strtolower(buf + strlen(_searchpaths[sp]) - 1);
-		if (FileExists(buf)) break;
+		 * disk. Of course Windows doesn't care about casing. */
+		if (strtolower(buf + strlen(_searchpaths[sp]) - 1) && FileExists(buf)) return buf;
 #endif
 	}
 
-	return buf;
+	return NULL;
 }
 
 char *FioAppendDirectory(char *buf, size_t buflen, Searchpath sp, Subdirectory subdir)
@@ -342,8 +421,7 @@ static FILE *FioFOpenFileSp(const char *filename, const char *mode, Searchpath s
 
 	f = fopen(buf, mode);
 #if !defined(WIN32)
-	if (f == NULL) {
-		strtolower(buf + ((subdir == NO_DIRECTORY) ? 0 : strlen(_searchpaths[sp]) - 1));
+	if (f == NULL && strtolower(buf + ((subdir == NO_DIRECTORY) ? 0 : strlen(_searchpaths[sp]) - 1))) {
 		f = fopen(buf, mode);
 	}
 #endif
@@ -356,6 +434,13 @@ static FILE *FioFOpenFileSp(const char *filename, const char *mode, Searchpath s
 	return f;
 }
 
+/**
+ * Opens a file from inside a tar archive.
+ * @param entry The entry to open.
+ * @param filesize [out] If not \c NULL, size of the opened file.
+ * @return File handle of the opened file, or \c NULL if the file is not available.
+ * @note The file is read from within the tar file, and may not return \c EOF after reading the whole file.
+ */
 FILE *FioFOpenFileTar(TarFileListEntry *entry, size_t *filesize)
 {
 	FILE *f = fopen(entry->tar_filename, "rb");
@@ -366,7 +451,13 @@ FILE *FioFOpenFileTar(TarFileListEntry *entry, size_t *filesize)
 	return f;
 }
 
-/** Opens OpenTTD files somewhere in a personal or global directory */
+/**
+ * Opens a OpenTTD file somewhere in a personal or global directory.
+ * @param filename Name of the file to open.
+ * @param subdir Subdirectory to open.
+ * @param filename Name of the file to open.
+ * @return File handle of the opened file, or \c NULL if the file is not available.
+ */
 FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir, size_t *filesize)
 {
 	FILE *f = NULL;
@@ -380,7 +471,7 @@ FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir,
 	}
 
 	/* We can only use .tar in case of data-dir, and read-mode */
-	if (f == NULL && mode[0] == 'r') {
+	if (f == NULL && mode[0] == 'r' && subdir != NO_DIRECTORY) {
 		static const uint MAX_RESOLVED_LENGTH = 2 * (100 + 100 + 155) + 1; // Enough space to hold two filenames plus link. See 'TarHeader'.
 		char resolved_name[MAX_RESOLVED_LENGTH];
 
@@ -391,7 +482,7 @@ FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir,
 		size_t resolved_len = strlen(resolved_name);
 
 		/* Resolve ONE directory link */
-		for (TarLinkList::iterator link = _tar_linklist.begin(); link != _tar_linklist.end(); link++) {
+		for (TarLinkList::iterator link = _tar_linklist[subdir].begin(); link != _tar_linklist[subdir].end(); link++) {
 			const std::string &src = link->first;
 			size_t len = src.length();
 			if (resolved_len >= len && resolved_name[len - 1] == PATHSEPCHAR && strncmp(src.c_str(), resolved_name, len) == 0) {
@@ -405,8 +496,8 @@ FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir,
 			}
 		}
 
-		TarFileList::iterator it = _tar_filelist.find(resolved_name);
-		if (it != _tar_filelist.end()) {
+		TarFileList::iterator it = _tar_filelist[subdir].find(resolved_name);
+		if (it != _tar_filelist[subdir].end()) {
 			f = FioFOpenFileTar(&((*it).second), filesize);
 		}
 	}
@@ -414,7 +505,19 @@ FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir,
 	/* Sometimes a full path is given. To support
 	 * the 'subdirectory' must be 'removed'. */
 	if (f == NULL && subdir != NO_DIRECTORY) {
-		f = FioFOpenFile(filename, mode, NO_DIRECTORY, filesize);
+		switch (subdir) {
+			case BASESET_DIR:
+				f = FioFOpenFile(filename, mode, OLD_GM_DIR, filesize);
+				if (f != NULL) break;
+				/* FALL THROUGH */
+			case NEWGRF_DIR:
+				f = FioFOpenFile(filename, mode, OLD_DATA_DIR, filesize);
+				break;
+
+			default:
+				f = FioFOpenFile(filename, mode, NO_DIRECTORY, filesize);
+				break;
+		}
 	}
 
 	return f;
@@ -449,7 +552,7 @@ static void FioCreateDirectory(const char *name)
  * Appends, if necessary, the path separator character to the end of the string.
  * It does not add the path separator to zero-sized strings.
  * @param buf    string to append the separator to
- * @param buflen the length of the buf
+ * @param buflen the length of \a buf.
  * @return true iff the operation succeeded
  */
 bool AppendPathSeparator(char *buf, size_t buflen)
@@ -492,14 +595,19 @@ char *BuildWithFullPath(const char *dir)
 	return dest;
 }
 
-const char *FioTarFirstDir(const char *tarname)
+/**
+ * Find the first directory in a tar archive.
+ * @param tarname the name of the tar archive to look in.
+ * @param subdir  the subdirectory to look in.
+ */
+const char *FioTarFirstDir(const char *tarname, Subdirectory subdir)
 {
-	TarList::iterator it = _tar_list.find(tarname);
-	if (it == _tar_list.end()) return NULL;
+	TarList::iterator it = _tar_list[subdir].find(tarname);
+	if (it == _tar_list[subdir].end()) return NULL;
 	return (*it).second.dirname;
 }
 
-static void TarAddLink(const std::string &srcParam, const std::string &destParam)
+static void TarAddLink(const std::string &srcParam, const std::string &destParam, Subdirectory subdir)
 {
 	std::string src = srcParam;
 	std::string dest = destParam;
@@ -507,27 +615,27 @@ static void TarAddLink(const std::string &srcParam, const std::string &destParam
 	std::transform(src.begin(), src.end(), src.begin(), tolower);
 	std::transform(dest.begin(), dest.end(), dest.begin(), tolower);
 
-	TarFileList::iterator dest_file = _tar_filelist.find(dest);
-	if (dest_file != _tar_filelist.end()) {
+	TarFileList::iterator dest_file = _tar_filelist[subdir].find(dest);
+	if (dest_file != _tar_filelist[subdir].end()) {
 		/* Link to file. Process the link like the destination file. */
-		_tar_filelist.insert(TarFileList::value_type(src, dest_file->second));
+		_tar_filelist[subdir].insert(TarFileList::value_type(src, dest_file->second));
 	} else {
 		/* Destination file not found. Assume 'link to directory'
 		 * Append PATHSEPCHAR to 'src' and 'dest' if needed */
 		const std::string src_path = ((*src.rbegin() == PATHSEPCHAR) ? src : src + PATHSEPCHAR);
 		const std::string dst_path = (dest.length() == 0 ? "" : ((*dest.rbegin() == PATHSEPCHAR) ? dest : dest + PATHSEPCHAR));
-		_tar_linklist.insert(TarLinkList::value_type(src_path, dst_path));
+		_tar_linklist[subdir].insert(TarLinkList::value_type(src_path, dst_path));
 	}
 }
 
-void FioTarAddLink(const char *src, const char *dest)
+void FioTarAddLink(const char *src, const char *dest, Subdirectory subdir)
 {
-	TarAddLink(src, dest);
+	TarAddLink(src, dest, subdir);
 }
 
 /**
  * Simplify filenames from tars.
- * Replace '/' by PATHSEPCHAR, and force 'name' to lowercase.
+ * Replace '/' by #PATHSEPCHAR, and force 'name' to lowercase.
  * @param name Filename to process.
  */
 static void SimplifyFileName(char *name)
@@ -535,29 +643,70 @@ static void SimplifyFileName(char *name)
 	/* Force lowercase */
 	strtolower(name);
 
-	/* Tar-files always have '/' path-seperator, but we want our PATHSEPCHAR */
+	/* Tar-files always have '/' path-separator, but we want our PATHSEPCHAR */
 #if (PATHSEPCHAR != '/')
 	for (char *n = name; *n != '\0'; n++) if (*n == '/') *n = PATHSEPCHAR;
 #endif
 }
 
-/* static */ uint TarScanner::DoScan()
+/**
+ * Perform the scanning of a particular subdirectory.
+ * @param subdir The subdirectory to scan.
+ * @return The number of found tar files.
+ */
+uint TarScanner::DoScan(Subdirectory sd)
 {
-	_tar_filelist.clear();
-	_tar_list.clear();
+	_tar_filelist[sd].clear();
+	_tar_list[sd].clear();
+	uint num = this->Scan(".tar", sd, false);
+	if (sd == BASESET_DIR || sd == NEWGRF_DIR) num += this->Scan(".tar", OLD_DATA_DIR, false);
+	return num;
+}
 
+/* static */ uint TarScanner::DoScan(TarScanner::Mode mode)
+{
 	DEBUG(misc, 1, "Scanning for tars");
 	TarScanner fs;
-	uint num = fs.Scan(".tar", DATA_DIR, false);
-	num += fs.Scan(".tar", AI_DIR, false);
-	num += fs.Scan(".tar", AI_LIBRARY_DIR, false);
-	num += fs.Scan(".tar", SCENARIO_DIR, false);
+	uint num = 0;
+	if (mode & TarScanner::BASESET) {
+		num += fs.DoScan(BASESET_DIR);
+	}
+	if (mode & TarScanner::NEWGRF) {
+		num += fs.DoScan(NEWGRF_DIR);
+	}
+	if (mode & TarScanner::AI) {
+		num += fs.DoScan(AI_DIR);
+		num += fs.DoScan(AI_LIBRARY_DIR);
+	}
+	if (mode & TarScanner::GAME) {
+		num += fs.DoScan(GAME_DIR);
+		num += fs.DoScan(GAME_LIBRARY_DIR);
+	}
+	if (mode & TarScanner::SCENARIO) {
+		num += fs.DoScan(SCENARIO_DIR);
+		num += fs.DoScan(HEIGHTMAP_DIR);
+	}
 	DEBUG(misc, 1, "Scan complete, found %d files", num);
 	return num;
 }
 
-bool TarScanner::AddFile(const char *filename, size_t basepath_length)
+/**
+ * Add a single file to the scanned files of a tar, circumventing the scanning code.
+ * @param sd       The sub directory the file is in.
+ * @param filename The name of the file to add.
+ * @return True if the additions went correctly.
+ */
+bool TarScanner::AddFile(Subdirectory sd, const char *filename)
 {
+	this->subdir = sd;
+	return this->AddFile(filename, 0);
+}
+
+bool TarScanner::AddFile(const char *filename, size_t basepath_length, const char *tar_filename)
+{
+	/* No tar within tar. */
+	assert(tar_filename == NULL);
+
 	/* The TAR-header, repeated for every file */
 	typedef struct TarHeader {
 		char name[100];      ///< Name of the file
@@ -581,8 +730,8 @@ bool TarScanner::AddFile(const char *filename, size_t basepath_length)
 	} TarHeader;
 
 	/* Check if we already seen this file */
-	TarList::iterator it = _tar_list.find(filename);
-	if (it != _tar_list.end()) return false;
+	TarList::iterator it = _tar_list[this->subdir].find(filename);
+	if (it != _tar_list[this->subdir].end()) return false;
 
 	FILE *f = fopen(filename, "rb");
 	/* Although the file has been found there can be
@@ -592,8 +741,8 @@ bool TarScanner::AddFile(const char *filename, size_t basepath_length)
 	if (f == NULL) return false;
 
 	const char *dupped_filename = strdup(filename);
-	_tar_list[filename].filename = dupped_filename;
-	_tar_list[filename].dirname = NULL;
+	_tar_list[this->subdir][filename].filename = dupped_filename;
+	_tar_list[this->subdir][filename].dirname = NULL;
 
 	TarLinkList links; ///< Temporary list to collect links
 
@@ -661,7 +810,7 @@ bool TarScanner::AddFile(const char *filename, size_t basepath_length)
 				SimplifyFileName(name);
 
 				DEBUG(misc, 6, "Found file in tar: %s (" PRINTF_SIZE " bytes, " PRINTF_SIZE " offset)", name, skip, pos);
-				if (_tar_filelist.insert(TarFileList::value_type(name, entry)).second) num++;
+				if (_tar_filelist[this->subdir].insert(TarFileList::value_type(name, entry)).second) num++;
 
 				break;
 			}
@@ -734,7 +883,7 @@ bool TarScanner::AddFile(const char *filename, size_t basepath_length)
 
 				/* Store the first directory name we detect */
 				DEBUG(misc, 6, "Found dir in tar: %s", name);
-				if (_tar_list[filename].dirname == NULL) _tar_list[filename].dirname = strdup(name);
+				if (_tar_list[this->subdir][filename].dirname == NULL) _tar_list[this->subdir][filename].dirname = strdup(name);
 				break;
 
 			default:
@@ -763,7 +912,7 @@ bool TarScanner::AddFile(const char *filename, size_t basepath_length)
 	for (TarLinkList::iterator link = links.begin(); link != links.end(); link++) {
 		const std::string &src = link->first;
 		const std::string &dest = link->second;
-		TarAddLink(src, dest);
+		TarAddLink(src, dest, this->subdir);
 	}
 
 	return true;
@@ -773,13 +922,14 @@ bool TarScanner::AddFile(const char *filename, size_t basepath_length)
  * Extract the tar with the given filename in the directory
  * where the tar resides.
  * @param tar_filename the name of the tar to extract.
+ * @param subdir The sub directory the tar is in.
  * @return false on failure.
  */
-bool ExtractTar(const char *tar_filename)
+bool ExtractTar(const char *tar_filename, Subdirectory subdir)
 {
-	TarList::iterator it = _tar_list.find(tar_filename);
+	TarList::iterator it = _tar_list[subdir].find(tar_filename);
 	/* We don't know the file. */
-	if (it == _tar_list.end()) return false;
+	if (it == _tar_list[subdir].end()) return false;
 
 	const char *dirname = (*it).second.dirname;
 
@@ -797,7 +947,7 @@ bool ExtractTar(const char *tar_filename)
 	DEBUG(misc, 8, "Extracting %s to directory %s", tar_filename, filename);
 	FioCreateDirectory(filename);
 
-	for (TarFileList::iterator it2 = _tar_filelist.begin(); it2 != _tar_filelist.end(); it2++) {
+	for (TarFileList::iterator it2 = _tar_filelist[subdir].begin(); it2 != _tar_filelist[subdir].end(); it2++) {
 		if (strcmp((*it2).second.tar_filename, tar_filename) != 0) continue;
 
 		strecpy(p, (*it2).first.c_str(), lastof(filename));
@@ -1021,9 +1171,7 @@ void DeterminePaths(const char *exe)
 		}
 	} else {
 		char personal_dir[MAX_PATH];
-		FioFindFullPath(personal_dir, lengthof(personal_dir), BASE_DIR, "openttd.cfg");
-
-		if (FileExists(personal_dir)) {
+		if (FioFindFullPath(personal_dir, lengthof(personal_dir), BASE_DIR, "openttd.cfg") != NULL) {
 			char *end = strrchr(personal_dir, PATHSEPCHAR);
 			if (end != NULL) end[1] = '\0';
 			_personal_dir = strdup(personal_dir);
@@ -1055,7 +1203,7 @@ void DeterminePaths(const char *exe)
 #endif
 
 	static const Subdirectory default_subdirs[] = {
-		SAVE_DIR, AUTOSAVE_DIR, SCENARIO_DIR, HEIGHTMAP_DIR
+		SAVE_DIR, AUTOSAVE_DIR, SCENARIO_DIR, HEIGHTMAP_DIR, BASESET_DIR, NEWGRF_DIR, AI_DIR, AI_LIBRARY_DIR, GAME_DIR, GAME_LIBRARY_DIR, SCREENSHOT_DIR
 	};
 
 	for (uint i = 0; i < lengthof(default_subdirs); i++) {
@@ -1070,7 +1218,7 @@ void DeterminePaths(const char *exe)
 	FioCreateDirectory(_searchpaths[SP_AUTODOWNLOAD_DIR]);
 
 	/* Create the directory for each of the types of content */
-	const Subdirectory dirs[] = { SCENARIO_DIR, HEIGHTMAP_DIR, DATA_DIR, AI_DIR, AI_LIBRARY_DIR, GM_DIR };
+	const Subdirectory dirs[] = { SCENARIO_DIR, HEIGHTMAP_DIR, BASESET_DIR, NEWGRF_DIR, AI_DIR, AI_LIBRARY_DIR, GAME_DIR, GAME_LIBRARY_DIR };
 	for (uint i = 0; i < lengthof(dirs); i++) {
 		char *tmp = str_fmt("%s%s", _searchpaths[SP_AUTODOWNLOAD_DIR], _subdirs[dirs[i]]);
 		FioCreateDirectory(tmp);
@@ -1083,12 +1231,10 @@ void DeterminePaths(const char *exe)
 	/* If we don't have networking, we don't need to make the directory. But
 	 * if it exists we keep it, otherwise remove it from the search paths. */
 	if (!FileExists(_searchpaths[SP_AUTODOWNLOAD_DIR]))  {
-		free((void*)_searchpaths[SP_AUTODOWNLOAD_DIR]);
+		free(_searchpaths[SP_AUTODOWNLOAD_DIR]);
 		_searchpaths[SP_AUTODOWNLOAD_DIR] = NULL;
 	}
 #endif /* ENABLE_NETWORK */
-
-	TarScanner::DoScan();
 }
 
 /**
@@ -1109,6 +1255,14 @@ void SanitizeFilename(char *filename)
 	}
 }
 
+/**
+ * Load a file into memory.
+ * @param filename Name of the file to load.
+ * @param lenp [out] Length of loaded data.
+ * @param maxsize Maximum size to load.
+ * @return Pointer to new memory containing the loaded data, or \c NULL if loading failed.
+ * @note If \a maxsize less than the length of the file, loading fails.
+ */
 void *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize)
 {
 	FILE *in = fopen(filename, "rb");
@@ -1134,6 +1288,19 @@ void *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize)
 	return mem;
 }
 
+/**
+ * Helper to see whether a given filename matches the extension.
+ * @param extension The extension to look for.
+ * @param filename  The filename to look in for the extension.
+ * @return True iff the extension is NULL, or the filename ends with it.
+ */
+static bool MatchesExtension(const char *extension, const char *filename)
+{
+	if (extension == NULL) return true;
+
+	const char *ext = strrchr(filename, extension[0]);
+	return ext != NULL && strcasecmp(ext, extension) == 0;
+}
 
 /**
  * Scan a single directory (and recursively its children) and add
@@ -1171,15 +1338,7 @@ static uint ScanPath(FileScanner *fs, const char *extension, const char *path, s
 			num += ScanPath(fs, extension, filename, basepath_length, recursive);
 		} else if (S_ISREG(sb.st_mode)) {
 			/* File */
-			if (extension != NULL) {
-				char *ext = strrchr(filename, '.');
-
-				/* If no extension or extension isn't .grf, skip the file */
-				if (ext == NULL) continue;
-				if (strcasecmp(ext, extension) != 0) continue;
-			}
-
-			if (fs->AddFile(filename, basepath_length)) num++;
+			if (MatchesExtension(extension, filename) && fs->AddFile(filename, basepath_length, NULL)) num++;
 		}
 	}
 
@@ -1199,21 +1358,13 @@ static uint ScanTar(FileScanner *fs, const char *extension, TarFileList::iterato
 	uint num = 0;
 	const char *filename = (*tar).first.c_str();
 
-	if (extension != NULL) {
-		const char *ext = strrchr(filename, '.');
-
-		/* If no extension or extension isn't .grf, skip the file */
-		if (ext == NULL) return false;
-		if (strcasecmp(ext, extension) != 0) return false;
-	}
-
-	if (fs->AddFile(filename, 0)) num++;
+	if (MatchesExtension(extension, filename) && fs->AddFile(filename, 0, (*tar).second.tar_filename)) num++;
 
 	return num;
 }
 
 /**
- * Scan for files with the given extention in the given search path.
+ * Scan for files with the given extension in the given search path.
  * @param extension the extension of files to search for.
  * @param sd        the sub directory to search in.
  * @param tars      whether to search in the tars too.
@@ -1223,6 +1374,8 @@ static uint ScanTar(FileScanner *fs, const char *extension, TarFileList::iterato
  */
 uint FileScanner::Scan(const char *extension, Subdirectory sd, bool tars, bool recursive)
 {
+	this->subdir = sd;
+
 	Searchpath sp;
 	char path[MAX_PATH];
 	TarFileList::iterator tar;
@@ -1236,17 +1389,28 @@ uint FileScanner::Scan(const char *extension, Subdirectory sd, bool tars, bool r
 		num += ScanPath(this, extension, path, strlen(path), recursive);
 	}
 
-	if (tars) {
-		FOR_ALL_TARS(tar) {
+	if (tars && sd != NO_DIRECTORY) {
+		FOR_ALL_TARS(tar, sd) {
 			num += ScanTar(this, extension, tar);
 		}
 	}
 
+	switch (sd) {
+		case BASESET_DIR:
+			num += this->Scan(extension, OLD_GM_DIR, tars, recursive);
+			/* FALL THROUGH */
+		case NEWGRF_DIR:
+			num += this->Scan(extension, OLD_DATA_DIR, tars, recursive);
+			break;
+
+		default: break;
+	}
+
 	return num;
 }
 
 /**
- * Scan for files with the given extention in the given search path.
+ * Scan for files with the given extension in the given search path.
  * @param extension the extension of files to search for.
  * @param directory the sub directory to search in.
  * @param recursive whether to search recursively
