@@ -1,4 +1,4 @@
-/* $Id: fileio_func.h 20689 2010-08-29 15:58:43Z alberth $ */
+/* $Id: fileio_func.h 24804 2012-12-09 16:52:43Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,6 +12,7 @@
 #ifndef FILEIO_FUNC_H
 #define FILEIO_FUNC_H
 
+#include "core/enum_type.hpp"
 #include "fileio_type.h"
 
 void FioSeekTo(size_t pos, int mode);
@@ -22,12 +23,12 @@ byte FioReadByte();
 uint16 FioReadWord();
 uint32 FioReadDword();
 void FioCloseAll();
-void FioOpenFile(int slot, const char *filename);
+void FioOpenFile(int slot, const char *filename, Subdirectory subdir);
 void FioReadBlock(void *ptr, size_t size);
 void FioSkipBytes(int n);
 
 /**
- * The searchpaths OpenTTD could search through.
+ * The search paths OpenTTD could search through.
  * At least one of the slots has to be filled with a path.
  * NULL paths tell that there is no such path for the
  * current operating system.
@@ -48,27 +49,30 @@ static inline bool IsValidSearchPath(Searchpath sp)
 #define FOR_ALL_SEARCHPATHS(sp) for (sp = SP_FIRST_DIR; sp < NUM_SEARCHPATHS; sp++) if (IsValidSearchPath(sp))
 
 void FioFCloseFile(FILE *f);
-FILE *FioFOpenFile(const char *filename, const char *mode = "rb", Subdirectory subdir = DATA_DIR, size_t *filesize = NULL);
-bool FioCheckFileExists(const char *filename, Subdirectory subdir = DATA_DIR);
+FILE *FioFOpenFile(const char *filename, const char *mode, Subdirectory subdir, size_t *filesize = NULL);
+bool FioCheckFileExists(const char *filename, Subdirectory subdir);
 char *FioGetFullPath(char *buf, size_t buflen, Searchpath sp, Subdirectory subdir, const char *filename);
 char *FioFindFullPath(char *buf, size_t buflen, Subdirectory subdir, const char *filename);
 char *FioAppendDirectory(char *buf, size_t buflen, Searchpath sp, Subdirectory subdir);
 char *FioGetDirectory(char *buf, size_t buflen, Subdirectory subdir);
 
+const char *FiosGetScreenshotDir();
+
 void SanitizeFilename(char *filename);
 bool AppendPathSeparator(char *buf, size_t buflen);
 void DeterminePaths(const char *exe);
 void *ReadFileToMem(const char *filename, size_t *lenp, size_t maxsize);
 bool FileExists(const char *filename);
-const char *FioTarFirstDir(const char *tarname);
-void FioTarAddLink(const char *src, const char *dest);
-bool ExtractTar(const char *tar_filename);
+const char *FioTarFirstDir(const char *tarname, Subdirectory subdir);
+void FioTarAddLink(const char *src, const char *dest, Subdirectory subdir);
+bool ExtractTar(const char *tar_filename, Subdirectory subdir);
 
 extern char *_personal_dir; ///< custom directory for personal settings, saves, newgrf, etc.
 
 /** Helper for scanning for files with a given name */
-class FileScanner
-{
+class FileScanner {
+protected:
+	Subdirectory subdir; ///< The current sub directory we are searching through
 public:
 	/** Destruct the proper one... */
 	virtual ~FileScanner() {}
@@ -81,23 +85,39 @@ public:
 	 * @param filename        the full path to the file to read
 	 * @param basepath_length amount of characters to chop of before to get a
 	 *                        filename relative to the search path.
+	 * @param tar_filename    the name of the tar file the file is read from.
 	 * @return true if the file is added.
 	 */
-	virtual bool AddFile(const char *filename, size_t basepath_length) = 0;
+	virtual bool AddFile(const char *filename, size_t basepath_length, const char *tar_filename) = 0;
 };
 
 /** Helper for scanning for files with tar as extension */
 class TarScanner : FileScanner {
+	uint DoScan(Subdirectory sd);
 public:
-	/* virtual */ bool AddFile(const char *filename, size_t basepath_length);
+	/** The mode of tar scanning. */
+	enum Mode {
+		NONE     = 0,      ///< Scan nothing.
+		BASESET  = 1 << 0, ///< Scan for base sets.
+		NEWGRF   = 1 << 1, ///< Scan for non-base sets.
+		AI       = 1 << 2, ///< Scan for AIs and its libraries.
+		SCENARIO = 1 << 3, ///< Scan for scenarios and heightmaps.
+		GAME     = 1 << 4, ///< Scan for game scripts.
+		ALL      = BASESET | NEWGRF | AI | SCENARIO | GAME, ///< Scan for everything.
+	};
+
+	/* virtual */ bool AddFile(const char *filename, size_t basepath_length, const char *tar_filename = NULL);
+
+	bool AddFile(Subdirectory sd, const char *filename);
 
 	/** Do the scan for Tars. */
-	static uint DoScan();
+	static uint DoScan(TarScanner::Mode mode);
 };
 
+DECLARE_ENUM_AS_BIT_SET(TarScanner::Mode)
+
 /* Implementation of opendir/readdir/closedir for Windows */
 #if defined(WIN32)
-#include <windows.h>
 struct DIR;
 
 struct dirent { // XXX - only d_name implemented
@@ -108,19 +128,6 @@ struct dirent { // XXX - only d_name implemented
 	DIR *dir;
 };
 
-struct DIR {
-	HANDLE hFind;
-	/* the dirent returned by readdir.
-	 * note: having only one global instance is not possible because
-	 * multiple independent opendir/readdir sequences must be supported. */
-	dirent ent;
-	WIN32_FIND_DATA fd;
-	/* since opendir calls FindFirstFile, we need a means of telling the
-	 * first call to readdir that we already have a file.
-	 * that's the case iff this is true */
-	bool at_first_entry;
-};
-
 DIR *opendir(const TCHAR *path);
 struct dirent *readdir(DIR *d);
 int closedir(DIR *d);
