@@ -1,4 +1,4 @@
-/* $Id: gfxinit.cpp 20430 2010-08-09 21:48:19Z rubidium $ */
+/* $Id: gfxinit.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -15,42 +15,53 @@
 #include "3rdparty/md5/md5.h"
 #include "fontcache.h"
 #include "gfx_func.h"
-#include "settings_type.h"
+#include "blitter/factory.hpp"
+#include "video/video_driver.hpp"
 
 /* The type of set we're replacing */
 #define SET_TYPE "graphics"
 #include "base_media_func.h"
 
 #include "table/sprites.h"
-#include "table/palette_convert.h"
 
-/** The currently used palette */
-PaletteType _use_palette = PAL_AUTODETECT;
-/** Whether the given NewGRFs must get a palette remap or not. */
+/** Whether the given NewGRFs must get a palette remap from windows to DOS or not. */
 bool _palette_remap_grf[MAX_FILE_SLOTS];
-/** Palette map to go from the !_use_palette to the _use_palette */
-const byte *_palette_remap = NULL;
-/** Palette map to go from the _use_palette to the !_use_palette */
-const byte *_palette_reverse_remap = NULL;
 
 #include "table/landscape_sprite.h"
 
+/** Offsets for loading the different "replacement" sprites in the files. */
 static const SpriteID * const _landscape_spriteindexes[] = {
-	_landscape_spriteindexes_1,
-	_landscape_spriteindexes_2,
-	_landscape_spriteindexes_3,
+	_landscape_spriteindexes_arctic,
+	_landscape_spriteindexes_tropic,
+	_landscape_spriteindexes_toyland,
 };
 
+/**
+ * Load an old fashioned GRF file.
+ * @param filename   The name of the file to open.
+ * @param load_index The offset of the first sprite.
+ * @param file_index The Fio offset to load the file in.
+ * @return The number of loaded sprites.
+ */
 static uint LoadGrfFile(const char *filename, uint load_index, int file_index)
 {
 	uint load_index_org = load_index;
 	uint sprite_id = 0;
 
-	FioOpenFile(file_index, filename);
+	FioOpenFile(file_index, filename, BASESET_DIR);
 
 	DEBUG(sprite, 2, "Reading grf-file '%s'", filename);
 
-	while (LoadNextSprite(load_index, file_index, sprite_id)) {
+	byte container_ver = GetGRFContainerVersion();
+	if (container_ver == 0) usererror("Base grf '%s' is corrupt", filename);
+	ReadGRFSpriteOffsets(container_ver);
+	if (container_ver >= 2) {
+		/* Read compression. */
+		byte compression = FioReadByte();
+		if (compression != 0) usererror("Unsupported compression format");
+	}
+
+	while (LoadNextSprite(load_index, file_index, sprite_id, container_ver)) {
 		load_index++;
 		sprite_id++;
 		if (load_index >= MAX_SPRITES) {
@@ -62,32 +73,42 @@ static uint LoadGrfFile(const char *filename, uint load_index, int file_index)
 	return load_index - load_index_org;
 }
 
-
-static void LoadSpritesIndexed(int file_index, uint *sprite_id, const SpriteID *index_tbl)
+/**
+ * Load an old fashioned GRF file to replace already loaded sprites.
+ * @param filename   The name of the file to open.
+ * @param index_tlb  The offsets of each of the sprites.
+ * @param file_index The Fio offset to load the file in.
+ * @return The number of loaded sprites.
+ */
+static void LoadGrfFileIndexed(const char *filename, const SpriteID *index_tbl, int file_index)
 {
 	uint start;
+	uint sprite_id = 0;
+
+	FioOpenFile(file_index, filename, BASESET_DIR);
+
+	DEBUG(sprite, 2, "Reading indexed grf-file '%s'", filename);
+
+	byte container_ver = GetGRFContainerVersion();
+	if (container_ver == 0) usererror("Base grf '%s' is corrupt", filename);
+	ReadGRFSpriteOffsets(container_ver);
+	if (container_ver >= 2) {
+		/* Read compression. */
+		byte compression = FioReadByte();
+		if (compression != 0) usererror("Unsupported compression format");
+	}
+
 	while ((start = *index_tbl++) != END) {
 		uint end = *index_tbl++;
 
 		do {
-			bool b = LoadNextSprite(start, file_index, *sprite_id);
+			bool b = LoadNextSprite(start, file_index, sprite_id, container_ver);
 			assert(b);
-			(*sprite_id)++;
+			sprite_id++;
 		} while (++start <= end);
 	}
 }
 
-static void LoadGrfIndexed(const char *filename, const SpriteID *index_tbl, int file_index)
-{
-	uint sprite_id = 0;
-
-	FioOpenFile(file_index, filename);
-
-	DEBUG(sprite, 2, "Reading indexed grf-file '%s'", filename);
-
-	LoadSpritesIndexed(file_index, &sprite_id, index_tbl);
-}
-
 /**
  * Checks whether the MD5 checksums of the files are correct.
  *
@@ -97,10 +118,9 @@ void CheckExternalFiles()
 {
 	if (BaseGraphics::GetUsedSet() == NULL || BaseSounds::GetUsedSet() == NULL) return;
 
-	BaseGraphics::DeterminePalette();
 	const GraphicsSet *used_set = BaseGraphics::GetUsedSet();
 
-	DEBUG(grf, 1, "Using the %s base graphics set with the %s palette", used_set->name, _use_palette == PAL_DOS ? "DOS" : "Windows");
+	DEBUG(grf, 1, "Using the %s base graphics set", used_set->name);
 
 	static const size_t ERROR_MESSAGE_LENGTH = 256;
 	static const size_t MISSING_FILE_MESSAGE_LENGTH = 128;
@@ -114,10 +134,10 @@ void CheckExternalFiles()
 	const char *last = lastof(error_msg);
 
 	if (used_set->GetNumInvalid() != 0) {
-		/* Not all files were loaded succesfully, see which ones */
+		/* Not all files were loaded successfully, see which ones */
 		add_pos += seprintf(add_pos, last, "Trying to load graphics set '%s', but it is incomplete. The game will probably not run correctly until you properly install this set or select another one. See section 4.1 of readme.txt.\n\nThe following files are corrupted or missing:\n", used_set->name);
 		for (uint i = 0; i < GraphicsSet::NUM_FILES; i++) {
-			MD5File::ChecksumResult res = used_set->files[i].CheckMD5(DATA_DIR);
+			MD5File::ChecksumResult res = GraphicsSet::CheckMD5(&used_set->files[i], BASESET_DIR);
 			if (res != MD5File::CR_MATCH) add_pos += seprintf(add_pos, last, "\t%s is %s (%s)\n", used_set->files[i].filename, res == MD5File::CR_MISMATCH ? "corrupt" : "missing", used_set->files[i].missing_warning);
 		}
 		add_pos += seprintf(add_pos, last, "\n");
@@ -130,20 +150,20 @@ void CheckExternalFiles()
 		assert_compile(SoundsSet::NUM_FILES == 1);
 		/* No need to loop each file, as long as there is only a single
 		 * sound file. */
-		add_pos += seprintf(add_pos, last, "\t%s is %s (%s)\n", sounds_set->files->filename, sounds_set->files->CheckMD5(DATA_DIR) == MD5File::CR_MISMATCH ? "corrupt" : "missing", sounds_set->files->missing_warning);
+		add_pos += seprintf(add_pos, last, "\t%s is %s (%s)\n", sounds_set->files->filename, SoundsSet::CheckMD5(sounds_set->files, BASESET_DIR) == MD5File::CR_MISMATCH ? "corrupt" : "missing", sounds_set->files->missing_warning);
 	}
 
 	if (add_pos != error_msg) ShowInfoF("%s", error_msg);
 }
 
-
+/** Actually load the sprite tables. */
 static void LoadSpriteTables()
 {
 	memset(_palette_remap_grf, 0, sizeof(_palette_remap_grf));
 	uint i = FIRST_GRF_SLOT;
 	const GraphicsSet *used_set = BaseGraphics::GetUsedSet();
 
-	_palette_remap_grf[i] = (_use_palette != used_set->palette);
+	_palette_remap_grf[i] = (PAL_DOS != used_set->palette);
 	LoadGrfFile(used_set->files[GFT_BASE].filename, 0, i++);
 
 	/*
@@ -152,7 +172,7 @@ static void LoadSpriteTables()
 	 * has a few sprites less. However, we do not care about those missing
 	 * sprites as they are not shown anyway (logos in intro game).
 	 */
-	_palette_remap_grf[i] = (_use_palette != used_set->palette);
+	_palette_remap_grf[i] = (PAL_DOS != used_set->palette);
 	LoadGrfFile(used_set->files[GFT_LOGOS].filename, 4793, i++);
 
 	/*
@@ -161,8 +181,8 @@ static void LoadSpriteTables()
 	 * and the ground sprites.
 	 */
 	if (_settings_game.game_creation.landscape != LT_TEMPERATE) {
-		_palette_remap_grf[i] = (_use_palette != used_set->palette);
-		LoadGrfIndexed(
+		_palette_remap_grf[i] = (PAL_DOS != used_set->palette);
+		LoadGrfFileIndexed(
 			used_set->files[GFT_ARCTIC + _settings_game.game_creation.landscape - 1].filename,
 			_landscape_spriteindexes[_settings_game.game_creation.landscape - 1],
 			i++
@@ -178,10 +198,19 @@ static void LoadSpriteTables()
 	 * so we have to manually add it, and then remove it later.
 	 */
 	GRFConfig *top = _grfconfig;
-	GRFConfig *master = CallocT<GRFConfig>(1);
-	master->filename = strdup(used_set->files[GFT_EXTRA].filename);
-	FillGRFDetails(master, false);
-	master->windows_paletted = (used_set->palette == PAL_WINDOWS);
+	GRFConfig *master = new GRFConfig(used_set->files[GFT_EXTRA].filename);
+
+	/* We know the palette of the base set, so if the base NewGRF is not
+	 * setting one, use the palette of the base set and not the global
+	 * one which might be the wrong palette for this base NewGRF.
+	 * The value set here might be overridden via action14 later. */
+	switch (used_set->palette) {
+		case PAL_DOS:     master->palette |= GRFP_GRF_DOS;     break;
+		case PAL_WINDOWS: master->palette |= GRFP_GRF_WINDOWS; break;
+		default: break;
+	}
+	FillGRFDetails(master, false, BASESET_DIR);
+
 	ClrBit(master->flags, GCF_INIT_ONLY);
 	master->next = top;
 	_grfconfig = master;
@@ -189,15 +218,45 @@ static void LoadSpriteTables()
 	LoadNewGRF(SPR_NEWGRFS_BASE, i);
 
 	/* Free and remove the top element. */
-	ClearGRFConfig(&master);
+	delete master;
 	_grfconfig = top;
 }
 
 
+/**
+ * Check blitter needed by NewGRF config and switch if needed.
+ */
+static void SwitchNewGRFBlitter()
+{
+	/* Get blitter of base set. */
+	bool is_32bpp = BaseGraphics::GetUsedSet()->blitter == BLT_32BPP;
+
+	/* Get combined blitter mode of all NewGRFs. */
+	for (GRFConfig *c = _grfconfig; c != NULL; c = c->next) {
+		if (c->status == GCS_DISABLED || c->status == GCS_NOT_FOUND || HasBit(c->flags, GCF_INIT_ONLY)) continue;
+
+		if (c->palette & GRFP_BLT_32BPP) is_32bpp = true;
+	}
+
+	/* A GRF would like a 32 bpp blitter, switch blitter if needed. Never switch if the blitter was specified by the user. */
+	if (_blitter_autodetected && is_32bpp && BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth() != 0 && BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth() < 16) {
+		const char *cur_blitter = BlitterFactoryBase::GetCurrentBlitter()->GetName();
+		if (BlitterFactoryBase::SelectBlitter("32bpp-anim") != NULL) {
+			if (!_video_driver->AfterBlitterChange()) {
+				/* Failed to switch blitter, let's hope we can return to the old one. */
+				if (BlitterFactoryBase::SelectBlitter(cur_blitter) == NULL || !_video_driver->AfterBlitterChange()) usererror("Failed to reinitialize video driver for 32 bpp blitter. Specify a fixed blitter in the config");
+			}
+		}
+	}
+}
+
+/** Initialise and load all the sprites. */
 void GfxLoadSprites()
 {
 	DEBUG(sprite, 2, "Loading sprite set %d", _settings_game.game_creation.landscape);
 
+	SwitchNewGRFBlitter();
+	ClearFontCache();
 	GfxInitSpriteMem();
 	LoadSpriteTables();
 	GfxInitPalettes();
@@ -207,33 +266,62 @@ void GfxLoadSprites()
 
 bool GraphicsSet::FillSetDetails(IniFile *ini, const char *path, const char *full_filename)
 {
-	bool ret = this->BaseSet<GraphicsSet, MAX_GFT, DATA_DIR>::FillSetDetails(ini, path, full_filename, false);
+	bool ret = this->BaseSet<GraphicsSet, MAX_GFT, true>::FillSetDetails(ini, path, full_filename, false);
 	if (ret) {
 		IniGroup *metadata = ini->GetGroup("metadata");
 		IniItem *item;
 
 		fetch_metadata("palette");
 		this->palette = (*item->value == 'D' || *item->value == 'd') ? PAL_DOS : PAL_WINDOWS;
+
+		/* Get optional blitter information. */
+		item = metadata->GetItem("blitter", false);
+		this->blitter = (item != NULL && *item->value == '3') ? BLT_32BPP : BLT_8BPP;
 	}
 	return ret;
 }
 
+/**
+ * Calculate and check the MD5 hash of the supplied GRF.
+ * @param file The file get the hash of.
+ * @param subdir The sub directory to get the files from.
+ * @return
+ * - #CR_MATCH if the MD5 hash matches
+ * - #CR_MISMATCH if the MD5 does not match
+ * - #CR_NO_FILE if the file misses
+ */
+/* static */ MD5File::ChecksumResult GraphicsSet::CheckMD5(const MD5File *file, Subdirectory subdir)
+{
+	size_t size = 0;
+	FILE *f = FioFOpenFile(file->filename, "rb", subdir, &size);
+	if (f == NULL) return MD5File::CR_NO_FILE;
+
+	size_t max = GRFGetSizeOfDataSection(f);
+
+	FioFCloseFile(f);
+
+	return file->CheckMD5(subdir, max);
+}
+
 
 /**
  * Calculate and check the MD5 hash of the supplied filename.
  * @param subdir The sub directory to get the files from
+ * @param max_size Only calculate the hash for this many bytes from the file start.
  * @return
- *  CR_MATCH if the MD5 hash matches
- *  CR_MISMATCH if the MD5 does not match
- *  CR_NO_FILE if the file misses
+ * - #CR_MATCH if the MD5 hash matches
+ * - #CR_MISMATCH if the MD5 does not match
+ * - #CR_NO_FILE if the file misses
  */
-MD5File::ChecksumResult MD5File::CheckMD5(Subdirectory subdir) const
+MD5File::ChecksumResult MD5File::CheckMD5(Subdirectory subdir, size_t max_size) const
 {
 	size_t size;
 	FILE *f = FioFOpenFile(this->filename, "rb", subdir, &size);
 
 	if (f == NULL) return CR_NO_FILE;
 
+	size = min(size, max_size);
+
 	Md5 checksum;
 	uint8 buffer[1024];
 	uint8 digest[16];
@@ -254,38 +342,8 @@ MD5File::ChecksumResult MD5File::CheckMD5(Subdirectory subdir) const
 static const char * const _graphics_file_names[] = { "base", "logos", "arctic", "tropical", "toyland", "extra" };
 
 /** Implementation */
-template <class T, size_t Tnum_files, Subdirectory Tsubdir>
-/* static */ const char * const *BaseSet<T, Tnum_files, Tsubdir>::file_names = _graphics_file_names;
-
-extern void UpdateNewGRFConfigPalette();
-
-/**
- * Determine the palette that has to be used.
- *  - forced palette via command line -> leave it that way
- *  - otherwise -> palette based on the graphics pack
- */
-/* static */ void BaseGraphics::DeterminePalette()
-{
-	assert(BaseGraphics::used_set != NULL);
-	if (_use_palette >= MAX_PAL) _use_palette = BaseGraphics::used_set->palette;
-
-	switch (_use_palette) {
-		case PAL_DOS:
-			_palette_remap = _palmap_w2d;
-			_palette_reverse_remap = _palmap_d2w;
-			break;
-
-		case PAL_WINDOWS:
-			_palette_remap = _palmap_d2w;
-			_palette_reverse_remap = _palmap_w2d;
-			break;
-
-		default:
-			NOT_REACHED();
-	}
-
-	UpdateNewGRFConfigPalette();
-}
+template <class T, size_t Tnum_files, bool Tsearch_in_tars>
+/* static */ const char * const *BaseSet<T, Tnum_files, Tsearch_in_tars>::file_names = _graphics_file_names;
 
 template <class Tbase_set>
 /* static */ bool BaseMedia<Tbase_set>::DetermineBestSet()
@@ -294,7 +352,7 @@ template <class Tbase_set>
 
 	const Tbase_set *best = NULL;
 	for (const Tbase_set *c = BaseMedia<Tbase_set>::available_sets; c != NULL; c = c->next) {
-		/* Skip unuseable sets */
+		/* Skip unusable sets */
 		if (c->GetNumMissing() != 0) continue;
 
 		if (best == NULL ||
@@ -302,7 +360,7 @@ template <class Tbase_set>
 				best->valid_files < c->valid_files ||
 				(best->valid_files == c->valid_files && (
 					(best->shortname == c->shortname && best->version < c->version) ||
-					(best->palette != _use_palette && c->palette == _use_palette)))) {
+					(best->palette != PAL_DOS && c->palette == PAL_DOS)))) {
 			best = c;
 		}
 	}
