@@ -1,4 +1,4 @@
-/* $Id: landscape.cpp 19665 2010-04-17 22:27:49Z rubidium $ */
+/* $Id: landscape.cpp 24132 2012-04-17 19:43:43Z michi_cc $ */
 
 /*
  * This file is part of OpenTTD.
@@ -18,19 +18,22 @@
 #include "viewport_func.h"
 #include "command_func.h"
 #include "landscape.h"
-#include "variables.h"
 #include "void_map.h"
 #include "tgp.h"
 #include "genworld.h"
 #include "fios.h"
-#include "functions.h"
 #include "date_func.h"
 #include "water.h"
 #include "effectvehicle_func.h"
 #include "landscape_type.h"
 #include "animated_tile_func.h"
 #include "core/random_func.hpp"
+#include "object_base.h"
+#include "company_func.h"
+#include "pathfinder/npf/aystar.h"
+#include <list>
 
+#include "table/strings.h"
 #include "table/sprites.h"
 
 extern const TileTypeProcs
@@ -41,14 +44,16 @@ extern const TileTypeProcs
 	_tile_type_trees_procs,
 	_tile_type_station_procs,
 	_tile_type_water_procs,
-	_tile_type_dummy_procs,
+	_tile_type_void_procs,
 	_tile_type_industry_procs,
 	_tile_type_tunnelbridge_procs,
-	_tile_type_unmovable_procs;
+	_tile_type_object_procs;
 
-/** Tile callback functions for each type of tile.
+/**
+ * Tile callback functions for each type of tile.
  * @ingroup TileCallbackGroup
- * @see TileType */
+ * @see TileType
+ */
 const TileTypeProcs * const _tile_type_procs[16] = {
 	&_tile_type_clear_procs,        ///< Callback functions for MP_CLEAR tiles
 	&_tile_type_rail_procs,         ///< Callback functions for MP_RAILWAY tiles
@@ -57,14 +62,14 @@ const TileTypeProcs * const _tile_type_procs[16] = {
 	&_tile_type_trees_procs,        ///< Callback functions for MP_TREES tiles
 	&_tile_type_station_procs,      ///< Callback functions for MP_STATION tiles
 	&_tile_type_water_procs,        ///< Callback functions for MP_WATER tiles
-	&_tile_type_dummy_procs,        ///< Callback functions for MP_VOID tiles
+	&_tile_type_void_procs,         ///< Callback functions for MP_VOID tiles
 	&_tile_type_industry_procs,     ///< Callback functions for MP_INDUSTRY tiles
 	&_tile_type_tunnelbridge_procs, ///< Callback functions for MP_TUNNELBRIDGE tiles
-	&_tile_type_unmovable_procs,    ///< Callback functions for MP_UNMOVABLE tiles
+	&_tile_type_object_procs,       ///< Callback functions for MP_OBJECT tiles
 };
 
-/* landscape slope => sprite */
-const byte _tileh_to_sprite[32] = {
+/** landscape slope => sprite */
+extern const byte _slope_to_sprite_offset[32] = {
 	0, 1, 2, 3, 4, 5, 6,  7, 8, 9, 10, 11, 12, 13, 14, 0,
 	0, 0, 0, 0, 0, 0, 0, 16, 0, 0,  0, 17,  0, 15, 18, 0,
 };
@@ -72,10 +77,11 @@ const byte _tileh_to_sprite[32] = {
 /**
  * Description of the snow line throughout the year.
  *
- * If it is \c NULL, a static snowline height is used, as set by \c _settings_game.game_creation.snow_line.
- * Otherwise it points to a table loaded from a newGRF file, that describes the variable snowline
+ * If it is \c NULL, a static snowline height is used, as set by \c _settings_game.game_creation.snow_line_height.
+ * Otherwise it points to a table loaded from a newGRF file that describes the variable snowline.
  * @ingroup SnowLineGroup
- * @see GetSnowLine() GameCreationSettings */
+ * @see GetSnowLine() GameCreationSettings
+ */
 static SnowLine *_snow_line = NULL;
 
 /**
@@ -91,7 +97,7 @@ uint ApplyFoundationToSlope(Foundation f, Slope *s)
 	if (!IsFoundation(f)) return 0;
 
 	if (IsLeveledFoundation(f)) {
-		uint dz = TILE_HEIGHT + (IsSteepSlope(*s) ? TILE_HEIGHT : 0);
+		uint dz = 1 + (IsSteepSlope(*s) ? 1 : 0);
 		*s = SLOPE_FLAT;
 		return dz;
 	}
@@ -106,7 +112,7 @@ uint ApplyFoundationToSlope(Foundation f, Slope *s)
 		return 0;
 	}
 
-	uint dz = IsSteepSlope(*s) ? TILE_HEIGHT : 0;
+	uint dz = IsSteepSlope(*s) ? 1 : 0;
 	Corner highest_corner = GetHighestSlopeCorner(*s);
 
 	switch (f) {
@@ -139,24 +145,24 @@ uint ApplyFoundationToSlope(Foundation f, Slope *s)
  * @param corners slope to examine
  * @return height of given point of given slope
  */
-uint GetPartialZ(int x, int y, Slope corners)
+uint GetPartialPixelZ(int x, int y, Slope corners)
 {
 	if (IsHalftileSlope(corners)) {
 		switch (GetHalftileSlopeCorner(corners)) {
 			case CORNER_W:
-				if (x - y >= 0) return GetSlopeMaxZ(corners);
+				if (x - y >= 0) return GetSlopeMaxPixelZ(corners);
 				break;
 
 			case CORNER_S:
-				if (x - (y ^ 0xF) >= 0) return GetSlopeMaxZ(corners);
+				if (x - (y ^ 0xF) >= 0) return GetSlopeMaxPixelZ(corners);
 				break;
 
 			case CORNER_E:
-				if (y - x >= 0) return GetSlopeMaxZ(corners);
+				if (y - x >= 0) return GetSlopeMaxPixelZ(corners);
 				break;
 
 			case CORNER_N:
-				if ((y ^ 0xF) - x >= 0) return GetSlopeMaxZ(corners);
+				if ((y ^ 0xF) - x >= 0) return GetSlopeMaxPixelZ(corners);
 				break;
 
 			default: NOT_REACHED();
@@ -266,7 +272,7 @@ uint GetPartialZ(int x, int y, Slope corners)
 	return z;
 }
 
-uint GetSlopeZ(int x, int y)
+int GetSlopePixelZ(int x, int y)
 {
 	TileIndex tile = TileVirtXY(x, y);
 
@@ -285,13 +291,13 @@ uint GetSlopeZ(int x, int y)
 int GetSlopeZInCorner(Slope tileh, Corner corner)
 {
 	assert(!IsHalftileSlope(tileh));
-	return ((tileh & SlopeWithOneCornerRaised(corner)) != 0 ? TILE_HEIGHT : 0) + (tileh == SteepSlope(corner) ? TILE_HEIGHT : 0);
+	return ((tileh & SlopeWithOneCornerRaised(corner)) != 0 ? 1 : 0) + (tileh == SteepSlope(corner) ? 1 : 0);
 }
 
 /**
  * Determine the Z height of the corners of a specific tile edge
  *
- * @note If a tile has a non-continuous halftile foundation, a corner can have different heights wrt. it's edges.
+ * @note If a tile has a non-continuous halftile foundation, a corner can have different heights wrt. its edges.
  *
  * @pre z1 and z2 must be initialized (typ. with TileZ). The corner heights just get added.
  *
@@ -300,7 +306,7 @@ int GetSlopeZInCorner(Slope tileh, Corner corner)
  * @param z1 Gets incremented by the height of the first corner of the edge. (near corner wrt. the camera)
  * @param z2 Gets incremented by the height of the second corner of the edge. (far corner wrt. the camera)
  */
-void GetSlopeZOnEdge(Slope tileh, DiagDirection edge, int *z1, int *z2)
+void GetSlopePixelZOnEdge(Slope tileh, DiagDirection edge, int *z1, int *z2)
 {
 	static const Slope corners[4][4] = {
 		/*    corner     |          steep slope
@@ -323,13 +329,13 @@ void GetSlopeZOnEdge(Slope tileh, DiagDirection edge, int *z1, int *z2)
 
 /**
  * Get slope of a tile on top of a (possible) foundation
- * If a tile does not have a foundation, the function returns the same as GetTileSlope.
+ * If a tile does not have a foundation, the function returns the same as GetTilePixelSlope.
  *
  * @param tile The tile of interest.
  * @param z returns the z of the foundation slope. (Can be NULL, if not needed)
  * @return The slope on top of the foundation.
  */
-Slope GetFoundationSlope(TileIndex tile, uint *z)
+Slope GetFoundationSlope(TileIndex tile, int *z)
 {
 	Slope tileh = GetTileSlope(tile, z);
 	Foundation f = _tile_type_procs[GetTileType(tile)]->get_foundation_proc(tile, tileh);
@@ -341,16 +347,16 @@ Slope GetFoundationSlope(TileIndex tile, uint *z)
 
 bool HasFoundationNW(TileIndex tile, Slope slope_here, uint z_here)
 {
-	uint z;
+	int z;
 
 	int z_W_here = z_here;
 	int z_N_here = z_here;
-	GetSlopeZOnEdge(slope_here, DIAGDIR_NW, &z_W_here, &z_N_here);
+	GetSlopePixelZOnEdge(slope_here, DIAGDIR_NW, &z_W_here, &z_N_here);
 
-	Slope slope = GetFoundationSlope(TILE_ADDXY(tile, 0, -1), &z);
+	Slope slope = GetFoundationPixelSlope(TILE_ADDXY(tile, 0, -1), &z);
 	int z_W = z;
 	int z_N = z;
-	GetSlopeZOnEdge(slope, DIAGDIR_SE, &z_W, &z_N);
+	GetSlopePixelZOnEdge(slope, DIAGDIR_SE, &z_W, &z_N);
 
 	return (z_N_here > z_N) || (z_W_here > z_W);
 }
@@ -358,16 +364,16 @@ bool HasFoundationNW(TileIndex tile, Slope slope_here, uint z_here)
 
 bool HasFoundationNE(TileIndex tile, Slope slope_here, uint z_here)
 {
-	uint z;
+	int z;
 
 	int z_E_here = z_here;
 	int z_N_here = z_here;
-	GetSlopeZOnEdge(slope_here, DIAGDIR_NE, &z_E_here, &z_N_here);
+	GetSlopePixelZOnEdge(slope_here, DIAGDIR_NE, &z_E_here, &z_N_here);
 
-	Slope slope = GetFoundationSlope(TILE_ADDXY(tile, -1, 0), &z);
+	Slope slope = GetFoundationPixelSlope(TILE_ADDXY(tile, -1, 0), &z);
 	int z_E = z;
 	int z_N = z;
-	GetSlopeZOnEdge(slope, DIAGDIR_SW, &z_E, &z_N);
+	GetSlopePixelZOnEdge(slope, DIAGDIR_SW, &z_E, &z_N);
 
 	return (z_N_here > z_N) || (z_E_here > z_E);
 }
@@ -385,8 +391,8 @@ void DrawFoundation(TileInfo *ti, Foundation f)
 	assert(f != FOUNDATION_STEEP_BOTH);
 
 	uint sprite_block = 0;
-	uint z;
-	Slope slope = GetFoundationSlope(ti->tile, &z);
+	int z;
+	Slope slope = GetFoundationPixelSlope(ti->tile, &z);
 
 	/* Select the needed block of foundations sprites
 	 * Block 0: Walls at NW and NE edge
@@ -411,7 +417,7 @@ void DrawFoundation(TileInfo *ti, Foundation f)
 		}
 
 		Corner highest_corner = GetHighestSlopeCorner(ti->tileh);
-		ti->z += ApplyFoundationToSlope(f, &ti->tileh);
+		ti->z += ApplyPixelFoundationToSlope(f, &ti->tileh);
 
 		if (IsInclinedFoundation(f)) {
 			/* inclined foundation */
@@ -473,7 +479,7 @@ void DrawFoundation(TileInfo *ti, Foundation f)
 			);
 			OffsetGroundSprite(31, 9);
 		}
-		ti->z += ApplyFoundationToSlope(f, &ti->tileh);
+		ti->z += ApplyPixelFoundationToSlope(f, &ti->tileh);
 	}
 }
 
@@ -486,7 +492,8 @@ void DoClearSquare(TileIndex tile)
 	MarkTileDirtyByTile(tile);
 }
 
-/** Returns information about trackdirs and signal states.
+/**
+ * Returns information about trackdirs and signal states.
  * If there is any trackbit at 'side', return all trackdirbits.
  * For TRANSPORT_ROAD, return no trackbits if there is no roadbit (of given subtype) at given side.
  * @param tile tile to get info about
@@ -552,7 +559,7 @@ void SetSnowLine(byte table[SNOW_LINE_MONTHS][SNOW_LINE_DAYS])
  */
 byte GetSnowLine()
 {
-	if (_snow_line == NULL) return _settings_game.game_creation.snow_line;
+	if (_snow_line == NULL) return _settings_game.game_creation.snow_line_height;
 
 	YearMonthDay ymd;
 	ConvertDateToYMD(_date, &ymd);
@@ -566,7 +573,7 @@ byte GetSnowLine()
  */
 byte HighestSnowLine()
 {
-	return _snow_line == NULL ? _settings_game.game_creation.snow_line : _snow_line->highest_value;
+	return _snow_line == NULL ? _settings_game.game_creation.snow_line_height : _snow_line->highest_value;
 }
 
 /**
@@ -576,7 +583,7 @@ byte HighestSnowLine()
  */
 byte LowestSnowLine()
 {
-	return _snow_line == NULL ? _settings_game.game_creation.snow_line : _snow_line->lowest_value;
+	return _snow_line == NULL ? _settings_game.game_creation.snow_line_height : _snow_line->lowest_value;
 }
 
 /**
@@ -589,7 +596,8 @@ void ClearSnowLine()
 	_snow_line = NULL;
 }
 
-/** Clear a piece of landscape
+/**
+ * Clear a piece of landscape
  * @param tile tile to clear
  * @param flags of operation to conduct
  * @param p1 unused
@@ -599,14 +607,51 @@ void ClearSnowLine()
  */
 CommandCost CmdLandscapeClear(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	return _tile_type_procs[GetTileType(tile)]->clear_tile_proc(tile, flags);
+	CommandCost cost(EXPENSES_CONSTRUCTION);
+	bool do_clear = false;
+	/* Test for stuff which results in water when cleared. Then add the cost to also clear the water. */
+	if ((flags & DC_FORCE_CLEAR_TILE) && HasTileWaterClass(tile) && IsTileOnWater(tile) && !IsWaterTile(tile) && !IsCoastTile(tile)) {
+		if ((flags & DC_AUTO) && GetWaterClass(tile) == WATER_CLASS_CANAL) return_cmd_error(STR_ERROR_MUST_DEMOLISH_CANAL_FIRST);
+		do_clear = true;
+		cost.AddCost(GetWaterClass(tile) == WATER_CLASS_CANAL ? _price[PR_CLEAR_CANAL] : _price[PR_CLEAR_WATER]);
+	}
+
+	Company *c = (flags & (DC_AUTO | DC_BANKRUPT)) ? NULL : Company::GetIfValid(_current_company);
+	if (c != NULL && (int)GB(c->clear_limit, 16, 16) < 1) {
+		return_cmd_error(STR_ERROR_CLEARING_LIMIT_REACHED);
+	}
+
+	const ClearedObjectArea *coa = FindClearedObject(tile);
+
+	/* If this tile was the first tile which caused object destruction, always
+	 * pass it on to the tile_type_proc. That way multiple test runs and the exec run stay consistent. */
+	if (coa != NULL && coa->first_tile != tile) {
+		/* If this tile belongs to an object which was already cleared via another tile, pretend it has been
+		 * already removed.
+		 * However, we need to check stuff, which is not the same for all object tiles. (e.g. being on water or not) */
+
+		/* If a object is removed, it leaves either bare land or water. */
+		if ((flags & DC_NO_WATER) && HasTileWaterClass(tile) && IsTileOnWater(tile)) {
+			return_cmd_error(STR_ERROR_CAN_T_BUILD_ON_WATER);
+		}
+	} else {
+		cost.AddCost(_tile_type_procs[GetTileType(tile)]->clear_tile_proc(tile, flags));
+	}
+
+	if (flags & DC_EXEC) {
+		if (c != NULL) c->clear_limit -= 1 << 16;
+		if (do_clear) DoClearSquare(tile);
+	}
+	return cost;
 }
 
-/** Clear a big piece of landscape
+/**
+ * Clear a big piece of landscape
  * @param tile end tile of area dragging
  * @param flags of operation to conduct
  * @param p1 start tile of area dragging
- * @param p2 unused
+ * @param p2 various bitstuffed data.
+ *  bit      0: Whether to use the Orthogonal (0) or Diagonal (1) iterator.
  * @param text unused
  * @return the cost of this operation or an error
  */
@@ -614,75 +659,96 @@ CommandCost CmdClearArea(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 {
 	if (p1 >= MapSize()) return CMD_ERROR;
 
-	/* make sure sx,sy are smaller than ex,ey */
-	int ex = TileX(tile);
-	int ey = TileY(tile);
-	int sx = TileX(p1);
-	int sy = TileY(p1);
-	if (ex < sx) Swap(ex, sx);
-	if (ey < sy) Swap(ey, sy);
-
 	Money money = GetAvailableMoneyForCommand();
 	CommandCost cost(EXPENSES_CONSTRUCTION);
-	bool success = false;
+	CommandCost last_error = CMD_ERROR;
+	bool had_success = false;
 
-	for (int x = sx; x <= ex; ++x) {
-		for (int y = sy; y <= ey; ++y) {
-			CommandCost ret = DoCommand(TileXY(x, y), 0, 0, flags & ~DC_EXEC, CMD_LANDSCAPE_CLEAR);
-			if (ret.Failed()) continue;
-			success = true;
+	const Company *c = (flags & (DC_AUTO | DC_BANKRUPT)) ? NULL : Company::GetIfValid(_current_company);
+	int limit = (c == NULL ? INT32_MAX : GB(c->clear_limit, 16, 16));
 
-			if (flags & DC_EXEC) {
-				money -= ret.GetCost();
-				if (ret.GetCost() > 0 && money < 0) {
-					_additional_cash_required = ret.GetCost();
-					return cost;
-				}
-				DoCommand(TileXY(x, y), 0, 0, flags, CMD_LANDSCAPE_CLEAR);
+	TileArea ta(tile, p1);
+	TileIterator *iter = HasBit(p2, 0) ? (TileIterator *)new DiagonalTileIterator(tile, p1) : new OrthogonalTileIterator(ta);
+	for (; *iter != INVALID_TILE; ++(*iter)) {
+		TileIndex t = *iter;
+		CommandCost ret = DoCommand(t, 0, 0, flags & ~DC_EXEC, CMD_LANDSCAPE_CLEAR);
+		if (ret.Failed()) {
+			last_error = ret;
 
-				/* draw explosion animation... */
-				if ((x == sx || x == ex) && (y == sy || y == ey)) {
-					/* big explosion in each corner, or small explosion for single tiles */
-					CreateEffectVehicleAbove(x * TILE_SIZE + TILE_SIZE / 2, y * TILE_SIZE + TILE_SIZE / 2, 2,
-						sy == ey && sx == ex ? EV_EXPLOSION_SMALL : EV_EXPLOSION_LARGE
-					);
-				}
-			}
-			cost.AddCost(ret);
+			/* We may not clear more tiles. */
+			if (c != NULL && GB(c->clear_limit, 16, 16) < 1) break;
+			continue;
 		}
+
+		had_success = true;
+		if (flags & DC_EXEC) {
+			money -= ret.GetCost();
+			if (ret.GetCost() > 0 && money < 0) {
+				_additional_cash_required = ret.GetCost();
+				delete iter;
+				return cost;
+			}
+			DoCommand(t, 0, 0, flags, CMD_LANDSCAPE_CLEAR);
+
+			/* draw explosion animation...
+			 * Disable explosions when game is paused. Looks silly and blocks the view. */
+			TileIndex off = t - ta.tile;
+			if ((TileX(off) == 0 || TileX(off) == ta.w - 1U) && (TileY(off) == 0 || TileY(off) == ta.h - 1U) && _pause_mode == PM_UNPAUSED) {
+				/* big explosion in each corner, or small explosion for single tiles */
+				CreateEffectVehicleAbove(TileX(t) * TILE_SIZE + TILE_SIZE / 2, TileY(t) * TILE_SIZE + TILE_SIZE / 2, 2,
+					ta.w == 1 && ta.h == 1 ? EV_EXPLOSION_SMALL : EV_EXPLOSION_LARGE
+				);
+			}
+		} else {
+			/* When we're at the clearing limit we better bail (unneed) testing as well. */
+			if (ret.GetCost() != 0 && --limit <= 0) break;
+		}
+		cost.AddCost(ret);
 	}
 
-	return (success) ? cost : CMD_ERROR;
+	delete iter;
+	return had_success ? cost : last_error;
 }
 
 
 TileIndex _cur_tileloop_tile;
-#define TILELOOP_BITS 4
-#define TILELOOP_SIZE (1 << TILELOOP_BITS)
-#define TILELOOP_ASSERTMASK ((TILELOOP_SIZE - 1) + ((TILELOOP_SIZE - 1) << MapLogX()))
-#define TILELOOP_CHKMASK (((1 << (MapLogX() - TILELOOP_BITS))-1) << TILELOOP_BITS)
 
+/**
+ * Gradually iterate over all tiles on the map, calling their TileLoopProcs once every 256 ticks.
+ */
 void RunTileLoop()
 {
-	TileIndex tile = _cur_tileloop_tile;
+	/* The pseudorandom sequence of tiles is generated using a Galois linear feedback
+	 * shift register (LFSR). This allows a deterministic pseudorandom ordering, but
+	 * still with minimal state and fast iteration. */
 
-	assert((tile & ~TILELOOP_ASSERTMASK) == 0);
-	uint count = (MapSizeX() / TILELOOP_SIZE) * (MapSizeY() / TILELOOP_SIZE);
-	do {
+	/* Maximal length LFSR feedback terms, from 12-bit (for 64x64 maps) to 22-bit (for 2048x2048 maps).
+	 * Extracted from http://www.ece.cmu.edu/~koopman/lfsr/ */
+	static const uint32 feedbacks[] = {
+		0xD8F, 0x1296, 0x2496, 0x4357, 0x8679, 0x1030E, 0x206CD, 0x403FE, 0x807B8, 0x1004B2, 0x2006A8
+	};
+	const uint32 feedback = feedbacks[MapLogX() + MapLogY() - 12];
+
+	/* We update every tile every 256 ticks, so divide the map size by 2^8 = 256 */
+	uint count = 1 << (MapLogX() + MapLogY() - 8);
+
+	TileIndex tile = _cur_tileloop_tile;
+	/* The LFSR cannot have a zeroed state. */
+	assert(tile != 0);
+
+	/* Manually update tile 0 every 256 ticks - the LFSR never iterates over it itself.  */
+	if (_tick_counter % 256 == 0) {
+		_tile_type_procs[GetTileType(0)]->tile_loop_proc(0);
+		count--;
+	}
+
+	while (count--) {
 		_tile_type_procs[GetTileType(tile)]->tile_loop_proc(tile);
 
-		if (TileX(tile) < MapSizeX() - TILELOOP_SIZE) {
-			tile += TILELOOP_SIZE; // no overflow
-		} else {
-			tile = TILE_MASK(tile - TILELOOP_SIZE * (MapSizeX() / TILELOOP_SIZE - 1) + TileDiffXY(0, TILELOOP_SIZE)); // x would overflow, also increase y
-		}
-	} while (--count != 0);
-	assert((tile & ~TILELOOP_ASSERTMASK) == 0);
-
-	tile += 9;
-	if (tile & TILELOOP_CHKMASK) {
-		tile = (tile + MapSizeX()) & TILELOOP_ASSERTMASK;
+		/* Get the next tile in sequence using a Galois LFSR. */
+		tile = (tile >> 1) ^ (-(int32)(tile & 1) & feedback);
 	}
+
 	_cur_tileloop_tile = tile;
 }
 
@@ -714,6 +780,7 @@ static void GenerateTerrain(int type, uint flag)
 	uint32 r = Random();
 
 	const Sprite *templ = GetSprite((((r >> 24) * _genterrain_tbl_1[type]) >> 8) + _genterrain_tbl_2[type] + 4845, ST_MAPGEN);
+	if (templ == NULL) usererror("Map generator sprites could not be loaded");
 
 	uint x = r & MapMaxX();
 	uint y = (r >> MapLogX()) & MapMaxY();
@@ -756,16 +823,16 @@ static void GenerateTerrain(int type, uint flag)
 	if (x + w >= MapMaxX() - 1) return;
 	if (y + h >= MapMaxY() - 1) return;
 
-	Tile *tile = &_m[TileXY(x, y)];
+	TileIndex tile = TileXY(x, y);
 
 	switch (direction) {
 		default: NOT_REACHED();
 		case DIAGDIR_NE:
 			do {
-				Tile *tile_cur = tile;
+				TileIndex tile_cur = tile;
 
 				for (uint w_cur = w; w_cur != 0; --w_cur) {
-					if (GB(*p, 0, 4) >= tile_cur->type_height) tile_cur->type_height = GB(*p, 0, 4);
+					if (GB(*p, 0, 4) >= TileHeight(tile_cur)) SetTileHeight(tile_cur, GB(*p, 0, 4));
 					p++;
 					tile_cur++;
 				}
@@ -775,10 +842,10 @@ static void GenerateTerrain(int type, uint flag)
 
 		case DIAGDIR_SE:
 			do {
-				Tile *tile_cur = tile;
+				TileIndex tile_cur = tile;
 
 				for (uint h_cur = h; h_cur != 0; --h_cur) {
-					if (GB(*p, 0, 4) >= tile_cur->type_height) tile_cur->type_height = GB(*p, 0, 4);
+					if (GB(*p, 0, 4) >= TileHeight(tile_cur)) SetTileHeight(tile_cur, GB(*p, 0, 4));
 					p++;
 					tile_cur += TileDiffXY(0, 1);
 				}
@@ -789,10 +856,10 @@ static void GenerateTerrain(int type, uint flag)
 		case DIAGDIR_SW:
 			tile += TileDiffXY(w - 1, 0);
 			do {
-				Tile *tile_cur = tile;
+				TileIndex tile_cur = tile;
 
 				for (uint w_cur = w; w_cur != 0; --w_cur) {
-					if (GB(*p, 0, 4) >= tile_cur->type_height) tile_cur->type_height = GB(*p, 0, 4);
+					if (GB(*p, 0, 4) >= TileHeight(tile_cur)) SetTileHeight(tile_cur, GB(*p, 0, 4));
 					p++;
 					tile_cur--;
 				}
@@ -803,10 +870,10 @@ static void GenerateTerrain(int type, uint flag)
 		case DIAGDIR_NW:
 			tile += TileDiffXY(0, h - 1);
 			do {
-				Tile *tile_cur = tile;
+				TileIndex tile_cur = tile;
 
 				for (uint h_cur = h; h_cur != 0; --h_cur) {
-					if (GB(*p, 0, 4) >= tile_cur->type_height) tile_cur->type_height = GB(*p, 0, 4);
+					if (GB(*p, 0, 4) >= TileHeight(tile_cur)) SetTileHeight(tile_cur, GB(*p, 0, 4));
 					p++;
 					tile_cur -= TileDiffXY(0, 1);
 				}
@@ -827,13 +894,16 @@ static void CreateDesertOrRainForest()
 	for (TileIndex tile = 0; tile != MapSize(); ++tile) {
 		if ((tile % update_freq) == 0) IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
+		if (!IsValidTile(tile)) continue;
+
 		for (data = _make_desert_or_rainforest_data;
 				data != endof(_make_desert_or_rainforest_data); ++data) {
 			TileIndex t = AddTileIndexDiffCWrap(tile, *data);
 			if (t != INVALID_TILE && (TileHeight(t) >= 4 || IsTileType(t, MP_WATER))) break;
 		}
-		if (data == endof(_make_desert_or_rainforest_data))
+		if (data == endof(_make_desert_or_rainforest_data)) {
 			SetTropicZone(tile, TROPICZONE_DESERT);
+		}
 	}
 
 	for (uint i = 0; i != 256; i++) {
@@ -845,13 +915,292 @@ static void CreateDesertOrRainForest()
 	for (TileIndex tile = 0; tile != MapSize(); ++tile) {
 		if ((tile % update_freq) == 0) IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
+		if (!IsValidTile(tile)) continue;
+
 		for (data = _make_desert_or_rainforest_data;
 				data != endof(_make_desert_or_rainforest_data); ++data) {
 			TileIndex t = AddTileIndexDiffCWrap(tile, *data);
 			if (t != INVALID_TILE && IsTileType(t, MP_CLEAR) && IsClearGround(t, CLEAR_DESERT)) break;
 		}
-		if (data == endof(_make_desert_or_rainforest_data))
+		if (data == endof(_make_desert_or_rainforest_data)) {
 			SetTropicZone(tile, TROPICZONE_RAINFOREST);
+		}
+	}
+}
+
+/**
+ * Find the spring of a river.
+ * @param tile The tile to consider for being the spring.
+ * @param user_data Ignored data.
+ * @return True iff it is suitable as a spring.
+ */
+static bool FindSpring(TileIndex tile, void *user_data)
+{
+	int referenceHeight;
+	Slope s = GetTileSlope(tile, &referenceHeight);
+	if (s != SLOPE_FLAT || IsWaterTile(tile)) return false;
+
+	/* In the tropics rivers start in the rainforest. */
+	if (_settings_game.game_creation.landscape == LT_TROPIC && GetTropicZone(tile) != TROPICZONE_RAINFOREST) return false;
+
+	/* Are there enough higher tiles to warrant a 'spring'? */
+	uint num = 0;
+	for (int dx = -1; dx <= 1; dx++) {
+		for (int dy = -1; dy <= 1; dy++) {
+			TileIndex t = TileAddWrap(tile, dx, dy);
+			if (t != INVALID_TILE && GetTileMaxZ(t) > referenceHeight) num++;
+		}
+	}
+
+	if (num < 4) return false;
+
+	/* Are we near the top of a hill? */
+	for (int dx = -16; dx <= 16; dx++) {
+		for (int dy = -16; dy <= 16; dy++) {
+			TileIndex t = TileAddWrap(tile, dx, dy);
+			if (t != INVALID_TILE && GetTileMaxZ(t) > referenceHeight + 2) return false;
+		}
+	}
+
+	return true;
+}
+
+/**
+ * Make a connected lake; fill all tiles in the circular tile search that are connected.
+ * @param tile The tile to consider for lake making.
+ * @param user_data The height of the lake.
+ * @return Always false, so it continues searching.
+ */
+static bool MakeLake(TileIndex tile, void *user_data)
+{
+	uint height = *(uint*)user_data;
+	if (!IsValidTile(tile) || TileHeight(tile) != height || GetTileSlope(tile) != SLOPE_FLAT) return false;
+	if (_settings_game.game_creation.landscape == LT_TROPIC && GetTropicZone(tile) == TROPICZONE_DESERT) return false;
+
+	for (DiagDirection d = DIAGDIR_BEGIN; d < DIAGDIR_END; d++) {
+		TileIndex t2 = tile + TileOffsByDiagDir(d);
+		if (IsWaterTile(t2)) {
+			MakeRiver(tile, Random());
+			return false;
+		}
+	}
+
+	return false;
+}
+
+/**
+ * Check whether a river at begin could (logically) flow down to end.
+ * @param begin The origin of the flow.
+ * @param end The destination of the flow.
+ * @return True iff the water can be flowing down.
+ */
+static bool FlowsDown(TileIndex begin, TileIndex end)
+{
+	assert(DistanceManhattan(begin, end) == 1);
+
+	int heightBegin;
+	int heightEnd;
+	Slope slopeBegin = GetTileSlope(begin, &heightBegin);
+	Slope slopeEnd   = GetTileSlope(end, &heightEnd);
+
+	return heightEnd <= heightBegin &&
+			/* Slope either is inclined or flat; rivers don't support other slopes. */
+			(slopeEnd == SLOPE_FLAT || IsInclinedSlope(slopeEnd)) &&
+			/* Slope continues, then it must be lower... or either end must be flat. */
+			((slopeEnd == slopeBegin && heightEnd < heightBegin) || slopeEnd == SLOPE_FLAT || slopeBegin == SLOPE_FLAT);
+}
+
+/* AyStar callback for checking whether we reached our destination. */
+static int32 River_EndNodeCheck(AyStar *aystar, OpenListNode *current)
+{
+	return current->path.node.tile == *(TileIndex*)aystar->user_target ? AYSTAR_FOUND_END_NODE : AYSTAR_DONE;
+}
+
+/* AyStar callback for getting the cost of the current node. */
+static int32 River_CalculateG(AyStar *aystar, AyStarNode *current, OpenListNode *parent)
+{
+	return 1 + RandomRange(_settings_game.game_creation.river_route_random);
+}
+
+/* AyStar callback for getting the estimated cost to the destination. */
+static int32 River_CalculateH(AyStar *aystar, AyStarNode *current, OpenListNode *parent)
+{
+	return DistanceManhattan(*(TileIndex*)aystar->user_target, current->tile);
+}
+
+/* AyStar callback for getting the neighbouring nodes of the given node. */
+static void River_GetNeighbours(AyStar *aystar, OpenListNode *current)
+{
+	TileIndex tile = current->path.node.tile;
+
+	aystar->num_neighbours = 0;
+	for (DiagDirection d = DIAGDIR_BEGIN; d < DIAGDIR_END; d++) {
+		TileIndex t2 = tile + TileOffsByDiagDir(d);
+		if (IsValidTile(t2) && FlowsDown(tile, t2)) {
+			aystar->neighbours[aystar->num_neighbours].tile = t2;
+			aystar->neighbours[aystar->num_neighbours].direction = INVALID_TRACKDIR;
+			aystar->num_neighbours++;
+		}
+	}
+}
+
+/* AyStar callback when an route has been found. */
+static void River_FoundEndNode(AyStar *aystar, OpenListNode *current)
+{
+	for (PathNode *path = &current->path; path != NULL; path = path->parent) {
+		TileIndex tile = path->node.tile;
+		if (!IsWaterTile(tile)) {
+			MakeRiver(tile, Random());
+			/* Remove desert directly around the river tile. */
+			CircularTileSearch(&tile, 5, RiverModifyDesertZone, NULL);
+		}
+	}
+}
+
+static const uint RIVER_HASH_SIZE = 8; ///< The number of bits the hash for river finding should have.
+
+/**
+ * Simple hash function for river tiles to be used by AyStar.
+ * @param tile The tile to hash.
+ * @param dir The unused direction.
+ * @return The hash for the tile.
+ */
+static uint River_Hash(uint tile, uint dir)
+{
+	return GB(TileHash(TileX(tile), TileY(tile)), 0, RIVER_HASH_SIZE);
+}
+
+/**
+ * Actually build the river between the begin and end tiles using AyStar.
+ * @param begin The begin of the river.
+ * @param end The end of the river.
+ */
+static void BuildRiver(TileIndex begin, TileIndex end)
+{
+	AyStar finder;
+	MemSetT(&finder, 0);
+	finder.CalculateG = River_CalculateG;
+	finder.CalculateH = River_CalculateH;
+	finder.GetNeighbours = River_GetNeighbours;
+	finder.EndNodeCheck = River_EndNodeCheck;
+	finder.FoundEndNode = River_FoundEndNode;
+	finder.user_target = &end;
+
+	finder.Init(River_Hash, 1 << RIVER_HASH_SIZE);
+
+	AyStarNode start;
+	start.tile = begin;
+	start.direction = INVALID_TRACKDIR;
+	finder.AddStartNode(&start, 0);
+	finder.Main();
+	finder.Free();
+}
+
+/**
+ * Try to flow the river down from a given begin.
+ * @param marks  Array for temporary of iterated tiles.
+ * @param spring The springing point of the river.
+ * @param begin  The begin point we are looking from; somewhere down hill from the spring.
+ * @return True iff a river could/has been built, otherwise false.
+ */
+static bool FlowRiver(bool *marks, TileIndex spring, TileIndex begin)
+{
+	uint height = TileHeight(begin);
+	if (IsWaterTile(begin)) return DistanceManhattan(spring, begin) > _settings_game.game_creation.min_river_length;
+
+	MemSetT(marks, 0, MapSize());
+	marks[begin] = true;
+
+	/* Breadth first search for the closest tile we can flow down to. */
+	std::list<TileIndex> queue;
+	queue.push_back(begin);
+
+	bool found = false;
+	uint count = 0; // Number of tiles considered; to be used for lake location guessing.
+	TileIndex end;
+	do {
+		end = queue.front();
+		queue.pop_front();
+
+		uint height2 = TileHeight(end);
+		if (GetTileSlope(end) == SLOPE_FLAT && (height2 < height || (height2 == height && IsWaterTile(end)))) {
+			found = true;
+			break;
+		}
+
+		for (DiagDirection d = DIAGDIR_BEGIN; d < DIAGDIR_END; d++) {
+			TileIndex t2 = end + TileOffsByDiagDir(d);
+			if (IsValidTile(t2) && !marks[t2] && FlowsDown(end, t2)) {
+				marks[t2] = true;
+				count++;
+				queue.push_back(t2);
+			}
+		}
+	} while (!queue.empty());
+
+	if (found) {
+		/* Flow further down hill. */
+		found = FlowRiver(marks, spring, end);
+	} else if (count > 32) {
+		/* Maybe we can make a lake. Find the Nth of the considered tiles. */
+		TileIndex lakeCenter = 0;
+		for (int i = RandomRange(count - 1); i != 0; lakeCenter++) {
+			if (marks[lakeCenter]) i--;
+		}
+
+		if (IsValidTile(lakeCenter) &&
+				/* A river, or lake, can only be built on flat slopes. */
+				GetTileSlope(lakeCenter) == SLOPE_FLAT &&
+				/* We want the lake to be built at the height of the river. */
+				TileHeight(begin) == TileHeight(lakeCenter) &&
+				/* We don't want the lake at the entry of the valley. */
+				lakeCenter != begin &&
+				/* We don't want lakes in the desert. */
+				(_settings_game.game_creation.landscape != LT_TROPIC || GetTropicZone(lakeCenter) != TROPICZONE_DESERT) &&
+				/* We only want a lake if the river is long enough. */
+				DistanceManhattan(spring, lakeCenter) > _settings_game.game_creation.min_river_length) {
+			end = lakeCenter;
+			MakeRiver(lakeCenter, Random());
+			uint range = RandomRange(8) + 3;
+			CircularTileSearch(&lakeCenter, range, MakeLake, &height);
+			/* Call the search a second time so artefacts from going circular in one direction get (mostly) hidden. */
+			lakeCenter = end;
+			CircularTileSearch(&lakeCenter, range, MakeLake, &height);
+			found = true;
+		}
+	}
+
+	if (found) BuildRiver(begin, end);
+	return found;
+}
+
+/**
+ * Actually (try to) create some rivers.
+ */
+static void CreateRivers()
+{
+	int amount = _settings_game.game_creation.amount_of_rivers;
+	if (amount == 0) return;
+
+	uint wells = ScaleByMapSize(4 << _settings_game.game_creation.amount_of_rivers);
+	SetGeneratingWorldProgress(GWP_RIVER, wells + 256 / 64); // Include the tile loop calls below.
+	bool *marks = CallocT<bool>(MapSize());
+
+	for (; wells != 0; wells--) {
+		IncreaseGeneratingWorldProgress(GWP_RIVER);
+		for (int tries = 0; tries < 128; tries++) {
+			TileIndex t = RandomTile();
+			if (!CircularTileSearch(&t, 8, FindSpring, NULL)) continue;
+			if (FlowRiver(marks, t, t)) break;
+		}
+	}
+
+	free(marks);
+
+	/* Run tile loop to update the ground density. */
+	for (uint i = 0; i != 256; i++) {
+		if (i % 64 == 0) IncreaseGeneratingWorldProgress(GWP_RIVER);
+		RunTileLoop();
 	}
 }
 
@@ -892,7 +1241,8 @@ void GenerateLandscape(byte mode)
 				for (uint i = ScaleByMapSize(GB(r, 9, 7) + 450); i != 0; --i) {
 					GenerateTerrain(4, flag);
 				}
-			} break;
+				break;
+			}
 
 			case LT_TROPIC: {
 				uint32 r = Random();
@@ -911,16 +1261,19 @@ void GenerateLandscape(byte mode)
 				for (uint i = ScaleByMapSize(GB(r, 17, 7) + 410); i != 0; --i) {
 					GenerateTerrain(3, flag);
 				}
-			} break;
+				break;
+			}
 
 			default: {
 				uint32 r = Random();
 
+				assert(_settings_game.difficulty.quantity_sea_lakes != CUSTOM_SEA_LEVEL_NUMBER_DIFFICULTY);
 				uint i = ScaleByMapSize(GB(r, 0, 7) + (3 - _settings_game.difficulty.quantity_sea_lakes) * 256 + 100);
 				for (; i != 0; --i) {
 					GenerateTerrain(_settings_game.difficulty.terrain_type, 0);
 				}
-			} break;
+				break;
+			}
 		}
 	}
 
@@ -932,6 +1285,8 @@ void GenerateLandscape(byte mode)
 	IncreaseGeneratingWorldProgress(GWP_LANDSCAPE);
 
 	if (_settings_game.game_creation.landscape == LT_TROPIC) CreateDesertOrRainForest();
+
+	CreateRivers();
 }
 
 void OnTick_Town();
