@@ -1,4 +1,4 @@
-/* $Id: misc_cmd.cpp 19854 2010-05-18 21:25:03Z rubidium $ */
+/* $Id: misc_cmd.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,14 +17,15 @@
 #include "network/network.h"
 #include "network/network_func.h"
 #include "strings_func.h"
-#include "functions.h"
 #include "company_func.h"
 #include "company_gui.h"
 #include "company_base.h"
+#include "core/backup_type.hpp"
 
 #include "table/strings.h"
 
-/** Increase the loan of your company.
+/**
+ * Increase the loan of your company.
  * @param tile unused
  * @param flags operation to perform
  * @param p1 amount to increase the loan with, multitude of LOAN_INTERVAL. Only used when p2 == 2.
@@ -53,7 +54,7 @@ CommandCost CmdIncreaseLoan(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
 			loan = _economy.max_loan - c->current_loan;
 			break;
 		case 2: // Take the given amount of loan
-			if ((((int32)p1 < LOAN_INTERVAL) || c->current_loan + (int32)p1 > _economy.max_loan || (p1 % LOAN_INTERVAL) != 0)) return CMD_ERROR;
+			if ((int32)p1 < LOAN_INTERVAL || c->current_loan + (int32)p1 > _economy.max_loan || p1 % LOAN_INTERVAL != 0) return CMD_ERROR;
 			loan = p1;
 			break;
 	}
@@ -70,7 +71,8 @@ CommandCost CmdIncreaseLoan(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
 	return CommandCost(EXPENSES_OTHER);
 }
 
-/** Decrease the loan of your company.
+/**
+ * Decrease the loan of your company.
  * @param tile unused
  * @param flags operation to perform
  * @param p1 amount to decrease the loan with, multitude of LOAN_INTERVAL. Only used when p2 == 2.
@@ -123,7 +125,9 @@ CommandCost CmdDecreaseLoan(TileIndex tile, DoCommandFlag flags, uint32 p1, uint
  */
 static void AskUnsafeUnpauseCallback(Window *w, bool confirmed)
 {
-	DoCommandP(0, PM_PAUSED_ERROR, confirmed ? 0 : 1, CMD_PAUSE);
+	if (confirmed) {
+		DoCommandP(0, PM_PAUSED_ERROR, 0, CMD_PAUSE);
+	}
 }
 
 /**
@@ -144,6 +148,7 @@ CommandCost CmdPause(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2,
 		case PM_PAUSED_SAVELOAD:
 		case PM_PAUSED_ERROR:
 		case PM_PAUSED_NORMAL:
+		case PM_PAUSED_GAME_SCRIPT:
 			break;
 
 #ifdef ENABLE_NETWORK
@@ -185,9 +190,8 @@ CommandCost CmdPause(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2,
 	return CommandCost();
 }
 
-/** Change the financial flow of your company.
- * This is normally only enabled in offline mode, but if there is a debug
- * build, you can cheat (to test).
+/**
+ * Change the financial flow of your company.
  * @param tile unused
  * @param flags operation to perform
  * @param p1 the amount of money to receive (if negative), or spend (if positive)
@@ -197,15 +201,13 @@ CommandCost CmdPause(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2,
  */
 CommandCost CmdMoneyCheat(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-#ifndef _DEBUG
-	if (_networking) return CMD_ERROR;
-#endif
 	return CommandCost(EXPENSES_OTHER, -(int32)p1);
 }
 
-/** Transfer funds (money) from one company to another.
+/**
+ * Transfer funds (money) from one company to another.
  * To prevent abuse in multiplayer games you can only send money to other
- * companies if you have paid off your loan (either explicitely, or implicitely
+ * companies if you have paid off your loan (either explicitly, or implicitly
  * given the fact that you have more money than loan).
  * @param tile unused
  * @param flags operation to perform
@@ -220,17 +222,17 @@ CommandCost CmdGiveMoney(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 
 	const Company *c = Company::Get(_current_company);
 	CommandCost amount(EXPENSES_OTHER, min((Money)p1, (Money)20000000LL));
+	CompanyID dest_company = (CompanyID)p2;
 
 	/* You can only transfer funds that is in excess of your loan */
 	if (c->money - c->current_loan < amount.GetCost() || amount.GetCost() < 0) return CMD_ERROR;
-	if (!_networking || !Company::IsValidID((CompanyID)p2)) return CMD_ERROR;
+	if (!_networking || !Company::IsValidID(dest_company)) return CMD_ERROR;
 
 	if (flags & DC_EXEC) {
 		/* Add money to company */
-		CompanyID old_company = _current_company;
-		_current_company = (CompanyID)p2;
+		Backup<CompanyByte> cur_company(_current_company, dest_company, FILE_LINE);
 		SubtractMoneyFromCompany(CommandCost(EXPENSES_OTHER, -amount.GetCost()));
-		_current_company = old_company;
+		cur_company.Restore();
 	}
 
 	/* Subtract money from local-company */
