@@ -1,4 +1,4 @@
-/* $Id: music.cpp 19741 2010-04-30 21:01:21Z rubidium $ */
+/* $Id: music.cpp 23214 2011-11-14 19:24:22Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,7 +11,7 @@
 
 #include "stdafx.h"
 
-/* The type of set we're replacing */
+/** The type of set we're replacing */
 #define SET_TYPE "music"
 #include "base_media_func.h"
 
@@ -24,10 +24,11 @@ static const char * const _music_file_names[] = {
 	"new_0", "new_1", "new_2", "new_3", "new_4", "new_5", "new_6", "new_7", "new_8", "new_9",
 	"ezy_0", "ezy_1", "ezy_2", "ezy_3", "ezy_4", "ezy_5", "ezy_6", "ezy_7", "ezy_8", "ezy_9",
 };
+/** Make sure we aren't messing things up. */
 assert_compile(lengthof(_music_file_names) == NUM_SONGS_AVAILABLE);
 
-template <class T, size_t Tnum_files, Subdirectory Tsubdir>
-/* static */ const char * const *BaseSet<T, Tnum_files, Tsubdir>::file_names = _music_file_names;
+template <class T, size_t Tnum_files, bool Tsearch_in_tars>
+/* static */ const char * const *BaseSet<T, Tnum_files, Tsearch_in_tars>::file_names = _music_file_names;
 
 template <class Tbase_set>
 /* static */ const char *BaseMedia<Tbase_set>::GetExtension()
@@ -59,7 +60,7 @@ template <class Tbase_set>
 
 bool MusicSet::FillSetDetails(IniFile *ini, const char *path, const char *full_filename)
 {
-	bool ret = this->BaseSet<MusicSet, NUM_SONGS_AVAILABLE, GM_DIR>::FillSetDetails(ini, path, full_filename);
+	bool ret = this->BaseSet<MusicSet, NUM_SONGS_AVAILABLE, false>::FillSetDetails(ini, path, full_filename);
 	if (ret) {
 		this->num_available = 0;
 		IniGroup *names = ini->GetGroup("names");
