@@ -1,4 +1,4 @@
-/* $Id: music_gui.cpp 23053 2011-10-22 21:06:58Z rubidium $ */
+/* $Id: music_gui.cpp 24700 2012-11-11 16:10:43Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -19,9 +19,12 @@
 #include "sound_func.h"
 #include "gfx_func.h"
 #include "core/random_func.hpp"
-#include "gui.h"
+#include "error.h"
 #include "core/geometry_func.hpp"
 #include "string_func.h"
+#include "settings_type.h"
+
+#include "widgets/music_widget.h"
 
 #include "table/strings.h"
 #include "table/sprites.h"
@@ -63,8 +66,8 @@ static byte _playlist_new_style[NUM_SONGS_CLASS + 1];
 /** Indices of all ezy street songs */
 static byte _playlist_ezy_street[NUM_SONGS_CLASS + 1];
 
-assert_compile(lengthof(_msf.custom_1) == NUM_SONGS_PLAYLIST + 1);
-assert_compile(lengthof(_msf.custom_2) == NUM_SONGS_PLAYLIST + 1);
+assert_compile(lengthof(_settings_client.music.custom_1) == NUM_SONGS_PLAYLIST + 1);
+assert_compile(lengthof(_settings_client.music.custom_2) == NUM_SONGS_PLAYLIST + 1);
 
 /** The different playlists that can be played. */
 static byte * const _playlists[] = {
@@ -72,8 +75,8 @@ static byte * const _playlists[] = {
 	_playlist_old_style,
 	_playlist_new_style,
 	_playlist_ezy_street,
-	_msf.custom_1,
-	_msf.custom_2,
+	_settings_client.music.custom_1,
+	_settings_client.music.custom_2,
 };
 
 /**
@@ -121,8 +124,8 @@ void InitializeMusic()
 		_playlists[k + 1][j] = 0;
 	}
 
-	ValidatePlaylist(_msf.custom_1, lastof(_msf.custom_1));
-	ValidatePlaylist(_msf.custom_2, lastof(_msf.custom_2));
+	ValidatePlaylist(_settings_client.music.custom_1, lastof(_settings_client.music.custom_1));
+	ValidatePlaylist(_settings_client.music.custom_2, lastof(_settings_client.music.custom_2));
 
 	if (BaseMusic::GetUsedSet()->num_available < _music_wnd_cursong) {
 		/* If there are less songs than the currently played song,
@@ -177,8 +180,9 @@ static void MusicVolumeChanged(byte new_vol)
 static void DoPlaySong()
 {
 	char filename[MAX_PATH];
-	FioFindFullPath(filename, lengthof(filename), GM_DIR,
-			BaseMusic::GetUsedSet()->files[_music_wnd_cursong - 1].filename);
+	if (FioFindFullPath(filename, lengthof(filename), BASESET_DIR, BaseMusic::GetUsedSet()->files[_music_wnd_cursong - 1].filename) == NULL) {
+		FioFindFullPath(filename, lengthof(filename), OLD_GM_DIR, BaseMusic::GetUsedSet()->files[_music_wnd_cursong - 1].filename);
+	}
 	_music_driver->PlaySong(filename);
 	SetWindowDirty(WC_MUSIC_WINDOW, 0);
 }
@@ -196,17 +200,17 @@ static void SelectSongToPlay()
 
 	memset(_cur_playlist, 0, sizeof(_cur_playlist));
 	do {
-		const char *filename = BaseMusic::GetUsedSet()->files[_playlists[_msf.playlist][i] - 1].filename;
+		const char *filename = BaseMusic::GetUsedSet()->files[_playlists[_settings_client.music.playlist][i] - 1].filename;
 		/* We are now checking for the existence of that file prior
 		 * to add it to the list of available songs */
-		if (!StrEmpty(filename) && FioCheckFileExists(filename, GM_DIR)) {
-			_cur_playlist[j] = _playlists[_msf.playlist][i];
+		if (!StrEmpty(filename) && FioCheckFileExists(filename, BASESET_DIR)) {
+			_cur_playlist[j] = _playlists[_settings_client.music.playlist][i];
 			j++;
 		}
-	} while (_playlists[_msf.playlist][++i] != 0 && j < lengthof(_cur_playlist) - 1);
+	} while (_playlists[_settings_client.music.playlist][++i] != 0 && j < lengthof(_cur_playlist) - 1);
 
 	/* Do not shuffle when on the intro-start window, as the song to play has to be the original TTD Theme*/
-	if (_msf.shuffle && _game_mode != GM_MENU) {
+	if (_settings_client.music.shuffle && _game_mode != GM_MENU) {
 		i = 500;
 		do {
 			uint32 r = InteractiveRandom();
@@ -240,7 +244,7 @@ static void PlayPlaylistSong()
 		if (_cur_playlist[0] == 0) {
 			_song_is_active = false;
 			_music_wnd_cursong = 0;
-			_msf.playing = false;
+			_settings_client.music.playing = false;
 			return;
 		}
 	}
@@ -259,9 +263,9 @@ void ResetMusic()
 
 void MusicLoop()
 {
-	if (!_msf.playing && _song_is_active) {
+	if (!_settings_client.music.playing && _song_is_active) {
 		StopMusic();
-	} else if (_msf.playing && !_song_is_active) {
+	} else if (_settings_client.music.playing && !_song_is_active) {
 		PlayPlaylistSong();
 	}
 
@@ -280,39 +284,26 @@ void MusicLoop()
 
 static void SelectPlaylist(byte list)
 {
-	_msf.playlist = list;
+	_settings_client.music.playlist = list;
 	InvalidateWindowData(WC_MUSIC_TRACK_SELECTION, 0);
 	InvalidateWindowData(WC_MUSIC_WINDOW, 0);
 }
 
-enum MusicTrackSelectionWidgets {
-	MTSW_LIST_LEFT,
-	MTSW_PLAYLIST,
-	MTSW_LIST_RIGHT,
-	MTSW_ALL,
-	MTSW_OLD,
-	MTSW_NEW,
-	MTSW_EZY,
-	MTSW_CUSTOM1,
-	MTSW_CUSTOM2,
-	MTSW_CLEAR,
-};
-
 struct MusicTrackSelectionWindow : public Window {
 	MusicTrackSelectionWindow(const WindowDesc *desc, WindowNumber number) : Window()
 	{
 		this->InitNested(desc, number);
-		this->LowerWidget(MTSW_LIST_LEFT);
-		this->LowerWidget(MTSW_LIST_RIGHT);
-		this->SetWidgetDisabledState(MTSW_CLEAR, _msf.playlist <= 3);
-		this->LowerWidget(MTSW_ALL + _msf.playlist);
+		this->LowerWidget(WID_MTS_LIST_LEFT);
+		this->LowerWidget(WID_MTS_LIST_RIGHT);
+		this->SetWidgetDisabledState(WID_MTS_CLEAR, _settings_client.music.playlist <= 3);
+		this->LowerWidget(WID_MTS_ALL + _settings_client.music.playlist);
 	}
 
 	virtual void SetStringParameters(int widget) const
 	{
 		switch (widget) {
-			case MTSW_PLAYLIST:
-				SetDParam(0, STR_MUSIC_PLAYLIST_ALL + _msf.playlist);
+			case WID_MTS_PLAYLIST:
+				SetDParam(0, STR_MUSIC_PLAYLIST_ALL + _settings_client.music.playlist);
 				break;
 		}
 	}
@@ -326,16 +317,16 @@ struct MusicTrackSelectionWindow : public Window {
 	{
 		if (!gui_scope) return;
 		for (int i = 0; i < 6; i++) {
-			this->SetWidgetLoweredState(MTSW_ALL + i, i == _msf.playlist);
+			this->SetWidgetLoweredState(WID_MTS_ALL + i, i == _settings_client.music.playlist);
 		}
-		this->SetWidgetDisabledState(MTSW_CLEAR, _msf.playlist <= 3);
+		this->SetWidgetDisabledState(WID_MTS_CLEAR, _settings_client.music.playlist <= 3);
 		this->SetDirty();
 	}
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
 		switch (widget) {
-			case MTSW_PLAYLIST: {
+			case WID_MTS_PLAYLIST: {
 				Dimension d = {0, 0};
 
 				for (int i = 0; i < 6; i++) {
@@ -348,7 +339,7 @@ struct MusicTrackSelectionWindow : public Window {
 				break;
 			}
 
-			case MTSW_LIST_LEFT: case MTSW_LIST_RIGHT: {
+			case WID_MTS_LIST_LEFT: case WID_MTS_LIST_RIGHT: {
 				Dimension d = {0, 0};
 
 				for (uint i = 0; i < NUM_SONGS_AVAILABLE; i++) {
@@ -373,8 +364,8 @@ struct MusicTrackSelectionWindow : public Window {
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		switch (widget) {
-			case MTSW_LIST_LEFT: {
-				GfxFillRect(r.left + 1, r.top + 1, r.right - 1, r.bottom - 1, 0);
+			case WID_MTS_LIST_LEFT: {
+				GfxFillRect(r.left + 1, r.top + 1, r.right - 1, r.bottom - 1, PC_BLACK);
 
 				int y = r.top + WD_FRAMERECT_TOP;
 				for (uint i = 0; i < NUM_SONGS_AVAILABLE; i++) {
@@ -390,11 +381,11 @@ struct MusicTrackSelectionWindow : public Window {
 				break;
 			}
 
-			case MTSW_LIST_RIGHT: {
-				GfxFillRect(r.left + 1, r.top + 1, r.right - 1, r.bottom - 1, 0);
+			case WID_MTS_LIST_RIGHT: {
+				GfxFillRect(r.left + 1, r.top + 1, r.right - 1, r.bottom - 1, PC_BLACK);
 
 				int y = r.top + WD_FRAMERECT_TOP;
-				for (const byte *p = _playlists[_msf.playlist]; *p != 0; p++) {
+				for (const byte *p = _playlists[_settings_client.music.playlist]; *p != 0; p++) {
 					uint i = *p - 1;
 					SetDParam(0, GetTrackNumber(i));
 					SetDParam(1, 2);
@@ -410,13 +401,13 @@ struct MusicTrackSelectionWindow : public Window {
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		switch (widget) {
-			case MTSW_LIST_LEFT: { // add to playlist
+			case WID_MTS_LIST_LEFT: { // add to playlist
 				int y = this->GetRowFromWidget(pt.y, widget, 0, FONT_HEIGHT_SMALL);
 
-				if (_msf.playlist < 4) return;
+				if (_settings_client.music.playlist < 4) return;
 				if (!IsInsideMM(y, 0, BaseMusic::GetUsedSet()->num_available)) return;
 
-				byte *p = _playlists[_msf.playlist];
+				byte *p = _playlists[_settings_client.music.playlist];
 				for (uint i = 0; i != NUM_SONGS_PLAYLIST - 1; i++) {
 					if (p[i] == 0) {
 						/* Find the actual song number */
@@ -435,13 +426,13 @@ struct MusicTrackSelectionWindow : public Window {
 				break;
 			}
 
-			case MTSW_LIST_RIGHT: { // remove from playlist
+			case WID_MTS_LIST_RIGHT: { // remove from playlist
 				int y = this->GetRowFromWidget(pt.y, widget, 0, FONT_HEIGHT_SMALL);
 
-				if (_msf.playlist < 4) return;
+				if (_settings_client.music.playlist < 4) return;
 				if (!IsInsideMM(y, 0, NUM_SONGS_PLAYLIST)) return;
 
-				byte *p = _playlists[_msf.playlist];
+				byte *p = _playlists[_settings_client.music.playlist];
 				for (uint i = y; i != NUM_SONGS_PLAYLIST - 1; i++) {
 					p[i] = p[i + 1];
 				}
@@ -451,16 +442,16 @@ struct MusicTrackSelectionWindow : public Window {
 				break;
 			}
 
-			case MTSW_CLEAR: // clear
-				for (uint i = 0; _playlists[_msf.playlist][i] != 0; i++) _playlists[_msf.playlist][i] = 0;
+			case WID_MTS_CLEAR: // clear
+				for (uint i = 0; _playlists[_settings_client.music.playlist][i] != 0; i++) _playlists[_settings_client.music.playlist][i] = 0;
 				this->SetDirty();
 				StopMusic();
 				SelectSongToPlay();
 				break;
 
-			case MTSW_ALL: case MTSW_OLD: case MTSW_NEW:
-			case MTSW_EZY: case MTSW_CUSTOM1: case MTSW_CUSTOM2: // set playlist
-				SelectPlaylist(widget - MTSW_ALL);
+			case WID_MTS_ALL: case WID_MTS_OLD: case WID_MTS_NEW:
+			case WID_MTS_EZY: case WID_MTS_CUSTOM1: case WID_MTS_CUSTOM2: // set playlist
+				SelectPlaylist(widget - WID_MTS_ALL);
 				StopMusic();
 				SelectSongToPlay();
 				break;
@@ -478,26 +469,26 @@ static const NWidgetPart _nested_music_track_selection_widgets[] = {
 			/* Left panel. */
 			NWidget(NWID_VERTICAL),
 				NWidget(WWT_LABEL, COLOUR_GREY), SetDataTip(STR_PLAYLIST_TRACK_INDEX, STR_NULL),
-				NWidget(WWT_PANEL, COLOUR_GREY, MTSW_LIST_LEFT), SetMinimalSize(180, 194), SetDataTip(0x0, STR_PLAYLIST_TOOLTIP_CLICK_TO_ADD_TRACK), EndContainer(),
+				NWidget(WWT_PANEL, COLOUR_GREY, WID_MTS_LIST_LEFT), SetMinimalSize(180, 194), SetDataTip(0x0, STR_PLAYLIST_TOOLTIP_CLICK_TO_ADD_TRACK), EndContainer(),
 				NWidget(NWID_SPACER), SetMinimalSize(0, 2),
 			EndContainer(),
 			/* Middle buttons. */
 			NWidget(NWID_VERTICAL),
 				NWidget(NWID_SPACER), SetMinimalSize(60, 30), // Space above the first button from the title bar.
-				NWidget(WWT_TEXTBTN, COLOUR_GREY, MTSW_ALL), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_ALL, STR_MUSIC_TOOLTIP_SELECT_ALL_TRACKS_PROGRAM),
-				NWidget(WWT_TEXTBTN, COLOUR_GREY, MTSW_OLD), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_OLD_STYLE, STR_MUSIC_TOOLTIP_SELECT_OLD_STYLE_MUSIC),
-				NWidget(WWT_TEXTBTN, COLOUR_GREY, MTSW_NEW), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_NEW_STYLE, STR_MUSIC_TOOLTIP_SELECT_NEW_STYLE_MUSIC),
-				NWidget(WWT_TEXTBTN, COLOUR_GREY, MTSW_EZY), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_EZY_STREET, STR_MUSIC_TOOLTIP_SELECT_EZY_STREET_STYLE),
-				NWidget(WWT_TEXTBTN, COLOUR_GREY, MTSW_CUSTOM1), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_CUSTOM_1, STR_MUSIC_TOOLTIP_SELECT_CUSTOM_1_USER_DEFINED),
-				NWidget(WWT_TEXTBTN, COLOUR_GREY, MTSW_CUSTOM2), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_CUSTOM_2, STR_MUSIC_TOOLTIP_SELECT_CUSTOM_2_USER_DEFINED),
+				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_MTS_ALL), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_ALL, STR_MUSIC_TOOLTIP_SELECT_ALL_TRACKS_PROGRAM),
+				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_MTS_OLD), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_OLD_STYLE, STR_MUSIC_TOOLTIP_SELECT_OLD_STYLE_MUSIC),
+				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_MTS_NEW), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_NEW_STYLE, STR_MUSIC_TOOLTIP_SELECT_NEW_STYLE_MUSIC),
+				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_MTS_EZY), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_EZY_STREET, STR_MUSIC_TOOLTIP_SELECT_EZY_STREET_STYLE),
+				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_MTS_CUSTOM1), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_CUSTOM_1, STR_MUSIC_TOOLTIP_SELECT_CUSTOM_1_USER_DEFINED),
+				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_MTS_CUSTOM2), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_CUSTOM_2, STR_MUSIC_TOOLTIP_SELECT_CUSTOM_2_USER_DEFINED),
 				NWidget(NWID_SPACER), SetMinimalSize(0, 16), // Space above 'clear' button
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, MTSW_CLEAR), SetFill(1, 0), SetDataTip(STR_PLAYLIST_CLEAR, STR_PLAYLIST_TOOLTIP_CLEAR_CURRENT_PROGRAM_CUSTOM1),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_MTS_CLEAR), SetFill(1, 0), SetDataTip(STR_PLAYLIST_CLEAR, STR_PLAYLIST_TOOLTIP_CLEAR_CURRENT_PROGRAM_CUSTOM1),
 				NWidget(NWID_SPACER), SetFill(0, 1),
 			EndContainer(),
 			/* Right panel. */
 			NWidget(NWID_VERTICAL),
-				NWidget(WWT_LABEL, COLOUR_GREY, MTSW_PLAYLIST), SetDataTip(STR_PLAYLIST_PROGRAM, STR_NULL),
-				NWidget(WWT_PANEL, COLOUR_GREY, MTSW_LIST_RIGHT), SetMinimalSize(180, 194), SetDataTip(0x0, STR_PLAYLIST_TOOLTIP_CLICK_TO_REMOVE_TRACK), EndContainer(),
+				NWidget(WWT_LABEL, COLOUR_GREY, WID_MTS_PLAYLIST), SetDataTip(STR_PLAYLIST_PROGRAM, STR_NULL),
+				NWidget(WWT_PANEL, COLOUR_GREY, WID_MTS_LIST_RIGHT), SetMinimalSize(180, 194), SetDataTip(0x0, STR_PLAYLIST_TOOLTIP_CLICK_TO_REMOVE_TRACK), EndContainer(),
 				NWidget(NWID_SPACER), SetMinimalSize(0, 2),
 			EndContainer(),
 		EndContainer(),
@@ -507,7 +498,7 @@ static const NWidgetPart _nested_music_track_selection_widgets[] = {
 static const WindowDesc _music_track_selection_desc(
 	WDP_AUTO, 0, 0,
 	WC_MUSIC_TRACK_SELECTION, WC_NONE,
-	WDF_UNCLICK_BUTTONS,
+	0,
 	_nested_music_track_selection_widgets, lengthof(_nested_music_track_selection_widgets)
 );
 
@@ -516,47 +507,23 @@ static void ShowMusicTrackSelection()
 	AllocateWindowDescFront<MusicTrackSelectionWindow>(&_music_track_selection_desc, 0);
 }
 
-enum MusicWidgets {
-	MW_PREV,
-	MW_NEXT,
-	MW_STOP,
-	MW_PLAY,
-	MW_SLIDERS,
-	MW_MUSIC_VOL,
-	MW_GAUGE,
-	MW_EFFECT_VOL,
-	MW_BACKGROUND,
-	MW_TRACK,
-	MW_TRACK_NR,
-	MW_TRACK_TITLE,
-	MW_TRACK_NAME,
-	MW_SHUFFLE,
-	MW_PROGRAMME,
-	MW_ALL,
-	MW_OLD,
-	MW_NEW,
-	MW_EZY,
-	MW_CUSTOM1,
-	MW_CUSTOM2,
-};
-
 struct MusicWindow : public Window {
 	static const int slider_width = 3;
 
 	MusicWindow(const WindowDesc *desc, WindowNumber number) : Window()
 	{
 		this->InitNested(desc, number);
-		this->LowerWidget(_msf.playlist + MW_ALL);
-		this->SetWidgetLoweredState(MW_SHUFFLE, _msf.shuffle);
+		this->LowerWidget(_settings_client.music.playlist + WID_M_ALL);
+		this->SetWidgetLoweredState(WID_M_SHUFFLE, _settings_client.music.shuffle);
 	}
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
 		switch (widget) {
-			/* Make sure that MW_SHUFFLE and MW_PROGRAMME have the same size.
-			 * This can't be done by using NC_EQUALSIZE as the MW_INFO is
+			/* Make sure that WID_M_SHUFFLE and WID_M_PROGRAMME have the same size.
+			 * This can't be done by using NC_EQUALSIZE as the WID_M_INFO is
 			 * between those widgets and of different size. */
-			case MW_SHUFFLE: case MW_PROGRAMME: {
+			case WID_M_SHUFFLE: case WID_M_PROGRAMME: {
 				Dimension d = maxdim(GetStringBoundingBox(STR_MUSIC_PROGRAM), GetStringBoundingBox(STR_MUSIC_SHUFFLE));
 				d.width += padding.width;
 				d.height += padding.height;
@@ -564,7 +531,7 @@ struct MusicWindow : public Window {
 				break;
 			}
 
-			case MW_TRACK_NR: {
+			case WID_M_TRACK_NR: {
 				Dimension d = GetStringBoundingBox(STR_MUSIC_TRACK_NONE);
 				d.width += WD_FRAMERECT_LEFT + WD_FRAMERECT_RIGHT;
 				d.height += WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM;
@@ -572,7 +539,7 @@ struct MusicWindow : public Window {
 				break;
 			}
 
-			case MW_TRACK_NAME: {
+			case WID_M_TRACK_NAME: {
 				Dimension d = GetStringBoundingBox(STR_MUSIC_TITLE_NONE);
 				for (uint i = 0; i < NUM_SONGS_AVAILABLE; i++) {
 					SetDParamStr(0, GetSongName(i));
@@ -586,32 +553,17 @@ struct MusicWindow : public Window {
 
 			/* Hack-ish: set the proper widget data; only needs to be done once
 			 * per (Re)Init as that's the only time the language changes. */
-			case MW_PREV: this->GetWidget<NWidgetCore>(MW_PREV)->widget_data = _current_text_dir == TD_RTL ? SPR_IMG_SKIP_TO_NEXT : SPR_IMG_SKIP_TO_PREV; break;
-			case MW_NEXT: this->GetWidget<NWidgetCore>(MW_NEXT)->widget_data = _current_text_dir == TD_RTL ? SPR_IMG_SKIP_TO_PREV : SPR_IMG_SKIP_TO_NEXT; break;
-			case MW_PLAY: this->GetWidget<NWidgetCore>(MW_PLAY)->widget_data = _current_text_dir == TD_RTL ? SPR_IMG_PLAY_MUSIC_RTL : SPR_IMG_PLAY_MUSIC; break;
+			case WID_M_PREV: this->GetWidget<NWidgetCore>(WID_M_PREV)->widget_data = _current_text_dir == TD_RTL ? SPR_IMG_SKIP_TO_NEXT : SPR_IMG_SKIP_TO_PREV; break;
+			case WID_M_NEXT: this->GetWidget<NWidgetCore>(WID_M_NEXT)->widget_data = _current_text_dir == TD_RTL ? SPR_IMG_SKIP_TO_PREV : SPR_IMG_SKIP_TO_NEXT; break;
+			case WID_M_PLAY: this->GetWidget<NWidgetCore>(WID_M_PLAY)->widget_data = _current_text_dir == TD_RTL ? SPR_IMG_PLAY_MUSIC_RTL : SPR_IMG_PLAY_MUSIC; break;
 		}
 	}
 
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		switch (widget) {
-			case MW_GAUGE:
-				GfxFillRect(r.left, r.top, r.right, r.bottom, 0);
-
-				for (uint i = 0; i != 8; i++) {
-					int colour = 0xD0;
-					if (i > 4) {
-						colour = 0xBF;
-						if (i > 6) {
-							colour = 0xB8;
-						}
-					}
-					GfxFillRect(r.left, r.bottom - i * 2, r.right, r.bottom - i * 2, colour);
-				}
-				break;
-
-			case MW_TRACK_NR: {
-				GfxFillRect(r.left + 1, r.top + 1, r.right, r.bottom, 0);
+			case WID_M_TRACK_NR: {
+				GfxFillRect(r.left + 1, r.top + 1, r.right, r.bottom, PC_BLACK);
 				StringID str = STR_MUSIC_TRACK_NONE;
 				if (_song_is_active != 0 && _music_wnd_cursong != 0) {
 					SetDParam(0, GetTrackNumber(_music_wnd_cursong - 1));
@@ -622,8 +574,8 @@ struct MusicWindow : public Window {
 				break;
 			}
 
-			case MW_TRACK_NAME: {
-				GfxFillRect(r.left, r.top + 1, r.right - 1, r.bottom, 0);
+			case WID_M_TRACK_NAME: {
+				GfxFillRect(r.left, r.top + 1, r.right - 1, r.bottom, PC_BLACK);
 				StringID str = STR_MUSIC_TITLE_NONE;
 				if (_song_is_active != 0 && _music_wnd_cursong != 0) {
 					str = STR_MUSIC_TITLE_NAME;
@@ -633,9 +585,9 @@ struct MusicWindow : public Window {
 				break;
 			}
 
-			case MW_MUSIC_VOL: case MW_EFFECT_VOL: {
+			case WID_M_MUSIC_VOL: case WID_M_EFFECT_VOL: {
 				DrawFrameRect(r.left, r.top + 2, r.right, r.bottom - 2, COLOUR_GREY, FR_LOWERED);
-				byte volume = (widget == MW_MUSIC_VOL) ? _msf.music_vol : _msf.effect_vol;
+				byte volume = (widget == WID_M_MUSIC_VOL) ? _settings_client.music.music_vol : _settings_client.music.effect_vol;
 				int x = (volume * (r.right - r.left) / 127);
 				if (_current_text_dir == TD_RTL) {
 					x = r.right - x;
@@ -657,7 +609,7 @@ struct MusicWindow : public Window {
 	{
 		if (!gui_scope) return;
 		for (int i = 0; i < 6; i++) {
-			this->SetWidgetLoweredState(MW_ALL + i, i == _msf.playlist);
+			this->SetWidgetLoweredState(WID_M_ALL + i, i == _settings_client.music.playlist);
 		}
 		this->SetDirty();
 	}
@@ -665,36 +617,36 @@ struct MusicWindow : public Window {
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		switch (widget) {
-			case MW_PREV: // skip to prev
+			case WID_M_PREV: // skip to prev
 				if (!_song_is_active) return;
 				SkipToPrevSong();
 				this->SetDirty();
 				break;
 
-			case MW_NEXT: // skip to next
+			case WID_M_NEXT: // skip to next
 				if (!_song_is_active) return;
 				SkipToNextSong();
 				this->SetDirty();
 				break;
 
-			case MW_STOP: // stop playing
-				_msf.playing = false;
+			case WID_M_STOP: // stop playing
+				_settings_client.music.playing = false;
 				break;
 
-			case MW_PLAY: // start playing
-				_msf.playing = true;
+			case WID_M_PLAY: // start playing
+				_settings_client.music.playing = true;
 				break;
 
-			case MW_MUSIC_VOL: case MW_EFFECT_VOL: { // volume sliders
+			case WID_M_MUSIC_VOL: case WID_M_EFFECT_VOL: { // volume sliders
 				int x = pt.x - this->GetWidget<NWidgetBase>(widget)->pos_x;
 
-				byte *vol = (widget == MW_MUSIC_VOL) ? &_msf.music_vol : &_msf.effect_vol;
+				byte *vol = (widget == WID_M_MUSIC_VOL) ? &_settings_client.music.music_vol : &_settings_client.music.effect_vol;
 
 				byte new_vol = x * 127 / this->GetWidget<NWidgetBase>(widget)->current_x;
 				if (_current_text_dir == TD_RTL) new_vol = 127 - new_vol;
 				if (new_vol != *vol) {
 					*vol = new_vol;
-					if (widget == MW_MUSIC_VOL) MusicVolumeChanged(new_vol);
+					if (widget == WID_M_MUSIC_VOL) MusicVolumeChanged(new_vol);
 					this->SetDirty();
 				}
 
@@ -702,22 +654,22 @@ struct MusicWindow : public Window {
 				break;
 			}
 
-			case MW_SHUFFLE: // toggle shuffle
-				_msf.shuffle ^= 1;
-				this->SetWidgetLoweredState(MW_SHUFFLE, _msf.shuffle);
-				this->SetWidgetDirty(MW_SHUFFLE);
+			case WID_M_SHUFFLE: // toggle shuffle
+				_settings_client.music.shuffle ^= 1;
+				this->SetWidgetLoweredState(WID_M_SHUFFLE, _settings_client.music.shuffle);
+				this->SetWidgetDirty(WID_M_SHUFFLE);
 				StopMusic();
 				SelectSongToPlay();
 				this->SetDirty();
 				break;
 
-			case MW_PROGRAMME: // show track selection
+			case WID_M_PROGRAMME: // show track selection
 				ShowMusicTrackSelection();
 				break;
 
-			case MW_ALL: case MW_OLD: case MW_NEW:
-			case MW_EZY: case MW_CUSTOM1: case MW_CUSTOM2: // playlist
-				SelectPlaylist(widget - MW_ALL);
+			case WID_M_ALL: case WID_M_OLD: case WID_M_NEW:
+			case WID_M_EZY: case WID_M_CUSTOM1: case WID_M_CUSTOM2: // playlist
+				SelectPlaylist(widget - WID_M_ALL);
 				StopMusic();
 				SelectSongToPlay();
 				this->SetDirty();
@@ -736,18 +688,18 @@ static const NWidgetPart _nested_music_window_widgets[] = {
 		NWidget(NWID_VERTICAL),
 			NWidget(WWT_PANEL, COLOUR_GREY, -1), SetFill(1, 1), EndContainer(),
 			NWidget(NWID_HORIZONTAL),
-				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, MW_PREV), SetMinimalSize(22, 22), SetDataTip(SPR_IMG_SKIP_TO_PREV, STR_MUSIC_TOOLTIP_SKIP_TO_PREVIOUS_TRACK),
-				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, MW_NEXT), SetMinimalSize(22, 22), SetDataTip(SPR_IMG_SKIP_TO_NEXT, STR_MUSIC_TOOLTIP_SKIP_TO_NEXT_TRACK_IN_SELECTION),
-				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, MW_STOP), SetMinimalSize(22, 22), SetDataTip(SPR_IMG_STOP_MUSIC, STR_MUSIC_TOOLTIP_STOP_PLAYING_MUSIC),
-				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, MW_PLAY), SetMinimalSize(22, 22), SetDataTip(SPR_IMG_PLAY_MUSIC, STR_MUSIC_TOOLTIP_START_PLAYING_MUSIC),
+				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_M_PREV), SetMinimalSize(22, 22), SetDataTip(SPR_IMG_SKIP_TO_PREV, STR_MUSIC_TOOLTIP_SKIP_TO_PREVIOUS_TRACK),
+				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_M_NEXT), SetMinimalSize(22, 22), SetDataTip(SPR_IMG_SKIP_TO_NEXT, STR_MUSIC_TOOLTIP_SKIP_TO_NEXT_TRACK_IN_SELECTION),
+				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_M_STOP), SetMinimalSize(22, 22), SetDataTip(SPR_IMG_STOP_MUSIC, STR_MUSIC_TOOLTIP_STOP_PLAYING_MUSIC),
+				NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_M_PLAY), SetMinimalSize(22, 22), SetDataTip(SPR_IMG_PLAY_MUSIC, STR_MUSIC_TOOLTIP_START_PLAYING_MUSIC),
 			EndContainer(),
 			NWidget(WWT_PANEL, COLOUR_GREY, -1), SetFill(1, 1), EndContainer(),
 		EndContainer(),
-		NWidget(WWT_PANEL, COLOUR_GREY, MW_SLIDERS),
-			NWidget(NWID_HORIZONTAL), SetPIP(20, 0, 20),
+		NWidget(WWT_PANEL, COLOUR_GREY, WID_M_SLIDERS),
+			NWidget(NWID_HORIZONTAL), SetPIP(20, 20, 20),
 				NWidget(NWID_VERTICAL),
 					NWidget(WWT_LABEL, COLOUR_GREY, -1), SetFill(1, 0), SetDataTip(STR_MUSIC_MUSIC_VOLUME, STR_NULL),
-					NWidget(WWT_EMPTY, COLOUR_GREY, MW_MUSIC_VOL), SetMinimalSize(67, 0), SetMinimalTextLines(1, 0), SetFill(1, 0), SetDataTip(0x0, STR_MUSIC_TOOLTIP_DRAG_SLIDERS_TO_SET_MUSIC),
+					NWidget(WWT_EMPTY, COLOUR_GREY, WID_M_MUSIC_VOL), SetMinimalSize(67, 0), SetMinimalTextLines(1, 0), SetFill(1, 0), SetDataTip(0x0, STR_MUSIC_TOOLTIP_DRAG_SLIDERS_TO_SET_MUSIC),
 					NWidget(NWID_HORIZONTAL),
 						NWidget(WWT_LABEL, COLOUR_GREY, -1), SetDataTip(STR_MUSIC_RULER_MIN, STR_NULL),
 						NWidget(WWT_LABEL, COLOUR_GREY, -1), SetDataTip(STR_MUSIC_RULER_MARKER, STR_NULL), SetFill(1, 0),
@@ -758,10 +710,9 @@ static const NWidgetPart _nested_music_window_widgets[] = {
 						NWidget(WWT_LABEL, COLOUR_GREY, -1), SetDataTip(STR_MUSIC_RULER_MAX, STR_NULL),
 					EndContainer(),
 				EndContainer(),
-				NWidget(WWT_PANEL, COLOUR_GREY, MW_GAUGE), SetMinimalSize(16, 20), SetPadding(1, 11, 1, 11), SetFill(0, 0), EndContainer(),
 				NWidget(NWID_VERTICAL),
 					NWidget(WWT_LABEL, COLOUR_GREY, -1), SetFill(1, 0), SetDataTip(STR_MUSIC_EFFECTS_VOLUME, STR_NULL),
-					NWidget(WWT_EMPTY, COLOUR_GREY, MW_EFFECT_VOL), SetMinimalSize(67, 0), SetMinimalTextLines(1, 0), SetFill(1, 0), SetDataTip(0x0, STR_MUSIC_TOOLTIP_DRAG_SLIDERS_TO_SET_MUSIC),
+					NWidget(WWT_EMPTY, COLOUR_GREY, WID_M_EFFECT_VOL), SetMinimalSize(67, 0), SetMinimalTextLines(1, 0), SetFill(1, 0), SetDataTip(0x0, STR_MUSIC_TOOLTIP_DRAG_SLIDERS_TO_SET_MUSIC),
 					NWidget(NWID_HORIZONTAL),
 						NWidget(WWT_LABEL, COLOUR_GREY, -1), SetDataTip(STR_MUSIC_RULER_MIN, STR_NULL),
 						NWidget(WWT_LABEL, COLOUR_GREY, -1), SetDataTip(STR_MUSIC_RULER_MARKER, STR_NULL), SetFill(1, 0),
@@ -775,42 +726,42 @@ static const NWidgetPart _nested_music_window_widgets[] = {
 			EndContainer(),
 		EndContainer(),
 	EndContainer(),
-	NWidget(WWT_PANEL, COLOUR_GREY, MW_BACKGROUND),
+	NWidget(WWT_PANEL, COLOUR_GREY, WID_M_BACKGROUND),
 		NWidget(NWID_HORIZONTAL), SetPIP(6, 0, 6),
 			NWidget(NWID_VERTICAL),
 				NWidget(NWID_SPACER), SetFill(0, 1),
-				NWidget(WWT_TEXTBTN, COLOUR_GREY, MW_SHUFFLE), SetMinimalSize(50, 8), SetDataTip(STR_MUSIC_SHUFFLE, STR_MUSIC_TOOLTIP_TOGGLE_PROGRAM_SHUFFLE),
+				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_M_SHUFFLE), SetMinimalSize(50, 8), SetDataTip(STR_MUSIC_SHUFFLE, STR_MUSIC_TOOLTIP_TOGGLE_PROGRAM_SHUFFLE),
 				NWidget(NWID_SPACER), SetFill(0, 1),
 			EndContainer(),
 			NWidget(NWID_VERTICAL), SetPadding(0, 0, 3, 3),
-				NWidget(WWT_LABEL, COLOUR_GREY, MW_TRACK), SetFill(0, 0), SetDataTip(STR_MUSIC_TRACK, STR_NULL),
-				NWidget(WWT_PANEL, COLOUR_GREY, MW_TRACK_NR), EndContainer(),
+				NWidget(WWT_LABEL, COLOUR_GREY, WID_M_TRACK), SetFill(0, 0), SetDataTip(STR_MUSIC_TRACK, STR_NULL),
+				NWidget(WWT_PANEL, COLOUR_GREY, WID_M_TRACK_NR), EndContainer(),
 			EndContainer(),
 			NWidget(NWID_VERTICAL), SetPadding(0, 3, 3, 0),
-				NWidget(WWT_LABEL, COLOUR_GREY, MW_TRACK_TITLE), SetFill(1, 0), SetDataTip(STR_MUSIC_XTITLE, STR_NULL),
-				NWidget(WWT_PANEL, COLOUR_GREY, MW_TRACK_NAME), SetFill(1, 0), EndContainer(),
+				NWidget(WWT_LABEL, COLOUR_GREY, WID_M_TRACK_TITLE), SetFill(1, 0), SetDataTip(STR_MUSIC_XTITLE, STR_NULL),
+				NWidget(WWT_PANEL, COLOUR_GREY, WID_M_TRACK_NAME), SetFill(1, 0), EndContainer(),
 			EndContainer(),
 			NWidget(NWID_VERTICAL),
 				NWidget(NWID_SPACER), SetFill(0, 1),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, MW_PROGRAMME), SetMinimalSize(50, 8), SetDataTip(STR_MUSIC_PROGRAM, STR_MUSIC_TOOLTIP_SHOW_MUSIC_TRACK_SELECTION),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_M_PROGRAMME), SetMinimalSize(50, 8), SetDataTip(STR_MUSIC_PROGRAM, STR_MUSIC_TOOLTIP_SHOW_MUSIC_TRACK_SELECTION),
 				NWidget(NWID_SPACER), SetFill(0, 1),
 			EndContainer(),
 		EndContainer(),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL, NC_EQUALSIZE),
-		NWidget(WWT_TEXTBTN, COLOUR_GREY, MW_ALL), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_ALL, STR_MUSIC_TOOLTIP_SELECT_ALL_TRACKS_PROGRAM),
-		NWidget(WWT_TEXTBTN, COLOUR_GREY, MW_OLD), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_OLD_STYLE, STR_MUSIC_TOOLTIP_SELECT_OLD_STYLE_MUSIC),
-		NWidget(WWT_TEXTBTN, COLOUR_GREY, MW_NEW), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_NEW_STYLE, STR_MUSIC_TOOLTIP_SELECT_NEW_STYLE_MUSIC),
-		NWidget(WWT_TEXTBTN, COLOUR_GREY, MW_EZY), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_EZY_STREET, STR_MUSIC_TOOLTIP_SELECT_EZY_STREET_STYLE),
-		NWidget(WWT_TEXTBTN, COLOUR_GREY, MW_CUSTOM1), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_CUSTOM_1, STR_MUSIC_TOOLTIP_SELECT_CUSTOM_1_USER_DEFINED),
-		NWidget(WWT_TEXTBTN, COLOUR_GREY, MW_CUSTOM2), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_CUSTOM_2, STR_MUSIC_TOOLTIP_SELECT_CUSTOM_2_USER_DEFINED),
+		NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_M_ALL), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_ALL, STR_MUSIC_TOOLTIP_SELECT_ALL_TRACKS_PROGRAM),
+		NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_M_OLD), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_OLD_STYLE, STR_MUSIC_TOOLTIP_SELECT_OLD_STYLE_MUSIC),
+		NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_M_NEW), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_NEW_STYLE, STR_MUSIC_TOOLTIP_SELECT_NEW_STYLE_MUSIC),
+		NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_M_EZY), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_EZY_STREET, STR_MUSIC_TOOLTIP_SELECT_EZY_STREET_STYLE),
+		NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_M_CUSTOM1), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_CUSTOM_1, STR_MUSIC_TOOLTIP_SELECT_CUSTOM_1_USER_DEFINED),
+		NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_M_CUSTOM2), SetFill(1, 0), SetDataTip(STR_MUSIC_PLAYLIST_CUSTOM_2, STR_MUSIC_TOOLTIP_SELECT_CUSTOM_2_USER_DEFINED),
 	EndContainer(),
 };
 
 static const WindowDesc _music_window_desc(
 	WDP_AUTO, 0, 0,
 	WC_MUSIC_WINDOW, WC_NONE,
-	WDF_UNCLICK_BUTTONS,
+	0,
 	_nested_music_window_widgets, lengthof(_nested_music_window_widgets)
 );
 
