@@ -1,4 +1,4 @@
-/* $Id: network.cpp 22935 2011-09-15 18:56:43Z rubidium $ */
+/* $Id: network.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -36,7 +36,7 @@
 #include "../rev.h"
 #include "../core/pool_func.hpp"
 #include "../gfx_func.h"
-#include "table/strings.h"
+#include "../error.h"
 
 #ifdef DEBUG_DUMP_COMMANDS
 #include "../fileio_func.h"
@@ -44,8 +44,10 @@
 bool _ddc_fastforward = true;
 #endif /* DEBUG_DUMP_COMMANDS */
 
+/** Make sure both pools have the same size. */
 assert_compile(NetworkClientInfoPool::MAX_SIZE == NetworkClientSocketPool::MAX_SIZE);
 
+/** The pool with client information. */
 NetworkClientInfoPool _networkclientinfo_pool("NetworkClientInfo");
 INSTANTIATE_POOL_METHODS(NetworkClientInfo)
 
@@ -54,30 +56,30 @@ bool _network_server;     ///< network-server is active
 bool _network_available;  ///< is network mode available?
 bool _network_dedicated;  ///< are we a dedicated server?
 bool _is_network_server;  ///< Does this client wants to be a network-server?
-NetworkServerGameInfo _network_game_info;
-NetworkCompanyState *_network_company_states = NULL;
-ClientID _network_own_client_id;
-ClientID _redirect_console_to_client;
-bool _network_need_advertise;
-uint32 _network_last_advertise_frame;
-uint8 _network_reconnect;
-StringList _network_bind_list;
-StringList _network_host_list;
-StringList _network_ban_list;
-uint32 _frame_counter_server; // The frame_counter of the server, if in network-mode
-uint32 _frame_counter_max; // To where we may go with our clients
-uint32 _frame_counter;
-uint32 _last_sync_frame; // Used in the server to store the last time a sync packet was sent to clients.
-NetworkAddressList _broadcast_list;
-uint32 _sync_seed_1;
+NetworkServerGameInfo _network_game_info; ///< Information about our game.
+NetworkCompanyState *_network_company_states = NULL; ///< Statistics about some companies.
+ClientID _network_own_client_id;      ///< Our client identifier.
+ClientID _redirect_console_to_client; ///< If not invalid, redirect the console output to a client.
+bool _network_need_advertise;         ///< Whether we need to advertise.
+uint32 _network_last_advertise_frame; ///< Last time we did advertise.
+uint8 _network_reconnect;             ///< Reconnect timeout
+StringList _network_bind_list;        ///< The addresses to bind on.
+StringList _network_host_list;        ///< The servers we know.
+StringList _network_ban_list;         ///< The banned clients.
+uint32 _frame_counter_server;         ///< The frame_counter of the server, if in network-mode
+uint32 _frame_counter_max;            ///< To where we may go with our clients
+uint32 _frame_counter;                ///< The current frame.
+uint32 _last_sync_frame;              ///< Used in the server to store the last time a sync packet was sent to clients.
+NetworkAddressList _broadcast_list;   ///< List of broadcast addresses.
+uint32 _sync_seed_1;                  ///< Seed to compare during sync checks.
 #ifdef NETWORK_SEND_DOUBLE_SEED
-uint32 _sync_seed_2;
+uint32 _sync_seed_2;                  ///< Second part of the seed.
 #endif
-uint32 _sync_frame;
-bool _network_first_time;
-bool _network_udp_server;
-uint16 _network_udp_broadcast;
-uint8 _network_advertise_retries;
+uint32 _sync_frame;                   ///< The frame to perform the sync check.
+bool _network_first_time;             ///< Whether we have finished joining or not.
+bool _network_udp_server;             ///< Is the UDP server started?
+uint16 _network_udp_broadcast;        ///< Timeout for the UDP broadcasts.
+uint8 _network_advertise_retries;     ///< The number of advertisement retries we did.
 CompanyMask _network_company_passworded; ///< Bitmask of the password status of all companies.
 
 /* Check whether NETWORK_NUM_LANDSCAPES is still in sync with NUM_LANDSCAPE */
@@ -88,7 +90,7 @@ extern NetworkUDPSocketHandler *_udp_client_socket; ///< udp client socket
 extern NetworkUDPSocketHandler *_udp_server_socket; ///< udp server socket
 extern NetworkUDPSocketHandler *_udp_master_socket; ///< udp master socket
 
-/* The amount of clients connected */
+/** The amount of clients connected */
 byte _network_clients_connected = 0;
 
 /* Some externs / forwards */
@@ -124,7 +126,7 @@ NetworkClientInfo::~NetworkClientInfo()
  * @param client_id the ClientID to search for
  * @return return a pointer to the corresponding NetworkClientSocket struct or NULL when not found
  */
-/* static */ NetworkClientSocket *NetworkClientSocket::GetByClientID(ClientID client_id)
+/* static */ ServerNetworkGameSocketHandler *ServerNetworkGameSocketHandler::GetByClientID(ClientID client_id)
 {
 	NetworkClientSocket *cs;
 
@@ -285,8 +287,7 @@ uint NetworkCalculateLag(const NetworkClientSocket *cs)
 void NetworkError(StringID error_string)
 {
 	_switch_mode = SM_MENU;
-	extern StringID _switch_mode_errorstr;
-	_switch_mode_errorstr = error_string;
+	ShowErrorMessage(error_string, INVALID_STRING_ID, WL_CRITICAL);
 }
 
 /**
@@ -314,8 +315,13 @@ StringID GetNetworkErrorMsg(NetworkErrorCode err)
 		STR_NETWORK_ERROR_CLIENT_KICKED,
 		STR_NETWORK_ERROR_CLIENT_CHEATER,
 		STR_NETWORK_ERROR_CLIENT_SERVER_FULL,
-		STR_NETWORK_ERROR_CLIENT_TOO_MANY_COMMANDS
+		STR_NETWORK_ERROR_CLIENT_TOO_MANY_COMMANDS,
+		STR_NETWORK_ERROR_CLIENT_TIMEOUT_PASSWORD,
+		STR_NETWORK_ERROR_CLIENT_TIMEOUT_COMPUTER,
+		STR_NETWORK_ERROR_CLIENT_TIMEOUT_MAP,
+		STR_NETWORK_ERROR_CLIENT_TIMEOUT_JOIN,
 	};
+	assert_compile(lengthof(network_error_strings) == NETWORK_ERROR_END);
 
 	if (err >= (ptrdiff_t)lengthof(network_error_strings)) err = NETWORK_ERROR_GENERAL;
 
@@ -334,6 +340,7 @@ void NetworkHandlePauseChange(PauseMode prev_mode, PauseMode changed_mode)
 	switch (changed_mode) {
 		case PM_PAUSED_NORMAL:
 		case PM_PAUSED_JOIN:
+		case PM_PAUSED_GAME_SCRIPT:
 		case PM_PAUSED_ACTIVE_CLIENTS: {
 			bool changed = ((_pause_mode == PM_UNPAUSED) != (prev_mode == PM_UNPAUSED));
 			bool paused = (_pause_mode != PM_UNPAUSED);
@@ -342,14 +349,17 @@ void NetworkHandlePauseChange(PauseMode prev_mode, PauseMode changed_mode)
 			StringID str;
 			if (!changed) {
 				int i = -1;
+
 				if ((_pause_mode & PM_PAUSED_NORMAL) != PM_UNPAUSED)         SetDParam(++i, STR_NETWORK_SERVER_MESSAGE_GAME_REASON_MANUAL);
 				if ((_pause_mode & PM_PAUSED_JOIN) != PM_UNPAUSED)           SetDParam(++i, STR_NETWORK_SERVER_MESSAGE_GAME_REASON_CONNECTING_CLIENTS);
+				if ((_pause_mode & PM_PAUSED_GAME_SCRIPT) != PM_UNPAUSED)    SetDParam(++i, STR_NETWORK_SERVER_MESSAGE_GAME_REASON_GAME_SCRIPT);
 				if ((_pause_mode & PM_PAUSED_ACTIVE_CLIENTS) != PM_UNPAUSED) SetDParam(++i, STR_NETWORK_SERVER_MESSAGE_GAME_REASON_NOT_ENOUGH_PLAYERS);
 				str = STR_NETWORK_SERVER_MESSAGE_GAME_STILL_PAUSED_1 + i;
 			} else {
 				switch (changed_mode) {
 					case PM_PAUSED_NORMAL:         SetDParam(0, STR_NETWORK_SERVER_MESSAGE_GAME_REASON_MANUAL); break;
 					case PM_PAUSED_JOIN:           SetDParam(0, STR_NETWORK_SERVER_MESSAGE_GAME_REASON_CONNECTING_CLIENTS); break;
+					case PM_PAUSED_GAME_SCRIPT:    SetDParam(0, STR_NETWORK_SERVER_MESSAGE_GAME_REASON_GAME_SCRIPT); break;
 					case PM_PAUSED_ACTIVE_CLIENTS: SetDParam(0, STR_NETWORK_SERVER_MESSAGE_GAME_REASON_NOT_ENOUGH_PLAYERS); break;
 					default: NOT_REACHED();
 				}
@@ -445,7 +455,7 @@ static void CheckPauseOnJoin()
  * Converts a string to ip/port/company
  *  Format: IP:port#company
  *
- * connection_string will be re-terminated to seperate out the hostname, and company and port will
+ * connection_string will be re-terminated to separate out the hostname, and company and port will
  * be set to the company and port strings given by the user, inside the memory area originally
  * occupied by connection_string.
  */
@@ -477,6 +487,11 @@ void ParseConnectionString(const char **company, const char **port, char *connec
 	}
 }
 
+/**
+ * Handle the accepting of a connection to the server.
+ * @param s The socket of the new connection.
+ * @param address The address of the peer.
+ */
 /* static */ void ServerNetworkGameSocketHandler::AcceptConnection(SOCKET s, const NetworkAddress &address)
 {
 	/* Register the login */
@@ -493,9 +508,7 @@ void ParseConnectionString(const char **company, const char **port, char *connec
  */
 static void InitializeNetworkPools(bool close_admins = true)
 {
-	_networkclientsocket_pool.CleanPool();
-	_networkclientinfo_pool.CleanPool();
-	if (close_admins) _networkadminsocket_pool.CleanPool();
+	PoolBase::Clean(PT_NCLIENT | (close_admins ? PT_NADMIN : PT_NONE));
 }
 
 /**
@@ -536,7 +549,7 @@ void NetworkClose(bool close_admins)
 	InitializeNetworkPools(close_admins);
 }
 
-/* Inits the network (cleans sockets and stuff) */
+/* Initializes the network (cleans sockets and stuff) */
 static void NetworkInitialize(bool close_admins = true)
 {
 	InitializeNetworkPools(close_admins);
@@ -786,7 +799,7 @@ void NetworkDisconnect(bool blocking, bool close_admins)
 
 	if (_settings_client.network.server_advertise) NetworkUDPRemoveAdvertise(blocking);
 
-	DeleteWindowById(WC_NETWORK_STATUS_WINDOW, 0);
+	DeleteWindowById(WC_NETWORK_STATUS_WINDOW, WN_NETWORK_STATUS_WINDOW_JOIN);
 
 	NetworkClose(close_admins);
 
@@ -796,7 +809,7 @@ void NetworkDisconnect(bool blocking, bool close_admins)
 
 /**
  * Receives something from the network.
- * @return true if everthing went fine, false when the connection got closed.
+ * @return true if everything went fine, false when the connection got closed.
  */
 static bool NetworkReceive()
 {
@@ -819,20 +832,18 @@ static void NetworkSend()
 	}
 }
 
-/* We have to do some UDP checking */
-void NetworkUDPGameLoop()
+/**
+ * We have to do some (simple) background stuff that runs normally,
+ * even when we are not in multiplayer. For example stuff needed
+ * for finding servers or downloading content.
+ */
+void NetworkBackgroundLoop()
 {
 	_network_content_client.SendReceive();
 	TCPConnecter::CheckCallbacks();
 	NetworkHTTPSocketHandler::HTTPReceive();
 
-	if (_network_udp_server) {
-		_udp_server_socket->ReceivePackets();
-		_udp_master_socket->ReceivePackets();
-	} else {
-		_udp_client_socket->ReceivePackets();
-		if (_network_udp_broadcast > 0) _network_udp_broadcast--;
-	}
+	NetworkBackgroundUDPLoop();
 }
 
 /* The main loop called from ttd.c
