@@ -1,4 +1,4 @@
-/* $Id: network_admin.cpp 23255 2011-11-18 21:04:08Z rubidium $ */
+/* $Id: network_admin.cpp 25599 2013-07-13 10:13:55Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -23,6 +23,7 @@
 #include "../core/pool_func.hpp"
 #include "../map_func.h"
 #include "../rev.h"
+#include "../game/game.hpp"
 
 
 /* This file handles all the admin network commands. */
@@ -33,6 +34,7 @@ AdminIndex _redirect_console_to_admin = INVALID_ADMIN_ID;
 /** The amount of admins connected. */
 byte _network_admins_connected = 0;
 
+/** The pool with sockets/clients. */
 NetworkAdminSocketPool _networkadminsocket_pool("NetworkAdminSocket");
 INSTANTIATE_POOL_METHODS(NetworkAdminSocket)
 
@@ -51,7 +53,9 @@ static const AdminUpdateFrequency _admin_update_type_frequencies[] = {
 	                       ADMIN_FREQUENCY_AUTOMATIC,                                                                                                      ///< ADMIN_UPDATE_CONSOLE
 	ADMIN_FREQUENCY_POLL,                                                                                                                                  ///< ADMIN_UPDATE_CMD_NAMES
 	                       ADMIN_FREQUENCY_AUTOMATIC,                                                                                                      ///< ADMIN_UPDATE_CMD_LOGGING
+	                       ADMIN_FREQUENCY_AUTOMATIC,                                                                                                      ///< ADMIN_UPDATE_GAMESCRIPT
 };
+/** Sanity check. */
 assert_compile(lengthof(_admin_update_type_frequencies) == ADMIN_UPDATE_END);
 
 /**
@@ -81,7 +85,12 @@ ServerNetworkAdminSocketHandler::~ServerNetworkAdminSocketHandler()
  */
 /* static */ bool ServerNetworkAdminSocketHandler::AllowConnection()
 {
-	return !StrEmpty(_settings_client.network.admin_password) && _network_admins_connected < MAX_ADMINS;
+	bool accept = !StrEmpty(_settings_client.network.admin_password) && _network_admins_connected < MAX_ADMINS;
+	/* We can't go over the MAX_ADMINS limit here. However, if we accept
+	 * the connection, there has to be space in the pool. */
+	assert_compile(NetworkAdminSocketPool::MAX_SIZE == MAX_ADMINS);
+	assert(!accept || ServerNetworkAdminSocketHandler::CanAllocateItem());
+	return accept;
 }
 
 /** Send the packets for the server sockets. */
@@ -100,6 +109,11 @@ ServerNetworkAdminSocketHandler::~ServerNetworkAdminSocketHandler()
 	}
 }
 
+/**
+ * Handle the acception of a connection.
+ * @param s The socket of the new connection.
+ * @param address The address of the peer.
+ */
 /* static */ void ServerNetworkAdminSocketHandler::AcceptConnection(SOCKET s, const NetworkAddress &address)
 {
 	ServerNetworkAdminSocketHandler *as = new ServerNetworkAdminSocketHandler(s);
@@ -110,6 +124,10 @@ ServerNetworkAdminSocketHandler::~ServerNetworkAdminSocketHandler()
  * Sending functions for admin network
  ************/
 
+/**
+ * Send an error to the admin.
+ * @param error The error to send.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendError(NetworkErrorCode error)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_ERROR);
@@ -126,6 +144,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendError(NetworkErrorCode er
 	return this->CloseConnection(true);
 }
 
+/** Send the protocol version to the admin. */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendProtocol()
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_PROTOCOL);
@@ -145,6 +164,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendProtocol()
 	return this->SendWelcome();
 }
 
+/** Send a welcome message to the admin. */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendWelcome()
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_WELCOME);
@@ -165,6 +185,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendWelcome()
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/** Tell the admin we started a new game. */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendNewGame()
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_NEWGAME);
@@ -172,6 +193,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendNewGame()
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/** Tell the admin we're shutting down. */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendShutdown()
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_SHUTDOWN);
@@ -179,6 +201,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendShutdown()
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/** Tell the admin the date. */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendDate()
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_DATE);
@@ -189,6 +212,10 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendDate()
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Tell the admin that a client joined.
+ * @param client_id The client that joined.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendClientJoin(ClientID client_id)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_CLIENT_JOIN);
@@ -223,6 +250,11 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendClientInfo(const NetworkC
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+
+/**
+ * Send an update for some client's information.
+ * @param ci The information about a client.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendClientUpdate(const NetworkClientInfo *ci)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_CLIENT_UPDATE);
@@ -236,6 +268,10 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendClientUpdate(const Networ
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Tell the admin that a client quit.
+ * @param client_id The client that quit.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendClientQuit(ClientID client_id)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_CLIENT_QUIT);
@@ -246,6 +282,11 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendClientQuit(ClientID clien
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Tell the admin that a client made an error.
+ * @param client_id The client that made the error.
+ * @param error The error that was made.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendClientError(ClientID client_id, NetworkErrorCode error)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_CLIENT_ERROR);
@@ -257,6 +298,10 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendClientError(ClientID clie
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Tell the admin that a new company was founded.
+ * @param company_id The company that was founded.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyNew(CompanyID company_id)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_COMPANY_NEW);
@@ -267,6 +312,10 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyNew(CompanyID comp
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Send the admin some information about a company.
+ * @param c The company to send the information about.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyInfo(const Company *c)
 {
 	char company_name[NETWORK_COMPANY_NAME_LENGTH];
@@ -293,6 +342,11 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyInfo(const Company
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+
+/**
+ * Send an update about a company.
+ * @param c The company to send the update of.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyUpdate(const Company *c)
 {
 	char company_name[NETWORK_COMPANY_NAME_LENGTH];
@@ -311,7 +365,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyUpdate(const Compa
 	p->Send_string(manager_name);
 	p->Send_uint8 (c->colour);
 	p->Send_bool  (NetworkCompanyIsPassworded(c->index));
-	p->Send_uint8 (c->quarters_of_bankruptcy);
+	p->Send_uint8 (CeilDiv(c->months_of_bankruptcy, 3)); // send as quarters_of_bankruptcy
 
 	for (size_t i = 0; i < lengthof(c->share_owners); i++) {
 		p->Send_uint8(c->share_owners[i]);
@@ -322,6 +376,11 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyUpdate(const Compa
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Tell the admin that a company got removed.
+ * @param company_id The company that got removed.
+ * @param acrr The reason for removal, e.g. bankruptcy or merger.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyRemove(CompanyID company_id, AdminCompanyRemoveReason acrr)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_COMPANY_REMOVE);
@@ -334,6 +393,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyRemove(CompanyID c
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/** Send economic information of all companies. */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyEconomy()
 {
 	const Company *company;
@@ -352,13 +412,13 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyEconomy()
 		p->Send_uint64(company->money);
 		p->Send_uint64(company->current_loan);
 		p->Send_uint64(income);
-		p->Send_uint16(company->cur_economy.delivered_cargo);
+		p->Send_uint16(min(UINT16_MAX, company->cur_economy.delivered_cargo.GetSum<OverflowSafeInt64>()));
 
 		/* Send stats for the last 2 quarters. */
 		for (uint i = 0; i < 2; i++) {
 			p->Send_uint64(company->old_economy[i].company_value);
 			p->Send_uint16(company->old_economy[i].performance_history);
-			p->Send_uint16(company->old_economy[i].delivered_cargo);
+			p->Send_uint16(min(UINT16_MAX, company->old_economy[i].delivered_cargo.GetSum<OverflowSafeInt64>()));
 		}
 
 		this->SendPacket(p);
@@ -368,6 +428,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyEconomy()
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/** Send statistics about the companies. */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyStats()
 {
 	/* Fetch the latest version of the stats. */
@@ -397,6 +458,14 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCompanyStats()
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Send a chat message.
+ * @param action The action associated with the message.
+ * @param desttype The destination type.
+ * @param client_id The origin of the chat message.
+ * @param msg The actual message.
+ * @param data Arbitrary extra data.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendChat(NetworkAction action, DestType desttype, ClientID client_id, const char *msg, int64 data)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_CHAT);
@@ -411,6 +480,25 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendChat(NetworkAction action
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Send a notification indicating the rcon command has completed.
+ * @param command The original command sent.
+ */
+NetworkRecvStatus ServerNetworkAdminSocketHandler::SendRconEnd(const char *command)
+{
+	Packet *p = new Packet(ADMIN_PACKET_SERVER_RCON_END);
+
+	p->Send_string(command);
+	this->SendPacket(p);
+
+	return NETWORK_RECV_STATUS_OKAY;
+}
+
+/**
+ * Send the reply of an rcon command.
+ * @param colour The colour of the text.
+ * @param result The result of the command.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendRcon(uint16 colour, const char *result)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_RCON);
@@ -422,7 +510,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendRcon(uint16 colour, const
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
-DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_RCON)
+NetworkRecvStatus ServerNetworkAdminSocketHandler::Receive_ADMIN_RCON(Packet *p)
 {
 	if (this->status == ADMIN_STATUS_INACTIVE) return this->SendError(NETWORK_ERROR_NOT_EXPECTED);
 
@@ -435,9 +523,39 @@ DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_RCON)
 	_redirect_console_to_admin = this->index;
 	IConsoleCmdExec(command);
 	_redirect_console_to_admin = INVALID_ADMIN_ID;
+	return this->SendRconEnd(command);
+}
+
+NetworkRecvStatus ServerNetworkAdminSocketHandler::Receive_ADMIN_GAMESCRIPT(Packet *p)
+{
+	if (this->status == ADMIN_STATUS_INACTIVE) return this->SendError(NETWORK_ERROR_NOT_EXPECTED);
+
+	char json[NETWORK_GAMESCRIPT_JSON_LENGTH];
+
+	p->Recv_string(json, sizeof(json));
+
+	DEBUG(net, 2, "[admin] GameScript JSON from '%s' (%s): '%s'", this->admin_name, this->admin_version, json);
+
+	Game::NewEvent(new ScriptEventAdminPort(json));
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+NetworkRecvStatus ServerNetworkAdminSocketHandler::Receive_ADMIN_PING(Packet *p)
+{
+	if (this->status == ADMIN_STATUS_INACTIVE) return this->SendError(NETWORK_ERROR_NOT_EXPECTED);
+
+	uint32 d1 = p->Recv_uint32();
+
+	DEBUG(net, 2, "[admin] Ping from '%s' (%s): '%d'", this->admin_name, this->admin_version, d1);
+
+	return this->SendPong(d1);
+}
+
+/**
+ * Send console output of other clients.
+ * @param origin The origin of the string.
+ * @param string The string that's put on the console.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendConsole(const char *origin, const char *string)
 {
 	/* If the length of both strings, plus the 2 '\0' terminations and 3 bytes of the packet
@@ -455,6 +573,37 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendConsole(const char *origi
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Send GameScript JSON output.
+ * @param json The JSON string.
+ */
+NetworkRecvStatus ServerNetworkAdminSocketHandler::SendGameScript(const char *json)
+{
+	/* At the moment we cannot transmit anything larger than MTU. So we limit
+	 *  the maximum amount of json data that can be sent. Account also for
+	 *  the trailing \0 of the string */
+	if (strlen(json) + 1 >= NETWORK_GAMESCRIPT_JSON_LENGTH) return NETWORK_RECV_STATUS_OKAY;
+
+	Packet *p = new Packet(ADMIN_PACKET_SERVER_GAMESCRIPT);
+
+	p->Send_string(json);
+	this->SendPacket(p);
+
+	return NETWORK_RECV_STATUS_OKAY;
+}
+
+/** Send ping-reply (pong) to admin **/
+NetworkRecvStatus ServerNetworkAdminSocketHandler::SendPong(uint32 d1)
+{
+	Packet *p = new Packet(ADMIN_PACKET_SERVER_PONG);
+
+	p->Send_uint32(d1);
+	this->SendPacket(p);
+
+	return NETWORK_RECV_STATUS_OKAY;
+}
+
+/** Send the names of the commands. */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCmdNames()
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_CMD_NAMES);
@@ -484,6 +633,11 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCmdNames()
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
+/**
+ * Send a command for logging purposes.
+ * @param client_id The client executing the command.
+ * @param cp The command that would be executed.
+ */
 NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCmdLogging(ClientID client_id, const CommandPacket *cp)
 {
 	Packet *p = new Packet(ADMIN_PACKET_SERVER_CMD_LOGGING);
@@ -506,7 +660,7 @@ NetworkRecvStatus ServerNetworkAdminSocketHandler::SendCmdLogging(ClientID clien
  * Receiving functions
  ************/
 
-DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_JOIN)
+NetworkRecvStatus ServerNetworkAdminSocketHandler::Receive_ADMIN_JOIN(Packet *p)
 {
 	if (this->status != ADMIN_STATUS_INACTIVE) return this->SendError(NETWORK_ERROR_NOT_EXPECTED);
 
@@ -534,13 +688,13 @@ DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_JOIN)
 	return this->SendProtocol();
 }
 
-DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_QUIT)
+NetworkRecvStatus ServerNetworkAdminSocketHandler::Receive_ADMIN_QUIT(Packet *p)
 {
 	/* The admin is leaving nothing else to do */
 	return this->CloseConnection();
 }
 
-DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_UPDATE_FREQUENCY)
+NetworkRecvStatus ServerNetworkAdminSocketHandler::Receive_ADMIN_UPDATE_FREQUENCY(Packet *p)
 {
 	if (this->status == ADMIN_STATUS_INACTIVE) return this->SendError(NETWORK_ERROR_NOT_EXPECTED);
 
@@ -558,7 +712,7 @@ DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_UPDATE_FREQUENCY)
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
-DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_POLL)
+NetworkRecvStatus ServerNetworkAdminSocketHandler::Receive_ADMIN_POLL(Packet *p)
 {
 	if (this->status == ADMIN_STATUS_INACTIVE) return this->SendError(NETWORK_ERROR_NOT_EXPECTED);
 
@@ -626,7 +780,7 @@ DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_POLL)
 	return NETWORK_RECV_STATUS_OKAY;
 }
 
-DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_CHAT)
+NetworkRecvStatus ServerNetworkAdminSocketHandler::Receive_ADMIN_CHAT(Packet *p)
 {
 	if (this->status == ADMIN_STATUS_INACTIVE) return this->SendError(NETWORK_ERROR_NOT_EXPECTED);
 
@@ -659,7 +813,7 @@ DEF_ADMIN_RECEIVE_COMMAND(Server, ADMIN_PACKET_ADMIN_CHAT)
 
 /**
  * Notify the admin network of a new client (if they did opt in for the respective update).
- * @param ci the client info.
+ * @param cs the client info.
  * @param new_client if this is a new client, send the respective packet too.
  */
 void NetworkAdminClientInfo(const NetworkClientSocket *cs, bool new_client)
@@ -812,6 +966,20 @@ void NetworkAdminConsole(const char *origin, const char *string)
 	}
 }
 
+/**
+ * Send GameScript JSON to the admin network (if they did opt in for the respective update).
+ * @param json The JSON data as received from the GameScript.
+ */
+void NetworkAdminGameScript(const char *json)
+{
+	ServerNetworkAdminSocketHandler *as;
+	FOR_ALL_ACTIVE_ADMIN_SOCKETS(as) {
+		if (as->update_frequency[ADMIN_UPDATE_GAMESCRIPT] & ADMIN_FREQUENCY_AUTOMATIC) {
+			as->SendGameScript(json);
+		}
+	}
+}
+
 /**
  * Distribute CommandPacket details over the admin network for logging purposes.
  * @param owner The owner of the CommandPacket (who sent us the CommandPacket).
@@ -842,7 +1010,7 @@ void ServerNetworkAdminSocketHandler::WelcomeAll()
 
 /**
  * Send (push) updates to the admin network as they have registered for these updates.
- * @param freq the frequency to be processd.
+ * @param freq the frequency to be processed.
  */
 void NetworkAdminUpdate(AdminUpdateFrequency freq)
 {
