@@ -1,4 +1,4 @@
-/* $Id: network_chat_gui.cpp 19646 2010-04-16 22:02:33Z rubidium $ */
+/* $Id: network_chat_gui.cpp 25982 2013-11-13 21:35:44Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,13 +14,10 @@
 #ifdef ENABLE_NETWORK
 
 #include "../stdafx.h"
-#include "../date_func.h"
-#include "../gfx_func.h"
 #include "../strings_func.h"
 #include "../blitter/factory.hpp"
 #include "../console_func.h"
 #include "../video/video_driver.hpp"
-#include "../table/sprites.h"
 #include "../querystring_gui.h"
 #include "../town.h"
 #include "../window_func.h"
@@ -29,34 +26,42 @@
 #include "network_client.h"
 #include "network_base.h"
 
+#include "../widgets/network_chat_widget.h"
+
 #include "table/strings.h"
 
-/* The draw buffer must be able to contain the chat message, client name and the "[All]" message,
+/** The draw buffer must be able to contain the chat message, client name and the "[All]" message,
  * some spaces and possible translations of [All] to other languages. */
 assert_compile((int)DRAW_STRING_BUFFER >= (int)NETWORK_CHAT_LENGTH + NETWORK_NAME_LENGTH + 40);
 
-enum {
-	NETWORK_CHAT_LINE_SPACING = 3,
-};
+/** Spacing between chat lines. */
+static const uint NETWORK_CHAT_LINE_SPACING = 3;
 
+/** Container for a message. */
 struct ChatMessage {
-	char message[DRAW_STRING_BUFFER];
-	TextColour colour;
-	Date end_date;
+	char message[DRAW_STRING_BUFFER]; ///< The action message.
+	TextColour colour;  ///< The colour of the message.
+	uint32 remove_time; ///< The time to remove the message.
 };
 
 /* used for chat window */
-static ChatMessage *_chatmsg_list = NULL;
-static bool _chatmessage_dirty = false;
-static bool _chatmessage_visible = false;
-static bool _chat_tab_completion_active;
-static uint MAX_CHAT_MESSAGES = 0;
+static ChatMessage *_chatmsg_list = NULL; ///< The actual chat message list.
+static bool _chatmessage_dirty = false;   ///< Does the chat message need repainting?
+static bool _chatmessage_visible = false; ///< Is a chat message visible.
+static bool _chat_tab_completion_active;  ///< Whether tab completion is active.
+static uint MAX_CHAT_MESSAGES = 0;        ///< The limit of chat messages to show.
 
-/* The chatbox grows from the bottom so the coordinates are pixels from
- * the left and pixels from the bottom. The height is the maximum height */
+/**
+ * The chatbox grows from the bottom so the coordinates are pixels from
+ * the left and pixels from the bottom. The height is the maximum height.
+ */
 static PointDimension _chatmsg_box;
-static uint8 *_chatmessage_backup = NULL;
+static uint8 *_chatmessage_backup = NULL; ///< Backup in case text is moved.
 
+/**
+ * Count the chat messages.
+ * @return The number of chat messages.
+ */
 static inline uint GetChatMessageCount()
 {
 	uint i = 0;
@@ -70,16 +75,13 @@ static inline uint GetChatMessageCount()
 /**
  * Add a text message to the 'chat window' to be shown
  * @param colour The colour this message is to be shown in
- * @param duration The duration of the chat message in game-days
+ * @param duration The duration of the chat message in seconds
  * @param message message itself in printf() style
  */
-void CDECL NetworkAddChatMessage(TextColour colour, uint8 duration, const char *message, ...)
+void CDECL NetworkAddChatMessage(TextColour colour, uint duration, const char *message, ...)
 {
 	char buf[DRAW_STRING_BUFFER];
-	const char *bufp;
 	va_list va;
-	uint msg_count;
-	uint16 lines;
 
 	va_start(va, message);
 	vsnprintf(buf, lengthof(buf), message, va);
@@ -87,43 +89,37 @@ void CDECL NetworkAddChatMessage(TextColour colour, uint8 duration, const char *
 
 	Utf8TrimString(buf, DRAW_STRING_BUFFER);
 
-	/* Force linebreaks for strings that are too long */
-	lines = GB(FormatStringLinebreaks(buf, lastof(buf), _chatmsg_box.width - 8), 0, 16) + 1;
-	if (lines >= MAX_CHAT_MESSAGES) return;
-
-	msg_count = GetChatMessageCount();
-	/* We want to add more chat messages than there is free space for, remove 'old' */
-	if (lines > MAX_CHAT_MESSAGES - msg_count) {
-		int i = lines - (MAX_CHAT_MESSAGES - msg_count);
-		memmove(&_chatmsg_list[0], &_chatmsg_list[i], sizeof(_chatmsg_list[0]) * (msg_count - i));
-		msg_count = MAX_CHAT_MESSAGES - lines;
+	uint msg_count = GetChatMessageCount();
+	if (MAX_CHAT_MESSAGES == msg_count) {
+		memmove(&_chatmsg_list[0], &_chatmsg_list[1], sizeof(_chatmsg_list[0]) * (msg_count - 1));
+		msg_count = MAX_CHAT_MESSAGES - 1;
 	}
 
-	for (bufp = buf; lines != 0; lines--) {
-		ChatMessage *cmsg = &_chatmsg_list[msg_count++];
-		strecpy(cmsg->message, bufp, lastof(cmsg->message));
-
-		/* The default colour for a message is company colour. Replace this with
-		 * white for any additional lines */
-		cmsg->colour = (bufp == buf && (colour & IS_PALETTE_COLOUR)) ? colour : TC_WHITE;
-		cmsg->end_date = _date + duration;
-
-		bufp += strlen(bufp) + 1; // jump to 'next line' in the formatted string
-	}
+	ChatMessage *cmsg = &_chatmsg_list[msg_count++];
+	strecpy(cmsg->message, buf, lastof(cmsg->message));
+	cmsg->colour = (colour & TC_IS_PALETTE_COLOUR) ? colour : TC_WHITE;
+	cmsg->remove_time = _realtime_tick + duration * 1000;
 
 	_chatmessage_dirty = true;
 }
 
+/** Initialize all font-dependent chat box sizes. */
+void NetworkReInitChatBoxSize()
+{
+	_chatmsg_box.y       = 3 * FONT_HEIGHT_NORMAL;
+	_chatmsg_box.height  = MAX_CHAT_MESSAGES * (FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING) + 2;
+	_chatmessage_backup  = ReallocT(_chatmessage_backup, _chatmsg_box.width * _chatmsg_box.height * BlitterFactoryBase::GetCurrentBlitter()->GetBytesPerPixel());
+}
+
+/** Initialize all buffers of the chat visualisation. */
 void NetworkInitChatMessage()
 {
 	MAX_CHAT_MESSAGES    = _settings_client.gui.network_chat_box_height;
 
 	_chatmsg_list        = ReallocT(_chatmsg_list, _settings_client.gui.network_chat_box_height);
 	_chatmsg_box.x       = 10;
-	_chatmsg_box.y       = 3 * FONT_HEIGHT_NORMAL;
 	_chatmsg_box.width   = _settings_client.gui.network_chat_box_width;
-	_chatmsg_box.height  = _settings_client.gui.network_chat_box_height * (FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING) + 2;
-	_chatmessage_backup  = ReallocT(_chatmessage_backup, _chatmsg_box.width * _chatmsg_box.height * BlitterFactoryBase::GetCurrentBlitter()->GetBytesPerPixel());
+	NetworkReInitChatBoxSize();
 	_chatmessage_visible = false;
 
 	for (uint i = 0; i < MAX_CHAT_MESSAGES; i++) {
@@ -177,15 +173,15 @@ void NetworkUndrawChatMessage()
 	}
 }
 
-/** Check if a message is expired every day */
-void NetworkChatMessageDailyLoop()
+/** Check if a message is expired. */
+void NetworkChatMessageLoop()
 {
 	for (uint i = 0; i < MAX_CHAT_MESSAGES; i++) {
 		ChatMessage *cmsg = &_chatmsg_list[i];
 		if (cmsg->message[0] == '\0') continue;
 
 		/* Message has expired, remove from the list */
-		if (cmsg->end_date < _date) {
+		if (cmsg->remove_time < _realtime_tick) {
 			/* Move the remaining messages over the current message */
 			if (i != MAX_CHAT_MESSAGES - 1) memmove(cmsg, cmsg + 1, sizeof(*cmsg) * (MAX_CHAT_MESSAGES - i - 1));
 
@@ -234,18 +230,27 @@ void NetworkDrawChatMessage()
 
 	_cur_dpi = &_screen; // switch to _screen painting
 
+	int string_height = 0;
+	for (uint i = 0; i < count; i++) {
+		SetDParamStr(0, _chatmsg_list[i].message);
+		string_height += GetStringLineCount(STR_JUST_RAW_STRING, width - 1) * FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING;
+	}
+
+	string_height = min(string_height, MAX_CHAT_MESSAGES * (FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING));
+
+	int top = _screen.height - _chatmsg_box.y - string_height - 2;
+	int bottom = _screen.height - _chatmsg_box.y - 2;
 	/* Paint a half-transparent box behind the chat messages */
-	GfxFillRect(
-			_chatmsg_box.x,
-			_screen.height - _chatmsg_box.y - count * (FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING) - 2,
-			_chatmsg_box.x + _chatmsg_box.width - 1,
-			_screen.height - _chatmsg_box.y - 2,
+	GfxFillRect(_chatmsg_box.x, top - 2, _chatmsg_box.x + _chatmsg_box.width - 1, bottom,
 			PALETTE_TO_TRANSPARENT, FILLRECT_RECOLOUR // black, but with some alpha for background
 		);
 
 	/* Paint the chat messages starting with the lowest at the bottom */
-	for (uint y = FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING; count-- != 0; y += (FONT_HEIGHT_NORMAL + NETWORK_CHAT_LINE_SPACING)) {
-		DrawString(_chatmsg_box.x + 3, _chatmsg_box.x + _chatmsg_box.width - 1, _screen.height - _chatmsg_box.y - y + 1, _chatmsg_list[count].message, _chatmsg_list[count].colour);
+	int ypos = bottom - 2;
+
+	for (int i = count - 1; i >= 0; i--) {
+		ypos = DrawStringMultiLine(_chatmsg_box.x + 3, _chatmsg_box.x + _chatmsg_box.width - 1, top, ypos, _chatmsg_list[i].message, _chatmsg_list[i].colour, SA_LEFT | SA_BOTTOM | SA_FORCE) - NETWORK_CHAT_LINE_SPACING;
+		if (ypos < top) break;
 	}
 
 	/* Make sure the data is updated next flush */
@@ -255,37 +260,42 @@ void NetworkDrawChatMessage()
 	_chatmessage_dirty = false;
 }
 
-
+/**
+ * Send an actual chat message.
+ * @param buf The message to send.
+ * @param type The type of destination.
+ * @param dest The actual destination index.
+ */
 static void SendChat(const char *buf, DestType type, int dest)
 {
 	if (StrEmpty(buf)) return;
 	if (!_network_server) {
-		SEND_COMMAND(PACKET_CLIENT_CHAT)((NetworkAction)(NETWORK_ACTION_CHAT + type), type, dest, buf, 0);
+		MyClient::SendChat((NetworkAction)(NETWORK_ACTION_CHAT + type), type, dest, buf, 0);
 	} else {
 		NetworkServerSendChat((NetworkAction)(NETWORK_ACTION_CHAT + type), type, dest, buf, CLIENT_ID_SERVER);
 	}
 }
 
-/** Widget numbers of the chat window. */
-enum NetWorkChatWidgets {
-	NWCW_CLOSE,
-	NWCW_BACKGROUND,
-	NWCW_DESTINATION,
-	NWCW_TEXTBOX,
-	NWCW_SENDBUTTON,
-};
+/** Window to enter the chat message in. */
+struct NetworkChatWindow : public Window {
+	DestType dtype;       ///< The type of destination.
+	StringID dest_string; ///< String representation of the destination.
+	int dest;             ///< The identifier of the destination.
+	QueryString message_editbox; ///< Message editbox.
 
-struct NetworkChatWindow : public QueryStringBaseWindow {
-	DestType dtype;
-	StringID dest_string;
-	int dest;
-
-	NetworkChatWindow(const WindowDesc *desc, DestType type, int dest) : QueryStringBaseWindow(NETWORK_CHAT_LENGTH)
+	/**
+	 * Create a chat input window.
+	 * @param desc Description of the looks of the window.
+	 * @param type The type of destination.
+	 * @param dest The actual destination index.
+	 */
+	NetworkChatWindow(const WindowDesc *desc, DestType type, int dest) : message_editbox(NETWORK_CHAT_LENGTH)
 	{
 		this->dtype   = type;
 		this->dest    = dest;
-		this->afilter = CS_ALPHANUMERAL;
-		InitializeTextBuffer(&this->text, this->edit_str_buf, this->edit_str_size, 0);
+		this->querystrings[WID_NC_TEXTBOX] = &this->message_editbox;
+		this->message_editbox.cancel_button = WID_NC_CLOSE;
+		this->message_editbox.ok_button = WID_NC_SENDBUTTON;
 
 		static const StringID chat_captions[] = {
 			STR_NETWORK_CHAT_ALL_CAPTION,
@@ -297,9 +307,11 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 
 		this->InitNested(desc, type);
 
-		this->SetFocusedWidget(NWCW_TEXTBOX);
+		this->SetFocusedWidget(WID_NC_TEXTBOX);
 		InvalidateWindowData(WC_NEWS_WINDOW, 0, this->height);
 		_chat_tab_completion_active = false;
+
+		PositionNetworkChatWindow(this);
 	}
 
 	~NetworkChatWindow()
@@ -365,9 +377,9 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 	void ChatTabCompletion()
 	{
 		static char _chat_tab_completion_buf[NETWORK_CHAT_LENGTH];
-		assert(this->edit_str_size == lengthof(_chat_tab_completion_buf));
+		assert(this->message_editbox.text.max_bytes == lengthof(_chat_tab_completion_buf));
 
-		Textbuf *tb = &this->text;
+		Textbuf *tb = &this->message_editbox.text;
 		size_t len, tb_len;
 		uint item;
 		char *tb_buf, *pre_buf;
@@ -395,11 +407,11 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 					/* If we are completing at the begin of the line, skip the ': ' we added */
 					if (tb_buf == pre_buf) {
 						offset = 0;
-						length = (tb->size - 1) - 2;
+						length = (tb->bytes - 1) - 2;
 					} else {
 						/* Else, find the place we are completing at */
 						offset = strlen(pre_buf) + 1;
-						length = (tb->size - 1) - offset;
+						length = (tb->bytes - 1) - offset;
 					}
 
 					/* Compare if we have a match */
@@ -419,14 +431,11 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 
 				/* Change to the found name. Add ': ' if we are at the start of the line (pretty) */
 				if (pre_buf == tb_buf) {
-					snprintf(tb->buf, this->edit_str_size, "%s: ", cur_name);
+					this->message_editbox.text.Print("%s: ", cur_name);
 				} else {
-					snprintf(tb->buf, this->edit_str_size, "%s %s", pre_buf, cur_name);
+					this->message_editbox.text.Print("%s %s", pre_buf, cur_name);
 				}
 
-				/* Update the textbuffer */
-				UpdateTextBufferSize(&this->text);
-
 				this->SetDirty();
 				free(pre_buf);
 				return;
@@ -434,36 +443,27 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 		}
 
 		if (second_scan) {
-			/* We walked all posibilities, and the user presses tab again.. revert to original text */
-			strcpy(tb->buf, _chat_tab_completion_buf);
+			/* We walked all possibilities, and the user presses tab again.. revert to original text */
+			this->message_editbox.text.Assign(_chat_tab_completion_buf);
 			_chat_tab_completion_active = false;
 
-			/* Update the textbuffer */
-			UpdateTextBufferSize(&this->text);
-
 			this->SetDirty();
 		}
 		free(pre_buf);
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-		this->DrawEditBox(NWCW_TEXTBOX);
-	}
-
 	virtual Point OnInitialPosition(const WindowDesc *desc, int16 sm_width, int16 sm_height, int window_number)
 	{
-		Point pt = { (_screen.width - max(sm_width, desc->default_width)) / 2, _screen.height - sm_height - FindWindowById(WC_STATUS_BAR, 0)->height };
+		Point pt = { 0, _screen.height - sm_height - FindWindowById(WC_STATUS_BAR, 0)->height };
 		return pt;
 	}
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
-		if (widget != NWCW_DESTINATION) return;
+		if (widget != WID_NC_DESTINATION) return;
 
 		if (this->dtype == DESTTYPE_CLIENT) {
-			SetDParamStr(0, NetworkFindClientInfoFromClientID((ClientID)this->dest)->client_name);
+			SetDParamStr(0, NetworkClientInfo::GetByClientID((ClientID)this->dest)->client_name);
 		}
 		Dimension d = GetStringBoundingBox(this->dest_string);
 		d.width  += WD_FRAMERECT_LEFT + WD_FRAMERECT_RIGHT;
@@ -473,10 +473,10 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
-		if (widget != NWCW_DESTINATION) return;
+		if (widget != WID_NC_DESTINATION) return;
 
 		if (this->dtype == DESTTYPE_CLIENT) {
-			SetDParamStr(0, NetworkFindClientInfoFromClientID((ClientID)this->dest)->client_name);
+			SetDParamStr(0, NetworkClientInfo::GetByClientID((ClientID)this->dest)->client_name);
 		}
 		DrawString(r.left + WD_FRAMERECT_LEFT, r.right - WD_FRAMERECT_RIGHT, r.top + WD_FRAMERECT_TOP, this->dest_string, TC_BLACK, SA_RIGHT);
 	}
@@ -485,66 +485,54 @@ struct NetworkChatWindow : public QueryStringBaseWindow {
 	{
 		switch (widget) {
 			/* Send */
-			case NWCW_SENDBUTTON: SendChat(this->text.buf, this->dtype, this->dest);
-			/* FALLTHROUGH */
-			case NWCW_CLOSE: /* Cancel */ delete this; break;
+			case WID_NC_SENDBUTTON: SendChat(this->message_editbox.text.buf, this->dtype, this->dest);
+				/* FALL THROUGH */
+			case WID_NC_CLOSE: /* Cancel */ delete this; break;
 		}
 	}
 
-	virtual void OnMouseLoop()
-	{
-		this->HandleEditBox(NWCW_TEXTBOX);
-	}
-
-	virtual EventState OnKeyPress(uint16 key, uint16 keycode)
+	virtual EventState OnKeyPress(WChar key, uint16 keycode)
 	{
 		EventState state = ES_NOT_HANDLED;
 		if (keycode == WKC_TAB) {
 			ChatTabCompletion();
 			state = ES_HANDLED;
-		} else {
-			_chat_tab_completion_active = false;
-			switch (this->HandleEditBoxKey(NWCW_TEXTBOX, key, keycode, state)) {
-				default: NOT_REACHED();
-				case HEBR_EDITING: {
-					Window *osk = FindWindowById(WC_OSK, 0);
-					if (osk != NULL && osk->parent == this) osk->InvalidateData();
-				} break;
-				case HEBR_CONFIRM:
-					SendChat(this->text.buf, this->dtype, this->dest);
-				/* FALLTHROUGH */
-				case HEBR_CANCEL: delete this; break;
-				case HEBR_NOT_FOCUSED: break;
-			}
 		}
 		return state;
 	}
 
-	virtual void OnOpenOSKWindow(int wid)
+	virtual void OnEditboxChanged(int wid)
 	{
-		ShowOnScreenKeyboard(this, wid, NWCW_CLOSE, NWCW_SENDBUTTON);
+		_chat_tab_completion_active = false;
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
 		if (data == this->dest) delete this;
 	}
 };
 
+/** The widgets of the chat window. */
 static const NWidgetPart _nested_chat_window_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_CLOSEBOX, COLOUR_GREY, NWCW_CLOSE),
-		NWidget(WWT_PANEL, COLOUR_GREY, NWCW_BACKGROUND),
+		NWidget(WWT_CLOSEBOX, COLOUR_GREY, WID_NC_CLOSE),
+		NWidget(WWT_PANEL, COLOUR_GREY, WID_NC_BACKGROUND),
 			NWidget(NWID_HORIZONTAL),
-				NWidget(WWT_TEXT, COLOUR_GREY, NWCW_DESTINATION), SetMinimalSize(62, 12), SetPadding(1, 0, 1, 0), SetDataTip(STR_NULL, STR_NULL),
-				NWidget(WWT_EDITBOX, COLOUR_GREY, NWCW_TEXTBOX), SetMinimalSize(100, 12), SetPadding(1, 0, 1, 0), SetResize(1, 0),
+				NWidget(WWT_TEXT, COLOUR_GREY, WID_NC_DESTINATION), SetMinimalSize(62, 12), SetPadding(1, 0, 1, 0), SetDataTip(STR_NULL, STR_NULL),
+				NWidget(WWT_EDITBOX, COLOUR_GREY, WID_NC_TEXTBOX), SetMinimalSize(100, 12), SetPadding(1, 0, 1, 0), SetResize(1, 0),
 																	SetDataTip(STR_NETWORK_CHAT_OSKTITLE, STR_NULL),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, NWCW_SENDBUTTON), SetMinimalSize(62, 12), SetPadding(1, 0, 1, 0), SetDataTip(STR_NETWORK_CHAT_SEND, STR_NULL),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_NC_SENDBUTTON), SetMinimalSize(62, 12), SetPadding(1, 0, 1, 0), SetDataTip(STR_NETWORK_CHAT_SEND, STR_NULL),
 			EndContainer(),
 		EndContainer(),
 	EndContainer(),
 };
 
+/** The description of the chat window. */
 static const WindowDesc _chat_window_desc(
 	WDP_MANUAL, 640, 14, // x, y, width, height
 	WC_SEND_NETWORK_MSG, WC_NONE,
@@ -552,6 +540,12 @@ static const WindowDesc _chat_window_desc(
 	_nested_chat_window_widgets, lengthof(_nested_chat_window_widgets)
 );
 
+
+/**
+ * Show the chat window.
+ * @param type The type of destination.
+ * @param dest The actual destination index.
+ */
 void ShowNetworkChatQueryWindow(DestType type, int dest)
 {
 	DeleteWindowByClass(WC_SEND_NETWORK_MSG);
