@@ -1,4 +1,4 @@
-/* $Id: network_content.cpp 20095 2010-07-08 19:59:13Z rubidium $ */
+/* $Id: network_content.cpp 25599 2013-07-13 10:13:55Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,9 +14,9 @@
 #include "../stdafx.h"
 #include "../rev.h"
 #include "../ai/ai.hpp"
+#include "../game/game.hpp"
 #include "../window_func.h"
-#include "../gui.h"
-#include "../variables.h"
+#include "../error.h"
 #include "../base_media_base.h"
 #include "../settings_type.h"
 #include "network_content.h"
@@ -28,12 +28,14 @@
 #endif
 
 extern bool HasScenario(const ContentInfo *ci, bool md5sum);
+
+/** The client we use to connect to the server. */
 ClientNetworkContentSocketHandler _network_content_client;
 
 /** Wrapper function for the HasProc */
 static bool HasGRFConfig(const ContentInfo *ci, bool md5sum)
 {
-	return FindGRFConfig(BSWAP32(ci->unique_id), md5sum ? ci->md5sum : NULL) != NULL;
+	return FindGRFConfig(BSWAP32(ci->unique_id), md5sum ? FGCM_EXACT : FGCM_ANY, md5sum ? ci->md5sum : NULL) != NULL;
 }
 
 /**
@@ -45,7 +47,7 @@ static bool HasGRFConfig(const ContentInfo *ci, bool md5sum)
  */
 typedef bool (*HasProc)(const ContentInfo *ci, bool md5sum);
 
-DEF_CONTENT_RECEIVE_COMMAND(Client, PACKET_CONTENT_SERVER_INFO)
+bool ClientNetworkContentSocketHandler::Receive_SERVER_INFO(Packet *p)
 {
 	ContentInfo *ci = new ContentInfo();
 	ci->type     = (ContentType)p->Recv_uint8();
@@ -55,7 +57,7 @@ DEF_CONTENT_RECEIVE_COMMAND(Client, PACKET_CONTENT_SERVER_INFO)
 	p->Recv_string(ci->name, lengthof(ci->name));
 	p->Recv_string(ci->version, lengthof(ci->name));
 	p->Recv_string(ci->url, lengthof(ci->url));
-	p->Recv_string(ci->description, lengthof(ci->description),  true);
+	p->Recv_string(ci->description, lengthof(ci->description), SVS_REPLACE_WITH_QUESTION_MARK | SVS_ALLOW_NEWLINE);
 
 	ci->unique_id = p->Recv_uint32();
 	for (uint j = 0; j < sizeof(ci->md5sum); j++) {
@@ -96,10 +98,21 @@ DEF_CONTENT_RECEIVE_COMMAND(Client, PACKET_CONTENT_SERVER_INFO)
 			break;
 
 		case CONTENT_TYPE_AI:
-		case CONTENT_TYPE_AI_LIBRARY:
 			proc = AI::HasAI; break;
 			break;
 
+		case CONTENT_TYPE_AI_LIBRARY:
+			proc = AI::HasAILibrary; break;
+			break;
+
+		case CONTENT_TYPE_GAME:
+			proc = Game::HasGame; break;
+			break;
+
+		case CONTENT_TYPE_GAME_LIBRARY:
+			proc = Game::HasGameLibrary; break;
+			break;
+
 		case CONTENT_TYPE_SCENARIO:
 		case CONTENT_TYPE_HEIGHTMAP:
 			proc = HasScenario;
@@ -164,6 +177,10 @@ DEF_CONTENT_RECEIVE_COMMAND(Client, PACKET_CONTENT_SERVER_INFO)
 	return true;
 }
 
+/**
+ * Request the content list for the given type.
+ * @param type The content type to request the list for.
+ */
 void ClientNetworkContentSocketHandler::RequestContentList(ContentType type)
 {
 	if (type == CONTENT_TYPE_END) {
@@ -172,10 +189,10 @@ void ClientNetworkContentSocketHandler::RequestContentList(ContentType type)
 		this->RequestContentList(CONTENT_TYPE_BASE_SOUNDS);
 		this->RequestContentList(CONTENT_TYPE_SCENARIO);
 		this->RequestContentList(CONTENT_TYPE_HEIGHTMAP);
-#ifdef ENABLE_AI
 		this->RequestContentList(CONTENT_TYPE_AI);
 		this->RequestContentList(CONTENT_TYPE_AI_LIBRARY);
-#endif /* ENABLE_AI */
+		this->RequestContentList(CONTENT_TYPE_GAME);
+		this->RequestContentList(CONTENT_TYPE_GAME_LIBRARY);
 		this->RequestContentList(CONTENT_TYPE_NEWGRF);
 		return;
 	}
@@ -186,9 +203,14 @@ void ClientNetworkContentSocketHandler::RequestContentList(ContentType type)
 	p->Send_uint8 ((byte)type);
 	p->Send_uint32(_openttd_newgrf_version);
 
-	this->Send_Packet(p);
+	this->SendPacket(p);
 }
 
+/**
+ * Request the content list for a given number of content IDs.
+ * @param count The number of IDs to request.
+ * @param content_ids The unique identifiers of the content to request information about.
+ */
 void ClientNetworkContentSocketHandler::RequestContentList(uint count, const ContentID *content_ids)
 {
 	this->Connect();
@@ -208,12 +230,17 @@ void ClientNetworkContentSocketHandler::RequestContentList(uint count, const Con
 			p->Send_uint32(content_ids[i]);
 		}
 
-		this->Send_Packet(p);
+		this->SendPacket(p);
 		count -= p_count;
 		content_ids += p_count;
 	}
 }
 
+/**
+ * Request the content list for a list of content.
+ * @param cv List with unique IDs and MD5 checksums.
+ * @param send_md5sum Whether we want a MD5 checksum matched set of files or not.
+ */
 void ClientNetworkContentSocketHandler::RequestContentList(ContentVector *cv, bool send_md5sum)
 {
 	if (cv == NULL) return;
@@ -238,7 +265,7 @@ void ClientNetworkContentSocketHandler::RequestContentList(ContentVector *cv, bo
 		}
 	}
 
-	this->Send_Packet(p);
+	this->SendPacket(p);
 
 	for (ContentIterator iter = cv->Begin(); iter != cv->End(); iter++) {
 		ContentInfo *ci = *iter;
@@ -259,6 +286,12 @@ void ClientNetworkContentSocketHandler::RequestContentList(ContentVector *cv, bo
 	}
 }
 
+/**
+ * Actually begin downloading the content we selected.
+ * @param[out] files The number of files we are going to download.
+ * @param[out] bytes The number of bytes we are going to download.
+ * @param fallback Whether to use the fallback or not.
+ */
 void ClientNetworkContentSocketHandler::DownloadSelectedContent(uint &files, uint &bytes, bool fallback)
 {
 	bytes = 0;
@@ -284,6 +317,10 @@ void ClientNetworkContentSocketHandler::DownloadSelectedContent(uint &files, uin
 	}
 }
 
+/**
+ * Initiate downloading the content over HTTP.
+ * @param content The content to download.
+ */
 void ClientNetworkContentSocketHandler::DownloadSelectedContentHTTP(const ContentIDList &content)
 {
 	uint count = content.Length();
@@ -308,6 +345,10 @@ void ClientNetworkContentSocketHandler::DownloadSelectedContentHTTP(const Conten
 	/* NetworkHTTPContentConnecter takes over freeing of content_request! */
 }
 
+/**
+ * Initiate downloading the content over the fallback protocol.
+ * @param content The content to download.
+ */
 void ClientNetworkContentSocketHandler::DownloadSelectedContentFallback(const ContentIDList &content)
 {
 	uint count = content.Length();
@@ -328,7 +369,7 @@ void ClientNetworkContentSocketHandler::DownloadSelectedContentFallback(const Co
 			p->Send_uint32(content_ids[i]);
 		}
 
-		this->Send_Packet(p);
+		this->SendPacket(p);
 		count -= p_count;
 		content_ids += p_count;
 	}
@@ -343,18 +384,8 @@ void ClientNetworkContentSocketHandler::DownloadSelectedContentFallback(const Co
  */
 static char *GetFullFilename(const ContentInfo *ci, bool compressed)
 {
-	Subdirectory dir;
-	switch (ci->type) {
-		default: return NULL;
-		case CONTENT_TYPE_BASE_GRAPHICS: dir = DATA_DIR;       break;
-		case CONTENT_TYPE_BASE_MUSIC:    dir = GM_DIR;         break;
-		case CONTENT_TYPE_BASE_SOUNDS:   dir = DATA_DIR;       break;
-		case CONTENT_TYPE_NEWGRF:        dir = DATA_DIR;       break;
-		case CONTENT_TYPE_AI:            dir = AI_DIR;         break;
-		case CONTENT_TYPE_AI_LIBRARY:    dir = AI_LIBRARY_DIR; break;
-		case CONTENT_TYPE_SCENARIO:      dir = SCENARIO_DIR;   break;
-		case CONTENT_TYPE_HEIGHTMAP:     dir = HEIGHTMAP_DIR;  break;
-	}
+	Subdirectory dir = GetContentInfoSubDir(ci->type);
+	if (dir == NO_DIRECTORY) return NULL;
 
 	static char buf[MAX_PATH];
 	FioGetFullPath(buf, lengthof(buf), SP_AUTODOWNLOAD_DIR, dir, ci->filename);
@@ -380,7 +411,7 @@ static bool GunzipFile(const ContentInfo *ci)
 		ret = false;
 	} else {
 		byte buff[8192];
-		while (1) {
+		for (;;) {
 			int read = gzread(fin, buff, sizeof(buff));
 			if (read == 0) {
 				/* If gzread() returns 0, either the end-of-file has been
@@ -426,7 +457,7 @@ static bool GunzipFile(const ContentInfo *ci)
 #endif /* defined(WITH_ZLIB) */
 }
 
-DEF_CONTENT_RECEIVE_COMMAND(Client, PACKET_CONTENT_SERVER_CONTENT)
+bool ClientNetworkContentSocketHandler::Receive_SERVER_CONTENT(Packet *p)
 {
 	if (this->curFile == NULL) {
 		delete this->curInfo;
@@ -445,8 +476,8 @@ DEF_CONTENT_RECEIVE_COMMAND(Client, PACKET_CONTENT_SERVER_CONTENT)
 		/* We have a file opened, thus are downloading internal content */
 		size_t toRead = (size_t)(p->size - p->pos);
 		if (fwrite(p->buffer + p->pos, 1, toRead, this->curFile) != toRead) {
-			DeleteWindowById(WC_NETWORK_STATUS_WINDOW, 0);
-			ShowErrorMessage(STR_CONTENT_ERROR_COULD_NOT_DOWNLOAD_FILE_NOT_WRITABLE, STR_CONTENT_ERROR_COULD_NOT_DOWNLOAD, 0, 0);
+			DeleteWindowById(WC_NETWORK_STATUS_WINDOW, WN_NETWORK_STATUS_WINDOW_CONTENT_DOWNLOAD);
+			ShowErrorMessage(STR_CONTENT_ERROR_COULD_NOT_DOWNLOAD, STR_CONTENT_ERROR_COULD_NOT_DOWNLOAD_FILE_NOT_WRITABLE, WL_ERROR);
 			this->Close();
 			fclose(this->curFile);
 			this->curFile = NULL;
@@ -454,7 +485,7 @@ DEF_CONTENT_RECEIVE_COMMAND(Client, PACKET_CONTENT_SERVER_CONTENT)
 			return false;
 		}
 
-		this->OnDownloadProgress(this->curInfo, (uint)toRead);
+		this->OnDownloadProgress(this->curInfo, (int)toRead);
 
 		if (toRead == 0) this->AfterDownload();
 	}
@@ -477,14 +508,12 @@ bool ClientNetworkContentSocketHandler::BeforeDownload()
 	if (this->curInfo->filesize != 0) {
 		/* The filesize is > 0, so we are going to download it */
 		const char *filename = GetFullFilename(this->curInfo, true);
-		if (filename == NULL) {
-			/* Unless that fails ofcourse... */
-			DeleteWindowById(WC_NETWORK_STATUS_WINDOW, 0);
-			ShowErrorMessage(STR_CONTENT_ERROR_COULD_NOT_DOWNLOAD_FILE_NOT_WRITABLE, STR_CONTENT_ERROR_COULD_NOT_DOWNLOAD, 0, 0);
+		if (filename == NULL || (this->curFile = fopen(filename, "wb")) == NULL) {
+			/* Unless that fails of course... */
+			DeleteWindowById(WC_NETWORK_STATUS_WINDOW, WN_NETWORK_STATUS_WINDOW_CONTENT_DOWNLOAD);
+			ShowErrorMessage(STR_CONTENT_ERROR_COULD_NOT_DOWNLOAD, STR_CONTENT_ERROR_COULD_NOT_DOWNLOAD_FILE_NOT_WRITABLE, WL_ERROR);
 			return false;
 		}
-
-		this->curFile = fopen(filename, "wb");
 	}
 	return true;
 }
@@ -503,18 +532,21 @@ void ClientNetworkContentSocketHandler::AfterDownload()
 	if (GunzipFile(this->curInfo)) {
 		unlink(GetFullFilename(this->curInfo, true));
 
+		Subdirectory sd = GetContentInfoSubDir(this->curInfo->type);
+		if (sd == NO_DIRECTORY) NOT_REACHED();
+
 		TarScanner ts;
-		ts.AddFile(GetFullFilename(this->curInfo, false), 0);
+		ts.AddFile(sd, GetFullFilename(this->curInfo, false));
 
 		if (this->curInfo->type == CONTENT_TYPE_BASE_MUSIC) {
 			/* Music can't be in a tar. So extract the tar! */
-			ExtractTar(GetFullFilename(this->curInfo, false));
+			ExtractTar(GetFullFilename(this->curInfo, false), BASESET_DIR);
 			unlink(GetFullFilename(this->curInfo, false));
 		}
 
 		this->OnDownloadComplete(this->curInfo->id);
 	} else {
-		ShowErrorMessage(STR_CONTENT_ERROR_COULD_NOT_EXTRACT, INVALID_STRING_ID, 0, 0);
+		ShowErrorMessage(STR_CONTENT_ERROR_COULD_NOT_EXTRACT, INVALID_STRING_ID, WL_ERROR);
 	}
 }
 
@@ -529,6 +561,10 @@ void ClientNetworkContentSocketHandler::OnFailure()
 	this->http_response_index = -2;
 
 	if (this->curFile != NULL) {
+		/* Revert the download progress when we are going for the old system. */
+		long size = ftell(this->curFile);
+		if (size > 0) this->OnDownloadProgress(this->curInfo, (int)-size);
+
 		fclose(this->curFile);
 		this->curFile = NULL;
 	}
@@ -562,7 +598,7 @@ void ClientNetworkContentSocketHandler::OnReceiveData(const char *data, size_t l
 			this->OnFailure();
 		} else {
 			/* Just received the data. */
-			this->OnDownloadProgress(this->curInfo, (uint)length);
+			this->OnDownloadProgress(this->curInfo, (int)length);
 		}
 		/* Nothing more to do now. */
 		return;
@@ -660,9 +696,7 @@ void ClientNetworkContentSocketHandler::OnReceiveData(const char *data, size_t l
 }
 
 /**
- * Create a socket handler with the given socket and (server) address.
- * @param s the socket to communicate over
- * @param sin the IP/port of the server
+ * Create a socket handler to handle the connection.
  */
 ClientNetworkContentSocketHandler::ClientNetworkContentSocketHandler() :
 	NetworkContentSocketHandler(),
@@ -682,8 +716,13 @@ ClientNetworkContentSocketHandler::~ClientNetworkContentSocketHandler()
 	for (ContentIterator iter = this->infos.Begin(); iter != this->infos.End(); iter++) delete *iter;
 }
 
+/** Connect to the content server. */
 class NetworkContentConnecter : TCPConnecter {
 public:
+	/**
+	 * Initiate the connecting.
+	 * @param address The address of the server.
+	 */
 	NetworkContentConnecter(const NetworkAddress &address) : TCPConnecter(address) {}
 
 	virtual void OnFailure()
@@ -738,28 +777,14 @@ void ClientNetworkContentSocketHandler::SendReceive()
 		return;
 	}
 
-	fd_set read_fd, write_fd;
-	struct timeval tv;
-
-	FD_ZERO(&read_fd);
-	FD_ZERO(&write_fd);
-
-	FD_SET(this->sock, &read_fd);
-	FD_SET(this->sock, &write_fd);
-
-	tv.tv_sec = tv.tv_usec = 0; // don't block at all.
-#if !defined(__MORPHOS__) && !defined(__AMIGA__)
-	select(FD_SETSIZE, &read_fd, &write_fd, NULL, &tv);
-#else
-	WaitSelect(FD_SETSIZE, &read_fd, &write_fd, NULL, &tv, NULL);
-#endif
-	if (FD_ISSET(this->sock, &read_fd)) {
-		this->Recv_Packets();
-		this->lastActivity = _realtime_tick;
+	if (this->CanSendReceive()) {
+		if (this->ReceivePackets()) {
+			/* Only update activity once a packet is received, instead of everytime we try it. */
+			this->lastActivity = _realtime_tick;
+		}
 	}
 
-	this->writable = !!FD_ISSET(this->sock, &write_fd);
-	this->Send_Packets();
+	this->SendPackets();
 }
 
 /**
@@ -850,7 +875,7 @@ void ClientNetworkContentSocketHandler::UnselectAll()
 	}
 }
 
-/** Toggle the state of a content info and check it's dependencies */
+/** Toggle the state of a content info and check its dependencies */
 void ClientNetworkContentSocketHandler::ToggleSelectedState(const ContentInfo *ci)
 {
 	switch (ci->state) {
@@ -897,10 +922,13 @@ void ClientNetworkContentSocketHandler::ReverseLookupTreeDependency(ConstContent
 {
 	*tree.Append() = child;
 
-	/* First find all direct parents */
-	for (ConstContentIterator iter = tree.Begin(); iter != tree.End(); iter++) {
+	/* First find all direct parents. We can't use the "normal" iterator as
+	 * we are including stuff into the vector and as such the vector's data
+	 * store can be reallocated (and thus move), which means out iterating
+	 * pointer gets invalid. So fall back to the indices. */
+	for (uint i = 0; i < tree.Length(); i++) {
 		ConstContentVector parents;
-		this->ReverseLookupDependency(parents, *iter);
+		this->ReverseLookupDependency(parents, tree[i]);
 
 		for (ConstContentIterator piter = parents.Begin(); piter != parents.End(); piter++) {
 			tree.Include(*piter);
@@ -989,7 +1017,7 @@ void ClientNetworkContentSocketHandler::CheckDependencyState(ContentInfo *ci)
 
 		/* Nothing depends on us, mark the whole graph as unselected.
 		 * After that's done run over them once again to test their children
-		 * to unselect. Don't do it immediatelly because it'll do exactly what
+		 * to unselect. Don't do it immediately because it'll do exactly what
 		 * we're doing now. */
 		for (ConstContentIterator iter = parents.Begin(); iter != parents.End(); iter++) {
 			const ContentInfo *c = *iter;
@@ -1001,6 +1029,7 @@ void ClientNetworkContentSocketHandler::CheckDependencyState(ContentInfo *ci)
 	}
 }
 
+/** Clear all downloaded content information. */
 void ClientNetworkContentSocketHandler::Clear()
 {
 	for (ContentIterator iter = this->infos.Begin(); iter != this->infos.End(); iter++) delete *iter;
@@ -1038,7 +1067,7 @@ void ClientNetworkContentSocketHandler::OnReceiveContentInfo(const ContentInfo *
 	}
 }
 
-void ClientNetworkContentSocketHandler::OnDownloadProgress(const ContentInfo *ci, uint bytes)
+void ClientNetworkContentSocketHandler::OnDownloadProgress(const ContentInfo *ci, int bytes)
 {
 	for (ContentCallback **iter = this->callbacks.Begin(); iter != this->callbacks.End(); /* nothing */) {
 		ContentCallback *cb = *iter;
