@@ -1,4 +1,4 @@
-/* $Id: newgrf_debug_gui.cpp 22252 2011-03-14 19:03:17Z rubidium $ */
+/* $Id: newgrf_debug_gui.cpp 25989 2013-11-13 21:58:36Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -32,9 +32,15 @@
 #include "newgrf_spritegroup.h"
 #include "newgrf_station.h"
 #include "newgrf_town.h"
+#include "newgrf_railtype.h"
+#include "newgrf_industries.h"
+#include "newgrf_industrytiles.h"
+
+#include "widgets/newgrf_debug_widget.h"
 
 #include "table/strings.h"
 
+/** The sprite picker. */
 NewGrfDebugSpritePicker _newgrf_debug_sprite_picker = { SPM_NONE, NULL, 0, SmallVector<SpriteID, 256>() };
 
 /**
@@ -62,7 +68,7 @@ static inline uint GetInspectWindowNumber(GrfSpecFeature feature, uint index)
 
 /**
  * The type of a property to show. This is used to
- * provide an appropriate represenation in the GUI.
+ * provide an appropriate representation in the GUI.
  */
 enum NIType {
 	NIT_INT,   ///< The property is a simple integer
@@ -139,6 +145,13 @@ public:
 	 */
 	virtual void SetStringParameters(uint index) const = 0;
 
+	/**
+	 * Get the GRFID of the file that includes this item.
+	 * @param index index to check.
+	 * @return GRFID of the item. 0 means that the item is not inspectable.
+	 */
+	virtual uint32 GetGRFID(uint index) const = 0;
+
 	/**
 	 * Resolve (action2) variable for a given index.
 	 * @param index The (instance) index to resolve the variable for.
@@ -147,23 +160,40 @@ public:
 	 * @param avail Return whether the variable is available.
 	 * @return The resolved variable's value.
 	 */
-	virtual uint Resolve(uint index, uint var, uint param, bool *avail) const
+	virtual uint Resolve(uint index, uint var, uint param, bool *avail) const = 0;
+
+	/**
+	 * Used to decide if the PSA needs a parameter or not.
+	 * @return True iff this item has a PSA that requires a parameter.
+	 */
+	virtual bool PSAWithParameter() const
 	{
-		ResolverObject ro;
-		memset(&ro, 0, sizeof(ro));
-		this->Resolve(&ro, index);
-		return ro.GetVariable(&ro, var, param, avail);
+		return false;
+	}
+
+	/**
+	 * Allows to know the size of the persistent storage.
+	 * @param index Index of the item.
+	 * @param grfid Parameter for the PSA. Only required for items with parameters.
+	 * @return Size of the persistent storage in indices.
+	 */
+	virtual uint GetPSASize(uint index, uint32 grfid) const
+	{
+		return 0;
+	}
+
+	/**
+	 * Gets the first position of the array containing the persistent storage.
+	 * @param index Index of the item.
+	 * @param grfid Parameter for the PSA. Only required for items with parameters.
+	 * @return Pointer to the first position of the storage array or NULL if not present.
+	 */
+	virtual const int32 *GetPSAFirstPosition(uint index, uint32 grfid) const
+	{
+		return NULL;
 	}
 
 protected:
-	/**
-	 * Actually execute the real resolving for a given (instance) index.
-	 * @param ro    The resolver object to fill with everything
-	 *              needed to be able to resolve a variable.
-	 * @param index The (instance) index of the to-be-resolved variable.
-	 */
-	virtual void Resolve(ResolverObject *ro, uint index) const {}
-
 	/**
 	 * Helper to make setting the strings easier.
 	 * @param string the string to actually draw.
@@ -198,8 +228,6 @@ struct NIFeature {
 	const NICallback *callbacks;  ///< The callbacks associated with this feature.
 	const NIVariable *variables;  ///< The variables associated with this feature.
 	const NIHelper   *helper;     ///< The class container all helper functions.
-	uint psa_size;                ///< The size of the persistent storage in indices.
-	size_t psa_offset;            ///< Offset to the array in the PSA.
 };
 
 /* Load all the NewGRF debug data; externalised as it is just a huge bunch of tables. */
@@ -237,15 +265,6 @@ static inline const NIHelper *GetFeatureHelper(uint window_number)
 	return GetFeature(window_number)->helper;
 }
 
-
-/** Widget numbers of settings window */
-enum NewGRFInspectWidgets {
-	NIW_CAPTION,   ///< The caption bar ofcourse
-	NIW_PARENT,    ///< Inspect the parent
-	NIW_MAINPANEL, ///< Panel widget containing the actual data
-	NIW_SCROLLBAR, ///< Scrollbar
-};
-
 /** Window used for inspecting NewGRFs. */
 struct NewGRFInspectWindow : Window {
 	static const int LEFT_OFFSET   = 5; ///< Position of left edge
@@ -254,9 +273,12 @@ struct NewGRFInspectWindow : Window {
 	static const int BOTTOM_OFFSET = 5; ///< Position of bottom edge
 
 	/** The value for the variable 60 parameters. */
-	static byte var60params[GSF_FAKE_END][0x20];
+	static uint32 var60params[GSF_FAKE_END][0x20];
 
-	/** The currently editted parameter, to update the right one. */
+	/** GRFID of the caller of this window, 0 if it has no caller. */
+	uint32 caller_grfid;
+
+	/** The currently edited parameter, to update the right one. */
 	byte current_edit_param;
 
 	Scrollbar *vscroll;
@@ -271,26 +293,36 @@ struct NewGRFInspectWindow : Window {
 		return IsInsideBS(variable, 0x60, 0x20);
 	}
 
+	/**
+	 * Set the GRFID of the item opening this window.
+	 * @param grfid GRFID of the item opening this window, or 0 if not opened by other window.
+	 */
+	void SetCallerGRFID(uint32 grfid)
+	{
+		this->caller_grfid = grfid;
+		this->SetDirty();
+	}
+
 	NewGRFInspectWindow(const WindowDesc *desc, WindowNumber wno) : Window()
 	{
 		this->CreateNestedTree(desc);
-		this->vscroll = this->GetScrollbar(NIW_SCROLLBAR);
+		this->vscroll = this->GetScrollbar(WID_NGRFI_SCROLLBAR);
 		this->FinishInitNested(desc, wno);
 
 		this->vscroll->SetCount(0);
-		this->SetWidgetDisabledState(NIW_PARENT, GetFeatureHelper(this->window_number)->GetParent(GetFeatureIndex(this->window_number)) == UINT32_MAX);
+		this->SetWidgetDisabledState(WID_NGRFI_PARENT, GetFeatureHelper(this->window_number)->GetParent(GetFeatureIndex(this->window_number)) == UINT32_MAX);
 	}
 
 	virtual void SetStringParameters(int widget) const
 	{
-		if (widget != NIW_CAPTION) return;
+		if (widget != WID_NGRFI_CAPTION) return;
 
 		GetFeatureHelper(this->window_number)->SetStringParameters(GetFeatureIndex(this->window_number));
 	}
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
-		if (widget != NIW_MAINPANEL) return;
+		if (widget != WID_NGRFI_MAINPANEL) return;
 
 		resize->height = max(11, FONT_HEIGHT_NORMAL + 1);
 		resize->width  = 1;
@@ -316,12 +348,12 @@ struct NewGRFInspectWindow : Window {
 		offset -= this->vscroll->GetPosition();
 		if (offset < 0 || offset >= this->vscroll->GetCapacity()) return;
 
-		::DrawString(r.left + LEFT_OFFSET, r.right + RIGHT_OFFSET, r.top + TOP_OFFSET + (offset * this->resize.step_height), buf, TC_BLACK);
+		::DrawString(r.left + LEFT_OFFSET, r.right - RIGHT_OFFSET, r.top + TOP_OFFSET + (offset * this->resize.step_height), buf, TC_BLACK);
 	}
 
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
-		if (widget != NIW_MAINPANEL) return;
+		if (widget != WID_NGRFI_MAINPANEL) return;
 
 		uint index = GetFeatureIndex(this->window_number);
 		const NIFeature *nif  = GetFeature(this->window_number);
@@ -347,11 +379,16 @@ struct NewGRFInspectWindow : Window {
 			}
 		}
 
-		if (nif->psa_size != 0) {
-			this->DrawString(r, i++, "Persistent storage:");
-			assert(nif->psa_size % 4 == 0);
-			int32 *psa = (int32*)((byte*)base + nif->psa_offset);
-			for (uint j = 0; j < nif->psa_size; j += 4, psa += 4) {
+		uint psa_size = nih->GetPSASize(index, this->caller_grfid);
+		const int32 *psa = nih->GetPSAFirstPosition(index, this->caller_grfid);
+		if (psa_size != 0 && psa != NULL) {
+			if (nih->PSAWithParameter()) {
+				this->DrawString(r, i++, "Persistent storage [%08X]:", BSWAP32(this->caller_grfid));
+			} else {
+				this->DrawString(r, i++, "Persistent storage:");
+			}
+			assert(psa_size % 4 == 0);
+			for (uint j = 0; j < psa_size; j += 4, psa += 4) {
 				this->DrawString(r, i++, "  %i: %i %i %i %i", j, psa[0], psa[1], psa[2], psa[3]);
 			}
 		}
@@ -359,12 +396,12 @@ struct NewGRFInspectWindow : Window {
 		if (nif->properties != NULL) {
 			this->DrawString(r, i++, "Properties:");
 			for (const NIProperty *nip = nif->properties; nip->name != NULL; nip++) {
-				void *ptr = (byte*)base + nip->offset;
+				const void *ptr = (const byte *)base + nip->offset;
 				uint value;
 				switch (nip->read_size) {
-					case 1: value = *(uint8  *)ptr; break;
-					case 2: value = *(uint16 *)ptr; break;
-					case 4: value = *(uint32 *)ptr; break;
+					case 1: value = *(const uint8  *)ptr; break;
+					case 2: value = *(const uint16 *)ptr; break;
+					case 4: value = *(const uint32 *)ptr; break;
 					default: NOT_REACHED();
 				}
 
@@ -393,12 +430,12 @@ struct NewGRFInspectWindow : Window {
 			this->DrawString(r, i++, "Callbacks:");
 			for (const NICallback *nic = nif->callbacks; nic->name != NULL; nic++) {
 				if (nic->cb_bit != CBM_NO_BIT) {
-					void *ptr = (byte*)base_spec + nic->offset;
+					const void *ptr = (const byte *)base_spec + nic->offset;
 					uint value;
 					switch (nic->read_size) {
-						case 1: value = *(uint8  *)ptr; break;
-						case 2: value = *(uint16 *)ptr; break;
-						case 4: value = *(uint32 *)ptr; break;
+						case 1: value = *(const uint8  *)ptr; break;
+						case 2: value = *(const uint16 *)ptr; break;
+						case 4: value = *(const uint32 *)ptr; break;
 						default: NOT_REACHED();
 					}
 
@@ -419,19 +456,20 @@ struct NewGRFInspectWindow : Window {
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		switch (widget) {
-			case NIW_PARENT: {
-				uint index = GetFeatureHelper(this->window_number)->GetParent(GetFeatureIndex(this->window_number));
-				::ShowNewGRFInspectWindow((GrfSpecFeature)GB(index, 24, 8), GetFeatureIndex(index));
+			case WID_NGRFI_PARENT: {
+				const NIHelper *nih   = GetFeatureHelper(this->window_number);
+				uint index = nih->GetParent(GetFeatureIndex(this->window_number));
+				::ShowNewGRFInspectWindow((GrfSpecFeature)GB(index, 24, 8), GetFeatureIndex(index), nih->GetGRFID(GetFeatureIndex(this->window_number)));
 				break;
 			}
 
-			case NIW_MAINPANEL: {
+			case WID_NGRFI_MAINPANEL: {
 				/* Does this feature have variables? */
 				const NIFeature *nif  = GetFeature(this->window_number);
 				if (nif->variables == NULL) return;
 
 				/* Get the line, make sure it's within the boundaries. */
-				int line = this->vscroll->GetScrolledRowFromWidget(pt.y, this, NIW_MAINPANEL, TOP_OFFSET);
+				int line = this->vscroll->GetScrolledRowFromWidget(pt.y, this, WID_NGRFI_MAINPANEL, TOP_OFFSET);
 				if (line == INT_MAX) return;
 
 				/* Find the variable related to the line */
@@ -441,7 +479,7 @@ struct NewGRFInspectWindow : Window {
 					if (!HasVariableParameter(niv->var)) break;
 
 					this->current_edit_param = niv->var;
-					ShowQueryString(STR_EMPTY, STR_NEWGRF_INSPECT_QUERY_CAPTION, 3, 100, this, CS_HEXADECIMAL, QSF_NONE);
+					ShowQueryString(STR_EMPTY, STR_NEWGRF_INSPECT_QUERY_CAPTION, 9, this, CS_HEXADECIMAL, QSF_NONE);
 				}
 			}
 		}
@@ -457,24 +495,24 @@ struct NewGRFInspectWindow : Window {
 
 	virtual void OnResize()
 	{
-		this->vscroll->SetCapacityFromWidget(this, NIW_MAINPANEL, TOP_OFFSET + BOTTOM_OFFSET);
+		this->vscroll->SetCapacityFromWidget(this, WID_NGRFI_MAINPANEL, TOP_OFFSET + BOTTOM_OFFSET);
 	}
 };
 
-/* static */ byte NewGRFInspectWindow::var60params[GSF_FAKE_END][0x20] = { {0} }; // Use spec to have 0s in whole array
+/* static */ uint32 NewGRFInspectWindow::var60params[GSF_FAKE_END][0x20] = { {0} }; // Use spec to have 0s in whole array
 
 static const NWidgetPart _nested_newgrf_inspect_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_GREY),
-		NWidget(WWT_CAPTION, COLOUR_GREY, NIW_CAPTION), SetDataTip(STR_NEWGRF_INSPECT_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
-		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, NIW_PARENT), SetDataTip(STR_NEWGRF_INSPECT_PARENT_BUTTON, STR_NEWGRF_INSPECT_PARENT_TOOLTIP),
+		NWidget(WWT_CAPTION, COLOUR_GREY, WID_NGRFI_CAPTION), SetDataTip(STR_NEWGRF_INSPECT_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
+		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_NGRFI_PARENT), SetDataTip(STR_NEWGRF_INSPECT_PARENT_BUTTON, STR_NEWGRF_INSPECT_PARENT_TOOLTIP),
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_GREY, NIW_MAINPANEL), SetMinimalSize(300, 0), SetScrollbar(NIW_SCROLLBAR), EndContainer(),
+		NWidget(WWT_PANEL, COLOUR_GREY, WID_NGRFI_MAINPANEL), SetMinimalSize(300, 0), SetScrollbar(WID_NGRFI_SCROLLBAR), EndContainer(),
 		NWidget(NWID_VERTICAL),
-			NWidget(NWID_VSCROLLBAR, COLOUR_GREY, NIW_SCROLLBAR),
+			NWidget(NWID_VSCROLLBAR, COLOUR_GREY, WID_NGRFI_SCROLLBAR),
 			NWidget(WWT_RESIZEBOX, COLOUR_GREY),
 		EndContainer(),
 	EndContainer(),
@@ -483,7 +521,7 @@ static const NWidgetPart _nested_newgrf_inspect_widgets[] = {
 static const WindowDesc _newgrf_inspect_desc(
 	WDP_AUTO, 400, 300,
 	WC_NEWGRF_INSPECT, WC_NONE,
-	WDF_UNCLICK_BUTTONS,
+	0,
 	_nested_newgrf_inspect_widgets, lengthof(_nested_newgrf_inspect_widgets)
 );
 
@@ -494,13 +532,16 @@ static const WindowDesc _newgrf_inspect_desc(
  * we want to inspect.
  * @param feature The feature we want to inspect.
  * @param index   The index/identifier of the feature to inspect.
+ * @param grfid   GRFID of the item opening this window, or 0 if not opened by other window.
  */
-void ShowNewGRFInspectWindow(GrfSpecFeature feature, uint index)
+void ShowNewGRFInspectWindow(GrfSpecFeature feature, uint index, const uint32 grfid)
 {
 	if (!IsNewGRFInspectable(feature, index)) return;
 
 	WindowNumber wno = GetInspectWindowNumber(feature, index);
-	AllocateWindowDescFront<NewGRFInspectWindow>(&_newgrf_inspect_desc, wno);
+	NewGRFInspectWindow *w = AllocateWindowDescFront<NewGRFInspectWindow>(&_newgrf_inspect_desc, wno);
+	if (w == NULL) w = (NewGRFInspectWindow *)FindWindowById(WC_NEWGRF_INSPECT, wno);
+	w->SetCallerGRFID(grfid);
 }
 
 /**
@@ -584,23 +625,6 @@ GrfSpecFeature GetGrfSpecFeature(VehicleType type)
 
 /**** Sprite Aligner ****/
 
-/** Widgets we want (some) influence over. */
-enum SpriteAlignerWidgets {
-	SAW_CAPTION,  ///< Caption of the window
-	SAW_PREVIOUS, ///< Skip to the previous sprite
-	SAW_GOTO,     ///< Go to a given sprite
-	SAW_NEXT,     ///< Skip to the next sprite
-	SAW_UP,       ///< Move the sprite up
-	SAW_LEFT,     ///< Move the sprite to the left
-	SAW_RIGHT,    ///< Move the sprite to the right
-	SAW_DOWN,     ///< Move the sprite down
-	SAW_SPRITE,   ///< The actual sprite
-	SAW_OFFSETS,  ///< The sprite offsets
-	SAW_PICKER,   ///< Sprite picker
-	SAW_LIST,     ///< Queried sprite list
-	SAW_SCROLLBAR,///< Scrollbar for sprite list
-};
-
 /** Window used for aligning sprites. */
 struct SpriteAlignerWindow : Window {
 	SpriteID current_sprite; ///< The currently shown sprite
@@ -609,7 +633,7 @@ struct SpriteAlignerWindow : Window {
 	SpriteAlignerWindow(const WindowDesc *desc, WindowNumber wno) : Window()
 	{
 		this->CreateNestedTree(desc);
-		this->vscroll = this->GetScrollbar(SAW_SCROLLBAR);
+		this->vscroll = this->GetScrollbar(WID_SA_SCROLLBAR);
 		this->FinishInitNested(desc, wno);
 
 		/* Oh yes, we assume there is at least one normal sprite! */
@@ -619,15 +643,15 @@ struct SpriteAlignerWindow : Window {
 	virtual void SetStringParameters(int widget) const
 	{
 		switch (widget) {
-			case SAW_CAPTION:
+			case WID_SA_CAPTION:
 				SetDParam(0, this->current_sprite);
 				SetDParamStr(1, FioGetFilename(GetOriginFileSlot(this->current_sprite)));
 				break;
 
-			case SAW_OFFSETS: {
+			case WID_SA_OFFSETS: {
 				const Sprite *spr = GetSprite(this->current_sprite, ST_NORMAL);
-				SetDParam(0, spr->x_offs);
-				SetDParam(1, spr->y_offs);
+				SetDParam(0, spr->x_offs / ZOOM_LVL_BASE);
+				SetDParam(1, spr->y_offs / ZOOM_LVL_BASE);
 				break;
 			}
 
@@ -638,7 +662,7 @@ struct SpriteAlignerWindow : Window {
 
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
-		if (widget != SAW_LIST) return;
+		if (widget != WID_SA_LIST) return;
 
 		resize->height = max(11, FONT_HEIGHT_NORMAL + 1);
 		resize->width  = 1;
@@ -650,27 +674,27 @@ struct SpriteAlignerWindow : Window {
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
 		switch (widget) {
-			case SAW_SPRITE: {
+			case WID_SA_SPRITE: {
 				/* Center the sprite ourselves */
 				const Sprite *spr = GetSprite(this->current_sprite, ST_NORMAL);
 				int width  = r.right  - r.left + 1;
 				int height = r.bottom - r.top  + 1;
-				int x = r.left - spr->x_offs + (width  - spr->width) / 2;
-				int y = r.top  - spr->y_offs + (height - spr->height) / 2;
+				int x = r.left - spr->x_offs / ZOOM_LVL_BASE + (width  - spr->width / ZOOM_LVL_BASE) / 2;
+				int y = r.top  - spr->y_offs / ZOOM_LVL_BASE + (height - spr->height / ZOOM_LVL_BASE) / 2;
 
 				/* And draw only the part within the sprite area */
 				SubSprite subspr = {
-					spr->x_offs + (spr->width  - width)  / 2 + 1,
-					spr->y_offs + (spr->height - height) / 2 + 1,
-					spr->x_offs + (spr->width  + width)  / 2 - 1,
-					spr->y_offs + (spr->height + height) / 2 - 1,
+					spr->x_offs + (spr->width  - width  * ZOOM_LVL_BASE) / 2 + 1,
+					spr->y_offs + (spr->height - height * ZOOM_LVL_BASE) / 2 + 1,
+					spr->x_offs + (spr->width  + width  * ZOOM_LVL_BASE) / 2 - 1,
+					spr->y_offs + (spr->height + height * ZOOM_LVL_BASE) / 2 - 1,
 				};
 
-				DrawSprite(this->current_sprite, PAL_NONE, x, y, &subspr);
+				DrawSprite(this->current_sprite, PAL_NONE, x, y, &subspr, ZOOM_LVL_GUI);
 				break;
 			}
 
-			case SAW_LIST: {
+			case WID_SA_LIST: {
 				const NWidgetBase *nwid = this->GetWidget<NWidgetBase>(widget);
 				int step_size = nwid->resize_y;
 
@@ -691,31 +715,31 @@ struct SpriteAlignerWindow : Window {
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
 		switch (widget) {
-			case SAW_PREVIOUS:
+			case WID_SA_PREVIOUS:
 				do {
 					this->current_sprite = (this->current_sprite == 0 ? GetMaxSpriteID() :  this->current_sprite) - 1;
 				} while (GetSpriteType(this->current_sprite) != ST_NORMAL);
 				this->SetDirty();
 				break;
 
-			case SAW_GOTO:
-				ShowQueryString(STR_EMPTY, STR_SPRITE_ALIGNER_GOTO_CAPTION, 7, 150, this, CS_NUMERAL, QSF_NONE);
+			case WID_SA_GOTO:
+				ShowQueryString(STR_EMPTY, STR_SPRITE_ALIGNER_GOTO_CAPTION, 7, this, CS_NUMERAL, QSF_NONE);
 				break;
 
-			case SAW_NEXT:
+			case WID_SA_NEXT:
 				do {
 					this->current_sprite = (this->current_sprite + 1) % GetMaxSpriteID();
 				} while (GetSpriteType(this->current_sprite) != ST_NORMAL);
 				this->SetDirty();
 				break;
 
-			case SAW_PICKER:
-				this->LowerWidget(SAW_PICKER);
+			case WID_SA_PICKER:
+				this->LowerWidget(WID_SA_PICKER);
 				_newgrf_debug_sprite_picker.mode = SPM_WAIT_CLICK;
 				this->SetDirty();
 				break;
 
-			case SAW_LIST: {
+			case WID_SA_LIST: {
 				const NWidgetBase *nwid = this->GetWidget<NWidgetBase>(widget);
 				int step_size = nwid->resize_y;
 
@@ -728,10 +752,10 @@ struct SpriteAlignerWindow : Window {
 				break;
 			}
 
-			case SAW_UP:
-			case SAW_DOWN:
-			case SAW_LEFT:
-			case SAW_RIGHT: {
+			case WID_SA_UP:
+			case WID_SA_DOWN:
+			case WID_SA_LEFT:
+			case WID_SA_RIGHT: {
 				/*
 				 * Yes... this is a hack.
 				 *
@@ -747,12 +771,12 @@ struct SpriteAlignerWindow : Window {
 				 */
 				Sprite *spr = const_cast<Sprite *>(GetSprite(this->current_sprite, ST_NORMAL));
 				switch (widget) {
-					case SAW_UP:    spr->y_offs--; break;
-					case SAW_DOWN:  spr->y_offs++; break;
-					case SAW_LEFT:  spr->x_offs--; break;
-					case SAW_RIGHT: spr->x_offs++; break;
+					case WID_SA_UP:    spr->y_offs -= ZOOM_LVL_BASE; break;
+					case WID_SA_DOWN:  spr->y_offs += ZOOM_LVL_BASE; break;
+					case WID_SA_LEFT:  spr->x_offs -= ZOOM_LVL_BASE; break;
+					case WID_SA_RIGHT: spr->x_offs += ZOOM_LVL_BASE; break;
 				}
-				/* Ofcourse, we need to redraw the sprite, but where is it used?
+				/* Of course, we need to redraw the sprite, but where is it used?
 				 * Everywhere is a safe bet. */
 				MarkWholeScreenDirty();
 				break;
@@ -782,22 +806,22 @@ struct SpriteAlignerWindow : Window {
 		if (!gui_scope) return;
 		if (data == 1) {
 			/* Sprite picker finished */
-			this->RaiseWidget(SAW_PICKER);
+			this->RaiseWidget(WID_SA_PICKER);
 			this->vscroll->SetCount(_newgrf_debug_sprite_picker.sprites.Length());
 		}
 	}
 
 	virtual void OnResize()
 	{
-		this->vscroll->SetCapacityFromWidget(this, SAW_LIST);
-		this->GetWidget<NWidgetCore>(SAW_LIST)->widget_data = (this->vscroll->GetCapacity() << MAT_ROW_START) + (1 << MAT_COL_START);
+		this->vscroll->SetCapacityFromWidget(this, WID_SA_LIST);
+		this->GetWidget<NWidgetCore>(WID_SA_LIST)->widget_data = (this->vscroll->GetCapacity() << MAT_ROW_START) + (1 << MAT_COL_START);
 	}
 };
 
 static const NWidgetPart _nested_sprite_aligner_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_GREY),
-		NWidget(WWT_CAPTION, COLOUR_GREY, SAW_CAPTION), SetDataTip(STR_SPRITE_ALIGNER_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
+		NWidget(WWT_CAPTION, COLOUR_GREY, WID_SA_CAPTION), SetDataTip(STR_SPRITE_ALIGNER_CAPTION, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
@@ -805,43 +829,43 @@ static const NWidgetPart _nested_sprite_aligner_widgets[] = {
 		NWidget(NWID_HORIZONTAL), SetPIP(0, 0, 10),
 			NWidget(NWID_VERTICAL), SetPIP(10, 5, 10),
 				NWidget(NWID_HORIZONTAL, NC_EQUALSIZE), SetPIP(10, 5, 10),
-					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, SAW_PREVIOUS), SetDataTip(STR_SPRITE_ALIGNER_PREVIOUS_BUTTON, STR_SPRITE_ALIGNER_PREVIOUS_TOOLTIP), SetFill(1, 0),
-					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, SAW_GOTO), SetDataTip(STR_SPRITE_ALIGNER_GOTO_BUTTON, STR_SPRITE_ALIGNER_GOTO_TOOLTIP), SetFill(1, 0),
-					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, SAW_NEXT), SetDataTip(STR_SPRITE_ALIGNER_NEXT_BUTTON, STR_SPRITE_ALIGNER_NEXT_TOOLTIP), SetFill(1, 0),
+					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_SA_PREVIOUS), SetDataTip(STR_SPRITE_ALIGNER_PREVIOUS_BUTTON, STR_SPRITE_ALIGNER_PREVIOUS_TOOLTIP), SetFill(1, 0),
+					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_SA_GOTO), SetDataTip(STR_SPRITE_ALIGNER_GOTO_BUTTON, STR_SPRITE_ALIGNER_GOTO_TOOLTIP), SetFill(1, 0),
+					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_SA_NEXT), SetDataTip(STR_SPRITE_ALIGNER_NEXT_BUTTON, STR_SPRITE_ALIGNER_NEXT_TOOLTIP), SetFill(1, 0),
 				EndContainer(),
 				NWidget(NWID_HORIZONTAL), SetPIP(10, 5, 10),
 					NWidget(NWID_SPACER), SetFill(1, 1),
-					NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, SAW_UP), SetDataTip(SPR_ARROW_UP, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
+					NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_SA_UP), SetDataTip(SPR_ARROW_UP, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
 					NWidget(NWID_SPACER), SetFill(1, 1),
 				EndContainer(),
 				NWidget(NWID_HORIZONTAL_LTR), SetPIP(10, 5, 10),
 					NWidget(NWID_VERTICAL),
 						NWidget(NWID_SPACER), SetFill(1, 1),
-						NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, SAW_LEFT), SetDataTip(SPR_ARROW_LEFT, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
+						NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_SA_LEFT), SetDataTip(SPR_ARROW_LEFT, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
 						NWidget(NWID_SPACER), SetFill(1, 1),
 					EndContainer(),
-					NWidget(WWT_PANEL, COLOUR_DARK_BLUE, SAW_SPRITE), SetDataTip(STR_NULL, STR_SPRITE_ALIGNER_SPRITE_TOOLTIP),
+					NWidget(WWT_PANEL, COLOUR_DARK_BLUE, WID_SA_SPRITE), SetDataTip(STR_NULL, STR_SPRITE_ALIGNER_SPRITE_TOOLTIP),
 					EndContainer(),
 					NWidget(NWID_VERTICAL),
 						NWidget(NWID_SPACER), SetFill(1, 1),
-						NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, SAW_RIGHT), SetDataTip(SPR_ARROW_RIGHT, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
+						NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_SA_RIGHT), SetDataTip(SPR_ARROW_RIGHT, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
 						NWidget(NWID_SPACER), SetFill(1, 1),
 					EndContainer(),
 				EndContainer(),
 				NWidget(NWID_HORIZONTAL), SetPIP(10, 5, 10),
 					NWidget(NWID_SPACER), SetFill(1, 1),
-					NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, SAW_DOWN), SetDataTip(SPR_ARROW_DOWN, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
+					NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_SA_DOWN), SetDataTip(SPR_ARROW_DOWN, STR_SPRITE_ALIGNER_MOVE_TOOLTIP), SetResize(0, 0),
 					NWidget(NWID_SPACER), SetFill(1, 1),
 				EndContainer(),
 				NWidget(NWID_HORIZONTAL), SetPIP(10, 5, 10),
-					NWidget(WWT_LABEL, COLOUR_GREY, SAW_OFFSETS), SetDataTip(STR_SPRITE_ALIGNER_OFFSETS, STR_NULL), SetFill(1, 0),
+					NWidget(WWT_LABEL, COLOUR_GREY, WID_SA_OFFSETS), SetDataTip(STR_SPRITE_ALIGNER_OFFSETS, STR_NULL), SetFill(1, 0),
 				EndContainer(),
 			EndContainer(),
 			NWidget(NWID_VERTICAL), SetPIP(10, 5, 10),
-				NWidget(WWT_TEXTBTN, COLOUR_GREY, SAW_PICKER), SetDataTip(STR_SPRITE_ALIGNER_PICKER_BUTTON, STR_SPRITE_ALIGNER_PICKER_TOOLTIP), SetFill(1, 0),
+				NWidget(WWT_TEXTBTN, COLOUR_GREY, WID_SA_PICKER), SetDataTip(STR_SPRITE_ALIGNER_PICKER_BUTTON, STR_SPRITE_ALIGNER_PICKER_TOOLTIP), SetFill(1, 0),
 				NWidget(NWID_HORIZONTAL),
-					NWidget(WWT_MATRIX, COLOUR_GREY, SAW_LIST), SetResize(1, 1), SetDataTip(0x101, STR_NULL), SetFill(1, 1), SetScrollbar(SAW_SCROLLBAR),
-					NWidget(NWID_VSCROLLBAR, COLOUR_GREY, SAW_SCROLLBAR),
+					NWidget(WWT_MATRIX, COLOUR_GREY, WID_SA_LIST), SetResize(1, 1), SetDataTip(0x101, STR_NULL), SetFill(1, 1), SetScrollbar(WID_SA_SCROLLBAR),
+					NWidget(NWID_VSCROLLBAR, COLOUR_GREY, WID_SA_SCROLLBAR),
 				EndContainer(),
 			EndContainer(),
 		EndContainer(),
@@ -851,7 +875,7 @@ static const NWidgetPart _nested_sprite_aligner_widgets[] = {
 static const WindowDesc _sprite_aligner_desc(
 	WDP_AUTO, 400, 300,
 	WC_SPRITE_ALIGNER, WC_NONE,
-	WDF_UNCLICK_BUTTONS,
+	0,
 	_nested_sprite_aligner_widgets, lengthof(_nested_sprite_aligner_widgets)
 );
 
