@@ -1,4 +1,4 @@
-/* $Id: newgrf_generic.cpp 22885 2011-09-03 18:51:59Z frosch $ */
+/* $Id: newgrf_generic.cpp 24846 2012-12-23 21:09:09Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,12 +11,49 @@
 
 #include "stdafx.h"
 #include "debug.h"
-#include "newgrf.h"
 #include "newgrf_spritegroup.h"
 #include "industrytype.h"
-#include "core/bitmath_func.hpp"
+#include "core/random_func.hpp"
+#include "newgrf_sound.h"
+#include "water_map.h"
 #include <list>
 
+/** Scope resolver for generic objects and properties. */
+struct GenericScopeResolver : public ScopeResolver {
+	CargoID cargo_type;
+	uint8 default_selection;
+	uint8 src_industry;        ///< Source industry substitute type. 0xFF for "town", 0xFE for "unknown".
+	uint8 dst_industry;        ///< Destination industry substitute type. 0xFF for "town", 0xFE for "unknown".
+	uint8 distance;
+	AIConstructionEvent event;
+	uint8 count;
+	uint8 station_size;
+
+	GenericScopeResolver(ResolverObject *ro, bool ai_callback);
+
+	/* virtual */ uint32 GetVariable(byte variable, uint32 parameter, bool *available) const;
+
+private:
+	bool ai_callback; ///< Callback comes from the AI.
+};
+
+
+/** Resolver object for generic objects/properties. */
+struct GenericResolverObject : public ResolverObject {
+	GenericScopeResolver generic_scope;
+
+	GenericResolverObject(bool ai_callback, CallbackID callback = CBID_NO_CALLBACK);
+
+	/* virtual */ ScopeResolver *GetScope(VarSpriteGroupScope scope = VSG_SCOPE_SELF, byte relative = 0)
+	{
+		switch (scope) {
+			case VSG_SCOPE_SELF: return &this->generic_scope;
+			default: return ResolverObject::GetScope(scope, relative);
+		}
+	}
+
+	/* virtual */ const SpriteGroup *ResolveReal(const RealSpriteGroup *group) const;
+};
 
 struct GenericCallback {
 	const GRFFile *file;
@@ -46,9 +83,9 @@ void ResetGenericCallbacks()
 
 /**
  * Add a generic feature callback sprite group to the appropriate feature list.
- * @param feature
- * @param file
- * @param group
+ * @param feature The feature for the callback.
+ * @param file The GRF of the callback.
+ * @param group The sprite group of the callback.
  */
 void AddGenericCallback(uint8 feature, const GRFFile *file, const SpriteGroup *group)
 {
@@ -63,41 +100,24 @@ void AddGenericCallback(uint8 feature, const GRFFile *file, const SpriteGroup *g
 	_gcl[feature].push_front(GenericCallback(file, group));
 }
 
-
-static uint32 GenericCallbackGetRandomBits(const ResolverObject *object)
+/* virtual */ uint32 GenericScopeResolver::GetVariable(byte variable, uint32 parameter, bool *available) const
 {
-	return 0;
-}
+	if (this->ai_callback) {
+		switch (variable) {
+			case 0x40: return this->ro->grffile->cargo_map[this->cargo_type];
 
+			case 0x80: return this->cargo_type;
+			case 0x81: return CargoSpec::Get(this->cargo_type)->bitnum;
+			case 0x82: return this->default_selection;
+			case 0x83: return this->src_industry;
+			case 0x84: return this->dst_industry;
+			case 0x85: return this->distance;
+			case 0x86: return this->event;
+			case 0x87: return this->count;
+			case 0x88: return this->station_size;
 
-static uint32 GenericCallbackGetTriggers(const ResolverObject *object)
-{
-	return 0;
-}
-
-
-static void GenericCallbackSetTriggers(const ResolverObject *object, int triggers)
-{
-	return;
-}
-
-
-static uint32 GenericCallbackGetVariable(const ResolverObject *object, byte variable, byte parameter, bool *available)
-{
-	switch (variable) {
-		case 0x40: return object->grffile->cargo_map[object->u.generic.cargo_type];
-
-		case 0x80: return object->u.generic.cargo_type;
-		case 0x81: return CargoSpec::Get(object->u.generic.cargo_type)->bitnum;
-		case 0x82: return object->u.generic.default_selection;
-		case 0x83: return object->u.generic.src_industry;
-		case 0x84: return object->u.generic.dst_industry;
-		case 0x85: return object->u.generic.distance;
-		case 0x86: return object->u.generic.event;
-		case 0x87: return object->u.generic.count;
-		case 0x88: return object->u.generic.station_size;
-
-		default: break;
+			default: break;
+		}
 	}
 
 	DEBUG(grf, 1, "Unhandled generic feature variable 0x%02X", variable);
@@ -107,26 +127,38 @@ static uint32 GenericCallbackGetVariable(const ResolverObject *object, byte vari
 }
 
 
-static const SpriteGroup *GenericCallbackResolveReal(const ResolverObject *object, const RealSpriteGroup *group)
+/* virtual */ const SpriteGroup *GenericResolverObject::ResolveReal(const RealSpriteGroup *group) const
 {
 	if (group->num_loaded == 0) return NULL;
 
 	return group->loaded[0];
 }
 
-
-static inline void NewGenericResolver(ResolverObject *res)
+/**
+ * Generic resolver.
+ * @param ai_callback Callback comes from the AI.
+ * @param callback Callback ID.
+ */
+GenericResolverObject::GenericResolverObject(bool ai_callback, CallbackID callback) : ResolverObject(NULL, callback), generic_scope(this, ai_callback)
 {
-	res->GetRandomBits = &GenericCallbackGetRandomBits;
-	res->GetTriggers   = &GenericCallbackGetTriggers;
-	res->SetTriggers   = &GenericCallbackSetTriggers;
-	res->GetVariable   = &GenericCallbackGetVariable;
-	res->ResolveReal   = &GenericCallbackResolveReal;
+}
 
-	res->callback        = CBID_NO_CALLBACK;
-	res->callback_param1 = 0;
-	res->callback_param2 = 0;
-	res->ResetState();
+/**
+ * Generic scope resolver.
+ * @param ro Surrounding resolver.
+ * @param ai_callback Callback comes from the AI.
+ */
+GenericScopeResolver::GenericScopeResolver(ResolverObject *ro, bool ai_callback) : ScopeResolver(ro)
+{
+	this->cargo_type = 0;
+	this->default_selection = 0;
+	this->src_industry = 0;
+	this->dst_industry = 0;
+	this->distance = 0;
+	this->event = (AIConstructionEvent)0;
+	this->count = 0;
+	this->station_size = 0;
+	this->ai_callback = ai_callback;
 }
 
 
@@ -135,11 +167,13 @@ static inline void NewGenericResolver(ResolverObject *res)
  * answer
  * @param feature GRF Feature of callback
  * @param object  pre-populated resolver object
+ * @param param1_grfv7 callback_param1 for GRFs up to version 7.
+ * @param param1_grfv8 callback_param1 for GRFs from version 8 on.
  * @param [out] file Optionally returns the GRFFile which made the final decision for the callback result.
  *                   May be NULL if not required.
  * @return callback value if successful or CALLBACK_FAILED
  */
-static uint16 GetGenericCallbackResult(uint8 feature, ResolverObject *object, const GRFFile **file)
+static uint16 GetGenericCallbackResult(uint8 feature, ResolverObject *object, uint32 param1_grfv7, uint32 param1_grfv8, const GRFFile **file)
 {
 	assert(feature < lengthof(_gcl));
 
@@ -147,6 +181,8 @@ static uint16 GetGenericCallbackResult(uint8 feature, ResolverObject *object, co
 	for (GenericCallbackList::const_iterator it = _gcl[feature].begin(); it != _gcl[feature].end(); ++it) {
 		const SpriteGroup *group = it->group;
 		object->grffile = it->file;
+		/* Set callback param based on GRF version. */
+		object->callback_param1 = it->file->grf_version >= 8 ? param1_grfv8 : param1_grfv7;
 		group = SpriteGroup::Resolve(group, object);
 		if (group == NULL || group->GetCallbackResult() == CALLBACK_FAILED) continue;
 
@@ -179,9 +215,7 @@ static uint16 GetGenericCallbackResult(uint8 feature, ResolverObject *object, co
  */
 uint16 GetAiPurchaseCallbackResult(uint8 feature, CargoID cargo_type, uint8 default_selection, IndustryType src_industry, IndustryType dst_industry, uint8 distance, AIConstructionEvent event, uint8 count, uint8 station_size, const GRFFile **file)
 {
-	ResolverObject object;
-
-	NewGenericResolver(&object);
+	GenericResolverObject object(true, CBID_GENERIC_AI_PURCHASE_SELECTION);
 
 	if (src_industry != IT_AI_UNKNOWN && src_industry != IT_AI_TOWN) {
 		const IndustrySpec *is = GetIndustrySpec(src_industry);
@@ -195,17 +229,42 @@ uint16 GetAiPurchaseCallbackResult(uint8 feature, CargoID cargo_type, uint8 defa
 		if (is->grf_prop.subst_id != INVALID_INDUSTRYTYPE) dst_industry = is->grf_prop.subst_id;
 	}
 
-	object.callback = CBID_GENERIC_AI_PURCHASE_SELECTION;
-	object.u.generic.cargo_type        = cargo_type;
-	object.u.generic.default_selection = default_selection;
-	object.u.generic.src_industry      = src_industry;
-	object.u.generic.dst_industry      = dst_industry;
-	object.u.generic.distance          = distance;
-	object.u.generic.event             = event;
-	object.u.generic.count             = count;
-	object.u.generic.station_size      = station_size;
+	object.generic_scope.cargo_type        = cargo_type;
+	object.generic_scope.default_selection = default_selection;
+	object.generic_scope.src_industry      = src_industry;
+	object.generic_scope.dst_industry      = dst_industry;
+	object.generic_scope.distance          = distance;
+	object.generic_scope.event             = event;
+	object.generic_scope.count             = count;
+	object.generic_scope.station_size      = station_size;
 
-	uint16 callback = GetGenericCallbackResult(feature, &object, file);
+	uint16 callback = GetGenericCallbackResult(feature, &object, 0, 0, file);
 	if (callback != CALLBACK_FAILED) callback = GB(callback, 0, 8);
 	return callback;
 }
+
+
+/**
+ * 'Execute' the ambient sound effect callback.
+ * @param tile Tile the sound effect should be generated for.
+ */
+void AmbientSoundEffectCallback(TileIndex tile)
+{
+	assert(IsTileType(tile, MP_CLEAR) || IsTileType(tile, MP_TREES) || IsTileType(tile, MP_WATER));
+
+	/* Only run every 1/200-th time. */
+	uint32 r; // Save for later
+	if (!Chance16R(1, 200, r) || !_settings_client.sound.ambient) return;
+
+	/* Prepare resolver object. */
+	GenericResolverObject object(false, CBID_SOUNDS_AMBIENT_EFFECT);
+
+	uint32 param1_v7 = GetTileType(tile) << 28 | Clamp(TileHeight(tile), 0, 15) << 24 | GB(r, 16, 8) << 16 | GetTerrainType(tile);
+	uint32 param1_v8 = GetTileType(tile) << 24 | GetTileZ(tile) << 16 | GB(r, 16, 8) << 8 | (HasTileWaterClass(tile) ? GetWaterClass(tile) : 0) << 3 | GetTerrainType(tile);
+
+	/* Run callback. */
+	const GRFFile *grf_file;
+	uint16 callback = GetGenericCallbackResult(GSF_SOUNDFX, &object, param1_v7, param1_v8, &grf_file);
+
+	if (callback != CALLBACK_FAILED) PlayTileSound(grf_file, callback, tile);
+}
