@@ -1,4 +1,4 @@
-/* $Id: newgrf_house.cpp 22838 2011-08-25 13:24:32Z rubidium $ */
+/* $Id: newgrf_house.cpp 25496 2013-06-28 19:09:47Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,9 +11,7 @@
 
 #include "stdafx.h"
 #include "debug.h"
-#include "viewport_func.h"
 #include "landscape.h"
-#include "newgrf.h"
 #include "newgrf_house.h"
 #include "newgrf_spritegroup.h"
 #include "newgrf_town.h"
@@ -21,15 +19,70 @@
 #include "company_func.h"
 #include "company_base.h"
 #include "town.h"
-#include "sprite.h"
 #include "genworld.h"
 #include "newgrf_animation_base.h"
+#include "newgrf_cargo.h"
+#include "station_base.h"
 
 static BuildingCounts<uint32> _building_counts;
 static HouseClassMapping _class_mapping[HOUSE_CLASS_MAX];
 
 HouseOverrideManager _house_mngr(NEW_HOUSE_OFFSET, HOUSE_MAX, INVALID_HOUSE_ID);
 
+/**
+ * Constructor of a house scope resolver.
+ * @param ro Surrounding resolver.
+ * @param house_id House type being queried.
+ * @param tile %Tile containing the house.
+ * @param town %Town containing the house.
+ * @param not_yet_constructed House is still under construction.
+ * @param initial_random_bits Random bits during construction checks.
+ * @param watched_cargo_triggers Cargo types that triggered the watched cargo callback.
+ */
+HouseScopeResolver::HouseScopeResolver(ResolverObject *ro, HouseID house_id, TileIndex tile, Town *town,
+			bool not_yet_constructed, uint8 initial_random_bits, uint32 watched_cargo_triggers)
+		: ScopeResolver(ro)
+{
+	this->house_id = house_id;
+	this->tile = tile;
+	this->town = town;
+	this->not_yet_constructed = not_yet_constructed;
+	this->initial_random_bits = initial_random_bits;
+	this->watched_cargo_triggers = watched_cargo_triggers;
+}
+
+/**
+ * Retrieve the grf file associated with a house.
+ * @param house_id House to query.
+ * @return The associated GRF file (may be \c NULL).
+ */
+static const GRFFile *GetHouseSpecGrf(HouseID house_id)
+{
+	const HouseSpec *hs  = HouseSpec::Get(house_id);
+	return (hs != NULL) ? hs->grf_prop.grffile : NULL;
+}
+
+/**
+ * Construct a resolver for a house.
+ * @param house_id House to query.
+ * @param tile %Tile containing the house.
+ * @param town %Town containing the house.
+ * @param callback Callback ID.
+ * @param param1 First parameter (var 10) of the callback.
+ * @param param2 Second parameter (var 18) of the callback.
+ * @param not_yet_constructed House is still under construction.
+ * @param initial_random_bits Random bits during construction checks.
+ * @param watched_cargo_triggers Cargo types that triggered the watched cargo callback.
+ */
+HouseResolverObject::HouseResolverObject(HouseID house_id, TileIndex tile, Town *town,
+		CallbackID callback, uint32 param1, uint32 param2,
+		bool not_yet_constructed, uint8 initial_random_bits, uint32 watched_cargo_triggers)
+	: ResolverObject(GetHouseSpecGrf(house_id), callback, param1, param2),
+	house_scope(this, house_id, tile, town, not_yet_constructed, initial_random_bits, watched_cargo_triggers),
+	town_scope(this, town, not_yet_constructed) // Don't access StorePSA if house is not yet constructed.
+{
+}
+
 HouseClassID AllocateHouseClassID(byte grf_class_id, uint32 grfid)
 {
 	/* Start from 1 because 0 means that no class has been assigned. */
@@ -50,6 +103,11 @@ HouseClassID AllocateHouseClassID(byte grf_class_id, uint32 grfid)
 void InitializeBuildingCounts()
 {
 	memset(&_building_counts, 0, sizeof(_building_counts));
+
+	Town *t;
+	FOR_ALL_TOWNS(t) {
+		memset(&t->cache.building_counts, 0, sizeof(t->cache.building_counts));
+	}
 }
 
 /**
@@ -64,12 +122,12 @@ void IncreaseBuildingCount(Town *t, HouseID house_id)
 
 	if (!_loaded_newgrf_features.has_newhouses) return;
 
-	t->building_counts.id_count[house_id]++;
+	t->cache.building_counts.id_count[house_id]++;
 	_building_counts.id_count[house_id]++;
 
 	if (class_id == HOUSE_NO_CLASS) return;
 
-	t->building_counts.class_count[class_id]++;
+	t->cache.building_counts.class_count[class_id]++;
 	_building_counts.class_count[class_id]++;
 }
 
@@ -85,36 +143,33 @@ void DecreaseBuildingCount(Town *t, HouseID house_id)
 
 	if (!_loaded_newgrf_features.has_newhouses) return;
 
-	if (t->building_counts.id_count[house_id] > 0) t->building_counts.id_count[house_id]--;
-	if (_building_counts.id_count[house_id] > 0)   _building_counts.id_count[house_id]--;
+	if (t->cache.building_counts.id_count[house_id] > 0) t->cache.building_counts.id_count[house_id]--;
+	if (_building_counts.id_count[house_id] > 0) _building_counts.id_count[house_id]--;
 
 	if (class_id == HOUSE_NO_CLASS) return;
 
-	if (t->building_counts.class_count[class_id] > 0) t->building_counts.class_count[class_id]--;
-	if (_building_counts.class_count[class_id] > 0)   _building_counts.class_count[class_id]--;
+	if (t->cache.building_counts.class_count[class_id] > 0) t->cache.building_counts.class_count[class_id]--;
+	if (_building_counts.class_count[class_id] > 0) _building_counts.class_count[class_id]--;
 }
 
-static uint32 HouseGetRandomBits(const ResolverObject *object)
+/* virtual */ uint32 HouseScopeResolver::GetRandomBits() const
 {
 	/* Note: Towns build houses over houses. So during construction checks 'tile' may be a valid but unrelated house. */
-	TileIndex tile = object->u.house.tile;
-	assert(IsValidTile(tile) && (object->u.house.not_yet_constructed || IsTileType(tile, MP_HOUSE)));
-	return object->u.house.not_yet_constructed ? object->u.house.initial_random_bits : GetHouseRandomBits(tile);
+	assert(IsValidTile(this->tile) && (this->not_yet_constructed || IsTileType(this->tile, MP_HOUSE)));
+	return this->not_yet_constructed ? this->initial_random_bits : GetHouseRandomBits(this->tile);
 }
 
-static uint32 HouseGetTriggers(const ResolverObject *object)
+/* virtual */ uint32 HouseScopeResolver::GetTriggers() const
 {
 	/* Note: Towns build houses over houses. So during construction checks 'tile' may be a valid but unrelated house. */
-	TileIndex tile = object->u.house.tile;
-	assert(IsValidTile(tile) && (object->u.house.not_yet_constructed || IsTileType(tile, MP_HOUSE)));
-	return object->u.house.not_yet_constructed ? 0 : GetHouseTriggers(tile);
+	assert(IsValidTile(this->tile) && (this->not_yet_constructed || IsTileType(this->tile, MP_HOUSE)));
+	return this->not_yet_constructed ? 0 : GetHouseTriggers(this->tile);
 }
 
-static void HouseSetTriggers(const ResolverObject *object, int triggers)
+/* virtual */ void HouseScopeResolver::SetTriggers(int triggers) const
 {
-	TileIndex tile = object->u.house.tile;
-	assert(!object->u.house.not_yet_constructed && IsValidTile(tile) && IsTileType(tile, MP_HOUSE));
-	SetHouseTriggers(tile, triggers);
+	assert(!this->not_yet_constructed && IsValidTile(this->tile) && IsTileType(this->tile, MP_HOUSE));
+	SetHouseTriggers(this->tile, triggers);
 }
 
 static uint32 GetNumHouses(HouseID house_id, const Town *town)
@@ -124,16 +179,23 @@ static uint32 GetNumHouses(HouseID house_id, const Town *town)
 
 	map_id_count     = ClampU(_building_counts.id_count[house_id], 0, 255);
 	map_class_count  = ClampU(_building_counts.class_count[class_id], 0, 255);
-	town_id_count    = ClampU(town->building_counts.id_count[house_id], 0, 255);
-	town_class_count = ClampU(town->building_counts.class_count[class_id], 0, 255);
+	town_id_count    = ClampU(town->cache.building_counts.id_count[house_id], 0, 255);
+	town_class_count = ClampU(town->cache.building_counts.class_count[class_id], 0, 255);
 
 	return map_class_count << 24 | town_class_count << 16 | map_id_count << 8 | town_id_count;
 }
 
-uint32 GetNearbyTileInformation(byte parameter, TileIndex tile)
+/**
+ * Get information about a nearby tile.
+ * @param parameter from callback. It's in fact a pair of coordinates
+ * @param tile TileIndex from which the callback was initiated
+ * @param grf_version8 True, if we are dealing with a new NewGRF which uses GRF version >= 8.
+ * @return a construction of bits obeying the newgrf format
+ */
+static uint32 GetNearbyTileInformation(byte parameter, TileIndex tile, bool grf_version8)
 {
 	tile = GetNearbyTile(parameter, tile);
-	return GetNearbyTileInformation(tile);
+	return GetNearbyTileInformation(tile, grf_version8);
 }
 
 /** Structure with user-data for SearchNearbyHouseXXX - functions */
@@ -248,89 +310,105 @@ static uint32 GetDistanceFromNearbyHouse(uint8 parameter, TileIndex tile, HouseI
 }
 
 /**
- * HouseGetVariable():
- *
- * Used by the resolver to get values for feature 07 deterministic spritegroups.
+ * @note Used by the resolver to get values for feature 07 deterministic spritegroups.
  */
-static uint32 HouseGetVariable(const ResolverObject *object, byte variable, byte parameter, bool *available)
+/* virtual */ uint32 HouseScopeResolver::GetVariable(byte variable, uint32 parameter, bool *available) const
 {
-	const Town *town = object->u.house.town;
-	TileIndex tile   = object->u.house.tile;
-	HouseID house_id = object->u.house.house_id;
-
-	if (object->scope == VSG_SCOPE_PARENT) {
-		return TownGetVariable(variable, parameter, available, town);
-	}
-
 	switch (variable) {
 		/* Construction stage. */
-		case 0x40: return (IsTileType(tile, MP_HOUSE) ? GetHouseBuildingStage(tile) : 0) | TileHash2Bit(TileX(tile), TileY(tile)) << 2;
+		case 0x40: return (IsTileType(this->tile, MP_HOUSE) ? GetHouseBuildingStage(this->tile) : 0) | TileHash2Bit(TileX(this->tile), TileY(this->tile)) << 2;
 
 		/* Building age. */
-		case 0x41: return IsTileType(tile, MP_HOUSE) ? GetHouseAge(tile) : 0;
+		case 0x41: return IsTileType(this->tile, MP_HOUSE) ? GetHouseAge(this->tile) : 0;
 
 		/* Town zone */
-		case 0x42: return GetTownRadiusGroup(town, tile);
+		case 0x42: return GetTownRadiusGroup(this->town, this->tile);
 
 		/* Terrain type */
-		case 0x43: return GetTerrainType(tile);
+		case 0x43: return GetTerrainType(this->tile);
 
 		/* Number of this type of building on the map. */
-		case 0x44: return GetNumHouses(house_id, town);
+		case 0x44: return GetNumHouses(this->house_id, this->town);
 
 		/* Whether the town is being created or just expanded. */
 		case 0x45: return _generating_world ? 1 : 0;
 
 		/* Current animation frame. */
-		case 0x46: return IsTileType(tile, MP_HOUSE) ? GetAnimationFrame(tile) : 0;
+		case 0x46: return IsTileType(this->tile, MP_HOUSE) ? GetAnimationFrame(this->tile) : 0;
 
 		/* Position of the house */
-		case 0x47: return TileY(tile) << 16 | TileX(tile);
+		case 0x47: return TileY(this->tile) << 16 | TileX(this->tile);
 
 		/* Building counts for old houses with id = parameter. */
-		case 0x60: return parameter < NEW_HOUSE_OFFSET ? GetNumHouses(parameter, town) : 0;
+		case 0x60: return parameter < NEW_HOUSE_OFFSET ? GetNumHouses(parameter, this->town) : 0;
 
 		/* Building counts for new houses with id = parameter. */
 		case 0x61: {
-			const HouseSpec *hs = HouseSpec::Get(house_id);
+			const HouseSpec *hs = HouseSpec::Get(this->house_id);
 			if (hs->grf_prop.grffile == NULL) return 0;
 
 			HouseID new_house = _house_mngr.GetID(parameter, hs->grf_prop.grffile->grfid);
-			return new_house == INVALID_HOUSE_ID ? 0 : GetNumHouses(new_house, town);
+			return new_house == INVALID_HOUSE_ID ? 0 : GetNumHouses(new_house, this->town);
 		}
 
 		/* Land info for nearby tiles. */
-		case 0x62: return GetNearbyTileInformation(parameter, tile);
+		case 0x62: return GetNearbyTileInformation(parameter, this->tile, this->ro->grffile->grf_version >= 8);
 
 		/* Current animation frame of nearby house tiles */
 		case 0x63: {
-			TileIndex testtile = GetNearbyTile(parameter, tile);
+			TileIndex testtile = GetNearbyTile(parameter, this->tile);
 			return IsTileType(testtile, MP_HOUSE) ? GetAnimationFrame(testtile) : 0;
 		}
 
 		/* Cargo acceptance history of nearby stations */
-		/*case 0x64: not implemented yet */
+		case 0x64: {
+			CargoID cid = GetCargoTranslation(parameter, this->ro->grffile);
+			if (cid == CT_INVALID) return 0;
+
+			/* Extract tile offset. */
+			int8 x_offs = GB(GetRegister(0x100), 0, 8);
+			int8 y_offs = GB(GetRegister(0x100), 8, 8);
+			TileIndex testtile = TILE_MASK(this->tile + TileDiffXY(x_offs, y_offs));
+
+			StationFinder stations(TileArea(testtile, 1, 1));
+			const StationList *sl = stations.GetStations();
+
+			/* Collect acceptance stats. */
+			uint32 res = 0;
+			for (Station * const * st_iter = sl->Begin(); st_iter != sl->End(); st_iter++) {
+				const Station *st = *st_iter;
+				if (HasBit(st->goods[cid].acceptance_pickup, GoodsEntry::GES_EVER_ACCEPTED))    SetBit(res, 0);
+				if (HasBit(st->goods[cid].acceptance_pickup, GoodsEntry::GES_LAST_MONTH))       SetBit(res, 1);
+				if (HasBit(st->goods[cid].acceptance_pickup, GoodsEntry::GES_CURRENT_MONTH))    SetBit(res, 2);
+				if (HasBit(st->goods[cid].acceptance_pickup, GoodsEntry::GES_ACCEPTED_BIGTICK)) SetBit(res, 3);
+			}
+
+			/* Cargo triggered CB 148? */
+			if (HasBit(this->watched_cargo_triggers, cid)) SetBit(res, 4);
+
+			return res;
+		}
 
 		/* Distance test for some house types */
-		case 0x65: return GetDistanceFromNearbyHouse(parameter, tile, object->u.house.house_id);
+		case 0x65: return GetDistanceFromNearbyHouse(parameter, this->tile, this->house_id);
 
 		/* Class and ID of nearby house tile */
 		case 0x66: {
-			TileIndex testtile = GetNearbyTile(parameter, tile);
+			TileIndex testtile = GetNearbyTile(parameter, this->tile);
 			if (!IsTileType(testtile, MP_HOUSE)) return 0xFFFFFFFF;
 			HouseSpec *hs = HouseSpec::Get(GetHouseType(testtile));
 			/* Information about the grf local classid if the house has a class */
 			uint houseclass = 0;
 			if (hs->class_id != HOUSE_NO_CLASS) {
-				houseclass = (hs->grf_prop.grffile == object->grffile ? 1 : 2) << 8;
+				houseclass = (hs->grf_prop.grffile == this->ro->grffile ? 1 : 2) << 8;
 				houseclass |= _class_mapping[hs->class_id].class_id;
 			}
 			/* old house type or grf-local houseid */
 			uint local_houseid = 0;
-			if (house_id < NEW_HOUSE_OFFSET) {
-				local_houseid = house_id;
+			if (this->house_id < NEW_HOUSE_OFFSET) {
+				local_houseid = this->house_id;
 			} else {
-				local_houseid = (hs->grf_prop.grffile == object->grffile ? 1 : 2) << 8;
+				local_houseid = (hs->grf_prop.grffile == this->ro->grffile ? 1 : 2) << 8;
 				local_houseid |= hs->grf_prop.local_id;
 			}
 			return houseclass << 16 | local_houseid;
@@ -338,13 +416,13 @@ static uint32 HouseGetVariable(const ResolverObject *object, byte variable, byte
 
 		/* GRFID of nearby house tile */
 		case 0x67: {
-			TileIndex testtile = GetNearbyTile(parameter, tile);
+			TileIndex testtile = GetNearbyTile(parameter, this->tile);
 			if (!IsTileType(testtile, MP_HOUSE)) return 0xFFFFFFFF;
 			HouseID house_id = GetHouseType(testtile);
 			if (house_id < NEW_HOUSE_OFFSET) return 0;
 			/* Checking the grffile information via HouseSpec doesn't work
 			 * in case the newgrf was removed. */
-			return _house_mngr.mapping_ID[house_id].grfid;
+			return _house_mngr.GetGRFID(house_id);
 		}
 	}
 
@@ -354,54 +432,15 @@ static uint32 HouseGetVariable(const ResolverObject *object, byte variable, byte
 	return UINT_MAX;
 }
 
-static const SpriteGroup *HouseResolveReal(const ResolverObject *object, const RealSpriteGroup *group)
+uint16 GetHouseCallback(CallbackID callback, uint32 param1, uint32 param2, HouseID house_id, Town *town, TileIndex tile,
+		bool not_yet_constructed, uint8 initial_random_bits, uint32 watched_cargo_triggers)
 {
-	/* Houses do not have 'real' groups */
-	return NULL;
-}
-
-/**
- * NewHouseResolver():
- *
- * Returns a resolver object to be used with feature 07 spritegroups.
- */
-static void NewHouseResolver(ResolverObject *res, HouseID house_id, TileIndex tile, const Town *town)
-{
-	res->GetRandomBits = HouseGetRandomBits;
-	res->GetTriggers   = HouseGetTriggers;
-	res->SetTriggers   = HouseSetTriggers;
-	res->GetVariable   = HouseGetVariable;
-	res->ResolveReal   = HouseResolveReal;
-
-	res->u.house.tile     = tile;
-	res->u.house.town     = town;
-	res->u.house.house_id = house_id;
-	res->u.house.not_yet_constructed = false;
-
-	res->callback        = CBID_NO_CALLBACK;
-	res->callback_param1 = 0;
-	res->callback_param2 = 0;
-	res->ResetState();
-
-	const HouseSpec *hs  = HouseSpec::Get(house_id);
-	res->grffile         = (hs != NULL ? hs->grf_prop.grffile : NULL);
-}
-
-uint16 GetHouseCallback(CallbackID callback, uint32 param1, uint32 param2, HouseID house_id, const Town *town, TileIndex tile, bool not_yet_constructed, uint8 initial_random_bits)
-{
-	ResolverObject object;
-	const SpriteGroup *group;
-
 	assert(IsValidTile(tile) && (not_yet_constructed || IsTileType(tile, MP_HOUSE)));
 
-	NewHouseResolver(&object, house_id, tile, town);
-	object.callback = callback;
-	object.callback_param1 = param1;
-	object.callback_param2 = param2;
-	object.u.house.not_yet_constructed = not_yet_constructed;
-	object.u.house.initial_random_bits = initial_random_bits;
+	HouseResolverObject object(house_id, tile, town, callback, param1, param2,
+			not_yet_constructed, initial_random_bits, watched_cargo_triggers);
 
-	group = SpriteGroup::Resolve(HouseSpec::Get(house_id)->grf_prop.spritegroup[0], &object);
+	const SpriteGroup *group = SpriteGroup::Resolve(HouseSpec::Get(house_id)->grf_prop.spritegroup[0], &object);
 	if (group == NULL) return CALLBACK_FAILED;
 
 	return group->GetCallbackResult();
@@ -409,7 +448,7 @@ uint16 GetHouseCallback(CallbackID callback, uint32 param1, uint32 param2, House
 
 static void DrawTileLayout(const TileInfo *ti, const TileLayoutSpriteGroup *group, byte stage, HouseID house_id)
 {
-	const DrawTileSprites *dts = group->dts;
+	const DrawTileSprites *dts = group->ProcessRegisters(&stage);
 
 	const HouseSpec *hs = HouseSpec::Get(house_id);
 	PaletteID palette = hs->random_colour[TileHash2Bit(ti->x, ti->y)] + PALETTE_RECOLOUR_START;
@@ -425,6 +464,7 @@ static void DrawTileLayout(const TileInfo *ti, const TileLayoutSpriteGroup *grou
 	PaletteID pal  = dts->ground.pal;
 
 	if (HasBit(image, SPRITE_MODIFIER_CUSTOM_SPRITE)) image += stage;
+	if (HasBit(pal, SPRITE_MODIFIER_CUSTOM_SPRITE)) pal += stage;
 
 	if (GB(image, 0, SPRITE_WIDTH) != 0) {
 		DrawGroundSprite(image, GroundSpritePaletteTransform(image, pal, palette));
@@ -436,42 +476,37 @@ static void DrawTileLayout(const TileInfo *ti, const TileLayoutSpriteGroup *grou
 void DrawNewHouseTile(TileInfo *ti, HouseID house_id)
 {
 	const HouseSpec *hs = HouseSpec::Get(house_id);
-	const SpriteGroup *group;
-	ResolverObject object;
 
 	if (ti->tileh != SLOPE_FLAT) {
 		bool draw_old_one = true;
 		if (HasBit(hs->callback_mask, CBM_HOUSE_DRAW_FOUNDATIONS)) {
 			/* Called to determine the type (if any) of foundation to draw for the house tile */
 			uint32 callback_res = GetHouseCallback(CBID_HOUSE_DRAW_FOUNDATIONS, 0, 0, house_id, Town::GetByTile(ti->tile), ti->tile);
-			draw_old_one = (callback_res != 0);
+			if (callback_res != CALLBACK_FAILED) draw_old_one = ConvertBooleanCallback(hs->grf_prop.grffile, CBID_HOUSE_DRAW_FOUNDATIONS, callback_res);
 		}
 
 		if (draw_old_one) DrawFoundation(ti, FOUNDATION_LEVELED);
 	}
 
-	NewHouseResolver(&object, house_id, ti->tile, Town::GetByTile(ti->tile));
+	HouseResolverObject object(house_id, ti->tile, Town::GetByTile(ti->tile));
 
-	group = SpriteGroup::Resolve(hs->grf_prop.spritegroup[0], &object);
-	if (group == NULL || group->type != SGT_TILELAYOUT) {
-		return;
-	} else {
+	const SpriteGroup *group = SpriteGroup::Resolve(hs->grf_prop.spritegroup[0], &object);
+	if (group != NULL && group->type == SGT_TILELAYOUT) {
 		/* Limit the building stage to the number of stages supplied. */
 		const TileLayoutSpriteGroup *tlgroup = (const TileLayoutSpriteGroup *)group;
 		byte stage = GetHouseBuildingStage(ti->tile);
-		stage = tlgroup->GetConstructionStageOffset(stage);
 		DrawTileLayout(ti, tlgroup, stage, house_id);
 	}
 }
 
 /* Simple wrapper for GetHouseCallback to keep the animation unified. */
-uint16 GetSimpleHouseCallback(CallbackID callback, uint32 param1, uint32 param2, const HouseSpec *spec, const Town *town, TileIndex tile)
+uint16 GetSimpleHouseCallback(CallbackID callback, uint32 param1, uint32 param2, const HouseSpec *spec, Town *town, TileIndex tile, uint32 extra_data)
 {
-	return GetHouseCallback(callback, param1, param2, spec - HouseSpec::Get(0), town, tile);
+	return GetHouseCallback(callback, param1, param2, spec - HouseSpec::Get(0), town, tile, false, 0, extra_data);
 }
 
 /** Helper class for animation control. */
-struct HouseAnimationBase : public AnimationBase<HouseAnimationBase, HouseSpec, Town, GetSimpleHouseCallback> {
+struct HouseAnimationBase : public AnimationBase<HouseAnimationBase, HouseSpec, Town, uint32, GetSimpleHouseCallback> {
 	static const CallbackID cb_animation_speed      = CBID_HOUSE_ANIMATION_SPEED;
 	static const CallbackID cb_animation_next_frame = CBID_HOUSE_ANIMATION_NEXT_FRAME;
 
@@ -500,15 +535,15 @@ bool CanDeleteHouse(TileIndex tile)
 {
 	const HouseSpec *hs = HouseSpec::Get(GetHouseType(tile));
 
-	/* Humans are always allowed to remove buildings, as is water and
+	/* Humans are always allowed to remove buildings, as is water and disasters and
 	 * anyone using the scenario editor. */
-	if (Company::IsValidHumanID(_current_company) || _current_company == OWNER_WATER || _current_company == OWNER_NONE) {
+	if (Company::IsValidHumanID(_current_company) || _current_company == OWNER_WATER || _current_company == OWNER_NONE || _game_mode == GM_EDITOR || _generating_world) {
 		return true;
 	}
 
 	if (HasBit(hs->callback_mask, CBM_HOUSE_DENY_DESTRUCTION)) {
 		uint16 callback_res = GetHouseCallback(CBID_HOUSE_DENY_DESTRUCTION, 0, 0, GetHouseType(tile), Town::GetByTile(tile), tile);
-		return (callback_res == CALLBACK_FAILED || callback_res == 0);
+		return (callback_res == CALLBACK_FAILED || !ConvertBooleanCallback(hs->grf_prop.grffile, CBID_HOUSE_DENY_DESTRUCTION, callback_res));
 	} else {
 		return !(hs->extra_flags & BUILDING_IS_PROTECTED);
 	}
@@ -556,7 +591,7 @@ bool NewHouseTileLoop(TileIndex tile)
 	/* Check callback 21, which determines if a house should be destroyed. */
 	if (HasBit(hs->callback_mask, CBM_HOUSE_DESTRUCTION)) {
 		uint16 callback_res = GetHouseCallback(CBID_HOUSE_DESTRUCTION, 0, 0, GetHouseType(tile), Town::GetByTile(tile), tile);
-		if (callback_res != CALLBACK_FAILED && GB(callback_res, 0, 8) > 0) {
+		if (callback_res != CALLBACK_FAILED && Convert8bitBooleanCallback(hs->grf_prop.grffile, CBID_HOUSE_DESTRUCTION, callback_res)) {
 			ClearTownHouse(Town::GetByTile(tile), tile);
 			return false;
 		}
@@ -569,8 +604,6 @@ bool NewHouseTileLoop(TileIndex tile)
 
 static void DoTriggerHouse(TileIndex tile, HouseTrigger trigger, byte base_random, bool first)
 {
-	ResolverObject object;
-
 	/* We can't trigger a non-existent building... */
 	assert(IsTileType(tile, MP_HOUSE));
 
@@ -579,9 +612,7 @@ static void DoTriggerHouse(TileIndex tile, HouseTrigger trigger, byte base_rando
 
 	if (hs->grf_prop.spritegroup == NULL) return;
 
-	NewHouseResolver(&object, hid, tile, Town::GetByTile(tile));
-
-	object.callback = CBID_RANDOM_TRIGGER;
+	HouseResolverObject object(hid, tile, Town::GetByTile(tile), CBID_RANDOM_TRIGGER);
 	object.trigger = trigger;
 
 	const SpriteGroup *group = SpriteGroup::Resolve(hs->grf_prop.spritegroup[0], &object);
@@ -619,11 +650,45 @@ void TriggerHouse(TileIndex t, HouseTrigger trigger)
 }
 
 /**
- * Resolve a house's spec and such so we can get a variable.
- * @param ro    The resolver object to fill.
- * @param index The house tile to get the data from.
+ * Run the watched cargo accepted callback for a single house tile.
+ * @param tile The house tile.
+ * @param origin The triggering tile.
+ * @param trigger_cargoes Cargo types that triggered the callback.
+ * @param random Random bits.
  */
-void GetHouseResolver(ResolverObject *ro, uint index)
+void DoWatchedCargoCallback(TileIndex tile, TileIndex origin, uint32 trigger_cargoes, uint16 random)
 {
-	NewHouseResolver(ro, GetHouseType(index), index, Town::GetByTile(index));
+	TileIndexDiffC diff = TileIndexToTileIndexDiffC(origin, tile);
+	uint32 cb_info = random << 16 | (uint8)diff.y << 8 | (uint8)diff.x;
+	HouseAnimationBase::ChangeAnimationFrame(CBID_HOUSE_WATCHED_CARGO_ACCEPTED, HouseSpec::Get(GetHouseType(tile)), Town::GetByTile(tile), tile, 0, cb_info, trigger_cargoes);
 }
+
+/**
+ * Run watched cargo accepted callback for a house.
+ * @param tile House tile.
+ * @param trigger_cargoes Triggering cargo types.
+ * @pre IsTileType(t, MP_HOUSE)
+ */
+void WatchedCargoCallback(TileIndex tile, uint32 trigger_cargoes)
+{
+	assert(IsTileType(tile, MP_HOUSE));
+	HouseID id = GetHouseType(tile);
+	const HouseSpec *hs = HouseSpec::Get(id);
+
+	trigger_cargoes &= hs->watched_cargoes;
+	/* None of the trigger cargoes is watched? */
+	if (trigger_cargoes == 0) return;
+
+	/* Same random value for all tiles of a multi-tile house. */
+	uint16 r = Random();
+
+	/* Do the callback, start at northern tile. */
+	TileIndex north = tile + GetHouseNorthPart(id);
+	hs = HouseSpec::Get(id);
+
+	DoWatchedCargoCallback(north, tile, trigger_cargoes, r);
+	if (hs->building_flags & BUILDING_2_TILES_Y)   DoWatchedCargoCallback(TILE_ADDXY(north, 0, 1), tile, trigger_cargoes, r);
+	if (hs->building_flags & BUILDING_2_TILES_X)   DoWatchedCargoCallback(TILE_ADDXY(north, 1, 0), tile, trigger_cargoes, r);
+	if (hs->building_flags & BUILDING_HAS_4_TILES) DoWatchedCargoCallback(TILE_ADDXY(north, 1, 1), tile, trigger_cargoes, r);
+}
+
