@@ -1,4 +1,4 @@
-/* $Id: newgrf_storage.h 22600 2011-06-18 19:40:56Z rubidium $ */
+/* $Id: newgrf_storage.h 25991 2013-11-13 22:00:46Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,22 +12,21 @@
 #ifndef NEWGRF_STORAGE_H
 #define NEWGRF_STORAGE_H
 
-#include "core/alloc_func.hpp"
+#include "core/pool_type.hpp"
 
 /**
  * Base class for all NewGRF storage arrays. Nothing fancy, only here
  * so we have a generalised class to use.
  */
-struct BaseStorageArray
-{
+struct BaseStorageArray {
 	virtual ~BaseStorageArray();
 
 	/**
-	 * Clear the changes made since the last ClearChanges.
+	 * Clear the changes made since the last #ClearChanges.
 	 * This can be done in two ways:
 	 *  - saving the changes permanently
 	 *  - reverting to the previous version
-	 * @param keep_changes do we save or revert the changes since the last ClearChanges?
+	 * @param keep_changes do we save or revert the changes since the last #ClearChanges?
 	 */
 	virtual void ClearChanges(bool keep_changes) = 0;
 
@@ -36,14 +35,14 @@ struct BaseStorageArray
 	 * @param pos   the position to write at
 	 * @param value the value to write
 	 */
-	virtual void Store(uint pos, int32 value) = 0;
+	virtual void StoreValue(uint pos, int32 value) = 0;
 };
 
 /**
  * Class for persistent storage of data.
- * On ClearChanges that data is either reverted or saved.
- * @param TYPE the type of variable to store.
- * @param SIZE the size of the array.
+ * On #ClearChanges that data is either reverted or saved.
+ * @tparam TYPE the type of variable to store.
+ * @tparam SIZE the size of the array.
  */
 template <typename TYPE, uint SIZE>
 struct PersistentStorageArray : BaseStorageArray {
@@ -75,7 +74,7 @@ struct PersistentStorageArray : BaseStorageArray {
 	 * @param pos   the position to write at
 	 * @param value the value to write
 	 */
-	void Store(uint pos, int32 value)
+	void StoreValue(uint pos, int32 value)
 	{
 		/* Out of the scope of the array */
 		if (pos >= SIZE) return;
@@ -85,7 +84,7 @@ struct PersistentStorageArray : BaseStorageArray {
 		if (this->storage[pos] == value) return;
 
 		/* We do not have made a backup; lets do so */
-		if (this->prev_storage != NULL) {
+		if (this->prev_storage == NULL) {
 			this->prev_storage = MallocT<TYPE>(SIZE);
 			memcpy(this->prev_storage, this->storage, sizeof(this->storage));
 
@@ -102,7 +101,7 @@ struct PersistentStorageArray : BaseStorageArray {
 	 * @param pos the position to get the data from
 	 * @return the data from that position
 	 */
-	TYPE Get(uint pos) const
+	TYPE GetValue(uint pos) const
 	{
 		/* Out of the scope of the array */
 		if (pos >= SIZE) return 0;
@@ -122,15 +121,16 @@ struct PersistentStorageArray : BaseStorageArray {
 			memcpy(this->storage, this->prev_storage, sizeof(this->storage));
 		}
 		free(this->prev_storage);
+		this->prev_storage = NULL;
 	}
 };
 
 
 /**
  * Class for temporary storage of data.
- * On ClearChanges that data is always zero-ed.
- * @param TYPE the type of variable to store.
- * @param SIZE the size of the array.
+ * On #ClearChanges that data is always zero-ed.
+ * @tparam TYPE the type of variable to store.
+ * @tparam SIZE the size of the array.
  */
 template <typename TYPE, uint SIZE>
 struct TemporaryStorageArray : BaseStorageArray {
@@ -147,7 +147,7 @@ struct TemporaryStorageArray : BaseStorageArray {
 	 * @param pos   the position to write at
 	 * @param value the value to write
 	 */
-	void Store(uint pos, int32 value)
+	void StoreValue(uint pos, int32 value)
 	{
 		/* Out of the scope of the array */
 		if (pos >= SIZE) return;
@@ -161,7 +161,7 @@ struct TemporaryStorageArray : BaseStorageArray {
 	 * @param pos the position to get the data from
 	 * @return the data from that position
 	 */
-	TYPE Get(uint pos) const
+	TYPE GetValue(uint pos) const
 	{
 		/* Out of the scope of the array */
 		if (pos >= SIZE) return 0;
@@ -178,4 +178,40 @@ struct TemporaryStorageArray : BaseStorageArray {
 void AddChangedStorage(BaseStorageArray *storage);
 void ClearStorageChanges(bool keep_changes);
 
+
+typedef PersistentStorageArray<int32, 16> OldPersistentStorage;
+
+typedef uint32 PersistentStorageID;
+
+struct PersistentStorage;
+typedef Pool<PersistentStorage, PersistentStorageID, 1, 0xFF000> PersistentStoragePool;
+
+extern PersistentStoragePool _persistent_storage_pool;
+
+/**
+ * Class for pooled persistent storage of data.
+ * On #ClearChanges that data is always zero-ed.
+ */
+struct PersistentStorage : PersistentStorageArray<int32, 16>, PersistentStoragePool::PoolItem<&_persistent_storage_pool> {
+	uint32 grfid; ///< GRFID associated to this persistent storage. A value of zero means "default".
+
+	/** We don't want GCC to zero our struct! It already is zeroed and has an index! */
+	PersistentStorage(const uint32 new_grfid) : grfid(new_grfid)
+	{
+		this->prev_storage = NULL;
+		memset(this->storage, 0, sizeof(this->storage));
+	}
+
+	/** Free the memory used by the persistent storage. */
+	~PersistentStorage()
+	{
+		free(this->prev_storage);
+	}
+};
+
+assert_compile(cpp_lengthof(OldPersistentStorage, storage) == cpp_lengthof(PersistentStorage, storage));
+
+#define FOR_ALL_STORAGES_FROM(var, start) FOR_ALL_ITEMS_FROM(PersistentStorage, storage_index, var, start)
+#define FOR_ALL_STORAGES(var) FOR_ALL_STORAGES_FROM(var, 0)
+
 #endif /* NEWGRF_STORAGE_H */
