@@ -1,4 +1,4 @@
-/* $Id: openttd.cpp 23795 2012-01-13 21:54:59Z rubidium $ */
+/* $Id: openttd.cpp 25583 2013-07-10 19:41:31Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,9 +17,8 @@
 #include "video/video_driver.hpp"
 
 #include "fontcache.h"
+#include "error.h"
 #include "gui.h"
-#include "sound_func.h"
-#include "window_func.h"
 
 #include "base_media_base.h"
 #include "saveload/saveload.h"
@@ -35,12 +34,11 @@
 #include "screenshot.h"
 #include "network/network.h"
 #include "network/network_func.h"
-#include "signs_base.h"
 #include "ai/ai.hpp"
 #include "ai/ai_config.hpp"
 #include "settings_func.h"
 #include "genworld.h"
-#include "group.h"
+#include "progress.h"
 #include "strings_func.h"
 #include "date_func.h"
 #include "vehicle_func.h"
@@ -50,7 +48,6 @@
 #include "elrail_func.h"
 #include "rev.h"
 #include "highscore.h"
-#include "thread/thread.h"
 #include "station_base.h"
 #include "crashlog.h"
 #include "engine_func.h"
@@ -59,16 +56,16 @@
 #include "core/backup_type.hpp"
 #include "hotkeys.h"
 #include "newgrf.h"
-
-
+#include "misc/getoptdata.h"
+#include "game/game.hpp"
+#include "game/game_config.hpp"
 #include "town.h"
-#include "industry.h"
+#include "subsidy_func.h"
+#include "gfx_layout.h"
+
 
 #include <stdarg.h>
 
-#include "table/strings.h"
-
-StringID _switch_mode_errorstr;
 
 void CallLandscapeTick();
 void IncreaseDate();
@@ -76,8 +73,8 @@ void DoPaletteAnimations();
 void MusicLoop();
 void ResetMusic();
 void CallWindowTickEvent();
+bool HandleBootstrap();
 
-extern void SetDifficultyLevel(int mode, DifficultySettings *gm_opt);
 extern Company *DoStartupNewCompany(bool is_ai, CompanyID company = INVALID_COMPANY);
 extern void ShowOSErrorBox(const char *buf, bool system);
 extern char *_config_file;
@@ -162,7 +159,7 @@ static void ShowHelp()
 		"  -g [savegame]       = Start new/save game immediately\n"
 		"  -G seed             = Set random seed\n"
 #if defined(ENABLE_NETWORK)
-		"  -n [ip:port#company]= Start networkgame\n"
+		"  -n [ip:port#company]= Join network game\n"
 		"  -p password         = Password to join server\n"
 		"  -P password         = Password to join company\n"
 		"  -D [ip][:port]      = Start dedicated server\n"
@@ -171,15 +168,12 @@ static void ShowHelp()
 		"  -f                  = Fork into the background (dedicated only)\n"
 #endif
 #endif /* ENABLE_NETWORK */
-		"  -i palette          = Force to use the DOS (0) or Windows (1) palette\n"
-		"                          (defines default setting when adding newgrfs)\n"
-		"                        Default value (2) lets OpenTTD use the palette\n"
-		"                          specified in graphics set file (see below)\n"
 		"  -I graphics_set     = Force the graphics set (see below)\n"
 		"  -S sounds_set       = Force the sounds set (see below)\n"
 		"  -M music_set        = Force the music set (see below)\n"
 		"  -c config_file      = Use 'config_file' instead of 'openttd.cfg'\n"
 		"  -x                  = Do not automatically save to config file on exit\n"
+		"  -q savegame         = Write some information about the savegame and exit\n"
 		"\n",
 		lastof(buf)
 	);
@@ -199,12 +193,19 @@ static void ShowHelp()
 	/* List the blitters */
 	p = BlitterFactoryBase::GetBlittersInfo(p, lastof(buf));
 
+	/* List the debug facilities. */
+	p = DumpDebugFacilityNames(p, lastof(buf));
+
 	/* We need to initialize the AI, so it finds the AIs */
-	TarScanner::DoScan();
 	AI::Initialize();
 	p = AI::GetConsoleList(p, lastof(buf), true);
 	AI::Uninitialize(true);
 
+	/* We need to initialize the GameScript, so it finds the GSs */
+	Game::Initialize();
+	p = Game::GetConsoleList(p, lastof(buf), true);
+	Game::Uninitialize(true);
+
 	/* ShowInfo put output to stderr, but version information should go
 	 * to stdout; this is the only exception */
 #if !defined(WIN32) && !defined(WIN64)
@@ -214,74 +215,45 @@ static void ShowHelp()
 #endif
 }
 
-
-struct MyGetOptData {
-	char *opt;
-	int numleft;
-	char **argv;
-	const char *options;
-	char *cont;
-
-	MyGetOptData(int argc, char **argv, const char *options)
-	{
-		opt = NULL;
-		numleft = argc;
-		this->argv = argv;
-		this->options = options;
-		cont = NULL;
-	}
-};
-
-static int MyGetOpt(MyGetOptData *md)
+static void WriteSavegameInfo(const char *name)
 {
-	char *s = md->cont;
-	if (s != NULL) {
-		goto md_continue_here;
+	extern uint16 _sl_version;
+	uint32 last_ottd_rev = 0;
+	byte ever_modified = 0;
+	bool removed_newgrfs = false;
+
+	GamelogInfo(_load_check_data.gamelog_action, _load_check_data.gamelog_actions, &last_ottd_rev, &ever_modified, &removed_newgrfs);
+
+	char buf[8192];
+	char *p = buf;
+	p += seprintf(p, lastof(buf), "Name:         %s\n", name);
+	p += seprintf(p, lastof(buf), "Savegame ver: %d\n", _sl_version);
+	p += seprintf(p, lastof(buf), "NewGRF ver:   0x%08X\n", last_ottd_rev);
+	p += seprintf(p, lastof(buf), "Modified:     %d\n", ever_modified);
+
+	if (removed_newgrfs) {
+		p += seprintf(p, lastof(buf), "NewGRFs have been removed\n");
 	}
 
-	for (;;) {
-		if (--md->numleft < 0) return -1;
-
-		s = *md->argv++;
-		if (*s == '-') {
-md_continue_here:;
-			s++;
-			if (*s != 0) {
-				const char *r;
-				/* Found argument, try to locate it in options. */
-				if (*s == ':' || (r = strchr(md->options, *s)) == NULL) {
-					/* ERROR! */
-					return -2;
-				}
-				if (r[1] == ':') {
-					char *t;
-					/* Item wants an argument. Check if the argument follows, or if it comes as a separate arg. */
-					if (!*(t = s + 1)) {
-						/* It comes as a separate arg. Check if out of args? */
-						if (--md->numleft < 0 || *(t = *md->argv) == '-') {
-							/* Check if item is optional? */
-							if (r[2] != ':') return -2;
-							md->numleft++;
-							t = NULL;
-						} else {
-							md->argv++;
-						}
-					}
-					md->opt = t;
-					md->cont = NULL;
-					return *s;
-				}
-				md->opt = NULL;
-				md->cont = s;
-				return *s;
-			}
-		} else {
-			/* This is currently not supported. */
-			return -2;
+	p = strecpy(p, "NewGRFs:\n", lastof(buf));
+	if (_load_check_data.HasNewGrfs()) {
+		for (GRFConfig *c = _load_check_data.grfconfig; c != NULL; c = c->next) {
+			char md5sum[33];
+			md5sumToString(md5sum, lastof(md5sum), HasBit(c->flags, GCF_COMPATIBLE) ? c->original_md5sum : c->ident.md5sum);
+			p += seprintf(p, lastof(buf), "%08X %s %s\n", c->ident.grfid, md5sum, c->filename);
 		}
 	}
+
+	/* ShowInfo put output to stderr, but version information should go
+	 * to stdout; this is the only exception */
+#if !defined(WIN32) && !defined(WIN64)
+	printf("%s\n", buf);
+#else
+	ShowInfo(buf);
+#endif
 }
 
+
 /**
  * Extract the resolution from the given string and store
  * it in the 'res' parameter.
@@ -315,45 +287,44 @@ static void ShutdownGame()
 
 	UnInitWindowSystem();
 
-	/* stop the AI */
+	/* stop the scripts */
 	AI::Uninitialize(false);
+	Game::Uninitialize(false);
 
 	/* Uninitialize variables that are allocated dynamically */
 	GamelogReset();
-	_town_pool.CleanPool();
-	_industry_pool.CleanPool();
-	_station_pool.CleanPool();
-	_roadstop_pool.CleanPool();
-	_vehicle_pool.CleanPool();
-	_sign_pool.CleanPool();
-	_order_pool.CleanPool();
-	_group_pool.CleanPool();
-	_cargopacket_pool.CleanPool();
-	_engine_pool.CleanPool();
-	_company_pool.CleanPool();
 
 #ifdef ENABLE_NETWORK
 	free(_config_file);
 #endif
 
-	ResetNewGRFData();
+	PoolBase::Clean(PT_ALL);
+
+	/* No NewGRFs were loaded when it was still bootstrapping. */
+	if (_game_mode != GM_BOOTSTRAP) ResetNewGRFData();
 
 	/* Close all and any open filehandles */
 	FioCloseAll();
+
+	UninitFreeType();
 }
 
-static void LoadIntroGame()
+/**
+ * Load the introduction game.
+ * @param load_newgrfs Whether to load the NewGRFs or not.
+ */
+static void LoadIntroGame(bool load_newgrfs = true)
 {
 	_game_mode = GM_MENU;
 
-	ResetGRFConfig(false);
+	if (load_newgrfs) ResetGRFConfig(false);
 
 	/* Setup main window */
 	ResetWindowSystem();
 	SetupColoursAndInitialWindow();
 
 	/* Load the default opening screen savegame */
-	if (SaveOrLoad("opntitle.dat", SL_LOAD, DATA_DIR) != SL_OK) {
+	if (SaveOrLoad("opntitle.dat", SL_LOAD, BASESET_DIR) != SL_OK) {
 		GenerateWorld(GWM_EMPTY, 64, 64); // if failed loading, make empty world.
 		WaitTillGeneratedWorld();
 		SetLocalCompany(COMPANY_SPECTATOR);
@@ -364,8 +335,8 @@ static void LoadIntroGame()
 	_pause_mode = PM_UNPAUSED;
 	_cursor.fix_at = false;
 
-	CheckForMissingSprites();
-	CheckForMissingGlyphsInLoadedLanguagePack();
+	if (load_newgrfs) CheckForMissingSprites();
+	CheckForMissingGlyphs();
 
 	/* Play main theme */
 	if (_music_driver->IsSongPlaying()) ResetMusic();
@@ -373,37 +344,186 @@ static void LoadIntroGame()
 
 void MakeNewgameSettingsLive()
 {
-#ifdef ENABLE_AI
 	for (CompanyID c = COMPANY_FIRST; c < MAX_COMPANIES; c++) {
 		if (_settings_game.ai_config[c] != NULL) {
 			delete _settings_game.ai_config[c];
 		}
 	}
-#endif /* ENABLE_AI */
+	if (_settings_game.game_config != NULL) {
+		delete _settings_game.game_config;
+	}
 
 	/* Copy newgame settings to active settings.
 	 * Also initialise old settings needed for savegame conversion. */
 	_settings_game = _settings_newgame;
 	_old_vds = _settings_client.company.vehicle;
 
-#ifdef ENABLE_AI
 	for (CompanyID c = COMPANY_FIRST; c < MAX_COMPANIES; c++) {
 		_settings_game.ai_config[c] = NULL;
 		if (_settings_newgame.ai_config[c] != NULL) {
 			_settings_game.ai_config[c] = new AIConfig(_settings_newgame.ai_config[c]);
 		}
 	}
-#endif /* ENABLE_AI */
+	_settings_game.game_config = NULL;
+	if (_settings_newgame.game_config != NULL) {
+		_settings_game.game_config = new GameConfig(_settings_newgame.game_config);
+	}
 }
 
+void OpenBrowser(const char *url)
+{
+	/* Make sure we only accept urls that are sure to open a browser. */
+	if (strstr(url, "http://") != url && strstr(url, "https://") != url) return;
+
+	extern void OSOpenBrowser(const char *url);
+	OSOpenBrowser(url);
+}
+
+/** Callback structure of statements to be executed after the NewGRF scan. */
+struct AfterNewGRFScan : NewGRFScanCallback {
+	Year startyear;                    ///< The start year.
+	uint generation_seed;              ///< Seed for the new game.
+	char *dedicated_host;              ///< Hostname for the dedicated server.
+	uint16 dedicated_port;             ///< Port for the dedicated server.
+	char *network_conn;                ///< Information about the server to connect to, or NULL.
+	const char *join_server_password;  ///< The password to join the server with.
+	const char *join_company_password; ///< The password to join the company with.
+	bool *save_config_ptr;             ///< The pointer to the save config setting.
+	bool save_config;                  ///< The save config setting.
+
+	/**
+	 * Create a new callback.
+	 * @param save_config_ptr Pointer to the save_config local variable which
+	 *                        decides whether to save of exit or not.
+	 */
+	AfterNewGRFScan(bool *save_config_ptr) :
+			startyear(INVALID_YEAR), generation_seed(GENERATE_NEW_SEED),
+			dedicated_host(NULL), dedicated_port(0), network_conn(NULL),
+			join_server_password(NULL), join_company_password(NULL),
+			save_config_ptr(save_config_ptr), save_config(true)
+	{
+	}
+
+	virtual void OnNewGRFsScanned()
+	{
+		ResetGRFConfig(false);
+
+		TarScanner::DoScan(TarScanner::SCENARIO);
+
+		AI::Initialize();
+		Game::Initialize();
+
+		/* We want the new (correct) NewGRF count to survive the loading. */
+		uint last_newgrf_count = _settings_client.gui.last_newgrf_count;
+		LoadFromConfig();
+		_settings_client.gui.last_newgrf_count = last_newgrf_count;
+		/* Since the default for the palette might have changed due to
+		 * reading the configuration file, recalculate that now. */
+		UpdateNewGRFConfigPalette();
+
+		Game::Uninitialize(true);
+		AI::Uninitialize(true);
+		CheckConfig();
+		LoadFromHighScore();
+		LoadHotkeysFromConfig();
+
+		/* We have loaded the config, so we may possibly save it. */
+		*save_config_ptr = save_config;
+
+		/* restore saved music volume */
+		_music_driver->SetVolume(_settings_client.music.music_vol);
+
+		if (startyear != INVALID_YEAR) _settings_newgame.game_creation.starting_year = startyear;
+		if (generation_seed != GENERATE_NEW_SEED) _settings_newgame.game_creation.generation_seed = generation_seed;
+
+#if defined(ENABLE_NETWORK)
+		if (dedicated_host != NULL) {
+			_network_bind_list.Clear();
+			*_network_bind_list.Append() = strdup(dedicated_host);
+		}
+		if (dedicated_port != 0) _settings_client.network.server_port = dedicated_port;
+#endif /* ENABLE_NETWORK */
+
+		/* initialize the ingame console */
+		IConsoleInit();
+		InitializeGUI();
+		IConsoleCmdExec("exec scripts/autoexec.scr 0");
+
+		/* Make sure _settings is filled with _settings_newgame if we switch to a game directly */
+		if (_switch_mode != SM_NONE) MakeNewgameSettingsLive();
+
+#ifdef ENABLE_NETWORK
+		if (_network_available && network_conn != NULL) {
+			const char *port = NULL;
+			const char *company = NULL;
+			uint16 rport = NETWORK_DEFAULT_PORT;
+			CompanyID join_as = COMPANY_NEW_COMPANY;
+
+			ParseConnectionString(&company, &port, network_conn);
+
+			if (company != NULL) {
+				join_as = (CompanyID)atoi(company);
+
+				if (join_as != COMPANY_SPECTATOR) {
+					join_as--;
+					if (join_as >= MAX_COMPANIES) {
+						delete this;
+						return;
+					}
+				}
+			}
+			if (port != NULL) rport = atoi(port);
+
+			LoadIntroGame();
+			_switch_mode = SM_NONE;
+			NetworkClientConnectGame(NetworkAddress(network_conn, rport), join_as, join_server_password, join_company_password);
+		}
+#endif /* ENABLE_NETWORK */
+
+		/* After the scan we're not used anymore. */
+		delete this;
+	}
+};
+
 #if defined(UNIX) && !defined(__MORPHOS__)
 extern void DedicatedFork();
 #endif
 
+/** Options of OpenTTD. */
+static const OptionData _options[] = {
+	 GETOPT_SHORT_VALUE('I'),
+	 GETOPT_SHORT_VALUE('S'),
+	 GETOPT_SHORT_VALUE('M'),
+	 GETOPT_SHORT_VALUE('m'),
+	 GETOPT_SHORT_VALUE('s'),
+	 GETOPT_SHORT_VALUE('v'),
+	 GETOPT_SHORT_VALUE('b'),
+#if defined(ENABLE_NETWORK)
+	GETOPT_SHORT_OPTVAL('D'),
+	GETOPT_SHORT_OPTVAL('n'),
+	 GETOPT_SHORT_VALUE('l'),
+	 GETOPT_SHORT_VALUE('p'),
+	 GETOPT_SHORT_VALUE('P'),
+#if !defined(__MORPHOS__) && !defined(__AMIGA__) && !defined(WIN32)
+	 GETOPT_SHORT_NOVAL('f'),
+#endif
+#endif /* ENABLE_NETWORK */
+	 GETOPT_SHORT_VALUE('r'),
+	 GETOPT_SHORT_VALUE('t'),
+	GETOPT_SHORT_OPTVAL('d'),
+	 GETOPT_SHORT_NOVAL('e'),
+	GETOPT_SHORT_OPTVAL('g'),
+	 GETOPT_SHORT_VALUE('G'),
+	 GETOPT_SHORT_VALUE('c'),
+	 GETOPT_SHORT_NOVAL('x'),
+	 GETOPT_SHORT_VALUE('q'),
+	 GETOPT_SHORT_NOVAL('h'),
+	GETOPT_END()
+};
+
+
 int ttd_main(int argc, char *argv[])
 {
-	int i;
-	const char *optformat;
 	char *musicdriver = NULL;
 	char *sounddriver = NULL;
 	char *videodriver = NULL;
@@ -412,18 +532,12 @@ int ttd_main(int argc, char *argv[])
 	char *sounds_set = NULL;
 	char *music_set = NULL;
 	Dimension resolution = {0, 0};
-	Year startyear = INVALID_YEAR;
-	uint generation_seed = GENERATE_NEW_SEED;
-	bool save_config = true;
+	/* AfterNewGRFScan sets save_config to true after scanning completed. */
+	bool save_config = false;
+	AfterNewGRFScan *scanner = new AfterNewGRFScan(&save_config);
 #if defined(ENABLE_NETWORK)
 	bool dedicated = false;
-	bool network   = false;
-	char *network_conn = NULL;
 	char *debuglog_conn = NULL;
-	char *dedicated_host = NULL;
-	uint16 dedicated_port = 0;
-	char *join_server_password = NULL;
-	char *join_company_password = NULL;
 
 	extern bool _dedicated_forks;
 	_dedicated_forks = false;
@@ -431,22 +545,12 @@ int ttd_main(int argc, char *argv[])
 
 	_game_mode = GM_MENU;
 	_switch_mode = SM_MENU;
-	_switch_mode_errorstr = INVALID_STRING_ID;
 	_config_file = NULL;
 
-	/* The last param of the following function means this:
-	 *   a letter means: it accepts that param (e.g.: -h)
-	 *   a ':' behind it means: it need a param (e.g.: -m<driver>)
-	 *   a '::' behind it means: it can optional have a param (e.g.: -d<debug>) */
-	optformat = "m:s:v:b:hD::n::ei::I:S:M:t:d::r:g::G:c:xl:p:P:"
-#if !defined(__MORPHOS__) && !defined(__AMIGA__) && !defined(WIN32)
-		"f"
-#endif
-	;
+	GetOptData mgo(argc - 1, argv + 1, _options);
 
-	MyGetOptData mgo(argc - 1, argv + 1, optformat);
-
-	while ((i = MyGetOpt(&mgo)) != -1) {
+	int i;
+	while ((i = mgo.GetOpt()) != -1) {
 		switch (i) {
 		case 'I': free(graphics_set); graphics_set = strdup(mgo.opt); break;
 		case 'S': free(sounds_set); sounds_set = strdup(mgo.opt); break;
@@ -473,27 +577,26 @@ int ttd_main(int argc, char *argv[])
 				const char *temp = NULL;
 				const char *port = NULL;
 				ParseConnectionString(&temp, &port, mgo.opt);
-				if (!StrEmpty(mgo.opt)) dedicated_host = mgo.opt;
-				if (port != NULL) dedicated_port = atoi(port);
+				if (!StrEmpty(mgo.opt)) scanner->dedicated_host = mgo.opt;
+				if (port != NULL) scanner->dedicated_port = atoi(port);
 			}
 			break;
 		case 'f': _dedicated_forks = true; break;
 		case 'n':
-			network = true;
-			network_conn = mgo.opt; // optional IP parameter, NULL if unset
+			scanner->network_conn = mgo.opt; // optional IP parameter, NULL if unset
 			break;
 		case 'l':
 			debuglog_conn = mgo.opt;
 			break;
 		case 'p':
-			join_server_password = mgo.opt;
+			scanner->join_server_password = mgo.opt;
 			break;
 		case 'P':
-			join_company_password = mgo.opt;
+			scanner->join_company_password = mgo.opt;
 			break;
 #endif /* ENABLE_NETWORK */
 		case 'r': ParseResolution(&resolution, mgo.opt); break;
-		case 't': startyear = atoi(mgo.opt); break;
+		case 't': scanner->startyear = atoi(mgo.opt); break;
 		case 'd': {
 #if defined(WIN32)
 				CreateConsole();
@@ -501,18 +604,11 @@ int ttd_main(int argc, char *argv[])
 				if (mgo.opt != NULL) SetDebugString(mgo.opt);
 				break;
 			}
-		case 'e': _switch_mode = SM_EDITOR; break;
-		case 'i':
-			/* there is an argument, it is not empty, and it is exactly 1 char long */
-			if (!StrEmpty(mgo.opt) && mgo.opt[1] == '\0') {
-				_use_palette = (PaletteType)(mgo.opt[0] - '0');
-				if (_use_palette <= MAX_PAL) break;
-			}
-			usererror("Valid value for '-i' is 0, 1 or 2");
+		case 'e': _switch_mode = (_switch_mode == SM_LOAD_GAME || _switch_mode == SM_LOAD_SCENARIO ? SM_LOAD_SCENARIO : SM_EDITOR); break;
 		case 'g':
 			if (mgo.opt != NULL) {
 				strecpy(_file_to_saveload.name, mgo.opt, lastof(_file_to_saveload.name));
-				_switch_mode = SM_LOAD;
+				_switch_mode = (_switch_mode == SM_EDITOR || _switch_mode == SM_LOAD_SCENARIO ? SM_LOAD_SCENARIO : SM_LOAD_GAME);
 				_file_to_saveload.mode = SL_LOAD;
 
 				/* if the file doesn't exist or it is not a valid savegame, let the saveload code show an error */
@@ -527,25 +623,58 @@ int ttd_main(int argc, char *argv[])
 
 			_switch_mode = SM_NEWGAME;
 			/* Give a random map if no seed has been given */
-			if (generation_seed == GENERATE_NEW_SEED) {
-				generation_seed = InteractiveRandom();
+			if (scanner->generation_seed == GENERATE_NEW_SEED) {
+				scanner->generation_seed = InteractiveRandom();
 			}
 			break;
-		case 'G': generation_seed = atoi(mgo.opt); break;
-		case 'c': _config_file = strdup(mgo.opt); break;
-		case 'x': save_config = false; break;
-		case -2:
-		case 'h':
-			/* The next two functions are needed to list the graphics sets.
-			 * We can't do them earlier because then we can't show it on
-			 * the debug console as that hasn't been configured yet. */
+		case 'q': {
 			DeterminePaths(argv[0]);
-			BaseGraphics::FindSets();
-			BaseSounds::FindSets();
-			BaseMusic::FindSets();
-			ShowHelp();
+			if (StrEmpty(mgo.opt)) return 1;
+			char title[80];
+			title[0] = '\0';
+			FiosGetSavegameListCallback(SLD_LOAD_GAME, mgo.opt, strrchr(mgo.opt, '.'), title, lastof(title));
+
+			_load_check_data.Clear();
+			SaveOrLoadResult res = SaveOrLoad(mgo.opt, SL_LOAD_CHECK, SAVE_DIR, false);
+			if (res != SL_OK || _load_check_data.HasErrors()) {
+				fprintf(stderr, "Failed to open savegame\n");
+				if (_load_check_data.HasErrors()) {
+					char buf[256];
+					SetDParamStr(0, _load_check_data.error_data);
+					GetString(buf, _load_check_data.error, lastof(buf));
+					fprintf(stderr, "%s\n", buf);
+				}
+				return 1;
+			}
+
+			WriteSavegameInfo(title);
+
 			return 0;
 		}
+		case 'G': scanner->generation_seed = atoi(mgo.opt); break;
+		case 'c': _config_file = strdup(mgo.opt); break;
+		case 'x': scanner->save_config = false; break;
+		case 'h':
+			i = -2; // Force printing of help.
+			break;
+		}
+		if (i == -2) break;
+	}
+
+	if (i == -2 || mgo.numleft > 0) {
+		/* Either the user typed '-h', he made an error, or he added unrecognized command line arguments.
+		 * In all cases, print the help, and exit.
+		 *
+		 * The next two functions are needed to list the graphics sets. We can't do them earlier
+		 * because then we cannot show it on the debug console as that hasn't been configured yet. */
+		DeterminePaths(argv[0]);
+		TarScanner::DoScan(TarScanner::BASESET);
+		BaseGraphics::FindSets();
+		BaseSounds::FindSets();
+		BaseMusic::FindSets();
+		ShowHelp();
+		delete scanner;
+		return 0;
 	}
 
 #if defined(WINCE) && defined(_DEBUG)
@@ -554,26 +683,21 @@ int ttd_main(int argc, char *argv[])
 #endif
 
 	DeterminePaths(argv[0]);
-	BaseGraphics::FindSets();
-	BaseSounds::FindSets();
-	BaseMusic::FindSets();
+	TarScanner::DoScan(TarScanner::BASESET);
 
-#if defined(ENABLE_NETWORK) && defined(UNIX) && !defined(__MORPHOS__)
+#if defined(ENABLE_NETWORK)
+	if (dedicated) DEBUG(net, 0, "Starting dedicated version %s", _openttd_revision);
+	if (_dedicated_forks && !dedicated) _dedicated_forks = false;
+
+#if defined(UNIX) && !defined(__MORPHOS__)
 	/* We must fork here, or we'll end up without some resources we need (like sockets) */
 	if (_dedicated_forks) DedicatedFork();
+#endif
 #endif
 
-	TarScanner::DoScan();
-	AI::Initialize();
-	LoadFromConfig();
-	AI::Uninitialize(true);
-	CheckConfig();
-	LoadFromHighScore();
-	LoadHotkeysFromConfig();
+	LoadFromConfig(true);
 
-	if (resolution.width != 0) { _cur_resolution = resolution; }
-	if (startyear != INVALID_YEAR) _settings_newgame.game_creation.starting_year = startyear;
-	if (generation_seed != GENERATE_NEW_SEED) _settings_newgame.game_creation.generation_seed = generation_seed;
+	if (resolution.width != 0) _cur_resolution = resolution;
 
 	/*
 	 * The width and height must be at least 1 pixel and width times
@@ -584,78 +708,50 @@ int ttd_main(int argc, char *argv[])
 	_cur_resolution.width  = ClampU(_cur_resolution.width,  1, UINT16_MAX / 2);
 	_cur_resolution.height = ClampU(_cur_resolution.height, 1, UINT16_MAX / 2);
 
-#if defined(ENABLE_NETWORK)
-	if (dedicated) DEBUG(net, 0, "Starting dedicated version %s", _openttd_revision);
-	if (dedicated_host) {
-		_network_bind_list.Clear();
-		*_network_bind_list.Append() = strdup(dedicated_host);
-	}
-	if (dedicated_port) _settings_client.network.server_port = dedicated_port;
-	if (_dedicated_forks && !dedicated) _dedicated_forks = false;
-#endif /* ENABLE_NETWORK */
+	/* Assume the cursor starts within the game as not all video drivers
+	 * get an event that the cursor is within the window when it is opened.
+	 * Saying the cursor is there makes no visible difference as it would
+	 * just be out of the bounds of the window. */
+	_cursor.in_window = true;
 
 	/* enumerate language files */
 	InitializeLanguagePacks();
 
-	/* initialize screenshot formats */
-	InitializeScreenshotFormats();
-
-	/* Initialize FreeType */
-	InitFreeType();
+	/* Initialize the regular font for FreeType */
+	InitFreeType(false);
 
 	/* This must be done early, since functions use the SetWindowDirty* calls */
 	InitWindowSystem();
 
-	/* Look for the sounds before the graphics. Otherwise none would be set and
-	 * the first initialisation of the video happens on the wrong data. Now it
-	 * can do the first initialisation right. */
-	if (sounds_set == NULL && BaseSounds::ini_set != NULL) sounds_set = strdup(BaseSounds::ini_set);
-	if (!BaseSounds::SetSet(sounds_set)) {
-		StrEmpty(sounds_set) ?
-			usererror("Failed to find a sounds set. Please acquire a sounds set for OpenTTD. See section 4.1 of readme.txt.") :
-			usererror("Failed to select requested sounds set '%s'", sounds_set);
-	}
-	free(sounds_set);
-
+	BaseGraphics::FindSets();
 	if (graphics_set == NULL && BaseGraphics::ini_set != NULL) graphics_set = strdup(BaseGraphics::ini_set);
 	if (!BaseGraphics::SetSet(graphics_set)) {
-		StrEmpty(graphics_set) ?
-			usererror("Failed to find a graphics set. Please acquire a graphics set for OpenTTD. See section 4.1 of readme.txt.") :
-			usererror("Failed to select requested graphics set '%s'", graphics_set);
+		if (!StrEmpty(graphics_set)) {
+			BaseGraphics::SetSet(NULL);
+
+			ErrorMessageData msg(STR_CONFIG_ERROR, STR_CONFIG_ERROR_INVALID_BASE_GRAPHICS_NOT_FOUND);
+			msg.SetDParamStr(0, graphics_set);
+			ScheduleErrorMessage(msg);
+		}
 	}
 	free(graphics_set);
 
-	if (music_set == NULL && BaseMusic::ini_set != NULL) music_set = strdup(BaseMusic::ini_set);
-	if (!BaseMusic::SetSet(music_set)) {
-		StrEmpty(music_set) ?
-			usererror("Failed to find a music set. Please acquire a music set for OpenTTD. See section 4.1 of readme.txt.") :
-			usererror("Failed to select requested music set '%s'", music_set);
-	}
-	free(music_set);
-
 	/* Initialize game palette */
 	GfxInitPalettes();
 
 	DEBUG(misc, 1, "Loading blitter...");
 	if (blitter == NULL && _ini_blitter != NULL) blitter = strdup(_ini_blitter);
-	if (BlitterFactoryBase::SelectBlitter(blitter) == NULL) {
-		StrEmpty(blitter) ?
-			usererror("Failed to autoprobe blitter") :
-			usererror("Failed to select requested blitter '%s'; does it exist?", blitter);
+	_blitter_autodetected = StrEmpty(blitter);
+	/* If we have a 32 bpp base set, try to select the 32 bpp blitter first, but only if we autoprobe the blitter. */
+	if (!_blitter_autodetected || BaseGraphics::GetUsedSet() == NULL || BaseGraphics::GetUsedSet()->blitter == BLT_8BPP || BlitterFactoryBase::SelectBlitter("32bpp-anim") == NULL) {
+		if (BlitterFactoryBase::SelectBlitter(blitter) == NULL) {
+			StrEmpty(blitter) ?
+				usererror("Failed to autoprobe blitter") :
+				usererror("Failed to select requested blitter '%s'; does it exist?", blitter);
+		}
 	}
 	free(blitter);
 
-	DEBUG(driver, 1, "Loading drivers...");
-
-	if (sounddriver == NULL && _ini_sounddriver != NULL) sounddriver = strdup(_ini_sounddriver);
-	_sound_driver = (SoundDriver*)SoundDriverFactoryBase::SelectDriver(sounddriver, Driver::DT_SOUND);
-	if (_sound_driver == NULL) {
-		StrEmpty(sounddriver) ?
-			usererror("Failed to autoprobe sound driver") :
-			usererror("Failed to select requested sound driver '%s'", sounddriver);
-	}
-	free(sounddriver);
-
 	if (videodriver == NULL && _ini_videodriver != NULL) videodriver = strdup(_ini_videodriver);
 	_video_driver = (VideoDriver*)VideoDriverFactoryBase::SelectDriver(videodriver, Driver::DT_VIDEO);
 	if (_video_driver == NULL) {
@@ -665,21 +761,9 @@ int ttd_main(int argc, char *argv[])
 	}
 	free(videodriver);
 
-	if (musicdriver == NULL && _ini_musicdriver != NULL) musicdriver = strdup(_ini_musicdriver);
-	_music_driver = (MusicDriver*)MusicDriverFactoryBase::SelectDriver(musicdriver, Driver::DT_MUSIC);
-	if (_music_driver == NULL) {
-		StrEmpty(musicdriver) ?
-			usererror("Failed to autoprobe music driver") :
-			usererror("Failed to select requested music driver '%s'", musicdriver);
-	}
-	free(musicdriver);
-
 	/* Initialize the zoom level of the screen to normal */
 	_screen.zoom = ZOOM_LVL_NORMAL;
 
-	/* restore saved music volume */
-	_music_driver->SetVolume(_msf.music_vol);
-
 	NetworkStartUp(); // initialize network-core
 
 #if defined(ENABLE_NETWORK)
@@ -697,54 +781,69 @@ int ttd_main(int argc, char *argv[])
 	}
 #endif /* ENABLE_NETWORK */
 
-	ScanNewGRFFiles();
+	if (!HandleBootstrap()) goto exit;
 
-	ResetGRFConfig(false);
+	_video_driver->ClaimMousePointer();
 
-	/* Make sure _settings is filled with _settings_newgame if we switch to a game directly */
-	if (_switch_mode != SM_NONE) MakeNewgameSettingsLive();
+	/* initialize screenshot formats */
+	InitializeScreenshotFormats();
 
-	/* initialize the ingame console */
-	IConsoleInit();
-	_cursor.in_window = true;
-	InitializeGUI();
-	IConsoleCmdExec("exec scripts/autoexec.scr 0");
+	BaseSounds::FindSets();
+	if (sounds_set == NULL && BaseSounds::ini_set != NULL) sounds_set = strdup(BaseSounds::ini_set);
+	if (!BaseSounds::SetSet(sounds_set)) {
+		if (StrEmpty(sounds_set) || !BaseSounds::SetSet(NULL)) {
+			usererror("Failed to find a sounds set. Please acquire a sounds set for OpenTTD. See section 4.1 of readme.txt.");
+		} else {
+			ErrorMessageData msg(STR_CONFIG_ERROR, STR_CONFIG_ERROR_INVALID_BASE_SOUNDS_NOT_FOUND);
+			msg.SetDParamStr(0, sounds_set);
+			ScheduleErrorMessage(msg);
+		}
+	}
+	free(sounds_set);
+
+	BaseMusic::FindSets();
+	if (music_set == NULL && BaseMusic::ini_set != NULL) music_set = strdup(BaseMusic::ini_set);
+	if (!BaseMusic::SetSet(music_set)) {
+		if (StrEmpty(music_set) || !BaseMusic::SetSet(NULL)) {
+			usererror("Failed to find a music set. Please acquire a music set for OpenTTD. See section 4.1 of readme.txt.");
+		} else {
+			ErrorMessageData msg(STR_CONFIG_ERROR, STR_CONFIG_ERROR_INVALID_BASE_MUSIC_NOT_FOUND);
+			msg.SetDParamStr(0, music_set);
+			ScheduleErrorMessage(msg);
+		}
+	}
+	free(music_set);
+
+	if (sounddriver == NULL && _ini_sounddriver != NULL) sounddriver = strdup(_ini_sounddriver);
+	_sound_driver = (SoundDriver*)SoundDriverFactoryBase::SelectDriver(sounddriver, Driver::DT_SOUND);
+	if (_sound_driver == NULL) {
+		StrEmpty(sounddriver) ?
+			usererror("Failed to autoprobe sound driver") :
+			usererror("Failed to select requested sound driver '%s'", sounddriver);
+	}
+	free(sounddriver);
+
+	if (musicdriver == NULL && _ini_musicdriver != NULL) musicdriver = strdup(_ini_musicdriver);
+	_music_driver = (MusicDriver*)MusicDriverFactoryBase::SelectDriver(musicdriver, Driver::DT_MUSIC);
+	if (_music_driver == NULL) {
+		StrEmpty(musicdriver) ?
+			usererror("Failed to autoprobe music driver") :
+			usererror("Failed to select requested music driver '%s'", musicdriver);
+	}
+	free(musicdriver);
 
 	/* Take our initial lock on whatever we might want to do! */
-	_genworld_paint_mutex->BeginCritical();
-	_genworld_mapgen_mutex->BeginCritical();
+	_modal_progress_paint_mutex->BeginCritical();
+	_modal_progress_work_mutex->BeginCritical();
 
 	GenerateWorld(GWM_EMPTY, 64, 64); // Make the viewport initialization happy
 	WaitTillGeneratedWorld();
 
-	CheckForMissingGlyphsInLoadedLanguagePack();
+	LoadIntroGame(false);
 
-#ifdef ENABLE_NETWORK
-	if (network && _network_available) {
-		if (network_conn != NULL) {
-			const char *port = NULL;
-			const char *company = NULL;
-			uint16 rport = NETWORK_DEFAULT_PORT;
-			CompanyID join_as = COMPANY_NEW_COMPANY;
+	CheckForMissingGlyphs();
 
-			ParseConnectionString(&company, &port, network_conn);
-
-			if (company != NULL) {
-				join_as = (CompanyID)atoi(company);
-
-				if (join_as != COMPANY_SPECTATOR) {
-					join_as--;
-					if (join_as >= MAX_COMPANIES) return false;
-				}
-			}
-			if (port != NULL) rport = atoi(port);
-
-			LoadIntroGame();
-			_switch_mode = SM_NONE;
-			NetworkClientConnectGame(NetworkAddress(network_conn, rport), join_as, join_server_password, join_company_password);
-		}
-	}
-#endif /* ENABLE_NETWORK */
+	ScanNewGRFFiles(scanner);
 
 	_video_driver->MainLoop();
 
@@ -757,12 +856,13 @@ int ttd_main(int argc, char *argv[])
 		SaveToHighScore();
 	}
 
+exit:
 	/* Reset windowing system, stop drivers, free used memory, ... */
 	ShutdownGame();
 
-	free(const_cast<char *>(BaseGraphics::ini_set));
-	free(const_cast<char *>(BaseSounds::ini_set));
-	free(const_cast<char *>(BaseMusic::ini_set));
+	free(BaseGraphics::ini_set);
+	free(BaseSounds::ini_set);
+	free(BaseMusic::ini_set);
 	free(_ini_musicdriver);
 	free(_ini_sounddriver);
 	free(_ini_videodriver);
@@ -773,7 +873,7 @@ int ttd_main(int argc, char *argv[])
 
 void HandleExitGameRequest()
 {
-	if (_game_mode == GM_MENU) { // do not ask to quit on the main screen
+	if (_game_mode == GM_MENU || _game_mode == GM_BOOTSTRAP) { // do not ask to quit on the main screen
 		_exit_game = true;
 	} else if (_settings_client.gui.autosave_on_exit) {
 		DoExitSave();
@@ -788,8 +888,9 @@ static void MakeNewGameDone()
 	SettingsDisableElrail(_settings_game.vehicle.disable_elrails);
 
 	/* In a dedicated server, the server does not play */
-	if (_network_dedicated || BlitterFactoryBase::GetCurrentBlitter()->GetScreenDepth() == 0) {
+	if (!_video_driver->HasGUI()) {
 		SetLocalCompany(COMPANY_SPECTATOR);
+		if (_settings_client.gui.pause_on_newgame) DoCommandP(0, PM_PAUSED_NORMAL, 1, CMD_PAUSE);
 		IConsoleCmdExec("exec scripts/game_start.scr 0");
 		return;
 	}
@@ -816,6 +917,7 @@ static void MakeNewGameDone()
 
 	if (_settings_client.gui.pause_on_newgame) DoCommandP(0, PM_PAUSED_NORMAL, 1, CMD_PAUSE);
 
+	CheckEngines();
 	MarkWholeScreenDirty();
 }
 
@@ -900,10 +1002,10 @@ void SwitchToMode(SwitchMode new_mode)
 {
 #ifdef ENABLE_NETWORK
 	/* If we are saving something, the network stays in his current state */
-	if (new_mode != SM_SAVE) {
+	if (new_mode != SM_SAVE_GAME) {
 		/* If the network is active, make it not-active */
 		if (_networking) {
-			if (_network_server && (new_mode == SM_LOAD || new_mode == SM_NEWGAME || new_mode == SM_RESTARTGAME)) {
+			if (_network_server && (new_mode == SM_LOAD_GAME || new_mode == SM_NEWGAME || new_mode == SM_RESTARTGAME)) {
 				NetworkReboot();
 			} else {
 				NetworkDisconnect();
@@ -928,8 +1030,8 @@ void SwitchToMode(SwitchMode new_mode)
 		}
 	}
 #endif /* ENABLE_NETWORK */
-	/* Make sure all AI controllers are gone at quiting game */
-	if (new_mode != SM_SAVE) AI::KillAll();
+	/* Make sure all AI controllers are gone at quitting game */
+	if (new_mode != SM_SAVE_GAME) AI::KillAll();
 
 	switch (new_mode) {
 		case SM_EDITOR: // Switch to scenario editor
@@ -946,7 +1048,7 @@ void SwitchToMode(SwitchMode new_mode)
 			MakeNewGame(false, new_mode == SM_NEWGAME);
 			break;
 
-		case SM_LOAD: { // Load game, Play Scenario
+		case SM_LOAD_GAME: { // Load game, Play Scenario
 			ResetGRFConfig(true);
 			ResetWindowSystem();
 
@@ -955,7 +1057,8 @@ void SwitchToMode(SwitchMode new_mode)
 				ShowErrorMessage(STR_JUST_RAW_STRING, INVALID_STRING_ID, WL_ERROR);
 			} else {
 				if (_saveload_mode == SLD_LOAD_SCENARIO) {
-					StartupEngines();
+					/* Reset engine pool to simplify changing engine NewGRFs in scenario editor. */
+					EngineOverrideManager::ResetToCurrentNewGRFConfig();
 				}
 				/* Update the local company for a loaded game. It is either always
 				 * company #1 (eg 0) or in the case of a dedicated server a spectator */
@@ -1010,7 +1113,7 @@ void SwitchToMode(SwitchMode new_mode)
 			}
 			break;
 
-		case SM_SAVE: // Save game
+		case SM_SAVE_GAME: // Save game.
 			/* Make network saved games on pause compatible to singleplayer */
 			if (SaveOrLoad(_file_to_saveload.name, SL_SAVE, NO_DIRECTORY) != SL_OK) {
 				SetDParamStr(0, GetSaveLoadErrorString());
@@ -1020,6 +1123,11 @@ void SwitchToMode(SwitchMode new_mode)
 			}
 			break;
 
+		case SM_SAVE_HEIGHTMAP: // Save heightmap.
+			MakeHeightmapScreenshot(_file_to_saveload.name);
+			DeleteWindowById(WC_SAVELOAD, 0);
+			break;
+
 		case SM_GENRANDLAND: // Generate random land within scenario editor
 			SetLocalCompany(OWNER_NONE);
 			GenerateWorld(GWM_RANDOM, 1 << _settings_game.game_creation.map_x, 1 << _settings_game.game_creation.map_y);
@@ -1029,11 +1137,6 @@ void SwitchToMode(SwitchMode new_mode)
 
 		default: NOT_REACHED();
 	}
-
-	if (_switch_mode_errorstr != INVALID_STRING_ID) {
-		ShowErrorMessage(_switch_mode_errorstr, INVALID_STRING_ID, WL_CRITICAL);
-		_switch_mode_errorstr = INVALID_STRING_ID;
-	}
 }
 
 
@@ -1049,6 +1152,41 @@ static void CheckCaches()
 	 * always to aid testing of caches. */
 	if (_debug_desync_level <= 1) return;
 
+	/* Check the town caches. */
+	SmallVector<TownCache, 4> old_town_caches;
+	Town *t;
+	FOR_ALL_TOWNS(t) {
+		MemCpyT(old_town_caches.Append(), &t->cache);
+	}
+
+	extern void RebuildTownCaches();
+	RebuildTownCaches();
+	RebuildSubsidisedSourceAndDestinationCache();
+
+	uint i = 0;
+	FOR_ALL_TOWNS(t) {
+		if (MemCmpT(old_town_caches.Get(i), &t->cache) != 0) {
+			DEBUG(desync, 2, "town cache mismatch: town %i", (int)t->index);
+		}
+		i++;
+	}
+
+	/* Check company infrastructure cache. */
+	SmallVector<CompanyInfrastructure, 4> old_infrastructure;
+	Company *c;
+	FOR_ALL_COMPANIES(c) MemCpyT(old_infrastructure.Append(), &c->infrastructure);
+
+	extern void AfterLoadCompanyStats();
+	AfterLoadCompanyStats();
+
+	i = 0;
+	FOR_ALL_COMPANIES(c) {
+		if (MemCmpT(old_infrastructure.Get(i), &c->infrastructure) != 0) {
+			DEBUG(desync, 2, "infrastructure cache mismatch: company %i", (int)c->index);
+		}
+		i++;
+	}
+
 	/* Strict checking of the road stop cache entries */
 	const RoadStop *rs;
 	FOR_ALL_ROADSTOPS(rs) {
@@ -1160,16 +1298,19 @@ static void CheckCaches()
  */
 void StateGameLoop()
 {
-	/* dont execute the state loop during pause */
+	/* don't execute the state loop during pause */
 	if (_pause_mode != PM_UNPAUSED) {
 		UpdateLandscapingLimits();
+		Game::GameLoop();
 		CallWindowTickEvent();
 		return;
 	}
-	if (IsGeneratingWorld()) return;
+	if (HasModalProgress()) return;
 
 	ClearStorageChanges(false);
 
+	Layouter::ReduceLineCache();
+
 	if (_game_mode == GM_EDITOR) {
 		RunTileLoop();
 		CallVehicleTicks();
@@ -1201,6 +1342,7 @@ void StateGameLoop()
 		ClearStorageChanges(true);
 
 		AI::GameLoop();
+		Game::GameLoop();
 		UpdateLandscapingLimits();
 
 		CallWindowTickEvent();
@@ -1244,6 +1386,15 @@ static void DoAutosave()
 
 void GameLoop()
 {
+	if (_game_mode == GM_BOOTSTRAP) {
+#ifdef ENABLE_NETWORK
+		/* Check for UDP stuff */
+		if (_network_available) NetworkBackgroundLoop();
+#endif
+		InputLoop();
+		return;
+	}
+
 	ProcessAsyncSaveFinish();
 
 	/* autosave game? */
@@ -1254,7 +1405,7 @@ void GameLoop()
 	}
 
 	/* switch game mode? */
-	if (_switch_mode != SM_NONE) {
+	if (_switch_mode != SM_NONE && !HasModalProgress()) {
 		SwitchToMode(_switch_mode);
 		_switch_mode = SM_NONE;
 	}
@@ -1270,7 +1421,7 @@ void GameLoop()
 	/* Check for UDP stuff */
 	if (_network_available) NetworkBackgroundLoop();
 
-	if (_networking && !IsGeneratingWorld()) {
+	if (_networking && !HasModalProgress()) {
 		/* Multiplayer */
 		NetworkGameLoop();
 	} else {
