@@ -1,4 +1,4 @@
-/* $Id: macos.mm 18074 2009-11-14 15:26:43Z michi_cc $ */
+/* $Id: macos.mm 25981 2013-11-13 21:30:39Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -7,6 +7,8 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
+/** @file macos.mm Code related to MacOSX. */
+
 #include "../../stdafx.h"
 #include "../../core/bitmath_func.hpp"
 #include "../../rev.h"
@@ -22,9 +24,6 @@
 /*
  * This file contains objective C
  * Apple uses objective C instead of plain C to interact with OS specific/native functions
- *
- * Note: TrueLight's crosscompiler can handle this, but it likely needs a manual modification for each change in this file.
- * To insure that the crosscompiler still works, let him try any changes before they are committed
  */
 
 
@@ -57,6 +56,13 @@ void GetMacOSVersion(int *return_major, int *return_minor, int *return_bugfix)
 
 #ifdef WITH_SDL
 
+/**
+ * Show the system dialogue message (SDL on MacOSX).
+ *
+ * @param title Window title.
+ * @param message Message text.
+ * @param buttonLabel Button text.
+ */
 void ShowMacDialog(const char *title, const char *message, const char *buttonLabel)
 {
 	NSRunAlertPanel([ NSString stringWithUTF8String:title ], [ NSString stringWithUTF8String:message ], [ NSString stringWithUTF8String:buttonLabel ], nil, nil);
@@ -66,6 +72,13 @@ void ShowMacDialog(const char *title, const char *message, const char *buttonLab
 
 extern void CocoaDialog(const char *title, const char *message, const char *buttonLabel);
 
+/**
+ * Show the system dialogue message (Cocoa on MacOSX).
+ *
+ * @param title Window title.
+ * @param message Message text.
+ * @param buttonLabel Button text.
+ */
 void ShowMacDialog(const char *title, const char *message, const char *buttonLabel)
 {
 	CocoaDialog(title, message, buttonLabel);
@@ -74,6 +87,13 @@ void ShowMacDialog(const char *title, const char *message, const char *buttonLab
 
 #else
 
+/**
+ * Show the system dialogue message (console on MacOSX).
+ *
+ * @param title Window title.
+ * @param message Message text.
+ * @param buttonLabel Button text.
+ */
 void ShowMacDialog(const char *title, const char *message, const char *buttonLabel)
 {
 	fprintf(stderr, "%s: %s\n", title, message);
@@ -82,18 +102,30 @@ void ShowMacDialog(const char *title, const char *message, const char *buttonLab
 #endif
 
 
+/**
+ * Show an error message.
+ *
+ * @param buf error message text.
+ * @param system message text originates from OS.
+ */
 void ShowOSErrorBox(const char *buf, bool system)
 {
 	/* Display the error in the best way possible. */
 	if (system) {
 		ShowMacDialog("OpenTTD has encountered an error", buf, "Quit");
 	} else {
-		ShowMacDialog(buf, "See the readme for more info.\nMost likely you are missing files from the original TTD.", "Quit");
+		ShowMacDialog(buf, "See the readme for more info.", "Quit");
 	}
 }
 
+void OSOpenBrowser(const char *url)
+{
+	[ [ NSWorkspace sharedWorkspace ] openURL:[ NSURL URLWithString:[ NSString stringWithUTF8String:url ] ] ];
+}
 
-/** Determine the current user's locale. */
+/**
+ * Determine and return the current user's locale.
+ */
 const char *GetCurrentLocale(const char *)
 {
 	static char retbuf[32] = { '\0' };
@@ -120,6 +152,13 @@ const char *GetCurrentLocale(const char *)
 
 
 #ifdef WITH_COCOA
+/**
+ * Return the contents of the clipboard (COCOA).
+ *
+ * @param buffer Clipboard content..
+ * @param buff_len Length of the clipboard content..
+ * @return Whether clipboard is empty or not.
+ */
 bool GetClipboardContents(char *buffer, size_t buff_len)
 {
 	NSPasteboard *pb = [ NSPasteboard generalPasteboard ];
@@ -137,3 +176,32 @@ bool GetClipboardContents(char *buffer, size_t buff_len)
 	return true;
 }
 #endif
+
+uint GetCPUCoreCount()
+{
+	uint count = 1;
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
+	if (MacOSVersionIsAtLeast(10, 5, 0)) {
+		count = [ [ NSProcessInfo processInfo ] activeProcessorCount ];
+	} else
+#endif
+	{
+#if (MAC_OS_X_VERSION_MIN_REQUIRED < MAC_OS_X_VERSION_10_5)
+		count = MPProcessorsScheduled();
+#endif
+	}
+
+	return count;
+}
+
+/**
+ * Check if a font is a monospace font.
+ * @param name Name of the font.
+ * @return True if the font is a monospace font.
+ */
+bool IsMonospaceFont(CFStringRef name)
+{
+	NSFont *font = [ NSFont fontWithName:(NSString *)name size:0.0f ];
+
+	return font != NULL ? [ font isFixedPitch ] : false;
+}
