@@ -1,4 +1,4 @@
-/* $Id: crashlog_win.cpp 18855 2010-01-18 10:11:27Z smatz $ */
+/* $Id: crashlog_win.cpp 25982 2013-11-13 21:35:44Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -19,10 +19,21 @@
 #include "../../strings_func.h"
 #include "../../gamelog.h"
 #include "../../saveload/saveload.h"
+#include "../../video/video_driver.hpp"
 
 #include <windows.h>
 #include <signal.h>
 
+static const uint MAX_SYMBOL_LEN = 512;
+static const uint MAX_FRAMES     = 64;
+
+/* printf format specification for 32/64-bit addresses. */
+#ifdef _M_AMD64
+#define PRINTF_PTR "0x%016IX"
+#else
+#define PRINTF_PTR "0x%08X"
+#endif
+
 /**
  * Windows implementation for the crash logger.
  */
@@ -38,6 +49,9 @@ class CrashLogWindows : public CrashLog {
 public:
 #if defined(_MSC_VER)
 	/* virtual */ int WriteCrashDump(char *filename, const char *filename_last) const;
+	char *AppendDecodedStacktrace(char *buffer, const char *last) const;
+#else
+	char *AppendDecodedStacktrace(char *buffer, const char *last) const { return buffer; }
 #endif /* _MSC_VER */
 
 	/** Buffer for the generated crash log */
@@ -152,8 +166,9 @@ static void GetFileInfo(DebugFileInfo *dfi, const TCHAR *filename)
 		uint32 crc = (uint32)-1;
 
 		for (;;) {
-			if (ReadFile(file, buffer, sizeof(buffer), &numread, NULL) == 0 || numread == 0)
+			if (ReadFile(file, buffer, sizeof(buffer), &numread, NULL) == 0 || numread == 0) {
 				break;
+			}
 			filesize += numread;
 			crc = CalcCRC(buffer, numread, crc);
 		}
@@ -176,7 +191,7 @@ static char *PrintModuleInfo(char *output, const char *last, HMODULE mod)
 	GetModuleFileName(mod, buffer, MAX_PATH);
 	GetFileInfo(&dfi, buffer);
 	output += seprintf(output, last, " %-20s handle: %p size: %d crc: %.8X date: %d-%.2d-%.2d %.2d:%.2d:%.2d\n",
-		WIDE_TO_MB(buffer),
+		FS2OTTD(buffer),
 		mod,
 		dfi.size,
 		dfi.crc32,
@@ -223,11 +238,11 @@ static char *PrintModuleInfo(char *output, const char *last, HMODULE mod)
 	buffer += seprintf(buffer, last, "Registers:\n");
 #ifdef _M_AMD64
 	buffer += seprintf(buffer, last,
-		" RAX: %.16llX RBX: %.16llX RCX: %.16llX RDX: %.16llX\n"
-		" RSI: %.16llX RDI: %.16llX RBP: %.16llX RSP: %.16llX\n"
-		" R8:  %.16llX R9:  %.16llX R10: %.16llX R11: %.16llX\n"
-		" R12: %.16llX R13: %.16llX R14: %.16llX R15: %.16llX\n"
-		" RIP: %.16llX EFLAGS: %.8X\n",
+		" RAX: %.16I64X RBX: %.16I64X RCX: %.16I64X RDX: %.16I64X\n"
+		" RSI: %.16I64X RDI: %.16I64X RBP: %.16I64X RSP: %.16I64X\n"
+		" R8:  %.16I64X R9:  %.16I64X R10: %.16I64X R11: %.16I64X\n"
+		" R12: %.16I64X R13: %.16I64X R14: %.16I64X R15: %.16I64X\n"
+		" RIP: %.16I64X EFLAGS: %.8lX\n",
 		ep->ContextRecord->Rax,
 		ep->ContextRecord->Rbx,
 		ep->ContextRecord->Rcx,
@@ -307,6 +322,119 @@ static char *PrintModuleInfo(char *output, const char *last, HMODULE mod)
 #if defined(_MSC_VER)
 #include <dbghelp.h>
 
+char *CrashLogWindows::AppendDecodedStacktrace(char *buffer, const char *last) const
+{
+#define M(x) x "\0"
+	static const char dbg_import[] =
+		M("dbghelp.dll")
+		M("SymInitialize")
+		M("SymSetOptions")
+		M("SymCleanup")
+		M("StackWalk64")
+		M("SymFunctionTableAccess64")
+		M("SymGetModuleBase64")
+		M("SymGetModuleInfo64")
+		M("SymGetSymFromAddr64")
+		M("SymGetLineFromAddr64")
+		M("")
+		;
+#undef M
+
+	struct ProcPtrs {
+		BOOL (WINAPI * pSymInitialize)(HANDLE, PCSTR, BOOL);
+		BOOL (WINAPI * pSymSetOptions)(DWORD);
+		BOOL (WINAPI * pSymCleanup)(HANDLE);
+		BOOL (WINAPI * pStackWalk64)(DWORD, HANDLE, HANDLE, LPSTACKFRAME64, PVOID, PREAD_PROCESS_MEMORY_ROUTINE64, PFUNCTION_TABLE_ACCESS_ROUTINE64, PGET_MODULE_BASE_ROUTINE64, PTRANSLATE_ADDRESS_ROUTINE64);
+		PVOID (WINAPI * pSymFunctionTableAccess64)(HANDLE, DWORD64);
+		DWORD64 (WINAPI * pSymGetModuleBase64)(HANDLE, DWORD64);
+		BOOL (WINAPI * pSymGetModuleInfo64)(HANDLE, DWORD64, PIMAGEHLP_MODULE64);
+		BOOL (WINAPI * pSymGetSymFromAddr64)(HANDLE, DWORD64, PDWORD64, PIMAGEHLP_SYMBOL64);
+		BOOL (WINAPI * pSymGetLineFromAddr64)(HANDLE, DWORD64, PDWORD, PIMAGEHLP_LINE64);
+	} proc;
+
+	buffer += seprintf(buffer, last, "\nDecoded stack trace:\n");
+
+	/* Try to load the functions from the DLL, if that fails because of a too old dbghelp.dll, just skip it. */
+	if (LoadLibraryList((Function*)&proc, dbg_import)) {
+		/* Initialize symbol handler. */
+		HANDLE hCur = GetCurrentProcess();
+		proc.pSymInitialize(hCur, NULL, TRUE);
+		/* Load symbols only when needed, fail silently on errors, demangle symbol names. */
+		proc.pSymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_FAIL_CRITICAL_ERRORS | SYMOPT_UNDNAME);
+
+		/* Initialize starting stack frame from context record. */
+		STACKFRAME64 frame;
+		memset(&frame, 0, sizeof(frame));
+#ifdef _M_AMD64
+		frame.AddrPC.Offset = ep->ContextRecord->Rip;
+		frame.AddrFrame.Offset = ep->ContextRecord->Rbp;
+		frame.AddrStack.Offset = ep->ContextRecord->Rsp;
+#else
+		frame.AddrPC.Offset = ep->ContextRecord->Eip;
+		frame.AddrFrame.Offset = ep->ContextRecord->Ebp;
+		frame.AddrStack.Offset = ep->ContextRecord->Esp;
+#endif
+		frame.AddrPC.Mode = AddrModeFlat;
+		frame.AddrFrame.Mode = AddrModeFlat;
+		frame.AddrStack.Mode = AddrModeFlat;
+
+		/* Copy context record as StackWalk64 may modify it. */
+		CONTEXT ctx;
+		memcpy(&ctx, ep->ContextRecord, sizeof(ctx));
+
+		/* Allocate space for symbol info. */
+		IMAGEHLP_SYMBOL64 *sym_info = (IMAGEHLP_SYMBOL64*)alloca(sizeof(IMAGEHLP_SYMBOL64) + MAX_SYMBOL_LEN - 1);
+		sym_info->SizeOfStruct = sizeof(IMAGEHLP_SYMBOL64);
+		sym_info->MaxNameLength = MAX_SYMBOL_LEN;
+
+		/* Walk stack at most MAX_FRAMES deep in case the stack is corrupt. */
+		for (uint num = 0; num < MAX_FRAMES; num++) {
+			if (!proc.pStackWalk64(
+#ifdef _M_AMD64
+				IMAGE_FILE_MACHINE_AMD64,
+#else
+				IMAGE_FILE_MACHINE_I386,
+#endif
+				hCur, GetCurrentThread(), &frame, &ctx, NULL, proc.pSymFunctionTableAccess64, proc.pSymGetModuleBase64, NULL)) break;
+
+			if (frame.AddrPC.Offset == frame.AddrReturn.Offset) {
+				buffer += seprintf(buffer, last, " <infinite loop>\n");
+				break;
+			}
+
+			/* Get module name. */
+			const char *mod_name = "???";
+
+			IMAGEHLP_MODULE64 module;
+			module.SizeOfStruct = sizeof(module);
+			if (proc.pSymGetModuleInfo64(hCur, frame.AddrPC.Offset, &module)) {
+				mod_name = module.ModuleName;
+			}
+
+			/* Print module and instruction pointer. */
+			buffer += seprintf(buffer, last, "[%02d] %-20s " PRINTF_PTR, num, mod_name, frame.AddrPC.Offset);
+
+			/* Get symbol name and line info if possible. */
+			DWORD64 offset;
+			if (proc.pSymGetSymFromAddr64(hCur, frame.AddrPC.Offset, &offset, sym_info)) {
+				buffer += seprintf(buffer, last, " %s + %I64u", sym_info->Name, offset);
+
+				DWORD line_offs;
+				IMAGEHLP_LINE64 line;
+				line.SizeOfStruct = sizeof(IMAGEHLP_LINE64);
+				if (proc.pSymGetLineFromAddr64(hCur, frame.AddrPC.Offset, &line_offs, &line)) {
+					buffer += seprintf(buffer, last, " (%s:%d)", line.FileName, line.LineNumber);
+				}
+			}
+			buffer += seprintf(buffer, last, "\n");
+		}
+
+		proc.pSymCleanup(hCur);
+	}
+
+	return buffer + seprintf(buffer, last, "\n*** End of additional info ***\n");
+}
+
 /* virtual */ int CrashLogWindows::WriteCrashDump(char *filename, const char *filename_last) const
 {
 	int ret = 0;
@@ -367,7 +495,7 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 
 	if (GamelogTestEmergency()) {
 		static const TCHAR _emergency_crash[] =
-			_T("A serious fault condition occured in the game. The game will shut down.\n")
+			_T("A serious fault condition occurred in the game. The game will shut down.\n")
 			_T("As you loaded an emergency savegame no crash information will be generated.\n");
 		MessageBox(NULL, _emergency_crash, _T("Fatal Application Failure"), MB_ICONERROR);
 		ExitProcess(3);
@@ -375,7 +503,7 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 
 	if (SaveloadCrashWithMissingNewGRFs()) {
 		static const TCHAR _saveload_crash[] =
-			_T("A serious fault condition occured in the game. The game will shut down.\n")
+			_T("A serious fault condition occurred in the game. The game will shut down.\n")
 			_T("As you loaded an savegame for which you do not have the required NewGRFs\n")
 			_T("no crash information will be generated.\n");
 		MessageBox(NULL, _saveload_crash, _T("Fatal Application Failure"), MB_ICONERROR);
@@ -384,15 +512,16 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 
 	CrashLogWindows *log = new CrashLogWindows(ep);
 	CrashLogWindows::current = log;
-	log->FillCrashLog(log->crashlog, lastof(log->crashlog));
-	log->WriteCrashLog(log->crashlog, log->crashlog_filename, lastof(log->crashlog_filename));
+	char *buf = log->FillCrashLog(log->crashlog, lastof(log->crashlog));
 	log->WriteCrashDump(log->crashdump_filename, lastof(log->crashdump_filename));
+	log->AppendDecodedStacktrace(buf, lastof(log->crashlog));
+	log->WriteCrashLog(log->crashlog, log->crashlog_filename, lastof(log->crashlog_filename));
 	log->WriteScreenshot(log->screenshot_filename, lastof(log->screenshot_filename));
 
 	/* Close any possible log files */
 	CloseConsoleLogIfActive();
 
-	if (_safe_esp) {
+	if ((_video_driver == NULL || _video_driver->HasGUI()) && _safe_esp != NULL) {
 #ifdef _M_AMD64
 		ep->ContextRecord->Rip = (DWORD64)ShowCrashlogWindow;
 		ep->ContextRecord->Rsp = (DWORD64)_safe_esp;
@@ -407,10 +536,6 @@ static LONG WINAPI ExceptionHandler(EXCEPTION_POINTERS *ep)
 	return EXCEPTION_EXECUTE_HANDLER;
 }
 
-#ifdef _M_AMD64
-extern "C" void *_get_safe_esp();
-#endif
-
 static void CDECL CustomAbort(int signal)
 {
 	RaiseException(0xE1212012, 0, 0, NULL);
@@ -418,16 +543,23 @@ static void CDECL CustomAbort(int signal)
 
 /* static */ void CrashLog::InitialiseCrashLog()
 {
-#if defined(_MSC_VER)
 #ifdef _M_AMD64
-	_safe_esp = _get_safe_esp();
+	CONTEXT ctx;
+	RtlCaptureContext(&ctx);
+
+	/* The stack pointer for AMD64 must always be 16-byte aligned inside a
+	 * function. As we are simulating a function call with the safe ESP value,
+	 * we need to subtract 8 for the imaginary return address otherwise stack
+	 * alignment would be wrong in the called function. */
+	_safe_esp = (void *)(ctx.Rsp - 8);
 #else
+#if defined(_MSC_VER)
 	_asm {
 		mov _safe_esp, esp
 	}
-#endif
 #else
 	asm("movl %esp, __safe_esp");
+#endif
 #endif
 
 	/* SIGABRT is not an unhandled exception, so we need to intercept it. */
@@ -444,7 +576,7 @@ static void CDECL CustomAbort(int signal)
 static bool _expanded;
 
 static const TCHAR _crash_desc[] =
-	_T("A serious fault condition occured in the game. The game will shut down.\n")
+	_T("A serious fault condition occurred in the game. The game will shut down.\n")
 	_T("Please send the crash information and the crash.dmp file (if any) to the developers.\n")
 	_T("This will greatly help debugging. The correct place to do this is http://bugs.openttd.org. ")
 	_T("The information contained in the report is displayed below.\n")
@@ -468,7 +600,7 @@ static void SetWndSize(HWND wnd, int mode)
 	if (mode >= 0) {
 		GetWindowRect(GetDlgItem(wnd, 11), &r2);
 		int offs = r2.bottom - r2.top + 10;
-		if (!mode) offs = -offs;
+		if (mode == 0) offs = -offs;
 		SetWindowPos(wnd, HWND_TOPMOST, 0, 0,
 			r.right - r.left, r.bottom - r.top + offs, SWP_NOMOVE | SWP_NOZORDER);
 	} else {
@@ -483,11 +615,9 @@ static INT_PTR CALLBACK CrashDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARA
 {
 	switch (msg) {
 		case WM_INITDIALOG: {
-#if defined(UNICODE)
-			/* We need to put the crash-log in a seperate buffer because the default
+			/* We need to put the crash-log in a separate buffer because the default
 			 * buffer in MB_TO_WIDE is not large enough (512 chars) */
-			wchar_t crash_msgW[lengthof(CrashLogWindows::current->crashlog)];
-#endif
+			TCHAR crash_msgW[lengthof(CrashLogWindows::current->crashlog)];
 			/* Convert unix -> dos newlines because the edit box only supports that properly :( */
 			const char *unix_nl = CrashLogWindows::current->crashlog;
 			char dos_nl[lengthof(CrashLogWindows::current->crashlog)];
@@ -517,7 +647,7 @@ static INT_PTR CALLBACK CrashDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARA
 			}
 
 			SetDlgItemText(wnd, 10, text);
-			SetDlgItemText(wnd, 11, MB_TO_WIDE_BUFFER(dos_nl, crash_msgW, lengthof(crash_msgW)));
+			SetDlgItemText(wnd, 11, convert_to_fs(dos_nl, crash_msgW, lengthof(crash_msgW)));
 			SendDlgItemMessage(wnd, 11, WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), FALSE);
 			SetWndSize(wnd, -1);
 		} return TRUE;
