@@ -1,4 +1,4 @@
-/* $Id: win32.cpp 22467 2011-05-15 18:06:13Z rubidium $ */
+/* $Id: win32.cpp 25982 2013-11-13 21:35:44Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,7 +17,9 @@
 #include "../../fios.h"
 #include <windows.h>
 #include <fcntl.h>
+#include <regstr.h>
 #include <shlobj.h> /* SHGetFolderPath */
+#include <Shellapi.h>
 #include "win32.h"
 #include "../../core/alloc_func.hpp"
 #include "../../openttd.h"
@@ -28,14 +30,16 @@
 #include <sys/stat.h>
 
 static bool _has_console;
+static bool _cursor_disable = true;
+static bool _cursor_visible = true;
 
-static bool cursor_visible = true;
-
-bool MyShowCursor(bool show)
+bool MyShowCursor(bool show, bool toggle)
 {
-	if (cursor_visible == show) return show;
+	if (toggle) _cursor_disable = !_cursor_disable;
+	if (_cursor_disable) return show;
+	if (_cursor_visible == show) return show;
 
-	cursor_visible = show;
+	_cursor_visible = show;
 	ShowCursor(show);
 
 	return !show;
@@ -74,13 +78,31 @@ bool LoadLibraryList(Function proc[], const char *dll)
 void ShowOSErrorBox(const char *buf, bool system)
 {
 	MyShowCursor(true);
-	MessageBox(GetActiveWindow(), MB_TO_WIDE(buf), _T("Error!"), MB_ICONSTOP);
+	MessageBox(GetActiveWindow(), OTTD2FS(buf), _T("Error!"), MB_ICONSTOP);
+}
+
+void OSOpenBrowser(const char *url)
+{
+	ShellExecute(GetActiveWindow(), _T("open"), OTTD2FS(url), NULL, NULL, SW_SHOWNORMAL);
 }
 
 /* Code below for windows version of opendir/readdir/closedir copied and
  * modified from Jan Wassenberg's GPL implementation posted over at
  * http://www.gamedev.net/community/forums/topic.asp?topic_id=364584&whichpage=1&#2398903 */
 
+struct DIR {
+	HANDLE hFind;
+	/* the dirent returned by readdir.
+	 * note: having only one global instance is not possible because
+	 * multiple independent opendir/readdir sequences must be supported. */
+	dirent ent;
+	WIN32_FIND_DATA fd;
+	/* since opendir calls FindFirstFile, we need a means of telling the
+	 * first call to readdir that we already have a file.
+	 * that's the case iff this is true */
+	bool at_first_entry;
+};
+
 /* suballocator - satisfies most requests with a reusable static instance.
  * this avoids hundreds of alloc/free which would fragment the heap.
  * To guarantee concurrency, we fall back to malloc if the instance is
@@ -217,8 +239,8 @@ bool FiosIsValidFile(const char *path, const struct dirent *ent, struct stat *sb
 	 * we just have to subtract POSIX epoch and scale down to units of seconds.
 	 * http://www.gamedev.net/community/forums/topic.asp?topic_id=294070&whichpage=1&#1860504
 	 * XXX - not entirely correct, since filetimes on FAT aren't UTC but local,
-	 * this won't entirely be correct, but we use the time only for comparsion. */
-	sb->st_mtime = (time_t)((*(uint64*)&fd->ftLastWriteTime - posix_epoch_hns) / 1E7);
+	 * this won't entirely be correct, but we use the time only for comparison. */
+	sb->st_mtime = (time_t)((*(const uint64*)&fd->ftLastWriteTime - posix_epoch_hns) / 1E7);
 	sb->st_mode  = (fd->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)? S_IFDIR : S_IFREG;
 
 	return true;
@@ -349,12 +371,10 @@ static INT_PTR CALLBACK HelpDialogFunc(HWND wnd, UINT msg, WPARAM wParam, LPARAM
 				*q++ = *p++;
 			}
 			*q = '\0';
-#if defined(UNICODE)
-			/* We need to put the text in a seperate buffer because the default
-			 * buffer in MB_TO_WIDE might not be large enough (512 chars) */
-			wchar_t help_msgW[8192];
-#endif
-			SetDlgItemText(wnd, 11, MB_TO_WIDE_BUFFER(help_msg, help_msgW, lengthof(help_msgW)));
+			/* We need to put the text in a separate buffer because the default
+			 * buffer in OTTD2FS might not be large enough (512 chars). */
+			TCHAR help_msg_buf[8192];
+			SetDlgItemText(wnd, 11, convert_to_fs(help_msg, help_msg_buf, lengthof(help_msg_buf)));
 			SendDlgItemMessage(wnd, 11, WM_SETFONT, (WPARAM)GetStockObject(ANSI_FIXED_FONT), FALSE);
 		} return TRUE;
 
@@ -385,14 +405,10 @@ void ShowInfo(const char *str)
 			_help_msg = str;
 			DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(101), NULL, HelpDialogFunc);
 		} else {
-#if defined(UNICODE)
-			/* We need to put the text in a seperate buffer because the default
-			 * buffer in MB_TO_WIDE might not be large enough (512 chars) */
-			wchar_t help_msgW[8192];
-#endif
-			if (MessageBox(GetActiveWindow(), MB_TO_WIDE_BUFFER(str, help_msgW, lengthof(help_msgW)), _T("OpenTTD"), MB_ICONINFORMATION | MB_OKCANCEL) == IDCANCEL) {
-				CreateConsole();
-			}
+			/* We need to put the text in a separate buffer because the default
+			 * buffer in OTTD2FS might not be large enough (512 chars). */
+			TCHAR help_msg_buf[8192];
+			MessageBox(GetActiveWindow(), convert_to_fs(str, help_msg_buf, lengthof(help_msg_buf)), _T("OpenTTD"), MB_ICONINFORMATION | MB_OK);
 		}
 		MyShowCursor(old);
 	}
@@ -406,28 +422,18 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLi
 {
 	int argc;
 	char *argv[64]; // max 64 command line arguments
-	char *cmdline;
-
-#if !defined(UNICODE)
-	_codepage = GetACP(); // get system codepage as some kind of a default
-#endif /* UNICODE */
 
 	CrashLog::InitialiseCrashLog();
 
-#if defined(UNICODE)
-
-#if !defined(WINCE)
+#if defined(UNICODE) && !defined(WINCE)
 	/* Check if a win9x user started the win32 version */
 	if (HasBit(GetVersion(), 31)) usererror("This version of OpenTTD doesn't run on windows 95/98/ME.\nPlease download the win9x binary and try again.");
 #endif
 
-	/* For UNICODE we need to convert the commandline to char* _AND_
-	 * save it because argv[] points into this buffer and thus needs to
-	 * be available between subsequent calls to FS2OTTD() */
-	char cmdlinebuf[MAX_PATH];
-#endif /* UNICODE */
-
-	cmdline = WIDE_TO_MB_BUFFER(GetCommandLine(), cmdlinebuf, lengthof(cmdlinebuf));
+	/* Convert the command line to UTF-8. We need a dedicated buffer
+	 * for this because argv[] points into this buffer and this needs to
+	 * be available between subsequent calls to FS2OTTD(). */
+	char *cmdline = strdup(FS2OTTD(GetCommandLine()));
 
 #if defined(_DEBUG)
 	CreateConsole();
@@ -443,6 +449,7 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLi
 	argc = ParseCommandLine(cmdline, argv, lengthof(argv));
 
 	ttd_main(argc, argv);
+	free(cmdline);
 	return 0;
 }
 
@@ -470,12 +477,10 @@ char *getcwd(char *buf, size_t size)
 	/* GetModuleFileName returns dir with file, so remove everything behind latest '\\' */
 	char *p = strrchr(buf, '\\');
 	if (p != NULL) *p = '\0';
-#elif defined(UNICODE)
+#else
 	TCHAR path[MAX_PATH];
 	GetCurrentDirectory(MAX_PATH - 1, path);
 	convert_from_fs(path, buf, size);
-#else
-	GetCurrentDirectory(size, buf);
 #endif
 	return buf;
 }
@@ -486,19 +491,25 @@ void DetermineBasePaths(const char *exe)
 	char tmp[MAX_PATH];
 	TCHAR path[MAX_PATH];
 #ifdef WITH_PERSONAL_DIR
-	SHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, SHGFP_TYPE_CURRENT, path);
-	strecpy(tmp, WIDE_TO_MB_BUFFER(path, tmp, lengthof(tmp)), lastof(tmp));
-	AppendPathSeparator(tmp, MAX_PATH);
-	ttd_strlcat(tmp, PERSONAL_DIR, MAX_PATH);
-	AppendPathSeparator(tmp, MAX_PATH);
-	_searchpaths[SP_PERSONAL_DIR] = strdup(tmp);
+	if (SUCCEEDED(OTTDSHGetFolderPath(NULL, CSIDL_PERSONAL, NULL, SHGFP_TYPE_CURRENT, path))) {
+		strecpy(tmp, FS2OTTD(path), lastof(tmp));
+		AppendPathSeparator(tmp, MAX_PATH);
+		ttd_strlcat(tmp, PERSONAL_DIR, MAX_PATH);
+		AppendPathSeparator(tmp, MAX_PATH);
+		_searchpaths[SP_PERSONAL_DIR] = strdup(tmp);
+	} else {
+		_searchpaths[SP_PERSONAL_DIR] = NULL;
+	}
 
-	SHGetFolderPath(NULL, CSIDL_COMMON_DOCUMENTS, NULL, SHGFP_TYPE_CURRENT, path);
-	strecpy(tmp, WIDE_TO_MB_BUFFER(path, tmp, lengthof(tmp)), lastof(tmp));
-	AppendPathSeparator(tmp, MAX_PATH);
-	ttd_strlcat(tmp, PERSONAL_DIR, MAX_PATH);
-	AppendPathSeparator(tmp, MAX_PATH);
-	_searchpaths[SP_SHARED_DIR] = strdup(tmp);
+	if (SUCCEEDED(OTTDSHGetFolderPath(NULL, CSIDL_COMMON_DOCUMENTS, NULL, SHGFP_TYPE_CURRENT, path))) {
+		strecpy(tmp, FS2OTTD(path), lastof(tmp));
+		AppendPathSeparator(tmp, MAX_PATH);
+		ttd_strlcat(tmp, PERSONAL_DIR, MAX_PATH);
+		AppendPathSeparator(tmp, MAX_PATH);
+		_searchpaths[SP_SHARED_DIR] = strdup(tmp);
+	} else {
+		_searchpaths[SP_SHARED_DIR] = NULL;
+	}
 #else
 	_searchpaths[SP_PERSONAL_DIR] = NULL;
 	_searchpaths[SP_SHARED_DIR]   = NULL;
@@ -514,12 +525,12 @@ void DetermineBasePaths(const char *exe)
 		_searchpaths[SP_BINARY_DIR] = NULL;
 	} else {
 		TCHAR exec_dir[MAX_PATH];
-		_tcsncpy(path, MB_TO_WIDE_BUFFER(exe, path, lengthof(path)), lengthof(path));
+		_tcsncpy(path, convert_to_fs(exe, path, lengthof(path)), lengthof(path));
 		if (!GetFullPathName(path, lengthof(exec_dir), exec_dir, NULL)) {
 			DEBUG(misc, 0, "GetFullPathName failed (%lu)\n", GetLastError());
 			_searchpaths[SP_BINARY_DIR] = NULL;
 		} else {
-			strecpy(tmp, WIDE_TO_MB_BUFFER(exec_dir, tmp, lengthof(tmp)), lastof(tmp));
+			strecpy(tmp, convert_from_fs(exec_dir, tmp, lengthof(tmp)), lastof(tmp));
 			char *s = strrchr(tmp, PATHSEPCHAR);
 			*(s + 1) = '\0';
 			_searchpaths[SP_BINARY_DIR] = strdup(tmp);
@@ -541,11 +552,11 @@ bool GetClipboardContents(char *buffer, size_t buff_len)
 		cbuf = GetClipboardData(CF_UNICODETEXT);
 
 		ptr = (const char*)GlobalLock(cbuf);
-		const char *ret = convert_from_fs((wchar_t*)ptr, buffer, buff_len);
+		int out_len = WideCharToMultiByte(CP_UTF8, 0, (LPCWSTR)ptr, -1, buffer, (int)buff_len, NULL, NULL);
 		GlobalUnlock(cbuf);
 		CloseClipboard();
 
-		if (*ret == '\0') return false;
+		if (out_len == 0) return false;
 #if !defined(UNICODE)
 	} else if (IsClipboardFormatAvailable(CF_TEXT)) {
 		OpenClipboard(NULL);
@@ -587,26 +598,7 @@ void CSleep(int milliseconds)
 const char *FS2OTTD(const TCHAR *name)
 {
 	static char utf8_buf[512];
-#if defined(UNICODE)
 	return convert_from_fs(name, utf8_buf, lengthof(utf8_buf));
-#else
-	char *s = utf8_buf;
-
-	for (; *name != '\0'; name++) {
-		wchar_t w;
-		int len = MultiByteToWideChar(_codepage, 0, name, 1, &w, 1);
-		if (len != 1) {
-			DEBUG(misc, 0, "[utf8] M2W error converting '%c'. Errno %lu", *name, GetLastError());
-			continue;
-		}
-
-		if (s + Utf8CharLen(w) >= lastof(utf8_buf)) break;
-		s += Utf8Encode(s, w);
-	}
-
-	*s = '\0';
-	return utf8_buf;
-#endif /* UNICODE */
 }
 
 /**
@@ -618,34 +610,13 @@ const char *FS2OTTD(const TCHAR *name)
  * The returned value's contents can only be guaranteed until the next call to
  * this function. So if the value is needed for anything else, use convert_from_fs
  * @param name pointer to a valid string that will be converted (UTF8)
+ * @param console_cp convert to the console encoding instead of the normal system encoding.
  * @return pointer to the converted string; if failed string is of zero-length
- * @see the current code-page comes from video\win32_v.cpp, event-notification
- * WM_INPUTLANGCHANGE
  */
-const TCHAR *OTTD2FS(const char *name)
+const TCHAR *OTTD2FS(const char *name, bool console_cp)
 {
 	static TCHAR system_buf[512];
-#if defined(UNICODE)
-	return convert_to_fs(name, system_buf, lengthof(system_buf));
-#else
-	char *s = system_buf;
-
-	for (WChar c; (c = Utf8Consume(&name)) != '\0';) {
-		if (s >= lastof(system_buf)) break;
-
-		char mb;
-		int len = WideCharToMultiByte(_codepage, 0, (wchar_t*)&c, 1, &mb, 1, NULL, NULL);
-		if (len != 1) {
-			DEBUG(misc, 0, "[utf8] W2M error converting '0x%X'. Errno %lu", c, GetLastError());
-			continue;
-		}
-
-		*s++ = mb;
-	}
-
-	*s = '\0';
-	return system_buf;
-#endif /* UNICODE */
+	return convert_to_fs(name, system_buf, lengthof(system_buf), console_cp);
 }
 
 
@@ -657,14 +628,26 @@ const TCHAR *OTTD2FS(const char *name)
  * @param buflen length in characters of the receiving buffer
  * @return pointer to utf8_buf. If conversion fails the string is of zero-length
  */
-char *convert_from_fs(const wchar_t *name, char *utf8_buf, size_t buflen)
+char *convert_from_fs(const TCHAR *name, char *utf8_buf, size_t buflen)
 {
-	int len = WideCharToMultiByte(CP_UTF8, 0, name, -1, utf8_buf, (int)buflen, NULL, NULL);
-	if (len == 0) {
-		DEBUG(misc, 0, "[utf8] W2M error converting wide-string. Errno %lu", GetLastError());
+#if defined(UNICODE)
+	const WCHAR *wide_buf = name;
+#else
+	/* Convert string from the local codepage to UTF-16. */
+	int wide_len = MultiByteToWideChar(CP_ACP, 0, name, -1, NULL, 0);
+	if (wide_len == 0) {
 		utf8_buf[0] = '\0';
+		return utf8_buf;
 	}
 
+	WCHAR *wide_buf = AllocaM(WCHAR, wide_len);
+	MultiByteToWideChar(CP_ACP, 0, name, -1, wide_buf, wide_len);
+#endif
+
+	/* Convert UTF-16 string to UTF-8. */
+	int len = WideCharToMultiByte(CP_UTF8, 0, wide_buf, -1, utf8_buf, (int)buflen, NULL, NULL);
+	if (len == 0) utf8_buf[0] = '\0';
+
 	return utf8_buf;
 }
 
@@ -676,17 +659,29 @@ char *convert_from_fs(const wchar_t *name, char *utf8_buf, size_t buflen)
  * @param utf16_buf pointer to a valid wide-char buffer that will receive the
  * converted string
  * @param buflen length in wide characters of the receiving buffer
+ * @param console_cp convert to the console encoding instead of the normal system encoding.
  * @return pointer to utf16_buf. If conversion fails the string is of zero-length
  */
-wchar_t *convert_to_fs(const char *name, wchar_t *utf16_buf, size_t buflen)
+TCHAR *convert_to_fs(const char *name, TCHAR *system_buf, size_t buflen, bool console_cp)
 {
-	int len = MultiByteToWideChar(CP_UTF8, 0, name, -1, utf16_buf, (int)buflen);
+#if defined(UNICODE)
+	int len = MultiByteToWideChar(CP_UTF8, 0, name, -1, system_buf, (int)buflen);
+	if (len == 0) system_buf[0] = '\0';
+#else
+	int len = MultiByteToWideChar(CP_UTF8, 0, name, -1, NULL, 0);
 	if (len == 0) {
-		DEBUG(misc, 0, "[utf8] M2W error converting '%s'. Errno %lu", name, GetLastError());
-		utf16_buf[0] = '\0';
+		system_buf[0] = '\0';
+		return system_buf;
 	}
 
-	return utf16_buf;
+	WCHAR *wide_buf = AllocaM(WCHAR, len);
+	MultiByteToWideChar(CP_UTF8, 0, name, -1, wide_buf, len);
+
+	len = WideCharToMultiByte(console_cp ? CP_OEMCP : CP_ACP, 0, wide_buf, len, system_buf, (int)buflen, NULL, NULL);
+	if (len == 0) system_buf[0] = '\0';
+#endif
+
+	return system_buf;
 }
 
 /**
@@ -707,8 +702,11 @@ HRESULT OTTDSHGetFolderPath(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,
 #else
 # define W(x) x "A"
 #endif
-		if (!LoadLibraryList((Function*)&SHGetFolderPath, "SHFolder.dll\0" W("SHGetFolderPath") "\0\0")) {
-			DEBUG(misc, 0, "Unable to load " W("SHGetFolderPath") "from SHFolder.dll");
+		/* The function lives in shell32.dll for all current Windows versions, but it first started to appear in SHFolder.dll. */
+		if (!LoadLibraryList((Function*)&SHGetFolderPath, "shell32.dll\0" W("SHGetFolderPath") "\0\0")) {
+			if (!LoadLibraryList((Function*)&SHGetFolderPath, "SHFolder.dll\0" W("SHGetFolderPath") "\0\0")) {
+				DEBUG(misc, 0, "Unable to load " W("SHGetFolderPath") "from either shell32.dll or SHFolder.dll");
+			}
 		}
 #undef W
 		first_time = false;
@@ -733,6 +731,17 @@ HRESULT OTTDSHGetFolderPath(HWND hwnd, int csidl, HANDLE hToken, DWORD dwFlags,
 
 				return (HRESULT)0;
 
+			case CSIDL_PERSONAL:
+			case CSIDL_COMMON_DOCUMENTS: {
+				HKEY key;
+				if (RegOpenKeyEx(csidl == CSIDL_PERSONAL ? HKEY_CURRENT_USER : HKEY_LOCAL_MACHINE, REGSTR_PATH_SPECIAL_FOLDERS, 0, KEY_READ, &key) != ERROR_SUCCESS) break;
+				DWORD len = MAX_PATH;
+				ret = RegQueryValueEx(key, csidl == CSIDL_PERSONAL ? _T("Personal") : _T("Common Documents"), NULL, NULL, (LPBYTE)pszPath, &len);
+				RegCloseKey(key);
+				if (ret == ERROR_SUCCESS) return (HRESULT)0;
+				break;
+			}
+
 			/* XXX - other types to go here when needed... */
 		}
 	}
@@ -753,3 +762,11 @@ const char *GetCurrentLocale(const char *)
 	static char retbuf[6] = {lang[0], lang[1], '_', country[0], country[1], 0};
 	return retbuf;
 }
+
+uint GetCPUCoreCount()
+{
+	SYSTEM_INFO info;
+
+	GetSystemInfo(&info);
+	return info.dwNumberOfProcessors;
+}
