@@ -1,4 +1,4 @@
-/* $Id: yapf.hpp 18420 2009-12-07 08:41:18Z rubidium $ */
+/* $Id: yapf.hpp 23640 2011-12-20 17:57:56Z truebrain $ */
 
 /*
  * This file is part of OpenTTD.
@@ -9,8 +9,8 @@
 
 /** @file yapf.hpp Base includes/functions for YAPF. */
 
-#ifndef  YAPF_HPP
-#define  YAPF_HPP
+#ifndef YAPF_HPP
+#define YAPF_HPP
 
 #include "../../landscape.h"
 #include "../pathfinder_func.h"
@@ -18,9 +18,8 @@
 #include "yapf.h"
 
 //#undef FORCEINLINE
-//#define FORCEINLINE inline
+//#define inline inline
 
-#include "../../misc/crc32.hpp"
 #include "../../misc/blob.hpp"
 #include "../../misc/str.hpp"
 #include "../../misc/fixedsizearray.hpp"
