@@ -1,4 +1,4 @@
-/* $Id: rail.cpp 21846 2011-01-18 23:09:43Z rubidium $ */
+/* $Id: rail.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -216,7 +216,7 @@ RailType GetBestRailtype(const CompanyID company)
 /**
  * Add the rail types that are to be introduced at the given date.
  * @param current The currently available railtypes.
- * @param date    The date for the introduction comparisions.
+ * @param date    The date for the introduction comparisons.
  * @return The rail types that should be available when date
  *         introduced rail types are taken into account as well.
  */
@@ -277,9 +277,10 @@ RailTypes GetCompanyRailtypes(CompanyID company)
 /**
  * Get the rail type for a given label.
  * @param label the railtype label.
+ * @param allow_alternate_labels Search in the alternate label lists as well.
  * @return the railtype.
  */
-RailType GetRailTypeByLabel(RailTypeLabel label)
+RailType GetRailTypeByLabel(RailTypeLabel label, bool allow_alternate_labels)
 {
 	/* Loop through each rail type until the label is found */
 	for (RailType r = RAILTYPE_BEGIN; r != RAILTYPE_END; r++) {
@@ -287,6 +288,14 @@ RailType GetRailTypeByLabel(RailTypeLabel label)
 		if (rti->label == label) return r;
 	}
 
+	if (allow_alternate_labels) {
+		/* Test if any rail type defines the label as an alternate. */
+		for (RailType r = RAILTYPE_BEGIN; r != RAILTYPE_END; r++) {
+			const RailtypeInfo *rti = GetRailTypeInfo(r);
+			if (rti->alternate_labels.Contains(label)) return r;
+		}
+	}
+
 	/* No matching label was found, so it is invalid */
 	return INVALID_RAILTYPE;
 }
