@@ -1,4 +1,11 @@
-/* $Id: rail_map.h 16324 2009-05-16 20:29:53Z rubidium $ */
+/* $Id: rail_map.h 24367 2012-07-01 23:12:50Z michi_cc $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file rail_map.h Hides the direct accesses to the map array with map accessors */
 
@@ -6,19 +13,17 @@
 #define RAIL_MAP_H
 
 #include "rail_type.h"
+#include "depot_type.h"
 #include "signal_func.h"
-#include "direction_func.h"
 #include "track_func.h"
 #include "tile_map.h"
 #include "signal_type.h"
-#include "waypoint_type.h"
 
 
 /** Different types of Rail-related tiles */
 enum RailTileType {
 	RAIL_TILE_NORMAL   = 0, ///< Normal rail tile without signals
 	RAIL_TILE_SIGNALS  = 1, ///< Normal rail tile with signals
-	RAIL_TILE_WAYPOINT = 2, ///< Waypoint (X or Y direction)
 	RAIL_TILE_DEPOT    = 3, ///< Depot (one entrance)
 };
 
@@ -42,12 +47,23 @@ static inline RailTileType GetRailTileType(TileIndex t)
  * @pre IsTileType(t, MP_RAILWAY)
  * @return true if and only if the tile is normal rail (with or without signals)
  */
-static inline bool IsPlainRailTile(TileIndex t)
+static inline bool IsPlainRail(TileIndex t)
 {
 	RailTileType rtt = GetRailTileType(t);
 	return rtt == RAIL_TILE_NORMAL || rtt == RAIL_TILE_SIGNALS;
 }
 
+/**
+ * Checks whether the tile is a rail tile or rail tile with signals.
+ * @param t the tile to get the information from
+ * @return true if and only if the tile is normal rail (with or without signals)
+ */
+static inline bool IsPlainRailTile(TileIndex t)
+{
+	return IsTileType(t, MP_RAILWAY) && IsPlainRail(t);
+}
+
+
 /**
  * Checks if a rail tile has signals.
  * @param t the tile to get the information from
@@ -71,27 +87,6 @@ static inline void SetHasSignals(TileIndex tile, bool signals)
 	SB(_m[tile].m5, 6, 1, signals);
 }
 
-/**
- * Is this rail tile a rail waypoint?
- * @param t the tile to get the information from
- * @pre IsTileType(t, MP_RAILWAY)
- * @return true if and only if the tile is a rail waypoint
- */
-static inline bool IsRailWaypoint(TileIndex t)
-{
-	return GetRailTileType(t) == RAIL_TILE_WAYPOINT;
-}
-
-/**
- * Is this tile rail tile and a rail waypoint?
- * @param t the tile to get the information from
- * @return true if and only if the tile is a rail waypoint
- */
-static inline bool IsRailWaypointTile(TileIndex t)
-{
-	return IsTileType(t, MP_RAILWAY) && IsRailWaypoint(t);
-}
-
 /**
  * Is this rail tile a rail depot?
  * @param t the tile to get the information from
@@ -136,7 +131,7 @@ static inline void SetRailType(TileIndex t, RailType r)
 
 /**
  * Gets the track bits of the given tile
- * @param t the tile to get the track bits from
+ * @param tile the tile to get the track bits from
  * @return the track bits of the tile
  */
 static inline TrackBits GetTrackBits(TileIndex tile)
@@ -191,58 +186,13 @@ static inline Track GetRailDepotTrack(TileIndex t)
 }
 
 
-/**
- * Returns the axis of the waypoint
- * @param t the tile to get the waypoint axis from
- * @pre IsRailWaypointTile(t)
- * @return the axis of the waypoint
- */
-static inline Axis GetWaypointAxis(TileIndex t)
-{
-	return (Axis)GB(_m[t].m5, 0, 1);
-}
-
-/**
- * Returns the track of the waypoint
- * @param t the tile to get the waypoint track from
- * @pre IsRailWaypointTile(t)
- * @return the track of the waypoint
- */
-static inline Track GetRailWaypointTrack(TileIndex t)
-{
-	return AxisToTrack(GetWaypointAxis(t));
-}
-
-/**
- * Returns the track bits of the waypoint
- * @param t the tile to get the waypoint track bits from
- * @pre IsRailWaypointTile(t)
- * @return the track bits of the waypoint
- */
-static inline TrackBits GetRailWaypointBits(TileIndex t)
-{
-	return TrackToTrackBits(GetRailWaypointTrack(t));
-}
-
-/**
- * Returns waypoint index (for the waypoint pool)
- * @param t the tile to get the waypoint index from
- * @pre IsRailWaypointTile(t)
- * @return the waypoint index
- */
-static inline WaypointID GetWaypointIndex(TileIndex t)
-{
-	return (WaypointID)_m[t].m2;
-}
-
-
 /**
  * Returns the reserved track bits of the tile
  * @pre IsPlainRailTile(t)
  * @param t the tile to query
  * @return the track bits
  */
-static inline TrackBits GetTrackReservation(TileIndex t)
+static inline TrackBits GetRailReservationTrackBits(TileIndex t)
 {
 	assert(IsPlainRailTile(t));
 	byte track_b = GB(_m[t].m2, 8, 3);
@@ -263,7 +213,7 @@ static inline void SetTrackReservation(TileIndex t, TrackBits b)
 	assert(b != INVALID_TRACK_BIT);
 	assert(!TracksOverlap(b));
 	Track track = RemoveFirstTrack(&b);
-	SB(_m[t].m2, 8, 3, track == INVALID_TRACK ? 0 : track+1);
+	SB(_m[t].m2, 8, 3, track == INVALID_TRACK ? 0 : track + 1);
 	SB(_m[t].m2, 11, 1, (byte)(b != TRACK_BIT_NONE));
 }
 
@@ -278,7 +228,7 @@ static inline bool TryReserveTrack(TileIndex tile, Track t)
 {
 	assert(HasTrack(tile, t));
 	TrackBits bits = TrackToTrackBits(t);
-	TrackBits res = GetTrackReservation(tile);
+	TrackBits res = GetRailReservationTrackBits(tile);
 	if ((res & bits) != TRACK_BIT_NONE) return false;  // already reserved
 	res |= bits;
 	if (TracksOverlap(res)) return false;  // crossing reservation present
@@ -295,57 +245,44 @@ static inline bool TryReserveTrack(TileIndex tile, Track t)
 static inline void UnreserveTrack(TileIndex tile, Track t)
 {
 	assert(HasTrack(tile, t));
-	TrackBits res = GetTrackReservation(tile);
+	TrackBits res = GetRailReservationTrackBits(tile);
 	res &= ~TrackToTrackBits(t);
 	SetTrackReservation(tile, res);
 }
 
 /**
- * Get the reservation state of the waypoint or depot
- * @note Works for both waypoints and rail depots
- * @pre IsRailWaypoint(t) || IsRailDepot(t)
- * @param t the waypoint/depot tile
+ * Get the reservation state of the depot
+ * @pre IsRailDepot(t)
+ * @param t the depot tile
  * @return reservation state
  */
-static inline bool GetDepotWaypointReservation(TileIndex t)
+static inline bool HasDepotReservation(TileIndex t)
 {
-	assert(IsRailWaypoint(t) || IsRailDepot(t));
+	assert(IsRailDepot(t));
 	return HasBit(_m[t].m5, 4);
 }
 
 /**
- * Set the reservation state of the waypoint or depot
- * @note Works for both waypoints and rail depots
- * @pre IsRailWaypoint(t) || IsRailDepot(t)
- * @param t the waypoint/depot tile
+ * Set the reservation state of the depot
+ * @pre IsRailDepot(t)
+ * @param t the depot tile
  * @param b the reservation state
  */
-static inline void SetDepotWaypointReservation(TileIndex t, bool b)
+static inline void SetDepotReservation(TileIndex t, bool b)
 {
-	assert(IsRailWaypoint(t) || IsRailDepot(t));
+	assert(IsRailDepot(t));
 	SB(_m[t].m5, 4, 1, (byte)b);
 }
 
-/**
- * Get the reserved track bits for a waypoint
- * @pre IsRailWaypoint(t)
- * @param t the tile
- * @return reserved track bits
- */
-static inline TrackBits GetRailWaypointReservation(TileIndex t)
-{
-	return GetDepotWaypointReservation(t) ? GetRailWaypointBits(t) : TRACK_BIT_NONE;
-}
-
 /**
  * Get the reserved track bits for a depot
  * @pre IsRailDepot(t)
  * @param t the tile
  * @return reserved track bits
  */
-static inline TrackBits GetRailDepotReservation(TileIndex t)
+static inline TrackBits GetDepotReservationTrackBits(TileIndex t)
 {
-	return GetDepotWaypointReservation(t) ? TrackToTrackBits(GetRailDepotTrack(t)) : TRACK_BIT_NONE;
+	return HasDepotReservation(t) ? TrackToTrackBits(GetRailDepotTrack(t)) : TRACK_BIT_NONE;
 }
 
 
@@ -408,15 +345,6 @@ static inline void SetSignalVariant(TileIndex t, Track track, SignalVariant v)
 	if (track == INVALID_TRACK) SB(_m[t].m2, 7, 1, v);
 }
 
-/** These are states in which a signal can be. Currently these are only two, so
- * simple boolean logic will do. But do try to compare to this enum instead of
- * normal boolean evaluation, since that will make future additions easier.
- */
-enum SignalState {
-	SIGNAL_STATE_RED   = 0, ///< The signal is red
-	SIGNAL_STATE_GREEN = 1, ///< The signal is green
-};
-
 /**
  * Set the states of the signals (Along/AgainstTrackDir)
  * @param tile  the tile to set the states for
@@ -430,7 +358,7 @@ static inline void SetSignalStates(TileIndex tile, uint state)
 /**
  * Set the states of the signals (Along/AgainstTrackDir)
  * @param tile  the tile to set the states for
- * @param state the new state
+ * @return the state of the signals
  */
 static inline uint GetSignalStates(TileIndex tile)
 {
@@ -486,9 +414,7 @@ static inline bool IsSignalPresent(TileIndex t, byte signalbit)
 static inline bool HasSignalOnTrack(TileIndex tile, Track track)
 {
 	assert(IsValidTrack(track));
-	return
-		GetRailTileType(tile) == RAIL_TILE_SIGNALS &&
-		(GetPresentSignals(tile) & SignalOnTrack(track)) != 0;
+	return GetRailTileType(tile) == RAIL_TILE_SIGNALS && (GetPresentSignals(tile) & SignalOnTrack(track)) != 0;
 }
 
 /**
@@ -501,9 +427,7 @@ static inline bool HasSignalOnTrack(TileIndex tile, Track track)
 static inline bool HasSignalOnTrackdir(TileIndex tile, Trackdir trackdir)
 {
 	assert (IsValidTrackdir(trackdir));
-	return
-		GetRailTileType(tile) == RAIL_TILE_SIGNALS &&
-		GetPresentSignals(tile) & SignalAlongTrackdir(trackdir);
+	return GetRailTileType(tile) == RAIL_TILE_SIGNALS && GetPresentSignals(tile) & SignalAlongTrackdir(trackdir);
 }
 
 /**
@@ -539,10 +463,8 @@ static inline void SetSignalStateByTrackdir(TileIndex tile, Trackdir trackdir, S
  */
 static inline bool HasPbsSignalOnTrackdir(TileIndex tile, Trackdir td)
 {
-	return
-		IsTileType(tile, MP_RAILWAY) &&
-		HasSignalOnTrackdir(tile, td) &&
-		IsPbsSignal(GetSignalType(tile, TrackdirToTrack(td)));
+	return IsTileType(tile, MP_RAILWAY) && HasSignalOnTrackdir(tile, td) &&
+			IsPbsSignal(GetSignalType(tile, TrackdirToTrack(td)));
 }
 
 /**
@@ -553,17 +475,11 @@ static inline bool HasPbsSignalOnTrackdir(TileIndex tile, Trackdir td)
  */
 static inline bool HasOnewaySignalBlockingTrackdir(TileIndex tile, Trackdir td)
 {
-	return
-		IsTileType(tile, MP_RAILWAY) &&
-		HasSignalOnTrackdir(tile, ReverseTrackdir(td)) &&
-		!HasSignalOnTrackdir(tile, td) &&
-		IsOnewaySignal(tile, TrackdirToTrack(td));
+	return IsTileType(tile, MP_RAILWAY) && HasSignalOnTrackdir(tile, ReverseTrackdir(td)) &&
+			!HasSignalOnTrackdir(tile, td) && IsOnewaySignal(tile, TrackdirToTrack(td));
 }
 
 
-/**
- * Return the rail type of tile, or INVALID_RAILTYPE if this is no rail tile.
- */
 RailType GetTileRailType(TileIndex tile);
 
 /** The ground 'under' the rail */
@@ -614,11 +530,11 @@ static inline void MakeRailNormal(TileIndex t, Owner o, TrackBits b, RailType r)
 }
 
 
-static inline void MakeRailDepot(TileIndex t, Owner o, DiagDirection d, RailType r)
+static inline void MakeRailDepot(TileIndex t, Owner o, DepotID did, DiagDirection d, RailType r)
 {
 	SetTileType(t, MP_RAILWAY);
 	SetTileOwner(t, o);
-	_m[t].m2 = 0;
+	_m[t].m2 = did;
 	_m[t].m3 = r;
 	_m[t].m4 = 0;
 	_m[t].m5 = RAIL_TILE_DEPOT << 6 | d;
@@ -626,17 +542,4 @@ static inline void MakeRailDepot(TileIndex t, Owner o, DiagDirection d, RailType
 	_me[t].m7 = 0;
 }
 
-
-static inline void MakeRailWaypoint(TileIndex t, Owner o, Axis a, RailType r, uint index)
-{
-	SetTileType(t, MP_RAILWAY);
-	SetTileOwner(t, o);
-	_m[t].m2 = index;
-	_m[t].m3 = r;
-	_m[t].m4 = 0;
-	_m[t].m5 = RAIL_TILE_WAYPOINT << 6 | a;
-	SB(_m[t].m6, 2, 4, 0);
-	_me[t].m7 = 0;
-}
-
 #endif /* RAIL_MAP_H */
