@@ -1,4 +1,4 @@
-/* $Id: road_func.h 21846 2011-01-18 23:09:43Z rubidium $ */
+/* $Id: road_func.h 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,9 +14,7 @@
 
 #include "core/bitmath_func.hpp"
 #include "road_type.h"
-#include "direction_type.h"
-#include "company_type.h"
-#include "tile_type.h"
+#include "economy_func.h"
 
 /**
  * Iterate through each set RoadType in a RoadTypes value.
@@ -57,7 +55,6 @@ static inline RoadTypes RoadTypeToRoadTypes(RoadType rt)
  *
  * @param r The given RoadTypes
  * @return The complement of the given RoadTypes
- * @note The unused value ROADTYPES_HWAY will be used, too.
  */
 static inline RoadTypes ComplementRoadTypes(RoadTypes r)
 {
@@ -95,7 +92,7 @@ static inline RoadBits MirrorRoadBits(RoadBits r)
 /**
  * Calculate rotated RoadBits
  *
- * Move the Roadbits clockwise til they are in their final position.
+ * Move the Roadbits clockwise until they are in their final position.
  *
  * @param r The given RoadBits value
  * @param rot The given Rotation angle
@@ -148,6 +145,19 @@ static inline RoadBits AxisToRoadBits(Axis a)
 	return a == AXIS_X ? ROAD_X : ROAD_Y;
 }
 
+
+/**
+ * Calculates the maintenance cost of a number of road bits.
+ * @param roadtype Road type to get the cost for.
+ * @param num Number of road bits.
+ * @return Total cost.
+ */
+static inline Money RoadMaintenanceCost(RoadType roadtype, uint32 num)
+{
+	assert(roadtype < ROADTYPE_END);
+	return (_price[PR_INFRASTRUCTURE_ROAD] * (roadtype == ROADTYPE_TRAM ? 3 : 2) * num * (1 + IntSqrt(num))) >> 9; // 2 bits fraction for the multiplier and 7 bits scaling.
+}
+
 bool HasRoadTypesAvail(const CompanyID company, const RoadTypes rts);
 bool ValParamRoadType(const RoadType rt);
 RoadTypes GetCompanyRoadtypes(const CompanyID company);
