@@ -1,10 +1,15 @@
-/* $Id: autoreplace_sl.cpp 15903 2009-03-30 23:15:05Z rubidium $ */
+/* $Id: autoreplace_sl.cpp 24136 2012-04-17 19:44:02Z michi_cc $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file autoreplace_sl.cpp Code handling saving and loading of autoreplace rules */
 
 #include "../stdafx.h"
-#include "../engine_type.h"
-#include "../group_type.h"
 #include "../autoreplace_base.h"
 
 #include "saveload.h"
@@ -15,6 +20,7 @@ static const SaveLoad _engine_renew_desc[] = {
 
 	    SLE_REF(EngineRenew, next,     REF_ENGINE_RENEWS),
 	SLE_CONDVAR(EngineRenew, group_id, SLE_UINT16, 60, SL_MAX_VERSION),
+	SLE_CONDVAR(EngineRenew, replace_when_old, SLE_BOOL, 175, SL_MAX_VERSION),
 	SLE_END()
 };
 
@@ -37,14 +43,23 @@ static void Load_ERNW()
 		SlObject(er, _engine_renew_desc);
 
 		/* Advanced vehicle lists, ungrouped vehicles got added */
-		if (CheckSavegameVersion(60)) {
+		if (IsSavegameVersionBefore(60)) {
 			er->group_id = ALL_GROUP;
-		} else if (CheckSavegameVersion(71)) {
+		} else if (IsSavegameVersionBefore(71)) {
 			if (er->group_id == DEFAULT_GROUP) er->group_id = ALL_GROUP;
 		}
 	}
 }
 
+static void Ptrs_ERNW()
+{
+	EngineRenew *er;
+
+	FOR_ALL_ENGINE_RENEWS(er) {
+		SlObject(er, _engine_renew_desc);
+	}
+}
+
 extern const ChunkHandler _autoreplace_chunk_handlers[] = {
-	{ 'ERNW', Save_ERNW,     Load_ERNW,     CH_ARRAY | CH_LAST},
+	{ 'ERNW', Save_ERNW, Load_ERNW, Ptrs_ERNW, NULL, CH_ARRAY | CH_LAST},
 };
