@@ -1,4 +1,4 @@
-/* $Id: economy_sl.cpp 18008 2009-11-08 12:23:02Z frosch $ */
+/* $Id: economy_sl.cpp 21284 2010-11-21 12:47:04Z alberth $ */
 
 /*
  * This file is part of OpenTTD.
@@ -19,7 +19,7 @@
 static void Load_PRIC()
 {
 	/* Old games store 49 base prices, very old games store them as int32 */
-	int vt = CheckSavegameVersion(65) ? SLE_FILE_I32 : SLE_FILE_I64;
+	int vt = IsSavegameVersionBefore(65) ? SLE_FILE_I32 : SLE_FILE_I64;
 	SlArray(NULL, 49, vt | SLE_VAR_NULL);
 	SlArray(NULL, 49, SLE_FILE_U16 | SLE_VAR_NULL);
 }
@@ -27,15 +27,15 @@ static void Load_PRIC()
 /** Cargo payment rates in pre 126 savegames */
 static void Load_CAPR()
 {
-	uint num_cargo = CheckSavegameVersion(55) ? 12 : NUM_CARGO;
-	int vt = CheckSavegameVersion(65) ? SLE_FILE_I32 : SLE_FILE_I64;
+	uint num_cargo = IsSavegameVersionBefore(55) ? 12 : NUM_CARGO;
+	int vt = IsSavegameVersionBefore(65) ? SLE_FILE_I32 : SLE_FILE_I64;
 	SlArray(NULL, num_cargo, vt | SLE_VAR_NULL);
 	SlArray(NULL, num_cargo, SLE_FILE_U16 | SLE_VAR_NULL);
 }
 
 static const SaveLoad _economy_desc[] = {
 	SLE_CONDNULL(4,                                                                  0, 64),             // max_loan
-	SLE_CONDNULL(8,                                                                 65, SL_MAX_VERSION), // max_loan
+	SLE_CONDNULL(8,                                                                 65, 143), // max_loan
 	SLE_CONDVAR(Economy, old_max_loan_unround,          SLE_FILE_I32 | SLE_VAR_I64,  0, 64),
 	SLE_CONDVAR(Economy, old_max_loan_unround,          SLE_INT64,                  65, 125),
 	SLE_CONDVAR(Economy, old_max_loan_unround_fract,    SLE_UINT16,                 70, 125),
@@ -59,7 +59,7 @@ static void Save_ECMY()
 static void Load_ECMY()
 {
 	SlObject(&_economy, _economy_desc);
-	StartupIndustryDailyChanges(CheckSavegameVersion(102));  // old savegames will need to be initialized
+	StartupIndustryDailyChanges(IsSavegameVersionBefore(102));  // old savegames will need to be initialized
 }
 
 static const SaveLoad _cargopayment_desc[] = {
@@ -99,8 +99,8 @@ static void Ptrs_CAPY()
 
 
 extern const ChunkHandler _economy_chunk_handlers[] = {
-	{ 'CAPY', Save_CAPY,     Load_CAPY,     Ptrs_CAPY, CH_ARRAY},
-	{ 'PRIC', NULL,          Load_PRIC,     NULL,      CH_RIFF | CH_AUTO_LENGTH},
-	{ 'CAPR', NULL,          Load_CAPR,     NULL,      CH_RIFF | CH_AUTO_LENGTH},
-	{ 'ECMY', Save_ECMY,     Load_ECMY,     NULL,      CH_RIFF | CH_LAST},
+	{ 'CAPY', Save_CAPY,     Load_CAPY,     Ptrs_CAPY, NULL, CH_ARRAY},
+	{ 'PRIC', NULL,          Load_PRIC,     NULL,      NULL, CH_RIFF | CH_AUTO_LENGTH},
+	{ 'CAPR', NULL,          Load_CAPR,     NULL,      NULL, CH_RIFF | CH_AUTO_LENGTH},
+	{ 'ECMY', Save_ECMY,     Load_ECMY,     NULL,      NULL, CH_RIFF | CH_LAST},
 };
