@@ -1,4 +1,4 @@
-/* $Id: engine_sl.cpp 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: engine_sl.cpp 24810 2012-12-09 16:55:03Z frosch $ */
 
 /*
  * This file is part of OpenTTD.
@@ -30,15 +30,16 @@ static const SaveLoad _engine_desc[] = {
 
 	SLE_CONDNULL(1,                                                        0, 120),
 	     SLE_VAR(Engine, flags,               SLE_UINT8),
-	     SLE_VAR(Engine, preview_company_rank,SLE_UINT8),
+	SLE_CONDNULL(1,                                                        0, 178), // old preview_company_rank
+	 SLE_CONDVAR(Engine, preview_asked,       SLE_UINT16,                179, SL_MAX_VERSION),
+	 SLE_CONDVAR(Engine, preview_company,     SLE_UINT8,                 179, SL_MAX_VERSION),
 	     SLE_VAR(Engine, preview_wait,        SLE_UINT8),
 	SLE_CONDNULL(1,                                                        0,  44),
 	 SLE_CONDVAR(Engine, company_avail,       SLE_FILE_U8  | SLE_VAR_U16,  0, 103),
 	 SLE_CONDVAR(Engine, company_avail,       SLE_UINT16,                104, SL_MAX_VERSION),
 	 SLE_CONDSTR(Engine, name,                SLE_STR, 0,                 84, SL_MAX_VERSION),
 
-	/* reserve extra space in savegame here. (currently 16 bytes) */
-	SLE_CONDNULL(16,                                                       2, SL_MAX_VERSION),
+	SLE_CONDNULL(16,                                                       2, 143), // old reserved space
 
 	SLE_END()
 };
@@ -68,6 +69,14 @@ static void Load_ENGN()
 	while ((index = SlIterateArray()) != -1) {
 		Engine *e = GetTempDataEngine(index);
 		SlObject(e, _engine_desc);
+
+		if (IsSavegameVersionBefore(179)) {
+			/* preview_company_rank was replaced with preview_company and preview_asked.
+			 * Just cancel any previews. */
+			e->flags &= ~4; // ENGINE_OFFER_WINDOW_OPEN
+			e->preview_company = INVALID_COMPANY;
+			e->preview_asked = (CompanyMask)-1;
+		}
 	}
 }
 
@@ -92,7 +101,8 @@ void CopyTempEngineData()
 		e->duration_phase_2    = se->duration_phase_2;
 		e->duration_phase_3    = se->duration_phase_3;
 		e->flags               = se->flags;
-		e->preview_company_rank= se->preview_company_rank;
+		e->preview_asked       = se->preview_asked;
+		e->preview_company     = se->preview_company;
 		e->preview_wait        = se->preview_wait;
 		e->company_avail       = se->company_avail;
 		if (se->name != NULL) e->name = strdup(se->name);
@@ -138,18 +148,16 @@ static void Save_EIDS()
 
 static void Load_EIDS()
 {
-	int index;
-
 	_engine_mngr.Clear();
 
-	while ((index = SlIterateArray()) != -1) {
+	while (SlIterateArray() != -1) {
 		EngineIDMapping *eid = _engine_mngr.Append();
 		SlObject(eid, _engine_id_mapping_desc);
 	}
 }
 
 extern const ChunkHandler _engine_chunk_handlers[] = {
-	{ 'EIDS', Save_EIDS, Load_EIDS, NULL, CH_ARRAY          },
-	{ 'ENGN', Save_ENGN, Load_ENGN, NULL, CH_ARRAY          },
-	{ 'ENGS', NULL,      Load_ENGS, NULL, CH_RIFF | CH_LAST },
+	{ 'EIDS', Save_EIDS, Load_EIDS, NULL, NULL, CH_ARRAY          },
+	{ 'ENGN', Save_ENGN, Load_ENGN, NULL, NULL, CH_ARRAY          },
+	{ 'ENGS', NULL,      Load_ENGS, NULL, NULL, CH_RIFF | CH_LAST },
 };
