@@ -1,4 +1,4 @@
-/* $Id: misc_sl.cpp 18809 2010-01-15 16:41:15Z rubidium $ */
+/* $Id: misc_sl.cpp 23740 2012-01-03 21:32:51Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,20 +11,19 @@
 
 #include "../stdafx.h"
 #include "../date_func.h"
-#include "../variables.h"
-#include "../openttd.h"
 #include "../zoom_func.h"
-#include "../vehicle_func.h"
 #include "../window_gui.h"
 #include "../window_func.h"
 #include "../viewport_func.h"
 #include "../gfx_func.h"
-#include "../company_base.h"
 #include "../core/random_func.hpp"
+#include "../fios.h"
 
 #include "saveload.h"
 
 extern TileIndex _cur_tileloop_tile;
+extern uint16 _disaster_delay;
+extern byte _trees_tick_ctr;
 
 /* Keep track of current game position */
 int _saved_scrollpos_x;
@@ -56,18 +55,26 @@ void ResetViewportAfterLoadGame()
 	vp->virtual_width = ScaleByZoom(vp->width, vp->zoom);
 	vp->virtual_height = ScaleByZoom(vp->height, vp->zoom);
 
+	/* If zoom_max is ZOOM_LVL_MIN then the setting has not been loaded yet, therefore all levels are allowed. */
+	if (_settings_client.gui.zoom_max != ZOOM_LVL_MIN) {
+		/* Ensure zoom level is allowed */
+		while (vp->zoom < _settings_client.gui.zoom_min) DoZoomInOutWindow(ZOOM_OUT, w);
+		while (vp->zoom > _settings_client.gui.zoom_max) DoZoomInOutWindow(ZOOM_IN, w);
+	}
+
 	DoZoomInOutWindow(ZOOM_NONE, w); // update button status
 	MarkWholeScreenDirty();
 }
 
+byte _age_cargo_skip_counter; ///< Skip aging of cargo? Used before savegame version 162.
 
 static const SaveLoadGlobVarList _date_desc[] = {
 	SLEG_CONDVAR(_date,                   SLE_FILE_U16 | SLE_VAR_I32,  0,  30),
 	SLEG_CONDVAR(_date,                   SLE_INT32,                  31, SL_MAX_VERSION),
 	    SLEG_VAR(_date_fract,             SLE_UINT16),
 	    SLEG_VAR(_tick_counter,           SLE_UINT16),
-	    SLEG_VAR(_vehicle_id_ctr_day,     SLE_UINT16),
-	    SLEG_VAR(_age_cargo_skip_counter, SLE_UINT8),
+	SLE_CONDNULL(2, 0, 156), // _vehicle_id_ctr_day
+	SLEG_CONDVAR(_age_cargo_skip_counter, SLE_UINT8,                   0, 161),
 	SLE_CONDNULL(1, 0, 45),
 	SLEG_CONDVAR(_cur_tileloop_tile,      SLE_FILE_U16 | SLE_VAR_U32,  0, 5),
 	SLEG_CONDVAR(_cur_tileloop_tile,      SLE_UINT32,                  6, SL_MAX_VERSION),
@@ -86,6 +93,31 @@ static const SaveLoadGlobVarList _date_desc[] = {
 	    SLEG_END()
 };
 
+static const SaveLoadGlobVarList _date_check_desc[] = {
+	SLEG_CONDVAR(_load_check_data.current_date,  SLE_FILE_U16 | SLE_VAR_I32,  0,  30),
+	SLEG_CONDVAR(_load_check_data.current_date,  SLE_INT32,                  31, SL_MAX_VERSION),
+	    SLE_NULL(2),                       // _date_fract
+	    SLE_NULL(2),                       // _tick_counter
+	SLE_CONDNULL(2, 0, 156),               // _vehicle_id_ctr_day
+	SLE_CONDNULL(1, 0, 161),               // _age_cargo_skip_counter
+	SLE_CONDNULL(1, 0, 45),
+	SLE_CONDNULL(2, 0, 5),                 // _cur_tileloop_tile
+	SLE_CONDNULL(4, 6, SL_MAX_VERSION),    // _cur_tileloop_tile
+	    SLE_NULL(2),                       // _disaster_delay
+	SLE_CONDNULL(2, 0, 119),
+	    SLE_NULL(4),                       // _random.state[0]
+	    SLE_NULL(4),                       // _random.state[1]
+	SLE_CONDNULL(1,  0,   9),
+	SLE_CONDNULL(4, 10, 119),
+	    SLE_NULL(1),                       // _cur_company_tick_index
+	SLE_CONDNULL(2, 0, 108),               // _next_competitor_start
+	SLE_CONDNULL(4, 109, SL_MAX_VERSION),  // _next_competitor_start
+	    SLE_NULL(1),                       // _trees_tick_ctr
+	SLE_CONDNULL(1, 4, SL_MAX_VERSION),    // _pause_mode
+	SLE_CONDNULL(4, 11, 119),
+	    SLEG_END()
+};
+
 /* Save load date related variables as well as persistent tick counters
  * XXX: currently some unrelated stuff is just put here */
 static void SaveLoad_DATE()
@@ -93,6 +125,14 @@ static void SaveLoad_DATE()
 	SlGlobList(_date_desc);
 }
 
+static void Check_DATE()
+{
+	SlGlobList(_date_check_desc);
+	if (IsSavegameVersionBefore(31)) {
+		_load_check_data.current_date += DAYS_TILL_ORIGINAL_BASE_YEAR;
+	}
+}
+
 
 static const SaveLoadGlobVarList _view_desc[] = {
 	SLEG_CONDVAR(_saved_scrollpos_x,    SLE_FILE_I16 | SLE_VAR_I32, 0, 5),
@@ -109,6 +149,6 @@ static void SaveLoad_VIEW()
 }
 
 extern const ChunkHandler _misc_chunk_handlers[] = {
-	{ 'DATE', SaveLoad_DATE, SaveLoad_DATE, NULL, CH_RIFF},
-	{ 'VIEW', SaveLoad_VIEW, SaveLoad_VIEW, NULL, CH_RIFF | CH_LAST},
+	{ 'DATE', SaveLoad_DATE, SaveLoad_DATE, NULL, Check_DATE, CH_RIFF},
+	{ 'VIEW', SaveLoad_VIEW, SaveLoad_VIEW, NULL, NULL,       CH_RIFF | CH_LAST},
 };
