@@ -1,4 +1,4 @@
-/* $Id: saveload.cpp 22701 2011-07-30 17:45:37Z rubidium $ */
+/* $Id: saveload.cpp 25168 2013-04-08 20:52:32Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -39,9 +39,8 @@
 #include "../fileio_func.h"
 #include "../gamelog.h"
 #include "../string_func.h"
-#include "../engine_base.h"
 #include "../fios.h"
-#include "../gui.h"
+#include "../error.h"
 
 #include "table/strings.h"
 
@@ -224,9 +223,29 @@
  *  157   21862
  *  158   21933
  *  159   21962
- *  160   21974
+ *  160   21974   1.1.x
+ *  161   22567
+ *  162   22713
+ *  163   22767
+ *  164   23290
+ *  165   23304
+ *  166   23415
+ *  167   23504
+ *  168   23637
+ *  169   23816
+ *  170   23826
+ *  171   23835
+ *  172   23947
+ *  173   23967   1.2.0-RC1
+ *  174   23973   1.2.x
+ *  175   24136
+ *  176   24446
+ *  177   24619
+ *  178   24789
+ *  179   24810
+ *  180   24998   1.3.x
  */
-extern const uint16 SAVEGAME_VERSION = 160; ///< Current savegame version of OpenTTD.
+extern const uint16 SAVEGAME_VERSION = 180; ///< Current savegame version of OpenTTD.
 
 SavegameType _savegame_type; ///< type of savegame we are loading
 
@@ -270,7 +289,7 @@ struct ReadBuffer {
 	{
 	}
 
-	FORCEINLINE byte ReadByte()
+	inline byte ReadByte()
 	{
 		if (this->bufp == this->bufe) {
 			size_t len = this->reader->Read(this->buf, lengthof(this->buf));
@@ -310,7 +329,7 @@ struct MemoryDumper {
 	 * Write a single byte into the dumper.
 	 * @param b The byte to write.
 	 */
-	FORCEINLINE void WriteByte(byte b)
+	inline void WriteByte(byte b)
 	{
 		/* Are we at the end of this chunk? */
 		if (this->buf == this->bufe) {
@@ -395,7 +414,10 @@ extern const ChunkHandler _station_chunk_handlers[];
 extern const ChunkHandler _industry_chunk_handlers[];
 extern const ChunkHandler _economy_chunk_handlers[];
 extern const ChunkHandler _subsidy_chunk_handlers[];
+extern const ChunkHandler _cargomonitor_chunk_handlers[];
+extern const ChunkHandler _goal_chunk_handlers[];
 extern const ChunkHandler _ai_chunk_handlers[];
+extern const ChunkHandler _game_chunk_handlers[];
 extern const ChunkHandler _animated_tile_chunk_handlers[];
 extern const ChunkHandler _newgrf_chunk_handlers[];
 extern const ChunkHandler _group_chunk_handlers[];
@@ -404,6 +426,7 @@ extern const ChunkHandler _autoreplace_chunk_handlers[];
 extern const ChunkHandler _labelmaps_chunk_handlers[];
 extern const ChunkHandler _airport_chunk_handlers[];
 extern const ChunkHandler _object_chunk_handlers[];
+extern const ChunkHandler _persistent_storage_chunk_handlers[];
 
 /** Array of all chunks in a savegame, \c NULL terminated. */
 static const ChunkHandler * const _chunk_handlers[] = {
@@ -420,12 +443,15 @@ static const ChunkHandler * const _chunk_handlers[] = {
 	_industry_chunk_handlers,
 	_economy_chunk_handlers,
 	_subsidy_chunk_handlers,
+	_cargomonitor_chunk_handlers,
+	_goal_chunk_handlers,
 	_engine_chunk_handlers,
 	_town_chunk_handlers,
 	_sign_chunk_handlers,
 	_station_chunk_handlers,
 	_company_chunk_handlers,
 	_ai_chunk_handlers,
+	_game_chunk_handlers,
 	_animated_tile_chunk_handlers,
 	_newgrf_chunk_handlers,
 	_group_chunk_handlers,
@@ -434,6 +460,7 @@ static const ChunkHandler * const _chunk_handlers[] = {
 	_labelmaps_chunk_handlers,
 	_airport_chunk_handlers,
 	_object_chunk_handlers,
+	_persistent_storage_chunk_handlers,
 	NULL,
 };
 
@@ -488,11 +515,12 @@ void NORETURN SlError(StringID string, const char *extra_msg)
 		_sl.error_str = string;
 		free(_sl.extra_msg);
 		_sl.extra_msg = (extra_msg == NULL) ? NULL : strdup(extra_msg);
-		/* We have to NULL all pointers here; we might be in a state where
-		 * the pointers are actually filled with indices, which means that
-		 * when we access them during cleaning the pool dereferences of
-		 * those indices will be made with segmentation faults as result. */
 	}
+
+	/* We have to NULL all pointers here; we might be in a state where
+	 * the pointers are actually filled with indices, which means that
+	 * when we access them during cleaning the pool dereferences of
+	 * those indices will be made with segmentation faults as result. */
 	if (_sl.action == SLA_LOAD || _sl.action == SLA_PTRS) SlNullPointers();
 	throw std::exception();
 }
@@ -765,7 +793,7 @@ int SlIterateArray()
 	 * we must have read in all the data, so we must be at end of current block. */
 	if (_next_offs != 0 && _sl.reader->GetSize() != _next_offs) SlErrorCorrupt("Invalid chunk size");
 
-	while (true) {
+	for (;;) {
 		uint length = SlReadArrayLength();
 		if (length == 0) {
 			_next_offs = 0;
@@ -878,15 +906,15 @@ size_t SlGetFieldLength()
 int64 ReadValue(const void *ptr, VarType conv)
 {
 	switch (GetVarMemType(conv)) {
-		case SLE_VAR_BL:  return (*(bool *)ptr != 0);
-		case SLE_VAR_I8:  return *(int8  *)ptr;
-		case SLE_VAR_U8:  return *(byte  *)ptr;
-		case SLE_VAR_I16: return *(int16 *)ptr;
-		case SLE_VAR_U16: return *(uint16*)ptr;
-		case SLE_VAR_I32: return *(int32 *)ptr;
-		case SLE_VAR_U32: return *(uint32*)ptr;
-		case SLE_VAR_I64: return *(int64 *)ptr;
-		case SLE_VAR_U64: return *(uint64*)ptr;
+		case SLE_VAR_BL:  return (*(const bool *)ptr != 0);
+		case SLE_VAR_I8:  return *(const int8  *)ptr;
+		case SLE_VAR_U8:  return *(const byte  *)ptr;
+		case SLE_VAR_I16: return *(const int16 *)ptr;
+		case SLE_VAR_U16: return *(const uint16*)ptr;
+		case SLE_VAR_I32: return *(const int32 *)ptr;
+		case SLE_VAR_U32: return *(const uint32*)ptr;
+		case SLE_VAR_I64: return *(const int64 *)ptr;
+		case SLE_VAR_U64: return *(const uint64*)ptr;
 		case SLE_VAR_NULL:return 0;
 		default: NOT_REACHED();
 	}
@@ -1006,12 +1034,12 @@ static inline size_t SlCalcStringLen(const void *ptr, size_t length, VarType con
 		default: NOT_REACHED();
 		case SLE_VAR_STR:
 		case SLE_VAR_STRQ:
-			str = *(const char**)ptr;
+			str = *(const char * const *)ptr;
 			len = SIZE_MAX;
 			break;
 		case SLE_VAR_STRB:
 		case SLE_VAR_STRBQ:
-			str = (const char*)ptr;
+			str = (const char *)ptr;
 			len = length;
 			break;
 	}
@@ -1070,6 +1098,7 @@ static void SlString(void *ptr, size_t length, VarType conv)
 					free(*(char **)ptr);
 					if (len == 0) {
 						*(char **)ptr = NULL;
+						return;
 					} else {
 						*(char **)ptr = MallocT<char>(len + 1); // terminating '\0'
 						ptr = *(char **)ptr;
@@ -1079,7 +1108,17 @@ static void SlString(void *ptr, size_t length, VarType conv)
 			}
 
 			((char *)ptr)[len] = '\0'; // properly terminate the string
-			str_validate((char *)ptr, (char *)ptr + len);
+			StringValidationSettings settings = SVS_REPLACE_WITH_QUESTION_MARK;
+			if ((conv & SLF_ALLOW_CONTROL) != 0) {
+				settings = settings | SVS_ALLOW_CONTROL_CODE;
+				if (IsSavegameVersionBefore(169)) {
+					str_fix_scc_encoded((char *)ptr, (char *)ptr + len);
+				}
+			}
+			if ((conv & SLF_ALLOW_NEWLINE) != 0) {
+				settings = settings | SVS_ALLOW_NEWLINE;
+			}
+			str_validate((char *)ptr, (char *)ptr + len, settings);
 			break;
 		}
 		case SLA_PTRS: break;
@@ -1166,22 +1205,23 @@ static size_t ReferenceToInt(const void *obj, SLRefType rt)
 	if (obj == NULL) return 0;
 
 	switch (rt) {
-		case REF_VEHICLE_OLD: // Old vehicles we save as new onces
+		case REF_VEHICLE_OLD: // Old vehicles we save as new ones
 		case REF_VEHICLE:   return ((const  Vehicle*)obj)->index + 1;
 		case REF_STATION:   return ((const  Station*)obj)->index + 1;
 		case REF_TOWN:      return ((const     Town*)obj)->index + 1;
 		case REF_ORDER:     return ((const    Order*)obj)->index + 1;
 		case REF_ROADSTOPS: return ((const RoadStop*)obj)->index + 1;
-		case REF_ENGINE_RENEWS: return ((const EngineRenew*)obj)->index + 1;
-		case REF_CARGO_PACKET:  return ((const CargoPacket*)obj)->index + 1;
-		case REF_ORDERLIST:     return ((const   OrderList*)obj)->index + 1;
+		case REF_ENGINE_RENEWS: return ((const       EngineRenew*)obj)->index + 1;
+		case REF_CARGO_PACKET:  return ((const       CargoPacket*)obj)->index + 1;
+		case REF_ORDERLIST:     return ((const         OrderList*)obj)->index + 1;
+		case REF_STORAGE:       return ((const PersistentStorage*)obj)->index + 1;
 		default: NOT_REACHED();
 	}
 }
 
 /**
  * Pointers cannot be loaded from a savegame, so this function
- * gets the index from the savegame and returns the appropiate
+ * gets the index from the savegame and returns the appropriate
  * pointer from the already loaded base.
  * Remember that an index of 0 is a NULL pointer so all indices
  * are +1 so vehicle 0 is saved as 1.
@@ -1244,6 +1284,10 @@ static void *IntToReference(size_t index, SLRefType rt)
 			if (CargoPacket::IsValidID(index)) return CargoPacket::Get(index);
 			SlErrorCorrupt("Referencing invalid CargoPacket");
 
+		case REF_STORAGE:
+			if (PersistentStorage::IsValidID(index)) return PersistentStorage::Get(index);
+			SlErrorCorrupt("Referencing invalid PersistentStorage");
+
 		default: NOT_REACHED();
 	}
 }
@@ -1254,7 +1298,7 @@ static void *IntToReference(size_t index, SLRefType rt)
  */
 static inline size_t SlCalcListLen(const void *list)
 {
-	std::list<void *> *l = (std::list<void *> *) list;
+	const std::list<void *> *l = (const std::list<void *> *) list;
 
 	int type_size = IsSavegameVersionBefore(69) ? 2 : 4;
 	/* Each entry is saved as type_size bytes, plus type_size bytes are used for the length
@@ -1325,7 +1369,7 @@ static void SlList(void *list, SLRefType conv)
 static inline bool SlIsObjectValidInSavegame(const SaveLoad *sld)
 {
 	if (_sl_version < sld->version_from || _sl_version > sld->version_to) return false;
-	if (sld->conv & SLF_SAVE_NO) return false;
+	if (sld->conv & SLF_NOT_IN_SAVE) return false;
 
 	return true;
 }
@@ -1337,7 +1381,7 @@ static inline bool SlIsObjectValidInSavegame(const SaveLoad *sld)
  */
 static inline bool SlSkipVariableOnLoad(const SaveLoad *sld)
 {
-	if ((sld->conv & SLF_NETWORK_NO) && _sl.action != SLA_SAVE && _networking && !_network_server) {
+	if ((sld->conv & SLF_NO_NETWORK_SYNC) && _sl.action != SLA_SAVE && _networking && !_network_server) {
 		SlSkipBytes(SlCalcConvMemLen(sld->conv) * sld->length);
 		return true;
 	}
@@ -1349,7 +1393,7 @@ static inline bool SlSkipVariableOnLoad(const SaveLoad *sld)
  * Calculate the size of an object.
  * @param object to be measured
  * @param sld The SaveLoad description of the object so we know how to manipulate it
- * @return size of given objetc
+ * @return size of given object
  */
 size_t SlCalcObjLength(const void *object, const SaveLoad *sld)
 {
@@ -1427,7 +1471,7 @@ bool SlObjectMember(void *ptr, const SaveLoad *sld)
 					}
 					break;
 				case SL_ARR: SlArray(ptr, sld->length, conv); break;
-				case SL_STR: SlString(ptr, sld->length, conv); break;
+				case SL_STR: SlString(ptr, sld->length, sld->conv); break;
 				case SL_LST: SlList(ptr, (SLRefType)conv); break;
 				default: NOT_REACHED();
 			}
@@ -1634,7 +1678,7 @@ static void SlStubSaveProc()
 
 /**
  * Save a chunk of data (eg. vehicles, stations, etc.). Each chunk is
- * prefixed by an ID identifying it, followed by data, and terminator where appropiate
+ * prefixed by an ID identifying it, followed by data, and terminator where appropriate
  * @param ch The chunkhandler that will be used for the operation
  */
 static void SlSaveChunk(const ChunkHandler *ch)
@@ -1689,7 +1733,7 @@ static void SlSaveChunks()
  * Find the ChunkHandler that will be used for processing the found
  * chunk in the savegame or in memory
  * @param id the chunk in question
- * @return returns the appropiate chunkhandler
+ * @return returns the appropriate chunkhandler
  */
 static const ChunkHandler *SlFindChunkHandler(uint32 id)
 {
@@ -1872,7 +1916,7 @@ struct LZOLoadFilter : LoadFilter {
 		if (tmp[0] != lzo_adler32(0, out, size + sizeof(uint32))) SlErrorCorrupt("Bad checksum");
 
 		/* Decompress */
-		lzo1x_decompress(out + sizeof(uint32) * 1, size, buf, &len, NULL);
+		lzo1x_decompress_safe(out + sizeof(uint32) * 1, size, buf, &len, NULL);
 		return len;
 	}
 };
@@ -2044,7 +2088,7 @@ struct ZlibSaveFilter : SaveFilter {
 			 * "Conditional jump or move depends on uninitialised value(s)" kind:
 			 * According to the author of zlib it is not a bug and it won't be fixed.
 			 * http://groups.google.com/group/comp.compression/browse_thread/thread/b154b8def8c2a3ef/cdf9b8729ce17ee2
-			 * [Mark Adler, Feb 24 2004, 'zlib-1.2.1 valgrind warnings' in the newgroup comp.compression]
+			 * [Mark Adler, Feb 24 2004, 'zlib-1.2.1 valgrind warnings' in the newsgroup comp.compression]
 			 */
 			int r = deflate(&this->z, mode);
 
@@ -2231,7 +2275,7 @@ static const SaveLoadFormat _saveload_formats[] = {
 	 * Higher compression levels are possible, and might improve savegame size by up to 25%, but are also up to 10 times slower.
 	 * The next significant reduction in file size is at level 4, but that is already 4 times slower. Level 3 is primarily 50%
 	 * slower while not improving the filesize, while level 0 and 1 are faster, but don't reduce savegame size much.
-	 * It's OTTX and not e.g. OTTL because liblzma is part of xz-utils and .tar.xz is prefered over .tar.lzma. */
+	 * It's OTTX and not e.g. OTTL because liblzma is part of xz-utils and .tar.xz is preferred over .tar.lzma. */
 	{"lzma",   TO_BE32X('OTTX'), CreateLoadFilter<LZMALoadFilter>,   CreateSaveFilter<LZMASaveFilter>,   0, 2, 9},
 #else
 	{"lzma",   TO_BE32X('OTTX'), NULL,                               NULL,                               0, 0, 0},
@@ -2271,7 +2315,8 @@ static const SaveLoadFormat *GetSavegameFormat(char *s, byte *compression_level)
 					char *end;
 					long level = strtol(complevel, &end, 10);
 					if (end == complevel || level != Clamp(level, slf->min_compression, slf->max_compression)) {
-						ShowInfoF("Compression level '%s' is not valid.", complevel);
+						SetDParamStr(0, complevel);
+						ShowErrorMessage(STR_CONFIG_ERROR, STR_CONFIG_ERROR_INVALID_SAVEGAME_COMPRESSION_LEVEL, WL_CRITICAL);
 					} else {
 						*compression_level = level;
 					}
@@ -2280,7 +2325,9 @@ static const SaveLoadFormat *GetSavegameFormat(char *s, byte *compression_level)
 			}
 		}
 
-		ShowInfoF("Savegame format '%s' is not available. Reverting to '%s'.", s, def->name);
+		SetDParamStr(0, s);
+		SetDParamStr(1, def->name);
+		ShowErrorMessage(STR_CONFIG_ERROR, STR_CONFIG_ERROR_INVALID_SAVEGAME_COMPRESSION_ALGORITHM, WL_CRITICAL);
 
 		/* Restore the string by adding the : back */
 		if (complevel != NULL) *complevel = ':';
@@ -2364,7 +2411,7 @@ static void SaveFileError()
 
 /**
  * We have written the whole game into memory, _memory_savegame, now find
- * and appropiate compressor and start writing to file.
+ * and appropriate compressor and start writing to file.
  */
 static SaveOrLoadResult SaveFileToDisk(bool threaded)
 {
@@ -2426,10 +2473,10 @@ void WaitTillSaved()
 
 /**
  * Actually perform the saving of the savegame.
- * General tactic is to first save the game to memory, then write it to file
+ * General tactics is to first save the game to memory, then write it to file
  * using the writer, either in threaded mode if possible, or single-threaded.
  * @param writer   The filter to write the savegame to.
- * @param threaded Whether to try to perform the saving asynchroniously.
+ * @param threaded Whether to try to perform the saving asynchronously.
  * @return Return the result of the action. #SL_OK or #SL_ERROR
  */
 static SaveOrLoadResult DoSave(SaveFilter *writer, bool threaded)
@@ -2460,7 +2507,7 @@ static SaveOrLoadResult DoSave(SaveFilter *writer, bool threaded)
 /**
  * Save the game using a (writer) filter.
  * @param writer   The filter to write the savegame to.
- * @param threaded Whether to try to perform the saving asynchroniously.
+ * @param threaded Whether to try to perform the saving asynchronously.
  * @return Return the result of the action. #SL_OK or #SL_ERROR
  */
 SaveOrLoadResult SaveWithFilter(SaveFilter *writer, bool threaded)
@@ -2522,8 +2569,7 @@ static SaveOrLoadResult DoLoad(LoadFilter *reader, bool load_check)
 			_sl_version = TO_BE32(hdr[1]) >> 16;
 			/* Minor is not used anymore from version 18.0, but it is still needed
 			 * in versions before that (4 cases) which can't be removed easy.
-			 * Therefor it is loaded, but never saved (or, it saves a 0 in any scenario).
-			 * So never EVER use this minor version again. -- TrueLight -- 22-11-2005 */
+			 * Therefore it is loaded, but never saved (or, it saves a 0 in any scenario). */
 			_sl_minor_version = (TO_BE32(hdr[1]) >> 8) & 0xFF;
 
 			DEBUG(sl, 1, "Loading savegame version %d", _sl_version);
@@ -2684,6 +2730,7 @@ SaveOrLoadResult SaveOrLoad(const char *filename, int mode, Subdirectory sb, boo
 		/* Make it a little easier to load savegames from the console */
 		if (fh == NULL && mode != SL_SAVE) fh = FioFOpenFile(filename, "rb", SAVE_DIR);
 		if (fh == NULL && mode != SL_SAVE) fh = FioFOpenFile(filename, "rb", BASE_DIR);
+		if (fh == NULL && mode != SL_SAVE) fh = FioFOpenFile(filename, "rb", SCENARIO_DIR);
 
 		if (fh == NULL) {
 			SlError(mode == SL_SAVE ? STR_GAME_SAVELOAD_ERROR_FILE_NOT_WRITEABLE : STR_GAME_SAVELOAD_ERROR_FILE_NOT_READABLE);
