@@ -1,4 +1,4 @@
-/* $Id: squirrel.cpp 23255 2011-11-18 21:04:08Z rubidium $ */
+/* $Id: squirrel.cpp 25560 2013-07-04 21:20:05Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,6 +14,7 @@
 #include "../debug.h"
 #include "squirrel_std.hpp"
 #include "../fileio_func.h"
+#include "../string_func.h"
 #include <sqstdaux.h>
 #include <../squirrel/sqpcheader.h>
 #include <../squirrel/sqvm.h>
@@ -33,7 +34,7 @@ void Squirrel::CompileError(HSQUIRRELVM vm, const SQChar *desc, const SQChar *so
 	engine->crashed = true;
 	SQPrintFunc *func = engine->print_func;
 	if (func == NULL) {
-		scfprintf(stderr, _SC("%s"), buf);
+		DEBUG(misc, 0, "[Squirrel] Compile error: %s", SQ2OTTD(buf));
 	} else {
 		(*func)(true, buf);
 	}
@@ -237,7 +238,7 @@ bool Squirrel::CallMethod(HSQOBJECT instance, const char *method_name, HSQOBJECT
 	sq_pushobject(this->vm, instance);
 	if (SQ_FAILED(sq_call(this->vm, 1, ret == NULL ? SQFalse : SQTrue, SQTrue, suspend))) return false;
 	if (ret != NULL) sq_getstackobj(vm, -1, ret);
-	/* Reset the top, but don't do so for the AI main function, as we need
+	/* Reset the top, but don't do so for the script main function, as we need
 	 *  a correct stack when resuming. */
 	if (suspend == -1 || !this->IsSuspended()) sq_settop(this->vm, top);
 	/* Restore the return-value location. */
@@ -252,6 +253,7 @@ bool Squirrel::CallStringMethodStrdup(HSQOBJECT instance, const char *method_nam
 	if (!this->CallMethod(instance, method_name, &ret, suspend)) return false;
 	if (ret._type != OT_STRING) return false;
 	*res = strdup(ObjectToString(&ret));
+	ValidateString(*res);
 	return true;
 }
 
@@ -273,22 +275,33 @@ bool Squirrel::CallBoolMethod(HSQOBJECT instance, const char *method_name, bool
 	return true;
 }
 
-/* static */ bool Squirrel::CreateClassInstanceVM(HSQUIRRELVM vm, const char *class_name, void *real_instance, HSQOBJECT *instance, SQRELEASEHOOK release_hook)
+/* static */ bool Squirrel::CreateClassInstanceVM(HSQUIRRELVM vm, const char *class_name, void *real_instance, HSQOBJECT *instance, SQRELEASEHOOK release_hook, bool prepend_API_name)
 {
+	Squirrel *engine = (Squirrel *)sq_getforeignptr(vm);
+
 	int oldtop = sq_gettop(vm);
 
 	/* First, find the class */
 	sq_pushroottable(vm);
-	sq_pushstring(vm, OTTD2SQ(class_name), -1);
+
+	if (prepend_API_name) {
+		char *class_name2 = (char *)alloca(strlen(class_name) + strlen(engine->GetAPIName()) + 1);
+		sprintf(class_name2, "%s%s", engine->GetAPIName(), class_name);
+
+		sq_pushstring(vm, OTTD2SQ(class_name2), -1);
+	} else {
+		sq_pushstring(vm, OTTD2SQ(class_name), -1);
+	}
+
 	if (SQ_FAILED(sq_get(vm, -2))) {
-		DEBUG(misc, 0, "[squirrel] Failed to find class by the name '%s'", class_name);
+		DEBUG(misc, 0, "[squirrel] Failed to find class by the name '%s%s'", prepend_API_name ? engine->GetAPIName() : "", class_name);
 		sq_settop(vm, oldtop);
 		return false;
 	}
 
 	/* Create the instance */
 	if (SQ_FAILED(sq_createinstance(vm, -1))) {
-		DEBUG(misc, 0, "[squirrel] Failed to create instance for class '%s'", class_name);
+		DEBUG(misc, 0, "[squirrel] Failed to create instance for class '%s%s'", prepend_API_name ? engine->GetAPIName() : "", class_name);
 		sq_settop(vm, oldtop);
 		return false;
 	}
@@ -316,13 +329,14 @@ bool Squirrel::CreateClassInstance(const char *class_name, void *real_instance,
 	return Squirrel::CreateClassInstanceVM(this->vm, class_name, real_instance, instance, NULL);
 }
 
-Squirrel::Squirrel()
+Squirrel::Squirrel(const char *APIName) :
+	global_pointer(NULL),
+	print_func(NULL),
+	crashed(false),
+	overdrawn_ops(0),
+	APIName(APIName)
 {
 	this->vm = sq_open(1024);
-	this->print_func = NULL;
-	this->global_pointer = NULL;
-	this->crashed = false;
-	this->overdrawn_ops = 0;
 
 	/* Handle compile-errors ourself, so we can display it nicely */
 	sq_setcompilererrorhandler(this->vm, &Squirrel::CompileError);
@@ -333,7 +347,7 @@ Squirrel::Squirrel()
 	sq_newclosure(this->vm, &Squirrel::_RunError, 0);
 	sq_seterrorhandler(this->vm);
 
-	/* Set the foreigh pointer, so we can always find this instance from within the VM */
+	/* Set the foreign pointer, so we can always find this instance from within the VM */
 	sq_setforeignptr(this->vm, this);
 
 	sq_pushroottable(this->vm);
@@ -401,14 +415,14 @@ static SQInteger _io_file_lexfeed_UTF8(SQUserPointer file)
 	return c;
 }
 
-static SQInteger _io_file_lexfeed_UCS2_LE(SQUserPointer file)
+static SQInteger _io_file_lexfeed_UCS2_no_swap(SQUserPointer file)
 {
 	wchar_t c;
 	if (((SQFile *)file)->Read(&c, sizeof(c), 1) > 0) return (SQChar)c;
 	return 0;
 }
 
-static SQInteger _io_file_lexfeed_UCS2_BE(SQUserPointer file)
+static SQInteger _io_file_lexfeed_UCS2_swap(SQUserPointer file)
 {
 	unsigned short c;
 	if (((SQFile *)file)->Read(&c, sizeof(c), 1) > 0) {
@@ -425,15 +439,25 @@ static SQInteger _io_file_read(SQUserPointer file, SQUserPointer buf, SQInteger
 	return ret;
 }
 
-/* static */ SQRESULT Squirrel::LoadFile(HSQUIRRELVM vm, const char *filename, SQBool printerror)
+SQRESULT Squirrel::LoadFile(HSQUIRRELVM vm, const char *filename, SQBool printerror)
 {
 	size_t size;
-	FILE *file = FioFOpenFile(filename, "rb", AI_DIR, &size);
+	FILE *file;
 	SQInteger ret;
 	unsigned short us;
 	unsigned char uc;
 	SQLEXREADFUNC func;
 
+	if (strncmp(this->GetAPIName(), "AI", 2) == 0) {
+		file = FioFOpenFile(filename, "rb", AI_DIR, &size);
+		if (file == NULL) file = FioFOpenFile(filename, "rb", AI_LIBRARY_DIR, &size);
+	} else if (strncmp(this->GetAPIName(), "GS", 2) == 0) {
+		file = FioFOpenFile(filename, "rb", GAME_DIR, &size);
+		if (file == NULL) file = FioFOpenFile(filename, "rb", GAME_LIBRARY_DIR, &size);
+	} else {
+		NOT_REACHED();
+	}
+
 	if (file != NULL) {
 		SQFile f(file, size);
 		ret = fread(&us, 1, sizeof(us), file);
@@ -450,9 +474,15 @@ static SQInteger _io_file_read(SQUserPointer file, SQUserPointer buf, SQInteger
 				FioFCloseFile(file);
 				return sq_throwerror(vm, _SC("Couldn't read bytecode"));
 			}
-			case 0xFFFE: func = _io_file_lexfeed_UCS2_BE; break; // UTF-16 little endian
-			case 0xFEFF: func = _io_file_lexfeed_UCS2_LE; break; // UTF-16 big endian
+			case 0xFFFE:
+				/* Either this file is encoded as big-endian and we're on a little-endian
+				 * machine, or this file is encoded as little-endian and we're on a big-endian
+				 * machine. Either way, swap the bytes of every word we read. */
+				func = _io_file_lexfeed_UCS2_swap;
+				break;
+			case 0xFEFF: func = _io_file_lexfeed_UCS2_no_swap; break;
 			case 0xBBEF: // UTF-8
+			case 0xEFBB: // UTF-8 on big-endian machine
 				if (fread(&uc, 1, sizeof(uc), file) == 0) {
 					FioFCloseFile(file);
 					return sq_throwerror(vm, _SC("I/O error"));
@@ -476,7 +506,7 @@ static SQInteger _io_file_read(SQUserPointer file, SQUserPointer buf, SQInteger
 	return sq_throwerror(vm, _SC("cannot open the file"));
 }
 
-/* static */ bool Squirrel::LoadScript(HSQUIRRELVM vm, const char *script, bool in_root)
+bool Squirrel::LoadScript(HSQUIRRELVM vm, const char *script, bool in_root)
 {
 	/* Make sure we are always in the root-table */
 	if (in_root) sq_pushroottable(vm);
@@ -513,15 +543,19 @@ Squirrel::~Squirrel()
 void Squirrel::InsertResult(bool result)
 {
 	sq_pushbool(this->vm, result);
-	vm->GetAt(vm->_stackbase + vm->_suspended_target) = vm->GetUp(-1);
-	vm->Pop();
+	if (this->IsSuspended()) { // Called before resuming a suspended script?
+		vm->GetAt(vm->_stackbase + vm->_suspended_target) = vm->GetUp(-1);
+		vm->Pop();
+	}
 }
 
 void Squirrel::InsertResult(int result)
 {
 	sq_pushinteger(this->vm, result);
-	vm->GetAt(vm->_stackbase + vm->_suspended_target) = vm->GetUp(-1);
-	vm->Pop();
+	if (this->IsSuspended()) { // Called before resuming a suspended script?
+		vm->GetAt(vm->_stackbase + vm->_suspended_target) = vm->GetUp(-1);
+		vm->Pop();
+	}
 }
 
 /* static */ void Squirrel::DecreaseOps(HSQUIRRELVM vm, int ops)
@@ -553,3 +587,8 @@ bool Squirrel::CanSuspend()
 {
 	return sq_can_suspend(this->vm);
 }
+
+SQInteger Squirrel::GetOpsTillSuspend()
+{
+	return this->vm->_ops_till_suspend;
+}
