@@ -1,4 +1,4 @@
-/* $Id: settings_func.h 20689 2010-08-29 15:58:43Z alberth $ */
+/* $Id: settings_func.h 23224 2011-11-14 21:42:54Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -20,7 +20,7 @@ void IConsoleSetSetting(const char *name, int32 value);
 void IConsoleGetSetting(const char *name, bool force_newgame = false);
 void IConsoleListSettings(const char *prefilter);
 
-void LoadFromConfig();
+void LoadFromConfig(bool minimal = false);
 void SaveToConfig();
 void CheckConfig();
 
