@@ -1,4 +1,4 @@
-/* $Id: signal.cpp 18809 2010-01-15 16:41:15Z rubidium $ */
+/* $Id: signal.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -14,18 +14,16 @@
 #include "station_map.h"
 #include "tunnelbridge_map.h"
 #include "vehicle_func.h"
-#include "functions.h"
+#include "viewport_func.h"
 #include "train.h"
 #include "company_base.h"
 
 
 /** these are the maximums used for updating signal blocks */
-enum {
-	SIG_TBU_SIZE    =  64, ///< number of signals entering to block
-	SIG_TBD_SIZE    = 256, ///< number of intersections - open nodes in current block
-	SIG_GLOB_SIZE   = 128, ///< number of open blocks (block can be opened more times until detected)
-	SIG_GLOB_UPDATE =  64, ///< how many items need to be in _globset to force update
-};
+static const uint SIG_TBU_SIZE    =  64; ///< number of signals entering to block
+static const uint SIG_TBD_SIZE    = 256; ///< number of intersections - open nodes in current block
+static const uint SIG_GLOB_SIZE   = 128; ///< number of open blocks (block can be opened more times until detected)
+static const uint SIG_GLOB_UPDATE =  64; ///< how many items need to be in _globset to force update
 
 assert_compile(SIG_GLOB_UPDATE <= SIG_GLOB_SIZE);
 
@@ -54,7 +52,7 @@ template <typename Tdir, uint items>
 struct SmallSet {
 private:
 	uint n;           // actual number of units
-	bool overflowed;  // did we try to oveflow the set?
+	bool overflowed;  // did we try to overflow the set?
 	const char *name; // name, used for debugging purposes...
 
 	/** Element of set */
@@ -75,7 +73,7 @@ public:
 	}
 
 	/**
-	 * Returns value of 'oveflowed'
+	 * Returns value of 'overflowed'
 	 * @return did we try to overflow the set?
 	 */
 	bool Overflowed()
@@ -133,7 +131,7 @@ public:
 	 * Tries to find given tile and dir in the set
 	 * @param tile tile
 	 * @param dir and dir to find
-	 * @return true iff the tile & dir elemnt was found
+	 * @return true iff the tile & dir element was found
 	 */
 	bool IsIn(TileIndex tile, Tdir dir)
 	{
@@ -203,7 +201,7 @@ static Vehicle *TrainOnTileEnum(Vehicle *v, void *)
  * The new and reverse direction is removed from _globset, because we are sure
  * it doesn't need to be checked again
  * Also, remove reverse direction from _tbdset
- * This is the 'core' part so the graph seaching won't enter any tile twice
+ * This is the 'core' part so the graph searching won't enter any tile twice
  *
  * @param t1 tile we are entering
  * @param d1 direction (tile side) we are entering
@@ -229,7 +227,7 @@ static inline bool CheckAddToTodoSet(TileIndex t1, DiagDirection d1, TileIndex t
  * The new and reverse direction is removed from Global set, because we are sure
  * it doesn't need to be checked again
  * Also, remove reverse direction from Todo set
- * This is the 'core' part so the graph seaching won't enter any tile twice
+ * This is the 'core' part so the graph searching won't enter any tile twice
  *
  * @param t1 tile we are entering
  * @param d1 direction (tile side) we are entering
@@ -301,7 +299,8 @@ static SigFlags ExploreSegment(Owner owner)
 
 				if (tracks == TRACK_BIT_HORZ || tracks == TRACK_BIT_VERT) { // there is exactly one incidating track, no need to check
 					tracks = tracks_masked;
-					if (!(flags & SF_TRAIN) && HasVehicleOnPos(tile, &tracks, &EnsureNoTrainOnTrackProc)) flags |= SF_TRAIN;
+					/* If no train detected yet, and there is not no train -> there is a train -> set the flag */
+					if (!(flags & SF_TRAIN) && EnsureNoTrainOnTrackBits(tile, tracks).Failed()) flags |= SF_TRAIN;
 				} else {
 					if (tracks_masked == TRACK_BIT_NONE) continue; // no incidating track
 					if (!(flags & SF_TRAIN) && HasVehicleOnPos(tile, NULL, &TrainOnTileEnum)) flags |= SF_TRAIN;
@@ -502,7 +501,7 @@ static SigSegState UpdateSignalsInBuffer(Owner owner)
 					_tbdset.Add(tile, INVALID_DIAGDIR); // start from depot inside
 					break;
 				}
-				/* FALLTHROUGH */
+				/* FALL THROUGH */
 			case MP_STATION:
 			case MP_ROAD:
 				if ((TrackStatusToTrackBits(GetTileTrackStatus(tile, TRANSPORT_RAIL, 0)) & _enterdir_to_trackbits[dir]) != TRACK_BIT_NONE) {
@@ -511,7 +510,7 @@ static SigSegState UpdateSignalsInBuffer(Owner owner)
 					_tbdset.Add(tile + TileOffsByDiagDir(dir), ReverseDiagDir(dir));
 					break;
 				}
-				/* FALLTHROUGH */
+				/* FALL THROUGH */
 			default:
 				/* jump to next tile */
 				tile = tile + TileOffsByDiagDir(dir);
