@@ -1,4 +1,11 @@
-/* $Id: cocoa_s.cpp 16804 2009-07-13 10:16:50Z rubidium $ */
+/* $Id: cocoa_s.cpp 26024 2013-11-17 13:35:48Z rubidium $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file cocoa_s.cpp Sound driver for cocoa. */
 
@@ -10,27 +17,18 @@
 
 #ifdef WITH_COCOA
 
-#define MAC_OS_X_VERSION_MIN_REQUIRED    MAC_OS_X_VERSION_10_3
-#include <AvailabilityMacros.h>
-
-#include <AudioUnit/AudioUnit.h>
-
-/* Name conflict */
-#define Rect        OTTDRect
-#define Point       OTTDPoint
-#define WindowClass OTTDWindowClass
-
 #include "../stdafx.h"
 #include "../debug.h"
 #include "../driver.h"
 #include "../mixer.h"
 #include "../core/endian_type.hpp"
-
 #include "cocoa_s.h"
 
-#undef WindowClass
-#undef Point
+#define Rect        OTTDRect
+#define Point       OTTDPoint
+#include <AudioUnit/AudioUnit.h>
 #undef Rect
+#undef Point
 
 static FSoundDriver_Cocoa iFSoundDriver_Cocoa;
 
@@ -56,7 +54,7 @@ const char *SoundDriver_Cocoa::Start(const char * const *parm)
 	requestedDesc.mFormatID = kAudioFormatLinearPCM;
 	requestedDesc.mFormatFlags = kLinearPCMFormatFlagIsPacked;
 	requestedDesc.mChannelsPerFrame = 2;
-	requestedDesc.mSampleRate = GetDriverParamInt(parm, "hz", 11025);
+	requestedDesc.mSampleRate = GetDriverParamInt(parm, "hz", 44100);
 
 	requestedDesc.mBitsPerChannel = 16;
 	requestedDesc.mFormatFlags |= kLinearPCMFormatFlagIsSignedInteger;
@@ -69,7 +67,7 @@ const char *SoundDriver_Cocoa::Start(const char * const *parm)
 	requestedDesc.mBytesPerFrame = requestedDesc.mBitsPerChannel * requestedDesc.mChannelsPerFrame / 8;
 	requestedDesc.mBytesPerPacket = requestedDesc.mBytesPerFrame * requestedDesc.mFramesPerPacket;
 
-	MxInitialize(requestedDesc.mSampleRate);
+	MxInitialize((uint)requestedDesc.mSampleRate);
 
 	/* Locate the default output audio unit */
 	desc.componentType = kAudioUnitType_Output;
