@@ -1,4 +1,4 @@
-/* $Id: statusbar_gui.h 17248 2009-08-21 20:21:05Z rubidium $ */
+/* $Id: statusbar_gui.h 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -12,10 +12,11 @@
 #ifndef STATUSBAR_GUI_H
 #define STATUSBAR_GUI_H
 
+/** What of the statusbar must be invalidated? */
 enum StatusBarInvalidate {
 	SBI_SAVELOAD_START,  ///< started saving
 	SBI_SAVELOAD_FINISH, ///< finished saving
-	SBI_SHOW_TICKER,     ///< start scolling news
+	SBI_SHOW_TICKER,     ///< start scrolling news
 	SBI_SHOW_REMINDER,   ///< show a reminder (dot on the right side of the statusbar)
 	SBI_NEWS_DELETED,    ///< abort current news display (active news were deleted)
 	SBI_END
@@ -24,4 +25,6 @@ enum StatusBarInvalidate {
 bool IsNewsTickerShown();
 void ShowStatusBar();
 
+extern int16 *_preferred_statusbar_size;
+
 #endif /* STATUSBAR_GUI_H */
