@@ -1,4 +1,4 @@
-/* $Id: strings.cpp 25204 2013-04-24 17:54:43Z rubidium $ */
+/* $Id: strings.cpp 26020 2013-11-17 11:24:39Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -20,7 +20,7 @@
 #include "newgrf_text.h"
 #include "fileio_func.h"
 #include "signs_base.h"
-#include "fontcache.h"
+#include "fontdetection.h"
 #include "error.h"
 #include "strings_func.h"
 #include "rev.h"
@@ -96,28 +96,31 @@ void StringParameters::ShiftParameters(uint amount)
  * @param max_value The biggest value which shall be displayed.
  *                  For the result only the number of digits of \a max_value matter.
  * @param min_count Minimum number of digits independent of \a max.
+ * @param size  Font of the number
  */
-void SetDParamMaxValue(uint n, uint64 max_value, uint min_count)
+void SetDParamMaxValue(uint n, uint64 max_value, uint min_count, FontSize size)
 {
 	uint num_digits = 1;
 	while (max_value >= 10) {
 		num_digits++;
 		max_value /= 10;
 	}
-	SetDParamMaxDigits(n, max(min_count, num_digits));
+	SetDParamMaxDigits(n, max(min_count, num_digits), size);
 }
 
 /**
  * Set DParam n to some number that is suitable for string size computations.
  * @param n Index of the string parameter.
  * @param count Number of digits which shall be displayable.
+ * @param size  Font of the number
  */
-void SetDParamMaxDigits(uint n, uint count)
+void SetDParamMaxDigits(uint n, uint count, FontSize size)
 {
-	static const uint biggest_digit = 8; ///< Digit with the biggest string width.
-	uint64 val = biggest_digit;
+	uint front, next;
+	GetBroadestDigit(&front, &next, size);
+	uint64 val = count > 1 ? front : next;
 	for (; count > 1; count--) {
-		val = 10 * val + biggest_digit;
+		val = 10 * val + next;
 	}
 	SetDParam(n, val);
 }
@@ -230,7 +233,10 @@ char *GetStringWithArgs(char *buffr, StringID string, StringParameters *args, co
 
 		case 15:
 			/* Old table for custom names. This is no longer used */
-			error("Incorrect conversion of custom name string.");
+			if (!game_script) {
+				error("Incorrect conversion of custom name string.");
+			}
+			break;
 
 		case GAME_TEXT_TAB:
 			return FormatString(buffr, GetGameStringPtr(index), args, last, case_index, true);
@@ -251,9 +257,6 @@ char *GetStringWithArgs(char *buffr, StringID string, StringParameters *args, co
 
 		case 30:
 			return FormatString(buffr, GetGRFStringPtr(index + 0x1000), args, last, case_index);
-
-		case 31:
-			NOT_REACHED();
 	}
 
 	if (index >= _langtab_num[tab]) {
@@ -1000,21 +1003,6 @@ static char *FormatString(char *buff, const char *str_arg, StringParameters *arg
 				break;
 			}
 
-			case SCC_SETX: // {SETX}
-				if (buff + Utf8CharLen(SCC_SETX) + 1 < last) {
-					buff += Utf8Encode(buff, SCC_SETX);
-					*buff++ = *str++;
-				}
-				break;
-
-			case SCC_SETXY: // {SETXY}
-				if (buff + Utf8CharLen(SCC_SETXY) + 2 < last) {
-					buff += Utf8Encode(buff, SCC_SETXY);
-					*buff++ = *str++;
-					*buff++ = *str++;
-				}
-				break;
-
 			case SCC_REVISION: // {REV}
 				buff = strecpy(buff, _openttd_revision, last);
 				break;
@@ -1490,7 +1478,7 @@ static char *FormatString(char *buff, const char *str_arg, StringParameters *arg
 
 					StringID str;
 					switch (v->type) {
-						default: NOT_REACHED();
+						default:           str = STR_INVALID_VEHICLE; break;
 						case VEH_TRAIN:    str = STR_SV_TRAIN_NAME; break;
 						case VEH_ROAD:     str = STR_SV_ROAD_VEHICLE_NAME; break;
 						case VEH_SHIP:     str = STR_SV_SHIP_NAME; break;
@@ -2033,14 +2021,7 @@ bool MissingGlyphSearcher::FindMissingGlyphs(const char **str)
 		FontSize size = this->DefaultSize();
 		if (str != NULL) *str = text;
 		for (WChar c = Utf8Consume(&text); c != '\0'; c = Utf8Consume(&text)) {
-			if (c == SCC_SETX) {
-				/* SetX is, together with SetXY as special character that
-					* uses the next (two) characters as data points. We have
-					* to skip those, otherwise the UTF8 reading will go haywire. */
-				text++;
-			} else if (c == SCC_SETXY) {
-				text += 2;
-			} else if (c == SCC_TINYFONT) {
+			if (c == SCC_TINYFONT) {
 				size = FS_SMALL;
 			} else if (c == SCC_BIGFONT) {
 				size = FS_LARGE;
@@ -2092,9 +2073,9 @@ class LanguagePackGlyphSearcher : public MissingGlyphSearcher {
 	/* virtual */ void SetFontNames(FreeTypeSettings *settings, const char *font_name)
 	{
 #ifdef WITH_FREETYPE
-		strecpy(settings->small_font,  font_name, lastof(settings->small_font));
-		strecpy(settings->medium_font, font_name, lastof(settings->medium_font));
-		strecpy(settings->large_font,  font_name, lastof(settings->large_font));
+		strecpy(settings->small.font,  font_name, lastof(settings->small.font));
+		strecpy(settings->medium.font, font_name, lastof(settings->medium.font));
+		strecpy(settings->large.font,  font_name, lastof(settings->large.font));
 #endif /* WITH_FREETYPE */
 	}
 };
