@@ -1,4 +1,4 @@
-/* $Id: townname.h 17597 2009-09-21 11:01:16Z smatz $ */
+/* $Id: townname.h 25984 2013-11-13 21:43:16Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -627,9 +627,9 @@ static const char * const _name_spanish_real[] = {
 	"Morrocoy",
 	"Cata",
 	"Cataito",
-	"Ciudad Bolivar",
+	"Ciudad Bol\xC3\xADvar",
 	"Barquisimeto",
-	"Merida",
+	"M\xC3\xA9rida",
 	"Puerto Ordaz",
 	"Santa Elena",
 	"San Juan",
@@ -638,12 +638,12 @@ static const char * const _name_spanish_real[] = {
 	"Santiago",
 	"Barcelona",
 	"Barinas",
-	"San Cristobal",
+	"San Crist\xC3\xB3""bal",
 	"San Fransisco",
-	"San Martin",
+	"San Mart\xC3\xADn",
 	"Guayana",
 	"San Carlos",
-	"El Limon",
+	"El Lim\xC3\xB3n",
 	"Coro",
 	"Corocoro",
 	"Puerto Ayacucho",
@@ -663,7 +663,7 @@ static const char * const _name_spanish_real[] = {
 	"Quito",
 	"Cuenca",
 	"Huacho",
-	"Tulcan",
+	"Tulc\xC3\xA1n",
 	"Esmeraldas",
 	"Ibarra",
 	"San Lorenzo",
@@ -674,20 +674,20 @@ static const char * const _name_spanish_real[] = {
 	"Latacunga",
 	"Tena",
 	"Cochabamba",
-	"Ascension",
+	"Ascensi\xC3\xB3n",
 	"Magdalena",
 	"Santa Ana",
 	"Manoa",
 	"Sucre",
 	"Oruro",
 	"Uyuni",
-	"Potosi",
+	"Potos\xC3\xAD",
 	"Tupiza",
 	"La Quiaca",
 	"Yacuiba",
 	"San Borja",
 	"Fuerte Olimpio",
-	"Fortin Esteros",
+	"Fort\xC3\xADn Esteros",
 	"Campo Grande",
 	"Bogota",
 	"El Banco",
@@ -1729,7 +1729,7 @@ static const char * const _name_czech_patmod[][3] = {
 	/* CZG_PNEUT */ { "\xC3\xAD", "\xC3\xA1", "a" }
 };
 
-/* This way the substantives can choose only some adjectives/endings:
+/* This way the substantive can choose only some adjectives/endings:
  * At least one of these flags must be satisfied: */
 enum CzechAllow {
 	CZA_SHORT = 1,
@@ -1738,7 +1738,7 @@ enum CzechAllow {
 	CZA_ALL = ~0
 };
 
-DECLARE_ENUM_AS_BIT_SET(CzechAllow);
+DECLARE_ENUM_AS_BIT_SET(CzechAllow)
 
 /* All these flags must be satisfied (in the stem->others direction): */
 enum CzechChoose {
@@ -1749,7 +1749,7 @@ enum CzechChoose {
 	CZC_ANY = ~0
 };
 
-DECLARE_ENUM_AS_BIT_SET(CzechChoose);
+DECLARE_ENUM_AS_BIT_SET(CzechChoose)
 
 struct CzechNameSubst {
 	CzechGender gender;
@@ -2115,7 +2115,7 @@ static const char * const _name_slovak_real[] = {
 	"Turcianske Teplice",
 	"Tvrdosin",
 	"Vrable",
-	"Vranov nad Toplov",
+	"Vranov nad Toplou",
 	"Zahorska Bystrica",
 	"Zdiar",
 	"Ziar nad Hronom",
@@ -3149,7 +3149,7 @@ static const char * const _name_italian_river2[] = {
 
 static const char * const _name_catalan_real[] = {
 	"Barcelona",
-	"Hospitalet",
+	"L'Hospitalet de Llobregat",
 	"Cerdanyola",
 	"Martorell",
 	"Badalona",
@@ -3161,7 +3161,7 @@ static const char * const _name_catalan_real[] = {
 	"Reus",
 	"Valls",
 	"Vic",
-	"Vielha",
+	"Vielha e Mijaran",
 	"Amposta",
 	"Tortosa",
 	"Berga",
@@ -3171,12 +3171,12 @@ static const char * const _name_catalan_real[] = {
 	"Figueres",
 	"Balaguer",
 	"Vilafranca del Pened\xC3\xA8s",
-	"La seu d'Urgell",
-	"Pont de Suert",
+	"La Seu d'Urgell",
+	"El Pont de Suert",
 	"Igualada",
 	"Manresa",
 	"Solsona",
-	"Les borges blanques",
+	"Les Borges Blanques",
 	"Tremp",
 	"Sort",
 	"Colera",
@@ -3194,21 +3194,21 @@ static const char * const _name_catalan_real[] = {
 	"Campdev\xC3\xA0nol",
 	"Cambrils",
 	"Begur",
-	"Set Cases",
+	"Setcases",
 	"Palafrugell",
 	"Begues",
 	"El Bruc",
 	"Cadaqu\xC3\xA9s",
 	"Collbat\xC3\xB3",
 	"Cervell\xC3\xB3",
-	"Esparraguera",
+	"Esparreguera",
 	"Abrera",
 	"Alp",
 	"Das",
 	"Cercs",
 	"Manlleu",
-	"Masnou",
-	"Molins de rei",
+	"El Masnou",
+	"Molins de Rei",
 	"Monistrol",
 	"Rocallaura",
 	"Rub\xC3\xAD",
@@ -3218,15 +3218,15 @@ static const char * const _name_catalan_real[] = {
 };
 
 static const char * const _name_catalan_pref[] = {
-	"Pont de ",
+	"El Pont de ",
 	"Parets de ",
 	"Canet de ",
 	"Castellar de ",
 	"Corbera de ",
 	"Arenys de ",
 	"Calella de ",
-	"La seu de ",
-	"La bisbal de ",
+	"La Seu de ",
+	"La Bisbal de ",
 	"Torroella de ",
 	"Port de ",
 	"Vilafranca de ",
@@ -3262,13 +3262,13 @@ static const char * const _name_catalan_1m[] = {
 };
 
 static const char * const _name_catalan_1f[] = {
-	"Aigua",
-	"Selva ",
+	"Pala",
+	"Selva",
 	"Vall",
 	"Serra",
 	"Torre",
 	"Riba",
-	"Vall",
+	"Cova",
 	"Terra",
 };
 
@@ -3280,7 +3280,7 @@ static const char * const _name_catalan_2m[] = {
 	"vent\xC3\xB3s",
 	"negre",
 	"roig",
-	"gris",
+	"gr\xC3\xADs",
 };
 
 static const char * const _name_catalan_2f[] = {
@@ -3295,16 +3295,16 @@ static const char * const _name_catalan_2f[] = {
 };
 
 static const char * const _name_catalan_3[] = {
-	" desp\xC3\xAD",
-	" desvern",
-	" del cam\xC3\xAD",
+	" Desp\xC3\xAD",
+	" Desvern",
+	" del Cam\xC3\xAD",
 	" de Mar",
 	" de Dalt",
 	" de Baix",
 	" del Vall\xC3\xA8s",
 	" de Bergued\xC3\xA0",
 	" de Conflent",
-	" de la plana",
+	" de la Plana",
 };
 
 static const char * const _name_catalan_river1[] = {
