@@ -1,29 +1,36 @@
-/* $Id: terraform_cmd.cpp 15434 2009-02-09 21:20:05Z rubidium $ */
+/* $Id: terraform_cmd.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
+
+/*
+ * This file is part of OpenTTD.
+ * OpenTTD is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
+ * OpenTTD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 /** @file terraform_cmd.cpp Commands related to terraforming. */
 
 #include "stdafx.h"
-#include "openttd.h"
 #include "command_func.h"
 #include "tunnel_map.h"
 #include "bridge_map.h"
-#include "variables.h"
-#include "functions.h"
-#include "economy_func.h"
-#include "settings_type.h"
+#include "viewport_func.h"
+#include "genworld.h"
+#include "object_base.h"
+#include "company_base.h"
+#include "company_func.h"
 
 #include "table/strings.h"
 
 /*
  * In one terraforming command all four corners of a initial tile can be raised/lowered (though this is not available to the player).
- * The maximal amount of height modifications is archieved when raising a complete flat land from sea level to MAX_TILE_HEIGHT or vice versa.
+ * The maximal amount of height modifications is achieved when raising a complete flat land from sea level to MAX_TILE_HEIGHT or vice versa.
  * This affects all corners with a manhatten distance smaller than MAX_TILE_HEIGHT to one of the initial 4 corners.
  * Their maximal amount is computed to 4 * \sum_{i=1}^{h_max} i  =  2 * h_max * (h_max + 1).
  */
 static const int TERRAFORMER_MODHEIGHT_SIZE = 2 * MAX_TILE_HEIGHT * (MAX_TILE_HEIGHT + 1);
 
 /*
- * The maximal amount of affected tiles (i.e. the tiles that incident with one of the corners above, is computed similiar to
+ * The maximal amount of affected tiles (i.e. the tiles that incident with one of the corners above, is computed similar to
  * 1 + 4 * \sum_{i=1}^{h_max} (i+1)  =  1 + 2 * h_max + (h_max + 3).
  */
 static const int TERRAFORMER_TILE_TABLE_SIZE = 1 + 2 * MAX_TILE_HEIGHT * (MAX_TILE_HEIGHT + 3);
@@ -142,15 +149,15 @@ static void TerraformAddDirtyTileAround(TerraformerState *ts, TileIndex tile)
  * @param ts TerraformerState.
  * @param tile Tile.
  * @param height Aimed height.
- * @param return Error code or cost.
+ * @return Error code or cost.
  */
 static CommandCost TerraformTileHeight(TerraformerState *ts, TileIndex tile, int height)
 {
 	assert(tile < MapSize());
 
 	/* Check range of destination height */
-	if (height < 0) return_cmd_error(STR_1003_ALREADY_AT_SEA_LEVEL);
-	if (height > MAX_TILE_HEIGHT) return_cmd_error(STR_1004_TOO_HIGH);
+	if (height < 0) return_cmd_error(STR_ERROR_ALREADY_AT_SEA_LEVEL);
+	if (height > (int)MAX_TILE_HEIGHT) return_cmd_error(STR_ERROR_TOO_HIGH);
 
 	/*
 	 * Check if the terraforming has any effect.
@@ -169,10 +176,10 @@ static CommandCost TerraformTileHeight(TerraformerState *ts, TileIndex tile, int
 		if (x == 1) x = 0;
 		if (y == 1) y = 0;
 		_terraform_err_tile = TileXY(x, y);
-		return_cmd_error(STR_0002_TOO_CLOSE_TO_EDGE_OF_MAP);
+		return_cmd_error(STR_ERROR_TOO_CLOSE_TO_EDGE_OF_MAP);
 	}
 
-	/* Mark incident tiles, that are involved in the terraforming */
+	/* Mark incident tiles that are involved in the terraforming. */
 	TerraformAddDirtyTileAround(ts, tile);
 
 	/* Store the height modification */
@@ -181,7 +188,7 @@ static CommandCost TerraformTileHeight(TerraformerState *ts, TileIndex tile, int
 	CommandCost total_cost(EXPENSES_CONSTRUCTION);
 
 	/* Increment cost */
-	total_cost.AddCost(_price.terraform);
+	total_cost.AddCost(_price[PR_TERRAFORM]);
 
 	/* Recurse to neighboured corners if height difference is larger than 1 */
 	{
@@ -212,7 +219,7 @@ static CommandCost TerraformTileHeight(TerraformerState *ts, TileIndex tile, int
 				/* Terraform the neighboured corner. The resulting height difference should be 1. */
 				height_diff += (height_diff < 0 ? 1 : -1);
 				CommandCost cost = TerraformTileHeight(ts, tile, r + height_diff);
-				if (CmdFailed(cost)) return cost;
+				if (cost.Failed()) return cost;
 				total_cost.AddCost(cost);
 			}
 		}
@@ -221,12 +228,14 @@ static CommandCost TerraformTileHeight(TerraformerState *ts, TileIndex tile, int
 	return total_cost;
 }
 
-/** Terraform land
+/**
+ * Terraform land
  * @param tile tile to terraform
  * @param flags for this command type
  * @param p1 corners to terraform (SLOPE_xxx)
  * @param p2 direction; eg up (non-zero) or down (zero)
- * @return error or cost of terraforming
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdTerraformLand(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
@@ -242,33 +251,35 @@ CommandCost CmdTerraformLand(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 	if ((p1 & SLOPE_W) != 0 && tile + TileDiffXY(1, 0) < MapSize()) {
 		TileIndex t = tile + TileDiffXY(1, 0);
 		CommandCost cost = TerraformTileHeight(&ts, t, TileHeight(t) + direction);
-		if (CmdFailed(cost)) return cost;
+		if (cost.Failed()) return cost;
 		total_cost.AddCost(cost);
 	}
 
 	if ((p1 & SLOPE_S) != 0 && tile + TileDiffXY(1, 1) < MapSize()) {
 		TileIndex t = tile + TileDiffXY(1, 1);
 		CommandCost cost = TerraformTileHeight(&ts, t, TileHeight(t) + direction);
-		if (CmdFailed(cost)) return cost;
+		if (cost.Failed()) return cost;
 		total_cost.AddCost(cost);
 	}
 
 	if ((p1 & SLOPE_E) != 0 && tile + TileDiffXY(0, 1) < MapSize()) {
 		TileIndex t = tile + TileDiffXY(0, 1);
 		CommandCost cost = TerraformTileHeight(&ts, t, TileHeight(t) + direction);
-		if (CmdFailed(cost)) return cost;
+		if (cost.Failed()) return cost;
 		total_cost.AddCost(cost);
 	}
 
 	if ((p1 & SLOPE_N) != 0) {
 		TileIndex t = tile + TileDiffXY(0, 0);
 		CommandCost cost = TerraformTileHeight(&ts, t, TileHeight(t) + direction);
-		if (CmdFailed(cost)) return cost;
+		if (cost.Failed()) return cost;
 		total_cost.AddCost(cost);
 	}
 
-	/* Check if the terraforming is valid wrt. tunnels, bridges and objects on the surface */
-	{
+	/* Check if the terraforming is valid wrt. tunnels, bridges and objects on the surface
+	 * Pass == 0: Collect tileareas which are caused to be auto-cleared.
+	 * Pass == 1: Collect the actual cost. */
+	for (int pass = 0; pass < 2; pass++) {
 		TileIndex *ti = ts.tile_table;
 
 		for (int count = ts.tile_table_count; count != 0; count--, ti++) {
@@ -280,14 +291,14 @@ CommandCost CmdTerraformLand(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 			if (IsTileType(tile, MP_VOID)) continue;
 
 			/* Find new heights of tile corners */
-			uint z_N = TerraformGetHeightOfTile(&ts, tile + TileDiffXY(0, 0));
-			uint z_W = TerraformGetHeightOfTile(&ts, tile + TileDiffXY(1, 0));
-			uint z_S = TerraformGetHeightOfTile(&ts, tile + TileDiffXY(1, 1));
-			uint z_E = TerraformGetHeightOfTile(&ts, tile + TileDiffXY(0, 1));
+			int z_N = TerraformGetHeightOfTile(&ts, tile + TileDiffXY(0, 0));
+			int z_W = TerraformGetHeightOfTile(&ts, tile + TileDiffXY(1, 0));
+			int z_S = TerraformGetHeightOfTile(&ts, tile + TileDiffXY(1, 1));
+			int z_E = TerraformGetHeightOfTile(&ts, tile + TileDiffXY(0, 1));
 
 			/* Find min and max height of tile */
-			uint z_min = min(min(z_N, z_W), min(z_S, z_E));
-			uint z_max = max(max(z_N, z_W), max(z_S, z_E));
+			int z_min = min(min(z_N, z_W), min(z_S, z_E));
+			int z_max = max(max(z_N, z_W), max(z_S, z_E));
 
 			/* Compute tile slope */
 			Slope tileh = (z_max > z_min + 1 ? SLOPE_STEEP : SLOPE_FLAT);
@@ -296,30 +307,52 @@ CommandCost CmdTerraformLand(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 			if (z_E > z_min) tileh |= SLOPE_E;
 			if (z_N > z_min) tileh |= SLOPE_N;
 
-			/* Check if bridge would take damage */
-			if (direction == 1 && MayHaveBridgeAbove(tile) && IsBridgeAbove(tile) &&
-					GetBridgeHeight(GetSouthernBridgeEnd(tile)) <= z_max * TILE_HEIGHT) {
-				_terraform_err_tile = tile; // highlight the tile under the bridge
-				return_cmd_error(STR_5007_MUST_DEMOLISH_BRIDGE_FIRST);
-			}
-			/* Check if tunnel would take damage */
-			if (direction == -1 && IsTunnelInWay(tile, z_min * TILE_HEIGHT)) {
-				_terraform_err_tile = tile; // highlight the tile above the tunnel
-				return_cmd_error(STR_1002_EXCAVATION_WOULD_DAMAGE);
+			if (pass == 0) {
+				/* Check if bridge would take damage */
+				if (direction == 1 && MayHaveBridgeAbove(tile) && IsBridgeAbove(tile) &&
+						GetBridgeHeight(GetSouthernBridgeEnd(tile)) <= z_max) {
+					_terraform_err_tile = tile; // highlight the tile under the bridge
+					return_cmd_error(STR_ERROR_MUST_DEMOLISH_BRIDGE_FIRST);
+				}
+				/* Check if tunnel would take damage */
+				if (direction == -1 && IsTunnelInWay(tile, z_min)) {
+					_terraform_err_tile = tile; // highlight the tile above the tunnel
+					return_cmd_error(STR_ERROR_EXCAVATION_WOULD_DAMAGE);
+				}
 			}
+
+			/* Is the tile already cleared? */
+			const ClearedObjectArea *coa = FindClearedObject(tile);
+			bool indirectly_cleared = coa != NULL && coa->first_tile != tile;
+
 			/* Check tiletype-specific things, and add extra-cost */
 			const bool curr_gen = _generating_world;
 			if (_game_mode == GM_EDITOR) _generating_world = true; // used to create green terraformed land
-			CommandCost cost = _tile_type_procs[GetTileType(tile)]->terraform_tile_proc(tile, flags | DC_AUTO, z_min * TILE_HEIGHT, tileh);
+			DoCommandFlag tile_flags = flags | DC_AUTO | DC_FORCE_CLEAR_TILE;
+			if (pass == 0) {
+				tile_flags &= ~DC_EXEC;
+				tile_flags |= DC_NO_MODIFY_TOWN_RATING;
+			}
+			CommandCost cost;
+			if (indirectly_cleared) {
+				cost = DoCommand(tile, 0, 0, tile_flags, CMD_LANDSCAPE_CLEAR);
+			} else {
+				cost = _tile_type_procs[GetTileType(tile)]->terraform_tile_proc(tile, tile_flags, z_min, tileh);
+			}
 			_generating_world = curr_gen;
-			if (CmdFailed(cost)) {
+			if (cost.Failed()) {
 				_terraform_err_tile = tile;
 				return cost;
 			}
-			total_cost.AddCost(cost);
+			if (pass == 1) total_cost.AddCost(cost);
 		}
 	}
 
+	Company *c = Company::GetIfValid(_current_company);
+	if (c != NULL && (int)GB(c->terraform_limit, 16, 16) < ts.modheight_count) {
+		return_cmd_error(STR_ERROR_TERRAFORM_LIMIT_REACHED);
+	}
+
 	if (flags & DC_EXEC) {
 		/* change the height */
 		{
@@ -342,17 +375,23 @@ CommandCost CmdTerraformLand(TileIndex tile, DoCommandFlag flags, uint32 p1, uin
 				MarkTileDirtyByTile(*ti);
 			}
 		}
+
+		if (c != NULL) c->terraform_limit -= ts.modheight_count << 16;
 	}
 	return total_cost;
 }
 
 
-/** Levels a selected (rectangle) area of land
+/**
+ * Levels a selected (rectangle) area of land
  * @param tile end tile of area-drag
  * @param flags for this command type
  * @param p1 start tile of area drag
- * @param p2 height difference; eg raise (+1), lower (-1) or level (0)
- * @return  error or cost of terraforming
+ * @param p2 various bitstuffed data.
+ *  bit      0: Whether to use the Orthogonal (0) or Diagonal (1) iterator.
+ *  bits 1 - 2: Mode of leveling \c LevelMode.
+ * @param text unused
+ * @return the cost of this operation or an error
  */
 CommandCost CmdLevelLand(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
@@ -364,46 +403,70 @@ CommandCost CmdLevelLand(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32
 	uint oldh = TileHeight(p1);
 
 	/* compute new height */
-	uint h = oldh + p2;
+	uint h = oldh;
+	LevelMode lm = (LevelMode)GB(p2, 1, 2);
+	switch (lm) {
+		case LM_LEVEL: break;
+		case LM_RAISE: h++; break;
+		case LM_LOWER: h--; break;
+		default: return CMD_ERROR;
+	}
 
 	/* Check range of destination height */
-	if (h > MAX_TILE_HEIGHT) return_cmd_error((oldh == 0) ? STR_1003_ALREADY_AT_SEA_LEVEL : STR_1004_TOO_HIGH);
-	if (p2 == 0) _error_message = STR_ALREADY_LEVELLED;
-
-	/* make sure sx,sy are smaller than ex,ey */
-	int ex = TileX(tile);
-	int ey = TileY(tile);
-	int sx = TileX(p1);
-	int sy = TileY(p1);
-	if (ex < sx) Swap(ex, sx);
-	if (ey < sy) Swap(ey, sy);
-	tile = TileXY(sx, sy);
-
-	int size_x = ex - sx + 1;
-	int size_y = ey - sy + 1;
+	if (h > MAX_TILE_HEIGHT) return_cmd_error((oldh == 0) ? STR_ERROR_ALREADY_AT_SEA_LEVEL : STR_ERROR_TOO_HIGH);
 
 	Money money = GetAvailableMoneyForCommand();
 	CommandCost cost(EXPENSES_CONSTRUCTION);
+	CommandCost last_error(lm == LM_LEVEL ? STR_ERROR_ALREADY_LEVELLED : INVALID_STRING_ID);
+	bool had_success = false;
 
-	BEGIN_TILE_LOOP(tile2, size_x, size_y, tile) {
-		uint curh = TileHeight(tile2);
+	const Company *c = Company::GetIfValid(_current_company);
+	int limit = (c == NULL ? INT32_MAX : GB(c->terraform_limit, 16, 16));
+	if (limit == 0) return_cmd_error(STR_ERROR_TERRAFORM_LIMIT_REACHED);
+
+	TileArea ta(tile, p1);
+	TileIterator *iter = HasBit(p2, 0) ? (TileIterator *)new DiagonalTileIterator(tile, p1) : new OrthogonalTileIterator(ta);
+	for (; *iter != INVALID_TILE; ++(*iter)) {
+		TileIndex t = *iter;
+		uint curh = TileHeight(t);
 		while (curh != h) {
-			CommandCost ret = DoCommand(tile2, SLOPE_N, (curh > h) ? 0 : 1, flags & ~DC_EXEC, CMD_TERRAFORM_LAND);
-			if (CmdFailed(ret)) break;
+			CommandCost ret = DoCommand(t, SLOPE_N, (curh > h) ? 0 : 1, flags & ~DC_EXEC, CMD_TERRAFORM_LAND);
+			if (ret.Failed()) {
+				last_error = ret;
+
+				/* Did we reach the limit? */
+				if (ret.GetErrorMessage() == STR_ERROR_TERRAFORM_LIMIT_REACHED) limit = 0;
+				break;
+			}
 
 			if (flags & DC_EXEC) {
 				money -= ret.GetCost();
 				if (money < 0) {
 					_additional_cash_required = ret.GetCost();
+					delete iter;
 					return cost;
 				}
-				DoCommand(tile2, SLOPE_N, (curh > h) ? 0 : 1, flags, CMD_TERRAFORM_LAND);
+				DoCommand(t, SLOPE_N, (curh > h) ? 0 : 1, flags, CMD_TERRAFORM_LAND);
+			} else {
+				/* When we're at the terraform limit we better bail (unneeded) testing as well.
+				 * This will probably cause the terraforming cost to be underestimated, but only
+				 * when it's near the terraforming limit. Even then, the estimation is
+				 * completely off due to it basically counting terraforming double, so it being
+				 * cut off earlier might even give a better estimate in some cases. */
+				if (--limit <= 0) {
+					had_success = true;
+					break;
+				}
 			}
 
 			cost.AddCost(ret);
 			curh += (curh > h) ? -1 : 1;
+			had_success = true;
 		}
-	} END_TILE_LOOP(tile2, size_x, size_y, tile)
 
-	return (cost.GetCost() == 0) ? CMD_ERROR : cost;
+		if (limit <= 0) break;
+	}
+
+	delete iter;
+	return had_success ? cost : last_error;
 }
