@@ -1,4 +1,4 @@
-/* $Id: timetable_cmd.cpp 19132 2010-02-14 16:31:35Z alberth $ */
+/* $Id: timetable_cmd.cpp 24053 2012-03-20 13:11:20Z michi_cc $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,33 +11,63 @@
 
 #include "stdafx.h"
 #include "command_func.h"
-#include "functions.h"
+#include "company_func.h"
 #include "date_func.h"
 #include "window_func.h"
 #include "vehicle_base.h"
+#include "cmd_helper.h"
 
 #include "table/strings.h"
 
-static void ChangeTimetable(Vehicle *v, VehicleOrderID order_number, uint16 time, bool is_journey)
+/**
+ * Change/update a particular timetable entry.
+ * @param v            The vehicle to change the timetable of.
+ * @param order_number The index of the timetable in the order list.
+ * @param val          The new data of the timetable entry.
+ * @param mtf          Which part of the timetable entry to change.
+ */
+static void ChangeTimetable(Vehicle *v, VehicleOrderID order_number, uint16 val, ModifyTimetableFlags mtf)
 {
 	Order *order = v->GetOrder(order_number);
-	int delta;
+	int delta = 0;
 
-	if (is_journey) {
-		delta = time - order->travel_time;
-		order->travel_time = time;
-	} else {
-		delta = time - order->wait_time;
-		order->wait_time = time;
+	switch (mtf) {
+		case MTF_WAIT_TIME:
+			delta = val - order->wait_time;
+			order->wait_time = val;
+			break;
+
+		case MTF_TRAVEL_TIME:
+			delta = val - order->travel_time;
+			order->travel_time = val;
+			break;
+
+		case MTF_TRAVEL_SPEED:
+			order->max_speed = val;
+			break;
+
+		default:
+			NOT_REACHED();
 	}
 	v->orders.list->UpdateOrderTimetable(delta);
 
 	for (v = v->FirstShared(); v != NULL; v = v->NextShared()) {
-		if (v->cur_order_index == order_number && v->current_order.Equals(*order)) {
-			if (is_journey) {
-				v->current_order.travel_time = time;
-			} else {
-				v->current_order.wait_time = time;
+		if (v->cur_real_order_index == order_number && v->current_order.Equals(*order)) {
+			switch (mtf) {
+				case MTF_WAIT_TIME:
+					v->current_order.wait_time = val;
+					break;
+
+				case MTF_TRAVEL_TIME:
+					v->current_order.travel_time = val;
+					break;
+
+				case MTF_TRAVEL_SPEED:
+					v->current_order.max_speed = val;
+					break;
+
+				default:
+					NOT_REACHED();
 			}
 		}
 		SetWindowDirty(WC_VEHICLE_TIMETABLE, v->index);
@@ -45,47 +75,54 @@ static void ChangeTimetable(Vehicle *v, VehicleOrderID order_number, uint16 time
 }
 
 /**
- * Add or remove waiting times from an order.
+ * Change timetable data of an order.
  * @param tile Not used.
  * @param flags Operation to perform.
  * @param p1 Various bitstuffed elements
- * - p1 = (bit  0-15) - Vehicle with the orders to change.
- * - p1 = (bit 16-23) - Order index to modify.
- * - p1 = (bit    24) - Whether to change the waiting time or the travelling
- *                      time.
- * - p1 = (bit    25) - Whether p2 contains waiting and travelling time.
+ * - p1 = (bit  0-19) - Vehicle with the orders to change.
+ * - p1 = (bit 20-27) - Order index to modify.
+ * - p1 = (bit 28-29) - Timetable data to change (@see ModifyTimetableFlags)
  * @param p2 The amount of time to wait.
- * - p2 = (bit  0-15) - Waiting or travelling time as specified by p1 bit 24 if p1 bit 25 is not set,
- *                      Travelling time if p1 bit 25 is set.
- * - p2 = (bit 16-31) - Waiting time if p1 bit 25 is set
+ * - p2 = (bit  0-15) - The data to modify as specified by p1 bits 28-29.
  * @param text unused
  * @return the cost of this operation or an error
  */
 CommandCost CmdChangeTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	if (!_settings_game.order.timetabling) return CMD_ERROR;
-
-	VehicleID veh = GB(p1, 0, 16);
+	VehicleID veh = GB(p1, 0, 20);
 
 	Vehicle *v = Vehicle::GetIfValid(veh);
-	if (v == NULL || !CheckOwnership(v->owner) || !v->IsPrimaryVehicle()) return CMD_ERROR;
+	if (v == NULL || !v->IsPrimaryVehicle()) return CMD_ERROR;
 
-	VehicleOrderID order_number = GB(p1, 16, 8);
+	CommandCost ret = CheckOwnership(v->owner);
+	if (ret.Failed()) return ret;
+
+	VehicleOrderID order_number = GB(p1, 20, 8);
 	Order *order = v->GetOrder(order_number);
-	if (order == NULL) return CMD_ERROR;
+	if (order == NULL || order->IsType(OT_IMPLICIT)) return CMD_ERROR;
 
-	bool packed_time = HasBit(p1, 25);
-	bool is_journey = HasBit(p1, 24) || packed_time;
+	ModifyTimetableFlags mtf = Extract<ModifyTimetableFlags, 28, 2>(p1);
+	if (mtf >= MTF_END) return CMD_ERROR;
 
 	int wait_time   = order->wait_time;
 	int travel_time = order->travel_time;
-	if (packed_time) {
-		travel_time = GB(p2, 0, 16);
-		wait_time   = GB(p2, 16, 16);
-	} else if (is_journey) {
-		travel_time = GB(p2, 0, 16);
-	} else {
-		wait_time   = GB(p2, 0, 16);
+	int max_speed   = order->max_speed;
+	switch (mtf) {
+		case MTF_WAIT_TIME:
+			wait_time = GB(p2, 0, 16);
+			break;
+
+		case MTF_TRAVEL_TIME:
+			travel_time = GB(p2, 0, 16);
+			break;
+
+		case MTF_TRAVEL_SPEED:
+			max_speed = GB(p2, 0, 16);
+			if (max_speed == 0) max_speed = UINT16_MAX; // Disable speed limit.
+			break;
+
+		default:
+			NOT_REACHED();
 	}
 
 	if (wait_time != order->wait_time) {
@@ -102,10 +139,12 @@ CommandCost CmdChangeTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1, u
 	}
 
 	if (travel_time != order->travel_time && order->IsType(OT_CONDITIONAL)) return CMD_ERROR;
+	if (max_speed != order->max_speed && (order->IsType(OT_CONDITIONAL) || v->type == VEH_AIRCRAFT)) return CMD_ERROR;
 
 	if (flags & DC_EXEC) {
-		if (wait_time   != order->wait_time)   ChangeTimetable(v, order_number, wait_time,   false);
-		if (travel_time != order->travel_time) ChangeTimetable(v, order_number, travel_time, true);
+		if (wait_time   != order->wait_time)   ChangeTimetable(v, order_number, wait_time,   MTF_WAIT_TIME);
+		if (travel_time != order->travel_time) ChangeTimetable(v, order_number, travel_time, MTF_TRAVEL_TIME);
+		if (max_speed   != order->max_speed)   ChangeTimetable(v, order_number, max_speed,   MTF_TRAVEL_SPEED);
 	}
 
 	return CommandCost();
@@ -116,19 +155,20 @@ CommandCost CmdChangeTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1, u
  * @param tile Not used.
  * @param flags Operation to perform.
  * @param p1 Various bitstuffed elements
- * - p1 = (bit  0-15) - Vehicle with the orders to change.
+ * - p1 = (bit  0-19) - Vehicle with the orders to change.
  * @param p2 unused
  * @param text unused
  * @return the cost of this operation or an error
  */
 CommandCost CmdSetVehicleOnTime(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	if (!_settings_game.order.timetabling) return CMD_ERROR;
-
-	VehicleID veh = GB(p1, 0, 16);
+	VehicleID veh = GB(p1, 0, 20);
 
 	Vehicle *v = Vehicle::GetIfValid(veh);
-	if (v == NULL || !CheckOwnership(v->owner) || !v->IsPrimaryVehicle()) return CMD_ERROR;
+	if (v == NULL || !v->IsPrimaryVehicle()) return CMD_ERROR;
+
+	CommandCost ret = CheckOwnership(v->owner);
+	if (ret.Failed()) return ret;
 
 	if (flags & DC_EXEC) {
 		v->lateness_counter = 0;
@@ -143,14 +183,17 @@ CommandCost CmdSetVehicleOnTime(TileIndex tile, DoCommandFlag flags, uint32 p1,
  * @param tile Not used.
  * @param flags Operation to perform.
  * @param p1 Vehicle id.
- * @param p2 The timetable start date in ticks.
+ * @param p2 The timetable start date.
+ * @param text Not used.
+ * @return The error or cost of the operation.
  */
 CommandCost CmdSetTimetableStart(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	if (!_settings_game.order.timetabling) return CMD_ERROR;
+	Vehicle *v = Vehicle::GetIfValid(GB(p1, 0, 20));
+	if (v == NULL || !v->IsPrimaryVehicle()) return CMD_ERROR;
 
-	Vehicle *v = Vehicle::GetIfValid(GB(p1, 0, 16));
-	if (v == NULL || !CheckOwnership(v->owner) || !v->IsPrimaryVehicle()) return CMD_ERROR;
+	CommandCost ret = CheckOwnership(v->owner);
+	if (ret.Failed()) return ret;
 
 	/* Don't let a timetable start more than 15 years into the future or 1 year in the past. */
 	Date start_date = (Date)p2;
@@ -185,12 +228,13 @@ CommandCost CmdSetTimetableStart(TileIndex tile, DoCommandFlag flags, uint32 p1,
  */
 CommandCost CmdAutofillTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1, uint32 p2, const char *text)
 {
-	if (!_settings_game.order.timetabling) return CMD_ERROR;
-
-	VehicleID veh = GB(p1, 0, 16);
+	VehicleID veh = GB(p1, 0, 20);
 
 	Vehicle *v = Vehicle::GetIfValid(veh);
-	if (v == NULL || !CheckOwnership(v->owner) || !v->IsPrimaryVehicle()) return CMD_ERROR;
+	if (v == NULL || !v->IsPrimaryVehicle()) return CMD_ERROR;
+
+	CommandCost ret = CheckOwnership(v->owner);
+	if (ret.Failed()) return ret;
 
 	if (flags & DC_EXEC) {
 		if (HasBit(p2, 0)) {
@@ -209,20 +253,25 @@ CommandCost CmdAutofillTimetable(TileIndex tile, DoCommandFlag flags, uint32 p1,
 			ClrBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE);
 			ClrBit(v->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME);
 		}
-	}
 
-	for (Vehicle *v2 = v->FirstShared(); v2 != NULL; v2 = v2->NextShared()) {
-		if (v2 != v) {
-			/* Stop autofilling; only one vehicle at a time can perform autofill */
-			ClrBit(v2->vehicle_flags, VF_AUTOFILL_TIMETABLE);
-			ClrBit(v2->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME);
+		for (Vehicle *v2 = v->FirstShared(); v2 != NULL; v2 = v2->NextShared()) {
+			if (v2 != v) {
+				/* Stop autofilling; only one vehicle at a time can perform autofill */
+				ClrBit(v2->vehicle_flags, VF_AUTOFILL_TIMETABLE);
+				ClrBit(v2->vehicle_flags, VF_AUTOFILL_PRES_WAIT_TIME);
+			}
+			SetWindowDirty(WC_VEHICLE_TIMETABLE, v2->index);
 		}
-		SetWindowDirty(WC_VEHICLE_TIMETABLE, v2->index);
 	}
 
 	return CommandCost();
 }
 
+/**
+ * Update the timetable for the vehicle.
+ * @param v The vehicle to update the timetable for.
+ * @param travelling Whether we just travelled or waited at a station.
+ */
 void UpdateVehicleTimetable(Vehicle *v, bool travelling)
 {
 	uint timetabled = travelling ? v->current_order.travel_time : v->current_order.wait_time;
@@ -230,12 +279,17 @@ void UpdateVehicleTimetable(Vehicle *v, bool travelling)
 
 	v->current_order_time = 0;
 
-	if (!_settings_game.order.timetabling) return;
+	if (v->current_order.IsType(OT_IMPLICIT)) return; // no timetabling of auto orders
+
+	VehicleOrderID first_manual_order = 0;
+	for (Order *o = v->GetFirstOrder(); o != NULL && o->IsType(OT_IMPLICIT); o = o->next) {
+		++first_manual_order;
+	}
 
 	bool just_started = false;
 
 	/* This vehicle is arriving at the first destination in the timetable. */
-	if (v->cur_order_index == 0 && travelling) {
+	if (v->cur_real_order_index == first_manual_order && travelling) {
 		/* If the start date hasn't been set, or it was set automatically when
 		 * the vehicle last arrived at the first destination, update it to the
 		 * current time. Otherwise set the late counter appropriately to when
@@ -266,13 +320,19 @@ void UpdateVehicleTimetable(Vehicle *v, bool travelling)
 		if (!v->current_order.IsType(OT_CONDITIONAL) && (travelling || time_taken > v->current_order.wait_time)) {
 			/* Round the time taken up to the nearest day, as this will avoid
 			 * confusion for people who are timetabling in days, and can be
-			 * adjusted later by people who aren't. */
-			time_taken = (((time_taken - 1) / DAY_TICKS) + 1) * DAY_TICKS;
+			 * adjusted later by people who aren't.
+			 * For trains/aircraft multiple movement cycles are done in one
+			 * tick. This makes it possible to leave the station and process
+			 * e.g. a depot order in the same tick, causing it to not fill
+			 * the timetable entry like is done for road vehicles/ships.
+			 * Thus always make sure at least one tick is used between the
+			 * processing of different orders when filling the timetable. */
+			time_taken = CeilDiv(max(time_taken, 1U), DAY_TICKS) * DAY_TICKS;
 
-			ChangeTimetable(v, v->cur_order_index, time_taken, travelling);
+			ChangeTimetable(v, v->cur_real_order_index, time_taken, travelling ? MTF_TRAVEL_TIME : MTF_WAIT_TIME);
 		}
 
-		if (v->cur_order_index == 0 && travelling) {
+		if (v->cur_real_order_index == first_manual_order && travelling) {
 			/* If we just started we would have returned earlier and have not reached
 			 * this code. So obviously, we have completed our round: So turn autofill
 			 * off again. */
@@ -291,6 +351,18 @@ void UpdateVehicleTimetable(Vehicle *v, bool travelling)
 
 	v->lateness_counter -= (timetabled - time_taken);
 
+	/* When we are more late than this timetabled bit takes we (somewhat expensively)
+	 * check how many ticks the (fully filled) timetable has. If a timetable cycle is
+	 * shorter than the amount of ticks we are late we reduce the lateness by the
+	 * length of a full cycle till lateness is less than the length of a timetable
+	 * cycle. When the timetable isn't fully filled the cycle will be INVALID_TICKS. */
+	if (v->lateness_counter > (int)timetabled) {
+		Ticks cycle = v->orders.list->GetTimetableTotalDuration();
+		if (cycle != INVALID_TICKS && v->lateness_counter > cycle) {
+			v->lateness_counter %= cycle;
+		}
+	}
+
 	for (v = v->FirstShared(); v != NULL; v = v->NextShared()) {
 		SetWindowDirty(WC_VEHICLE_TIMETABLE, v->index);
 	}
