@@ -1,4 +1,4 @@
-/* $Id: timetable_gui.cpp 18980 2010-02-01 12:43:28Z rubidium $ */
+/* $Id: timetable_gui.cpp 25340 2013-06-09 09:38:21Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -25,28 +25,11 @@
 #include "vehicle_gui.h"
 #include "settings_type.h"
 
+#include "widgets/timetable_widget.h"
+
 #include "table/sprites.h"
 #include "table/strings.h"
 
-enum TimetableViewWindowWidgets {
-	TTV_CAPTION,
-	TTV_ORDER_VIEW,
-	TTV_TIMETABLE_PANEL,
-	TTV_FAKE_SCROLLBAR,               ///< So the timetable panel 'sees' the scrollbar too
-	TTV_ARRIVAL_DEPARTURE_PANEL,      ///< Panel with the expected/scheduled arrivals
-	TTV_SCROLLBAR,
-	TTV_SUMMARY_PANEL,
-	TTV_START_DATE,
-	TTV_CHANGE_TIME,
-	TTV_CLEAR_TIME,
-	TTV_RESET_LATENESS,
-	TTV_AUTOFILL,
-	TTV_EXPECTED,                    ///< Toggle between expected and scheduled arrivals
-	TTV_SHARED_ORDER_LIST,           ///< Show the shared order list
-	TTV_ARRIVAL_DEPARTURE_SELECTION, ///< Disable/hide the arrival departure panel
-	TTV_EXPECTED_SELECTION,          ///< Disable/hide the expected selection button
-};
-
 /** Container for the arrival/departure dates of a vehicle */
 struct TimetableArrivalDeparture {
 	Ticks arrival;   ///< The arrival time
@@ -91,7 +74,7 @@ static void SetArrivalDepartParams(int param1, int param2, Ticks ticks)
 static bool CanDetermineTimeTaken(const Order *order, bool travelling)
 {
 	/* Current order is conditional */
-	if (order->IsType(OT_CONDITIONAL)) return false;
+	if (order->IsType(OT_CONDITIONAL) || order->IsType(OT_IMPLICIT)) return false;
 	/* No travel time and we have not already finished travelling */
 	if (travelling && order->travel_time == 0) return false;
 	/* No wait time but we are loading at this timetabled station */
@@ -127,15 +110,20 @@ static void FillTimetableArrivalDepartureTable(const Vehicle *v, VehicleOrderID
 	/* Cyclically loop over all orders until we reach the current one again.
 	 * As we may start at the current order, do a post-checking loop */
 	do {
-		if (travelling || i != start) {
-			if (!CanDetermineTimeTaken(order, true)) return;
-			sum += order->travel_time;
-			table[i].arrival = sum;
-		}
+		/* Automatic orders don't influence the overall timetable;
+		 * they just add some untimetabled entries, but the time till
+		 * the next non-implicit order can still be known. */
+		if (!order->IsType(OT_IMPLICIT)) {
+			if (travelling || i != start) {
+				if (!CanDetermineTimeTaken(order, true)) return;
+				sum += order->travel_time;
+				table[i].arrival = sum;
+			}
 
-		if (!CanDetermineTimeTaken(order, false)) return;
-		sum += order->wait_time;
-		table[i].departure = sum;
+			if (!CanDetermineTimeTaken(order, false)) return;
+			sum += order->wait_time;
+			table[i].departure = sum;
+		}
 
 		++i;
 		order = order->next;
@@ -173,6 +161,8 @@ struct TimetableWindow : Window {
 	bool show_expected;     ///< Whether we show expected arrival or scheduled
 	uint deparr_time_width; ///< The width of the departure/arrival time
 	uint deparr_abbr_width; ///< The width of the departure/arrival abbreviation
+	Scrollbar *vscroll;
+	bool query_is_speed_query; ///< The currently open query window is a speed query and not a time query.
 
 	TimetableWindow(const WindowDesc *desc, WindowNumber window_number) :
 			Window(),
@@ -181,6 +171,7 @@ struct TimetableWindow : Window {
 			show_expected(true)
 	{
 		this->CreateNestedTree(desc);
+		this->vscroll = this->GetScrollbar(WID_VT_SCROLLBAR);
 		this->UpdateSelectionStates();
 		this->FinishInitNested(desc, window_number);
 
@@ -200,7 +191,7 @@ struct TimetableWindow : Window {
 		bool travelling = (!v->current_order.IsType(OT_LOADING) || v->current_order.GetNonStopType() == ONSF_STOP_EVERYWHERE);
 		Ticks start_time = _date_fract - v->current_order_time;
 
-		FillTimetableArrivalDepartureTable(v, v->cur_order_index % v->GetNumOrders(), travelling, table, start_time);
+		FillTimetableArrivalDepartureTable(v, v->cur_real_order_index % v->GetNumOrders(), travelling, table, start_time);
 
 		return (travelling && v->lateness_counter < 0);
 	}
@@ -208,19 +199,19 @@ struct TimetableWindow : Window {
 	virtual void UpdateWidgetSize(int widget, Dimension *size, const Dimension &padding, Dimension *fill, Dimension *resize)
 	{
 		switch (widget) {
-			case TTV_ARRIVAL_DEPARTURE_PANEL:
-				SetDParam(0, MAX_YEAR * DAYS_IN_YEAR);
+			case WID_VT_ARRIVAL_DEPARTURE_PANEL:
+				SetDParamMaxValue(0, MAX_YEAR * DAYS_IN_YEAR, 0, FS_SMALL);
 				this->deparr_time_width = GetStringBoundingBox(STR_JUST_DATE_TINY).width;
 				this->deparr_abbr_width = max(GetStringBoundingBox(STR_TIMETABLE_ARRIVAL_ABBREVIATION).width, GetStringBoundingBox(STR_TIMETABLE_DEPARTURE_ABBREVIATION).width);
 				size->width = WD_FRAMERECT_LEFT + this->deparr_abbr_width + 10 + this->deparr_time_width + WD_FRAMERECT_RIGHT;
-				/* fall through */
-			case TTV_ARRIVAL_DEPARTURE_SELECTION:
-			case TTV_TIMETABLE_PANEL:
+				/* FALL THROUGH */
+			case WID_VT_ARRIVAL_DEPARTURE_SELECTION:
+			case WID_VT_TIMETABLE_PANEL:
 				resize->height = FONT_HEIGHT_NORMAL;
 				size->height = WD_FRAMERECT_TOP + 8 * resize->height + WD_FRAMERECT_BOTTOM;
 				break;
 
-			case TTV_SUMMARY_PANEL:
+			case WID_VT_SUMMARY_PANEL:
 				size->height = WD_FRAMERECT_TOP + 2 * FONT_HEIGHT_NORMAL + WD_FRAMERECT_BOTTOM;
 				break;
 		}
@@ -228,24 +219,29 @@ struct TimetableWindow : Window {
 
 	int GetOrderFromTimetableWndPt(int y, const Vehicle *v)
 	{
-		int sel = (y - this->GetWidget<NWidgetBase>(TTV_TIMETABLE_PANEL)->pos_y - WD_FRAMERECT_TOP) / FONT_HEIGHT_NORMAL;
+		int sel = (y - this->GetWidget<NWidgetBase>(WID_VT_TIMETABLE_PANEL)->pos_y - WD_FRAMERECT_TOP) / FONT_HEIGHT_NORMAL;
 
-		if ((uint)sel >= this->vscroll.GetCapacity()) return INVALID_ORDER;
+		if ((uint)sel >= this->vscroll->GetCapacity()) return INVALID_ORDER;
 
-		sel += this->vscroll.GetPosition();
+		sel += this->vscroll->GetPosition();
 
 		return (sel < v->GetNumOrders() * 2 && sel >= 0) ? sel : INVALID_ORDER;
 	}
 
-	virtual void OnInvalidateData(int data)
+	/**
+	 * Some data on this window has become invalid.
+	 * @param data Information about the changed data.
+	 * @param gui_scope Whether the call is done from GUI scope. You may not do everything when not in GUI scope. See #InvalidateWindowData() for details.
+	 */
+	virtual void OnInvalidateData(int data = 0, bool gui_scope = true)
 	{
 		switch (data) {
-			case 0:
+			case VIWD_AUTOREPLACE:
 				/* Autoreplace replaced the vehicle */
 				this->vehicle = Vehicle::Get(this->window_number);
 				break;
 
-			case -1:
+			case VIWD_REMOVE_ALL_ORDERS:
 				/* Removed / replaced all orders (after deleting / sharing) */
 				if (this->sel_index == -1) break;
 
@@ -253,12 +249,15 @@ struct TimetableWindow : Window {
 				this->sel_index = -1;
 				break;
 
-			case -2:
+			case VIWD_MODIFY_ORDERS:
+				if (!gui_scope) break;
 				this->UpdateSelectionStates();
 				this->ReInit();
 				break;
 
 			default: {
+				if (gui_scope) break; // only do this once; from command scope
+
 				/* Moving an order. If one of these is INVALID_VEH_ORDER_ID, then
 				 * the order is being created / removed */
 				if (this->sel_index == -1) break;
@@ -298,7 +297,8 @@ struct TimetableWindow : Window {
 				this->sel_index = 2 * selected_order - (int)travel;
 				/* travel time of first order needs special handling */
 				if (this->sel_index == -1) this->sel_index = this->vehicle->GetNumOrders() * 2 - 1;
-			} break;
+				break;
+			}
 		}
 	}
 
@@ -308,36 +308,41 @@ struct TimetableWindow : Window {
 		const Vehicle *v = this->vehicle;
 		int selected = this->sel_index;
 
-		this->vscroll.SetCount(v->GetNumOrders() * 2);
+		this->vscroll->SetCount(v->GetNumOrders() * 2);
 
 		if (v->owner == _local_company) {
 			bool disable = true;
 			if (selected != -1) {
 				const Order *order = v->GetOrder(((selected + 1) / 2) % v->GetNumOrders());
 				if (selected % 2 == 1) {
-					disable = order != NULL && order->IsType(OT_CONDITIONAL);
+					disable = order != NULL && (order->IsType(OT_CONDITIONAL) || order->IsType(OT_IMPLICIT));
 				} else {
 					disable = order == NULL || ((!order->IsType(OT_GOTO_STATION) || (order->GetNonStopType() & ONSF_NO_STOP_AT_DESTINATION_STATION)) && !order->IsType(OT_CONDITIONAL));
 				}
 			}
+			bool disable_speed = disable || selected % 2 != 1 || v->type == VEH_AIRCRAFT;
 
-			this->SetWidgetDisabledState(TTV_CHANGE_TIME, disable);
-			this->SetWidgetDisabledState(TTV_CLEAR_TIME, disable);
-			this->SetWidgetDisabledState(TTV_SHARED_ORDER_LIST, !v->IsOrderListShared());
+			this->SetWidgetDisabledState(WID_VT_CHANGE_TIME, disable);
+			this->SetWidgetDisabledState(WID_VT_CLEAR_TIME, disable);
+			this->SetWidgetDisabledState(WID_VT_CHANGE_SPEED, disable_speed);
+			this->SetWidgetDisabledState(WID_VT_CLEAR_SPEED, disable_speed);
+			this->SetWidgetDisabledState(WID_VT_SHARED_ORDER_LIST, !v->IsOrderListShared());
 
-			this->EnableWidget(TTV_START_DATE);
-			this->EnableWidget(TTV_RESET_LATENESS);
-			this->EnableWidget(TTV_AUTOFILL);
+			this->EnableWidget(WID_VT_START_DATE);
+			this->EnableWidget(WID_VT_RESET_LATENESS);
+			this->EnableWidget(WID_VT_AUTOFILL);
 		} else {
-			this->DisableWidget(TTV_START_DATE);
-			this->DisableWidget(TTV_CHANGE_TIME);
-			this->DisableWidget(TTV_CLEAR_TIME);
-			this->DisableWidget(TTV_RESET_LATENESS);
-			this->DisableWidget(TTV_AUTOFILL);
-			this->DisableWidget(TTV_SHARED_ORDER_LIST);
+			this->DisableWidget(WID_VT_START_DATE);
+			this->DisableWidget(WID_VT_CHANGE_TIME);
+			this->DisableWidget(WID_VT_CLEAR_TIME);
+			this->DisableWidget(WID_VT_CHANGE_SPEED);
+			this->DisableWidget(WID_VT_CLEAR_SPEED);
+			this->DisableWidget(WID_VT_RESET_LATENESS);
+			this->DisableWidget(WID_VT_AUTOFILL);
+			this->DisableWidget(WID_VT_SHARED_ORDER_LIST);
 		}
 
-		this->SetWidgetLoweredState(TTV_AUTOFILL, HasBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE));
+		this->SetWidgetLoweredState(WID_VT_AUTOFILL, HasBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE));
 
 		this->DrawWidgets();
 	}
@@ -345,8 +350,8 @@ struct TimetableWindow : Window {
 	virtual void SetStringParameters(int widget) const
 	{
 		switch (widget) {
-			case TTV_CAPTION: SetDParam(0, this->vehicle->index); break;
-			case TTV_EXPECTED: SetDParam(0, this->show_expected ? STR_TIMETABLE_EXPECTED : STR_TIMETABLE_SCHEDULED); break;
+			case WID_VT_CAPTION: SetDParam(0, this->vehicle->index); break;
+			case WID_VT_EXPECTED: SetDParam(0, this->show_expected ? STR_TIMETABLE_EXPECTED : STR_TIMETABLE_SCHEDULED); break;
 		}
 	}
 
@@ -356,21 +361,21 @@ struct TimetableWindow : Window {
 		int selected = this->sel_index;
 
 		switch (widget) {
-			case TTV_TIMETABLE_PANEL: {
+			case WID_VT_TIMETABLE_PANEL: {
 				int y = r.top + WD_FRAMERECT_TOP;
-				int i = this->vscroll.GetPosition();
+				int i = this->vscroll->GetPosition();
 				VehicleOrderID order_id = (i + 1) / 2;
 				bool final_order = false;
 
-				bool rtl = _dynlang.text_dir == TD_RTL;
-				SetDParam(0, 99);
-				int index_column_width = GetStringBoundingBox(STR_ORDER_INDEX).width + GetSpriteSize(rtl ? SPR_ARROW_RIGHT : SPR_ARROW_LEFT).width + 3;
+				bool rtl = _current_text_dir == TD_RTL;
+				SetDParamMaxValue(0, v->GetNumOrders(), 2);
+				int index_column_width = GetStringBoundingBox(STR_ORDER_INDEX).width + 2 * GetSpriteSize(rtl ? SPR_ARROW_RIGHT : SPR_ARROW_LEFT).width + 3;
 				int middle = rtl ? r.right - WD_FRAMERECT_RIGHT - index_column_width : r.left + WD_FRAMERECT_LEFT + index_column_width;
 
 				const Order *order = v->GetOrder(order_id);
 				while (order != NULL) {
 					/* Don't draw anything if it extends past the end of the window. */
-					if (!this->vscroll.IsVisible(i)) break;
+					if (!this->vscroll->IsVisible(i)) break;
 
 					if (i % 2 == 0) {
 						DrawOrderString(v, order, order_id, y, i == selected, true, r.left + WD_FRAMERECT_LEFT, middle, r.right - WD_FRAMERECT_RIGHT);
@@ -385,17 +390,21 @@ struct TimetableWindow : Window {
 						}
 					} else {
 						StringID string;
-
+						TextColour colour = (i == selected) ? TC_WHITE : TC_BLACK;
 						if (order->IsType(OT_CONDITIONAL)) {
 							string = STR_TIMETABLE_NO_TRAVEL;
+						} else if (order->IsType(OT_IMPLICIT)) {
+							string = STR_TIMETABLE_NOT_TIMETABLEABLE;
+							colour = ((i == selected) ? TC_SILVER : TC_GREY) | TC_NO_SHADE;
 						} else if (order->travel_time == 0) {
-							string = STR_TIMETABLE_TRAVEL_NOT_TIMETABLED;
+							string = order->max_speed != UINT16_MAX ? STR_TIMETABLE_TRAVEL_NOT_TIMETABLED_SPEED : STR_TIMETABLE_TRAVEL_NOT_TIMETABLED;
 						} else {
 							SetTimetableParams(0, 1, order->travel_time);
-							string = STR_TIMETABLE_TRAVEL_FOR;
+							string = order->max_speed != UINT16_MAX ? STR_TIMETABLE_TRAVEL_FOR_SPEED : STR_TIMETABLE_TRAVEL_FOR;
 						}
+						SetDParam(2, order->max_speed);
 
-						DrawString(rtl ? r.left + WD_FRAMERECT_LEFT : middle, rtl ? middle : r.right - WD_FRAMERECT_LEFT, y, string, (i == selected) ? TC_WHITE : TC_BLACK);
+						DrawString(rtl ? r.left + WD_FRAMERECT_LEFT : middle, rtl ? middle : r.right - WD_FRAMERECT_LEFT, y, string, colour);
 
 						if (final_order) break;
 					}
@@ -406,7 +415,7 @@ struct TimetableWindow : Window {
 				break;
 			}
 
-			case TTV_ARRIVAL_DEPARTURE_PANEL: {
+			case WID_VT_ARRIVAL_DEPARTURE_PANEL: {
 				/* Arrival and departure times are handled in an all-or-nothing approach,
 				 * i.e. are only shown if we can calculate all times.
 				 * Excluding order lists with only one order makes some things easier.
@@ -415,7 +424,7 @@ struct TimetableWindow : Window {
 				if (total_time <= 0 || v->GetNumOrders() <= 1 || !HasBit(v->vehicle_flags, VF_TIMETABLE_STARTED)) break;
 
 				TimetableArrivalDeparture *arr_dep = AllocaM(TimetableArrivalDeparture, v->GetNumOrders());
-				const VehicleOrderID cur_order = v->cur_order_index % v->GetNumOrders();
+				const VehicleOrderID cur_order = v->cur_real_order_index % v->GetNumOrders();
 
 				VehicleOrderID earlyID = BuildArrivalDepartureList(v, arr_dep) ? cur_order : (VehicleOrderID)INVALID_VEH_ORDER_ID;
 
@@ -424,15 +433,15 @@ struct TimetableWindow : Window {
 				bool show_late = this->show_expected && v->lateness_counter > DAY_TICKS;
 				Ticks offset = show_late ? 0 : -v->lateness_counter;
 
-				bool rtl = _dynlang.text_dir == TD_RTL;
+				bool rtl = _current_text_dir == TD_RTL;
 				int abbr_left  = rtl ? r.right - WD_FRAMERECT_RIGHT - this->deparr_abbr_width : r.left + WD_FRAMERECT_LEFT;
 				int abbr_right = rtl ? r.right - WD_FRAMERECT_RIGHT : r.left + WD_FRAMERECT_LEFT + this->deparr_abbr_width;
 				int time_left  = rtl ? r.left + WD_FRAMERECT_LEFT : r.right - WD_FRAMERECT_RIGHT - this->deparr_time_width;
 				int time_right = rtl ? r.left + WD_FRAMERECT_LEFT + this->deparr_time_width : r.right - WD_FRAMERECT_RIGHT;
 
-				for (int i = this->vscroll.GetPosition(); i / 2 < v->GetNumOrders(); ++i) { // note: i is also incremented in the loop
+				for (int i = this->vscroll->GetPosition(); i / 2 < v->GetNumOrders(); ++i) { // note: i is also incremented in the loop
 					/* Don't draw anything if it extends past the end of the window. */
-					if (!this->vscroll.IsVisible(i)) break;
+					if (!this->vscroll->IsVisible(i)) break;
 
 					if (i % 2 == 0) {
 						if (arr_dep[i / 2].arrival != INVALID_TICKS) {
@@ -454,9 +463,10 @@ struct TimetableWindow : Window {
 					}
 					y += FONT_HEIGHT_NORMAL;
 				}
-			} break;
+				break;
+			}
 
-			case TTV_SUMMARY_PANEL: {
+			case WID_VT_SUMMARY_PANEL: {
 				int y = r.top + WD_FRAMERECT_TOP;
 
 				Ticks total_time = v->orders.list != NULL ? v->orders.list->GetTimetableDurationIncomplete() : 0;
@@ -487,14 +497,14 @@ struct TimetableWindow : Window {
 		}
 	}
 
-	static inline uint32 PackTimetableArgs(const Vehicle *v, uint selected)
+	static inline uint32 PackTimetableArgs(const Vehicle *v, uint selected, bool speed)
 	{
 		uint order_number = (selected + 1) / 2;
-		uint is_journey   = (selected % 2 == 1) ? 1 : 0;
+		ModifyTimetableFlags mtf = (selected % 2 == 1) ? (speed ? MTF_TRAVEL_SPEED : MTF_TRAVEL_TIME) : MTF_WAIT_TIME;
 
 		if (order_number >= v->GetNumOrders()) order_number = 0;
 
-		return v->index | (order_number << 16) | (is_journey << 24);
+		return v->index | (order_number << 20) | (mtf << 28);
 	}
 
 	virtual void OnClick(Point pt, int widget, int click_count)
@@ -502,22 +512,23 @@ struct TimetableWindow : Window {
 		const Vehicle *v = this->vehicle;
 
 		switch (widget) {
-			case TTV_ORDER_VIEW: // Order view button
+			case WID_VT_ORDER_VIEW: // Order view button
 				ShowOrdersWindow(v);
 				break;
 
-			case TTV_TIMETABLE_PANEL: { // Main panel.
+			case WID_VT_TIMETABLE_PANEL: { // Main panel.
 				int selected = GetOrderFromTimetableWndPt(pt.y, v);
 
 				this->DeleteChildWindows();
 				this->sel_index = (selected == INVALID_ORDER || selected == this->sel_index) ? -1 : selected;
-			} break;
+				break;
+			}
 
-			case TTV_START_DATE: // Change the date that the timetable starts.
+			case WID_VT_START_DATE: // Change the date that the timetable starts.
 				ShowSetDateWindow(this, v->index, _date, _cur_year, _cur_year + 15, ChangeTimetableStartCallback);
 				break;
 
-			case TTV_CHANGE_TIME: { // "Wait For" button.
+			case WID_VT_CHANGE_TIME: { // "Wait For" button.
 				int selected = this->sel_index;
 				VehicleOrderID real = (selected + 1) / 2;
 
@@ -536,30 +547,60 @@ struct TimetableWindow : Window {
 					}
 				}
 
-				ShowQueryString(current, STR_TIMETABLE_CHANGE_TIME, 31, 150, this, CS_NUMERAL, QSF_NONE);
-			} break;
+				this->query_is_speed_query = false;
+				ShowQueryString(current, STR_TIMETABLE_CHANGE_TIME, 31, this, CS_NUMERAL, QSF_NONE);
+				break;
+			}
 
-			case TTV_CLEAR_TIME: { // Clear waiting time button.
-				uint32 p1 = PackTimetableArgs(v, this->sel_index);
+			case WID_VT_CHANGE_SPEED: { // Change max speed button.
+				int selected = this->sel_index;
+				VehicleOrderID real = (selected + 1) / 2;
+
+				if (real >= v->GetNumOrders()) real = 0;
+
+				StringID current = STR_EMPTY;
+				const Order *order = v->GetOrder(real);
+				if (order != NULL) {
+					if (order->max_speed != UINT16_MAX) {
+						SetDParam(0, ConvertKmhishSpeedToDisplaySpeed(order->max_speed));
+						current = STR_JUST_INT;
+					}
+				}
+
+				this->query_is_speed_query = true;
+				ShowQueryString(current, STR_TIMETABLE_CHANGE_SPEED, 31, this, CS_NUMERAL, QSF_NONE);
+				break;
+			}
+
+			case WID_VT_CLEAR_TIME: { // Clear waiting time.
+				uint32 p1 = PackTimetableArgs(v, this->sel_index, false);
 				DoCommandP(0, p1, 0, CMD_CHANGE_TIMETABLE | CMD_MSG(STR_ERROR_CAN_T_TIMETABLE_VEHICLE));
-			} break;
+				break;
+			}
 
-			case TTV_RESET_LATENESS: // Reset the vehicle's late counter.
+			case WID_VT_CLEAR_SPEED: { // Clear max speed button.
+				uint32 p1 = PackTimetableArgs(v, this->sel_index, true);
+				DoCommandP(0, p1, UINT16_MAX, CMD_CHANGE_TIMETABLE | CMD_MSG(STR_ERROR_CAN_T_TIMETABLE_VEHICLE));
+				break;
+			}
+
+			case WID_VT_RESET_LATENESS: // Reset the vehicle's late counter.
 				DoCommandP(0, v->index, 0, CMD_SET_VEHICLE_ON_TIME | CMD_MSG(STR_ERROR_CAN_T_TIMETABLE_VEHICLE));
 				break;
 
-			case TTV_AUTOFILL: { // Autofill the timetable.
+			case WID_VT_AUTOFILL: { // Autofill the timetable.
 				uint32 p2 = 0;
 				if (!HasBit(v->vehicle_flags, VF_AUTOFILL_TIMETABLE)) SetBit(p2, 0);
 				if (_ctrl_pressed) SetBit(p2, 1);
 				DoCommandP(0, v->index, p2, CMD_AUTOFILL_TIMETABLE | CMD_MSG(STR_ERROR_CAN_T_TIMETABLE_VEHICLE));
-			} break;
+				break;
+			}
 
-			case TTV_EXPECTED:
+			case WID_VT_EXPECTED:
 				this->show_expected = !this->show_expected;
 				break;
 
-			case TTV_SHARED_ORDER_LIST:
+			case WID_VT_SHARED_ORDER_LIST:
 				ShowVehicleListWindow(v);
 				break;
 		}
@@ -573,12 +614,16 @@ struct TimetableWindow : Window {
 
 		const Vehicle *v = this->vehicle;
 
-		uint32 p1 = PackTimetableArgs(v, this->sel_index);
+		uint32 p1 = PackTimetableArgs(v, this->sel_index, this->query_is_speed_query);
 
-		uint64 time = StrEmpty(str) ? 0 : strtoul(str, NULL, 10);
-		if (!_settings_client.gui.timetable_in_ticks) time *= DAY_TICKS;
+		uint64 val = StrEmpty(str) ? 0 : strtoul(str, NULL, 10);
+		if (this->query_is_speed_query) {
+			val = ConvertDisplaySpeedToKmhishSpeed(val);
+		} else {
+			if (!_settings_client.gui.timetable_in_ticks) val *= DAY_TICKS;
+		}
 
-		uint32 p2 = minu(time, UINT16_MAX);
+		uint32 p2 = minu(val, UINT16_MAX);
 
 		DoCommandP(0, p1, p2, CMD_CHANGE_TIMETABLE | CMD_MSG(STR_ERROR_CAN_T_TIMETABLE_VEHICLE));
 	}
@@ -586,7 +631,7 @@ struct TimetableWindow : Window {
 	virtual void OnResize()
 	{
 		/* Update the scroll bar */
-		this->vscroll.SetCapacityFromWidget(this, TTV_TIMETABLE_PANEL, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM);
+		this->vscroll->SetCapacityFromWidget(this, WID_VT_TIMETABLE_PANEL, WD_FRAMERECT_TOP + WD_FRAMERECT_BOTTOM);
 	}
 
 	/**
@@ -594,48 +639,51 @@ struct TimetableWindow : Window {
 	 */
 	void UpdateSelectionStates()
 	{
-		this->GetWidget<NWidgetStacked>(TTV_ARRIVAL_DEPARTURE_SELECTION)->SetDisplayedPlane(_settings_client.gui.timetable_arrival_departure ? 0 : SZSP_NONE);
-		this->GetWidget<NWidgetStacked>(TTV_EXPECTED_SELECTION)->SetDisplayedPlane(_settings_client.gui.timetable_arrival_departure ? 0 : 1);
+		this->GetWidget<NWidgetStacked>(WID_VT_ARRIVAL_DEPARTURE_SELECTION)->SetDisplayedPlane(_settings_client.gui.timetable_arrival_departure ? 0 : SZSP_NONE);
+		this->GetWidget<NWidgetStacked>(WID_VT_EXPECTED_SELECTION)->SetDisplayedPlane(_settings_client.gui.timetable_arrival_departure ? 0 : 1);
 	}
 };
 
 static const NWidgetPart _nested_timetable_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
 		NWidget(WWT_CLOSEBOX, COLOUR_GREY),
-		NWidget(WWT_CAPTION, COLOUR_GREY, TTV_CAPTION), SetDataTip(STR_TIMETABLE_TITLE, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
-		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, TTV_ORDER_VIEW), SetMinimalSize(61, 14), SetDataTip( STR_TIMETABLE_ORDER_VIEW, STR_TIMETABLE_ORDER_VIEW_TOOLTIP),
+		NWidget(WWT_CAPTION, COLOUR_GREY, WID_VT_CAPTION), SetDataTip(STR_TIMETABLE_TITLE, STR_TOOLTIP_WINDOW_TITLE_DRAG_THIS),
+		NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_ORDER_VIEW), SetMinimalSize(61, 14), SetDataTip( STR_TIMETABLE_ORDER_VIEW, STR_TIMETABLE_ORDER_VIEW_TOOLTIP),
 		NWidget(WWT_SHADEBOX, COLOUR_GREY),
 		NWidget(WWT_STICKYBOX, COLOUR_GREY),
 	EndContainer(),
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_GREY, TTV_TIMETABLE_PANEL), SetMinimalSize(388, 82), SetResize(1, 10), SetDataTip(STR_NULL, STR_TIMETABLE_TOOLTIP), EndContainer(),
-		NWidget(WWT_SCROLLBAR, COLOUR_GREY, TTV_FAKE_SCROLLBAR), SetMinimalSize(0, 0), // Hack so the timetable panel can 'use' the scrollbar too
-		NWidget(NWID_SELECTION, INVALID_COLOUR, TTV_ARRIVAL_DEPARTURE_SELECTION),
-			NWidget(WWT_PANEL, COLOUR_GREY, TTV_ARRIVAL_DEPARTURE_PANEL), SetMinimalSize(110, 0), SetFill(0, 1), SetDataTip(STR_NULL, STR_TIMETABLE_TOOLTIP), EndContainer(),
+		NWidget(WWT_PANEL, COLOUR_GREY, WID_VT_TIMETABLE_PANEL), SetMinimalSize(388, 82), SetResize(1, 10), SetDataTip(STR_NULL, STR_TIMETABLE_TOOLTIP), SetScrollbar(WID_VT_SCROLLBAR), EndContainer(),
+		NWidget(NWID_SELECTION, INVALID_COLOUR, WID_VT_ARRIVAL_DEPARTURE_SELECTION),
+			NWidget(WWT_PANEL, COLOUR_GREY, WID_VT_ARRIVAL_DEPARTURE_PANEL), SetMinimalSize(110, 0), SetFill(0, 1), SetDataTip(STR_NULL, STR_TIMETABLE_TOOLTIP), SetScrollbar(WID_VT_SCROLLBAR), EndContainer(),
 		EndContainer(),
-		NWidget(WWT_SCROLLBAR, COLOUR_GREY, TTV_SCROLLBAR),
+		NWidget(NWID_VSCROLLBAR, COLOUR_GREY, WID_VT_SCROLLBAR),
 	EndContainer(),
-	NWidget(WWT_PANEL, COLOUR_GREY, TTV_SUMMARY_PANEL), SetMinimalSize(400, 22), SetResize(1, 0), EndContainer(),
+	NWidget(WWT_PANEL, COLOUR_GREY, WID_VT_SUMMARY_PANEL), SetMinimalSize(400, 22), SetResize(1, 0), EndContainer(),
 	NWidget(NWID_HORIZONTAL),
 		NWidget(NWID_HORIZONTAL, NC_EQUALSIZE),
 			NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, TTV_CHANGE_TIME), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CHANGE_TIME, STR_TIMETABLE_WAIT_TIME_TOOLTIP),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, TTV_CLEAR_TIME), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CLEAR_TIME, STR_TIMETABLE_CLEAR_TIME_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CHANGE_TIME), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CHANGE_TIME, STR_TIMETABLE_WAIT_TIME_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CLEAR_TIME), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CLEAR_TIME, STR_TIMETABLE_CLEAR_TIME_TOOLTIP),
 			EndContainer(),
 			NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, TTV_START_DATE), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_STARTING_DATE, STR_TIMETABLE_STARTING_DATE_TOOLTIP),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, TTV_RESET_LATENESS), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_RESET_LATENESS, STR_TIMETABLE_RESET_LATENESS_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CHANGE_SPEED), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CHANGE_SPEED, STR_TIMETABLE_CHANGE_SPEED_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_CLEAR_SPEED), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_CLEAR_SPEED, STR_TIMETABLE_CLEAR_SPEED_TOOLTIP),
 			EndContainer(),
 			NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, TTV_AUTOFILL), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_AUTOFILL, STR_TIMETABLE_AUTOFILL_TOOLTIP),
-				NWidget(NWID_SELECTION, INVALID_COLOUR, TTV_EXPECTED_SELECTION),
-					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, TTV_EXPECTED), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_BLACK_STRING, STR_TIMETABLE_EXPECTED_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_START_DATE), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_STARTING_DATE, STR_TIMETABLE_STARTING_DATE_TOOLTIP),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_RESET_LATENESS), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_RESET_LATENESS, STR_TIMETABLE_RESET_LATENESS_TOOLTIP),
+			EndContainer(),
+			NWidget(NWID_VERTICAL, NC_EQUALSIZE),
+				NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_AUTOFILL), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_TIMETABLE_AUTOFILL, STR_TIMETABLE_AUTOFILL_TOOLTIP),
+				NWidget(NWID_SELECTION, INVALID_COLOUR, WID_VT_EXPECTED_SELECTION),
+					NWidget(WWT_PUSHTXTBTN, COLOUR_GREY, WID_VT_EXPECTED), SetResize(1, 0), SetFill(1, 1), SetDataTip(STR_BLACK_STRING, STR_TIMETABLE_EXPECTED_TOOLTIP),
 					NWidget(WWT_PANEL, COLOUR_GREY), SetResize(1, 0), SetFill(1, 1), EndContainer(),
 				EndContainer(),
 			EndContainer(),
 		EndContainer(),
 		NWidget(NWID_VERTICAL, NC_EQUALSIZE),
-			NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, TTV_SHARED_ORDER_LIST), SetFill(0, 1), SetDataTip(SPR_SHARED_ORDERS_ICON, STR_ORDERS_VEH_WITH_SHARED_ORDERS_LIST_TOOLTIP),
+			NWidget(WWT_PUSHIMGBTN, COLOUR_GREY, WID_VT_SHARED_ORDER_LIST), SetFill(0, 1), SetDataTip(SPR_SHARED_ORDERS_ICON, STR_ORDERS_VEH_WITH_SHARED_ORDERS_LIST_TOOLTIP),
 			NWidget(WWT_RESIZEBOX, COLOUR_GREY), SetFill(0, 1),
 		EndContainer(),
 	EndContainer(),
@@ -644,10 +692,14 @@ static const NWidgetPart _nested_timetable_widgets[] = {
 static const WindowDesc _timetable_desc(
 	WDP_AUTO, 400, 130,
 	WC_VEHICLE_TIMETABLE, WC_VEHICLE_VIEW,
-	WDF_UNCLICK_BUTTONS | WDF_CONSTRUCTION,
+	WDF_CONSTRUCTION,
 	_nested_timetable_widgets, lengthof(_nested_timetable_widgets)
 );
 
+/**
+ * Show the timetable for a given vehicle.
+ * @param v The vehicle to show the timetable for.
+ */
 void ShowTimetableWindow(const Vehicle *v)
 {
 	DeleteWindowById(WC_VEHICLE_DETAILS, v->index, false);
