@@ -1,4 +1,4 @@
-/* $Id: townname_func.h 17612 2009-09-22 13:54:54Z smatz $ */
+/* $Id: townname_func.h 23704 2012-01-01 17:22:32Z alberth $ */
 
 /*
  * This file is part of OpenTTD.
@@ -7,15 +7,17 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file townname_func.h Town name generator stuff. */
+/** @file townname_func.h %Town name generator stuff. */
 
 #ifndef TOWNNAME_FUNC_H
 #define TOWNNAME_FUNC_H
 
+#include "townname_type.h"
+
 char *GenerateTownNameString(char *buf, const char *last, size_t lang, uint32 seed);
-char *GetTownName(char *buff, const struct TownNameParams *par, uint32 townnameparts, const char *last);
-char *GetTownName(char *buff, const struct Town *t, const char *last);
-bool VerifyTownName(uint32 r, const struct TownNameParams *par);
+char *GetTownName(char *buff, const TownNameParams *par, uint32 townnameparts, const char *last);
+char *GetTownName(char *buff, const Town *t, const char *last);
+bool VerifyTownName(uint32 r, const TownNameParams *par);
 bool GenerateTownName(uint32 *townnameparts);
 
 #endif /* TOWNNAME_FUNC_H */
