@@ -1,4 +1,4 @@
-/* $Id: event.mm 18545 2009-12-19 18:46:40Z frosch $ */
+/* $Id: event.mm 26024 2013-11-17 13:35:48Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -26,7 +26,6 @@
 #include "../../openttd.h"
 #include "../../debug.h"
 #include "../../os/macosx/splash.h"
-#include "../../variables.h"
 #include "../../settings_type.h"
 #include "../../core/geometry_type.hpp"
 #include "cocoa_v.h"
@@ -35,7 +34,9 @@
 #include "../../gfx_func.h"
 #include "../../network/network.h"
 #include "../../core/random_func.hpp"
+#include "../../core/math_func.hpp"
 #include "../../texteff.hpp"
+#include "../../window_func.h"
 
 #import <sys/time.h> /* gettimeofday */
 
@@ -49,22 +50,38 @@
 
 
 /* Right Mouse Button Emulation enum */
-enum {
+enum RightMouseButtonEmulationState {
 	RMBE_COMMAND,
 	RMBE_CONTROL,
 	RMBE_OFF,
 };
 
 
-static bool _show_mouse = true;
 static unsigned int _current_mods;
 static bool _tab_is_down;
 static bool _emulating_right_button;
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
+static float _current_magnification;
+#endif
 #ifdef _DEBUG
 static uint32 _tEvent;
 #endif
 
 
+/* Support for touch gestures is only available starting with the
+ * 10.6 SDK, even if it says that support starts in fact with 10.5.2.
+ * Replicate the needed stuff for older SDKs. */
+#if MAC_OS_X_VERSION_MAX_ALLOWED == MAC_OS_X_VERSION_10_5
+static const NSUInteger NSEventTypeMagnify    = 30;
+static const NSUInteger NSEventTypeEndGesture = 20;
+
+@interface NSEvent ()
+/* This message is valid for events of type NSEventTypeMagnify, on 10.5.2 or later */
+- (CGFloat)magnification WEAK_IMPORT_ATTRIBUTE;
+@end
+#endif
+
+
 static uint32 GetTick()
 {
 	struct timeval tim;
@@ -73,34 +90,6 @@ static uint32 GetTick()
 	return tim.tv_usec / 1000 + tim.tv_sec * 1000;
 }
 
-
-void QZ_ShowMouse()
-{
-	if (!_show_mouse) {
-		[ NSCursor unhide ];
-		_show_mouse = true;
-
-		/* Hide the openttd cursor when leaving the window */
-		if (_cocoa_subdriver != NULL) UndrawMouseCursor();
-		_cursor.in_window = false;
-	}
-}
-
-void QZ_HideMouse()
-{
-	if (_show_mouse) {
-		/* Don't hide the cursor when compiling in debug mode.
-		 * Note: Not hiding the cursor will cause artefacts around it in 8bpp fullscreen mode. */
-#ifndef _DEBUG
-		[ NSCursor hide ];
-#endif
-		_show_mouse = false;
-
-		/* Show the openttd cursor again */
-		_cursor.in_window = true;
-	}
-}
-
 static void QZ_WarpCursor(int x, int y)
 {
 	assert(_cocoa_subdriver != NULL);
@@ -120,16 +109,16 @@ static void QZ_WarpCursor(int x, int y)
 
 static void QZ_CheckPaletteAnim()
 {
-	if (_pal_count_dirty != 0) {
+	if (_cur_palette.count_dirty != 0) {
 		Blitter *blitter = BlitterFactoryBase::GetCurrentBlitter();
 
 		switch (blitter->UsePaletteAnimation()) {
 			case Blitter::PALETTE_ANIMATION_VIDEO_BACKEND:
-				_cocoa_subdriver->UpdatePalette(_pal_first_dirty, _pal_count_dirty);
+				_cocoa_subdriver->UpdatePalette(_cur_palette.first_dirty, _cur_palette.count_dirty);
 				break;
 
 			case Blitter::PALETTE_ANIMATION_BLITTER:
-				blitter->PaletteAnimate(_pal_first_dirty, _pal_count_dirty);
+				blitter->PaletteAnimate(_cur_palette);
 				break;
 
 			case Blitter::PALETTE_ANIMATION_NONE:
@@ -138,7 +127,7 @@ static void QZ_CheckPaletteAnim()
 			default:
 				NOT_REACHED();
 		}
-		_pal_count_dirty = 0;
+		_cur_palette.count_dirty = 0;
 	}
 }
 
@@ -280,11 +269,13 @@ static uint32 QZ_MapKey(unsigned short sym)
 	if (_current_mods & NSAlternateKeyMask) key |= WKC_ALT;
 	if (_current_mods & NSCommandKeyMask)   key |= (_settings_client.gui.right_mouse_btn_emulation != RMBE_CONTROL ? WKC_META : WKC_CTRL);
 
-	return key << 16;
+	return key;
 }
 
-static void QZ_KeyEvent(unsigned short keycode, unsigned short unicode, BOOL down)
+static bool QZ_KeyEvent(unsigned short keycode, unsigned short unicode, BOOL down)
 {
+	bool interpret_keys = true;
+
 	switch (keycode) {
 		case QZ_UP:    SB(_dirkeys, 1, 1, down); break;
 		case QZ_DOWN:  SB(_dirkeys, 3, 1, down); break;
@@ -302,12 +293,32 @@ static void QZ_KeyEvent(unsigned short keycode, unsigned short unicode, BOOL dow
 	}
 
 	if (down) {
-		uint32 pressed_key = QZ_MapKey(keycode) | unicode;
-		HandleKeypress(pressed_key);
+		uint32 pressed_key = QZ_MapKey(keycode);
+
+		static bool console = false;
+
+		if (pressed_key == WKC_BACKQUOTE && unicode == 0) {
+			if (!console) {
+				/* Backquote is a dead key, require a double press for hotkey behaviour (i.e. console). */
+				console = true;
+				return true;
+			} else {
+				/* Second backquote, don't interpret as text input. */
+				interpret_keys = false;
+			}
+		}
+		console = false;
+
+		/* Don't handle normal characters if an edit box has the focus. */
+		if (!EditBoxInGlobalFocus() || ((pressed_key & ~WKC_SPECIAL_KEYS) <= WKC_TAB) || IsInsideMM(pressed_key & ~WKC_SPECIAL_KEYS, WKC_F1, WKC_PAUSE + 1)) {
+			HandleKeypress(pressed_key, unicode);
+		}
 		DEBUG(driver, 2, "cocoa_v: QZ_KeyEvent: %x (%x), down, mapping: %x", keycode, unicode, pressed_key);
 	} else {
 		DEBUG(driver, 2, "cocoa_v: QZ_KeyEvent: %x (%x), up", keycode, unicode);
 	}
+
+	return interpret_keys;
 }
 
 static void QZ_DoUnsidedModifiers(unsigned int newMods)
@@ -404,7 +415,6 @@ static bool QZ_PollEvent()
 
 	if (event == nil) return false;
 	if (!_cocoa_subdriver->IsActive()) {
-		QZ_ShowMouse();
 		[ NSApp sendEvent:event ];
 		return true;
 	}
@@ -419,18 +429,15 @@ static bool QZ_PollEvent()
 		case NSLeftMouseDragged:
 			pt = _cocoa_subdriver->GetMouseLocation(event);
 			if (!_cocoa_subdriver->MouseIsInsideView(&pt) && !_emulating_right_button) {
-				QZ_ShowMouse();
 				[ NSApp sendEvent:event ];
 				break;
 			}
 
-			QZ_HideMouse();
 			QZ_MouseMovedEvent((int)pt.x, (int)pt.y);
 			break;
 
 		case NSRightMouseDragged:
 			pt = _cocoa_subdriver->GetMouseLocation(event);
-			QZ_HideMouse();
 			QZ_MouseMovedEvent((int)pt.x, (int)pt.y);
 			break;
 
@@ -446,12 +453,6 @@ static bool QZ_PollEvent()
 				[ NSApp sendEvent:event ];
 			}
 
-			if (!_cocoa_subdriver->MouseIsInsideView(&pt)) {
-				QZ_ShowMouse();
-				break;
-			}
-
-			QZ_HideMouse();
 			QZ_MouseMovedEvent((int)pt.x, (int)pt.y);
 
 			/* Right mouse button emulation */
@@ -467,12 +468,7 @@ static bool QZ_PollEvent()
 			[ NSApp sendEvent:event ];
 
 			pt = _cocoa_subdriver->GetMouseLocation(event);
-			if (!_cocoa_subdriver->MouseIsInsideView(&pt)) {
-				QZ_ShowMouse();
-				break;
-			}
 
-			QZ_HideMouse();
 			QZ_MouseMovedEvent((int)pt.x, (int)pt.y);
 
 			/* Right mouse button emulation */
@@ -487,12 +483,10 @@ static bool QZ_PollEvent()
 		case NSRightMouseDown:
 			pt = _cocoa_subdriver->GetMouseLocation(event);
 			if (!_cocoa_subdriver->MouseIsInsideView(&pt)) {
-				QZ_ShowMouse();
 				[ NSApp sendEvent:event ];
 				break;
 			}
 
-			QZ_HideMouse();
 			QZ_MouseMovedEvent((int)pt.x, (int)pt.y);
 			QZ_MouseButtonEvent(1, YES);
 			break;
@@ -500,12 +494,10 @@ static bool QZ_PollEvent()
 		case NSRightMouseUp:
 			pt = _cocoa_subdriver->GetMouseLocation(event);
 			if (!_cocoa_subdriver->MouseIsInsideView(&pt)) {
-				QZ_ShowMouse();
 				[ NSApp sendEvent:event ];
 				break;
 			}
 
-			QZ_HideMouse();
 			QZ_MouseMovedEvent((int)pt.x, (int)pt.y);
 			QZ_MouseButtonEvent(1, NO);
 			break;
@@ -515,12 +507,10 @@ static bool QZ_PollEvent()
 		case NSOtherMouseDown:
 			pt = QZ_GetMouseLocation(event);
 			if (!QZ_MouseIsInsideView(&pt)) {
-				QZ_ShowMouse();
 				[ NSApp sendEvent:event ];
 				break;
 			}
 
-			QZ_HideMouse();
 			QZ_MouseMovedEvent((int)pt.x, (int)pt.y);
 			QZ_MouseButtonEvent([ event buttonNumber ], YES);
 			break;
@@ -528,12 +518,10 @@ static bool QZ_PollEvent()
 		case NSOtherMouseUp:
 			pt = QZ_GetMouseLocation(event);
 			if (!QZ_MouseIsInsideView(&pt)) {
-				QZ_ShowMouse();
 				[ NSApp sendEvent:event ];
 				break;
 			}
 
-			QZ_HideMouse();
 			QZ_MouseMovedEvent((int)pt.x, (int)pt.y);
 			QZ_MouseButtonEvent([ event buttonNumber ], NO);
 			break;
@@ -551,8 +539,21 @@ static bool QZ_PollEvent()
 					break;
 			}
 
-			chars = [ event characters ];
-			QZ_KeyEvent([ event keyCode ], [ chars length ] ? [ chars characterAtIndex:0 ] : 0, YES);
+			if (EditBoxInGlobalFocus()) {
+				if (QZ_KeyEvent([ event keyCode ], 0, YES)) {
+					[ _cocoa_subdriver->cocoaview interpretKeyEvents:[ NSArray arrayWithObject:event ] ];
+				}
+			} else {
+				chars = [ event characters ];
+				if ([ chars length ] == 0) {
+					QZ_KeyEvent([ event keyCode ], 0, YES);
+				} else {
+					QZ_KeyEvent([ event keyCode ], [ chars characterAtIndex:0 ], YES);
+					for (uint i = 1; i < [ chars length ]; i++) {
+						QZ_KeyEvent(0, [ chars characterAtIndex:i ], YES);
+					}
+				}
+			}
 			break;
 
 		case NSKeyUp:
@@ -583,6 +584,38 @@ static bool QZ_PollEvent()
 			_cursor.v_wheel -= (int)([ event deltaY ] * 5 * _settings_client.gui.scrollwheel_multiplier);
 			break;
 
+#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
+		case NSEventTypeMagnify:
+			/* Pinch open or close gesture. */
+			_current_magnification += [ event magnification ] * 5.0f;
+
+			while (_current_magnification >= 1.0f) {
+				_current_magnification -= 1.0f;
+				_cursor.wheel++;
+				HandleMouseEvents();
+			}
+			while (_current_magnification <= -1.0f) {
+				_current_magnification += 1.0f;
+				_cursor.wheel--;
+				HandleMouseEvents();
+			}
+			break;
+
+		case NSEventTypeEndGesture:
+			/* Gesture ended. */
+			_current_magnification = 0.0f;
+			break;
+#endif
+
+		case NSCursorUpdate:
+		case NSMouseEntered:
+		case NSMouseExited:
+			/* Catch these events if the cursor is dragging. During dragging, we reset
+			 * the mouse position programmatically, which would trigger OS X to show
+			 * the default arrow cursor if the events are propagated. */
+			if (_cursor.fix_at) break;
+			/* FALL THROUGH */
+
 		default:
 			[ NSApp sendEvent:event ];
 	}
@@ -595,8 +628,7 @@ void QZ_GameLoop()
 {
 	uint32 cur_ticks = GetTick();
 	uint32 last_cur_ticks = cur_ticks;
-	uint32 next_tick = cur_ticks + 30;
-	uint32 pal_tick = 0;
+	uint32 next_tick = cur_ticks + MILLISECONDS_PER_TICK;
 
 #ifdef _DEBUG
 	uint32 et0 = GetTick();
@@ -615,6 +647,12 @@ void QZ_GameLoop()
 	_cocoa_subdriver->Draw();
 	CSleep(1);
 
+	/* Set the proper OpenTTD palette which got spoilt by the splash
+	 * image when using 8bpp blitter */
+	GfxInitPalettes();
+	QZ_CheckPaletteAnim();
+	_cocoa_subdriver->Draw(true);
+
 	for (;;) {
 		uint32 prev_cur_ticks = cur_ticks; // to check for wrapping
 		InteractiveRandom(); // randomness
@@ -638,7 +676,7 @@ void QZ_GameLoop()
 		if (cur_ticks >= next_tick || (_fast_forward && !_pause_mode) || cur_ticks < prev_cur_ticks) {
 			_realtime_tick += cur_ticks - last_cur_ticks;
 			last_cur_ticks = cur_ticks;
-			next_tick = cur_ticks + 30;
+			next_tick = cur_ticks + MILLISECONDS_PER_TICK;
 
 			bool old_ctrl_pressed = _ctrl_pressed;
 
@@ -650,10 +688,7 @@ void QZ_GameLoop()
 			GameLoop();
 
 			UpdateWindows();
-			if (++pal_tick > 4) {
-				QZ_CheckPaletteAnim();
-				pal_tick = 1;
-			}
+			QZ_CheckPaletteAnim();
 			_cocoa_subdriver->Draw();
 		} else {
 #ifdef _DEBUG
