@@ -1,4 +1,4 @@
-/* $Id: fullscreen.mm 22992 2011-10-04 20:12:02Z rubidium $ */
+/* $Id: fullscreen.mm 26024 2013-11-17 13:35:48Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,6 +17,8 @@
 
 #include "../../stdafx.h"
 
+#if (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_9)
+
 #define Rect  OTTDRect
 #define Point OTTDPoint
 #import <Cocoa/Cocoa.h>
@@ -65,7 +67,7 @@ struct OTTD_QuartzGammaTable {
 @end
 
 @implementation NSScreen (NSScreenAccess)
-- (void) setFrame:(NSRect)frame;
+- (void) setFrame:(NSRect)frame
 {
 /* The 64 bits libraries don't seem to know about _frame, so this hack won't work. */
 #if !__LP64__
@@ -74,80 +76,6 @@ struct OTTD_QuartzGammaTable {
 }
 @end
 
-
-static int CDECL ModeSorter(const OTTD_Point *p1, const OTTD_Point *p2)
-{
-	if (p1->x < p2->x) return -1;
-	if (p1->x > p2->x) return +1;
-	if (p1->y < p2->y) return -1;
-	if (p1->y > p2->y) return +1;
-	return 0;
-}
-
-uint QZ_ListModes(OTTD_Point *modes, uint max_modes, CGDirectDisplayID display_id, int device_depth)
-{
-	CFArrayRef mode_list  = CGDisplayAvailableModes(display_id);
-	CFIndex    num_modes = CFArrayGetCount(mode_list);
-
-	/* Build list of modes with the requested bpp */
-	uint count = 0;
-	for (CFIndex i = 0; i < num_modes && count < max_modes; i++) {
-		int intvalue, bpp;
-		uint16 width, height;
-
-		CFDictionaryRef onemode = (const __CFDictionary*)CFArrayGetValueAtIndex(mode_list, i);
-		CFNumberRef number = (const __CFNumber*)CFDictionaryGetValue(onemode, kCGDisplayBitsPerPixel);
-		CFNumberGetValue(number, kCFNumberSInt32Type, &bpp);
-
-		if (bpp != device_depth) continue;
-
-		number = (const __CFNumber*)CFDictionaryGetValue(onemode, kCGDisplayWidth);
-		CFNumberGetValue(number, kCFNumberSInt32Type, &intvalue);
-		width = (uint16)intvalue;
-
-		number = (const __CFNumber*)CFDictionaryGetValue(onemode, kCGDisplayHeight);
-		CFNumberGetValue(number, kCFNumberSInt32Type, &intvalue);
-		height = (uint16)intvalue;
-
-		/* Check if mode is already in the list */
-		bool hasMode = false;
-		for (uint i = 0; i < count; i++) {
-			if (modes[i].x == width &&  modes[i].y == height) {
-				hasMode = true;
-				break;
-			}
-		}
-
-		if (hasMode) continue;
-
-		/* Add mode to the list */
-		modes[count].x = width;
-		modes[count].y = height;
-		count++;
-	}
-
-	/* Sort list smallest to largest */
-	QSortT(modes, count, &ModeSorter);
-
-	return count;
-}
-
-/** Small function to test if the main display can display 8 bpp in fullscreen */
-bool QZ_CanDisplay8bpp()
-{
-	/* 8bpp modes are deprecated starting in 10.5. CoreGraphics will return them
-	 * as available in the display list, but many features (e.g. palette animation)
-	 * will be broken. */
-	if (MacOSVersionIsAtLeast(10, 5, 0)) return false;
-
-	OTTD_Point p;
-
-	/* We want to know if 8 bpp is possible in fullscreen and not anything about
-	 * resolutions. Because of this we want to fill a list of 1 resolution of 8 bpp
-	 * on display 0 (main) and return if we found one. */
-	return QZ_ListModes(&p, 1, 0, 8);
-}
-
 class FullscreenSubdriver: public CocoaSubdriver {
 	CGDirectDisplayID  display_id;         ///< 0 == main display (only support single display)
 	CFDictionaryRef    cur_mode;           ///< current mode of the display
@@ -250,7 +178,7 @@ class FullscreenSubdriver: public CocoaSubdriver {
 	}
 
 
-	bool SetVideoMode(int w, int h)
+	bool SetVideoMode(int w, int h, int bpp)
 	{
 		/* Define this variables at the top (against coding style) because
 		 * otherwise GCC 4.2 barfs at the goto's jumping over variable initialization. */
@@ -266,14 +194,14 @@ class FullscreenSubdriver: public CocoaSubdriver {
 
 		/* See if requested mode exists */
 		boolean_t exact_match;
-		this->cur_mode = CGDisplayBestModeForParameters(this->display_id, this->device_depth, w, h, &exact_match);
+		this->cur_mode = CGDisplayBestModeForParameters(this->display_id, bpp, w, h, &exact_match);
 
 		/* If the mode wasn't an exact match, check if it has the right bpp, and update width and height */
 		if (!exact_match) {
-			int bpp;
+			int act_bpp;
 			CFNumberRef number = (const __CFNumber*) CFDictionaryGetValue(this->cur_mode, kCGDisplayBitsPerPixel);
-			CFNumberGetValue(number, kCFNumberSInt32Type, &bpp);
-			if (bpp != this->device_depth) {
+			CFNumberGetValue(number, kCFNumberSInt32Type, &act_bpp);
+			if (act_bpp != bpp) {
 				DEBUG(driver, 0, "Failed to find display resolution");
 				goto ERR_NO_MATCH;
 			}
@@ -313,7 +241,7 @@ class FullscreenSubdriver: public CocoaSubdriver {
 		 * disable until a replacement can be found. */
 		if (MacOSVersionIsAtLeast(10, 7, 0)) {
 			this->window_buffer = NULL;
-			this->window_pitch  = NULL;
+			this->window_pitch  = 0;
 		} else {
 #if (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_7)
 			this->window_buffer = CGDisplayBaseAddress(this->display_id);
@@ -323,6 +251,7 @@ class FullscreenSubdriver: public CocoaSubdriver {
 
 		this->device_width  = CGDisplayPixelsWide(this->display_id);
 		this->device_height = CGDisplayPixelsHigh(this->display_id);
+		this->device_depth  = bpp;
 
 		/* Setup double-buffer emulation */
 		this->pixel_buffer = malloc(this->device_width * this->device_height * this->device_depth / 8);
@@ -356,7 +285,7 @@ class FullscreenSubdriver: public CocoaSubdriver {
 
 		this->UpdatePalette(0, 256);
 
-		/* Move the mouse cursor to approx the same location */
+		/* Move the mouse cursor to approx. the same location */
 		CGPoint display_mouseLocation;
 		display_mouseLocation.x = mouseLocation.x * this->device_width;
 		display_mouseLocation.y = this->device_height - (mouseLocation.y * this->device_height);
@@ -418,22 +347,18 @@ ERR_NO_MATCH:
 	}
 
 public:
-	FullscreenSubdriver(int bpp)
+	FullscreenSubdriver()
 	{
-		if (bpp != 8 && bpp != 32) {
-			error("Cocoa: This video driver only supports 8 and 32 bpp blitters.");
-		}
-
 		/* Initialize the video settings; this data persists between mode switches */
 		this->display_id = kCGDirectMainDisplay;
 		this->save_mode  = CGDisplayCurrentMode(this->display_id);
 
-		if (bpp == 8) this->palette = CGPaletteCreateDefaultColorPalette();
+		this->palette = CGPaletteCreateDefaultColorPalette();
 
 		this->device_width  = CGDisplayPixelsWide(this->display_id);
 		this->device_height = CGDisplayPixelsHigh(this->display_id);
-		this->device_depth  = bpp;
-		this->pixel_buffer   = NULL;
+		this->device_depth  = 0;
+		this->pixel_buffer  = NULL;
 
 		this->num_dirty_rects = MAX_DIRTY_RECTS;
 	}
@@ -497,9 +422,9 @@ public:
 		for (uint32_t index = first_color; index < first_color + num_colors; index++) {
 			/* Clamp colors between 0.0 and 1.0 */
 			CGDeviceColor color;
-			color.red   = _cur_palette[index].r / 255.0;
-			color.blue  = _cur_palette[index].b / 255.0;
-			color.green = _cur_palette[index].g / 255.0;
+			color.red   = _cur_palette.palette[index].r / 255.0;
+			color.blue  = _cur_palette.palette[index].b / 255.0;
+			color.green = _cur_palette.palette[index].g / 255.0;
 
 			CGPaletteSetColorAtIndex(this->palette, color, index);
 		}
@@ -512,14 +437,16 @@ public:
 		return QZ_ListModes(modes, max_modes, this->display_id, this->device_depth);
 	}
 
-	virtual bool ChangeResolution(int w, int h)
+	virtual bool ChangeResolution(int w, int h, int bpp)
 	{
 		int old_width  = this->device_width;
 		int old_height = this->device_height;
+		int old_bpp    = this->device_depth;
 
-		if (SetVideoMode(w, h)) return true;
+		if (bpp != 8 && bpp != 32) error("Cocoa: This video driver only supports 8 and 32 bpp blitters.");
 
-		if (old_width != 0 && old_height != 0) SetVideoMode(old_width, old_height);
+		if (SetVideoMode(w, h, bpp)) return true;
+		if (old_width != 0 && old_height != 0) SetVideoMode(old_width, old_height, old_bpp);
 
 		return false;
 	}
@@ -583,9 +510,9 @@ CocoaSubdriver *QZ_CreateFullscreenSubdriver(int width, int height, int bpp)
 		return NULL;
 	}
 
-	FullscreenSubdriver *ret = new FullscreenSubdriver(bpp);
+	FullscreenSubdriver *ret = new FullscreenSubdriver();
 
-	if (!ret->ChangeResolution(width, height)) {
+	if (!ret->ChangeResolution(width, height, bpp)) {
 		delete ret;
 		return NULL;
 	}
@@ -593,4 +520,5 @@ CocoaSubdriver *QZ_CreateFullscreenSubdriver(int width, int height, int bpp)
 	return ret;
 }
 
+#endif /* (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_9) */
 #endif /* WITH_COCOA */
