@@ -1,4 +1,4 @@
-/* $Id: wnd_quartz.mm 22992 2011-10-04 20:12:02Z rubidium $ */
+/* $Id: wnd_quartz.mm 26024 2013-11-17 13:35:48Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -34,6 +34,13 @@
 #include "../../core/math_func.hpp"
 #include "../../gfx_func.h"
 
+/* On some old versions of MAC OS this may not be defined.
+ * Those versions generally only produce code for PPC. So it should be safe to
+ * set this to 0. */
+#ifndef kCGBitmapByteOrder32Host
+#define kCGBitmapByteOrder32Host 0
+#endif
+
 /**
  * Important notice regarding all modifications!!!!!!!
  * There are certain limitations because the file is objective C++.
@@ -63,10 +70,10 @@ private:
 	void BlitIndexedToView32(int left, int top, int right, int bottom);
 
 	virtual void GetDeviceInfo();
-	virtual bool SetVideoMode(int width, int height);
+	virtual bool SetVideoMode(int width, int height, int bpp);
 
 public:
-	WindowQuartzSubdriver(int bpp);
+	WindowQuartzSubdriver();
 	virtual ~WindowQuartzSubdriver();
 
 	virtual void Draw(bool force_update);
@@ -75,7 +82,7 @@ public:
 
 	virtual uint ListModes(OTTD_Point *modes, uint max_modes);
 
-	virtual bool ChangeResolution(int w, int h);
+	virtual bool ChangeResolution(int w, int h, int bpp);
 
 	virtual bool IsFullscreen() { return false; }
 	virtual bool ToggleFullscreen(); /* Full screen mode on OSX 10.7 */
@@ -144,7 +151,7 @@ static CGColorSpaceRef QZ_GetCorrectColorSpace()
 	/* Calculate total area we are blitting */
 	uint32 blitArea = 0;
 	for (int n = 0; n < dirtyRectCount; n++) {
-		blitArea += dirtyRects[n].size.width * dirtyRects[n].size.height;
+		blitArea += (uint32)(dirtyRects[n].size.width * dirtyRects[n].size.height);
 	}
 
 	/*
@@ -240,15 +247,15 @@ void WindowQuartzSubdriver::GetDeviceInfo()
  */
 bool WindowQuartzSubdriver::ToggleFullscreen()
 {
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7)
-	[this->window toggleFullScreen:this->window];
-	return true;
-#else
+	if ([ this->window respondsToSelector:@selector(toggleFullScreen:) ]) {
+		[ this->window performSelector:@selector(toggleFullScreen:) withObject:this->window ];
+		return true;
+	}
+
 	return false;
-#endif
 }
 
-bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
+bool WindowQuartzSubdriver::SetVideoMode(int width, int height, int bpp)
 {
 	this->setup = true;
 	this->GetDeviceInfo();
@@ -280,26 +287,27 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
 			return false;
 		}
 
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5
 		/* Add built in full-screen support when available (OS X 10.7 and higher)
 		 * This code actually compiles for 10.5 and later, but only makes sense in conjunction
 		 * with the quartz fullscreen support as found only in 10.7 and later
 		 */
-#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7)
 		if ([this->window respondsToSelector:@selector(toggleFullScreen:)]) {
-			/* Constants needed to build on pre-10.7 systems. Source: NSWindow documentation. */
+#if MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_7
+			/* Constants needed to build on pre-10.7 SDKs. Source: NSWindow documentation. */
 			const int NSWindowCollectionBehaviorFullScreenPrimary = 1 << 7;
 			const int NSWindowFullScreenButton = 7;
+#endif
 
-			NSWindowCollectionBehavior behavior = [this->window collectionBehavior];
+			NSWindowCollectionBehavior behavior = [ this->window collectionBehavior ];
 			behavior |= NSWindowCollectionBehaviorFullScreenPrimary;
-			[window setCollectionBehavior:behavior];
+			[ this->window setCollectionBehavior:behavior ];
 
-			NSButton* fullscreenButton =
-			[this->window standardWindowButton:NSWindowFullScreenButton];
-			[fullscreenButton setAction:@selector(toggleFullScreen:)];
-			[fullscreenButton setTarget:this->window];
+			NSButton* fullscreenButton = [ this->window standardWindowButton:NSWindowFullScreenButton ];
+			[ fullscreenButton setAction:@selector(toggleFullScreen:) ];
+			[ fullscreenButton setTarget:this->window ];
 
-			[this->window setCollectionBehavior: NSWindowCollectionBehaviorFullScreenPrimary];
+			[ this->window setCollectionBehavior: NSWindowCollectionBehaviorFullScreenPrimary ];
 		}
 #endif
 
@@ -327,16 +335,17 @@ bool WindowQuartzSubdriver::SetVideoMode(int width, int height)
 		[ this->window setContentSize:contentRect.size ];
 
 		/* Ensure frame height - title bar height >= view height */
-		contentRect.size.height = Clamp(height, 0, [ this->window frame ].size.height - 22 /* 22 is the height of title bar of window*/);
+		contentRect.size.height = Clamp(height, 0, (int)[ this->window frame ].size.height - 22 /* 22 is the height of title bar of window*/);
 
 		if (this->cocoaview != nil) {
-			height = contentRect.size.height;
+			height = (int)contentRect.size.height;
 			[ this->cocoaview setFrameSize:contentRect.size ];
 		}
 	}
 
 	this->window_width = width;
 	this->window_height = height;
+	this->buffer_depth = bpp;
 
 	[ this->window center ];
 
@@ -381,11 +390,11 @@ void WindowQuartzSubdriver::BlitIndexedToView32(int left, int top, int right, in
 }
 
 
-WindowQuartzSubdriver::WindowQuartzSubdriver(int bpp)
+WindowQuartzSubdriver::WindowQuartzSubdriver()
 {
 	this->window_width  = 0;
 	this->window_height = 0;
-	this->buffer_depth  = bpp;
+	this->buffer_depth  = 0;
 	this->window_buffer  = NULL;
 	this->pixel_buffer  = NULL;
 	this->active        = false;
@@ -467,9 +476,9 @@ void WindowQuartzSubdriver::UpdatePalette(uint first_color, uint num_colors)
 
 	for (uint i = first_color; i < first_color + num_colors; i++) {
 		uint32 clr = 0xff000000;
-		clr |= (uint32)_cur_palette[i].r << 16;
-		clr |= (uint32)_cur_palette[i].g << 8;
-		clr |= (uint32)_cur_palette[i].b;
+		clr |= (uint32)_cur_palette.palette[i].r << 16;
+		clr |= (uint32)_cur_palette.palette[i].g << 8;
+		clr |= (uint32)_cur_palette.palette[i].b;
 		this->palette[i] = clr;
 	}
 
@@ -481,13 +490,14 @@ uint WindowQuartzSubdriver::ListModes(OTTD_Point *modes, uint max_modes)
 	return QZ_ListModes(modes, max_modes, kCGDirectMainDisplay, this->buffer_depth);
 }
 
-bool WindowQuartzSubdriver::ChangeResolution(int w, int h)
+bool WindowQuartzSubdriver::ChangeResolution(int w, int h, int bpp)
 {
 	int old_width  = this->window_width;
 	int old_height = this->window_height;
+	int old_bpp    = this->buffer_depth;
 
-	if (this->SetVideoMode(w, h)) return true;
-	if (old_width != 0 && old_height != 0) this->SetVideoMode(old_width, old_height);
+	if (this->SetVideoMode(w, h, bpp)) return true;
+	if (old_width != 0 && old_height != 0) this->SetVideoMode(old_width, old_height, old_bpp);
 
 	return false;
 }
@@ -511,8 +521,21 @@ CGPoint WindowQuartzSubdriver::PrivateLocalToCG(NSPoint *p)
 
 NSPoint WindowQuartzSubdriver::GetMouseLocation(NSEvent *event)
 {
-	NSPoint pt = [ event locationInWindow ];
-	pt = [ this->cocoaview convertPoint:pt fromView:nil ];
+	NSPoint pt;
+
+	if ( [ event window ] == nil) {
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_7
+		if ([ this->cocoaview respondsToSelector:@selector(convertRectFromScreen:) ]) {
+			pt = [ this->cocoaview convertPoint:[ [ this->cocoaview window ] convertRectFromScreen:NSMakeRect([ event locationInWindow ].x, [ event locationInWindow ].y, 0, 0) ].origin fromView:nil ];
+		}
+		else
+#endif
+		{
+			pt = [ this->cocoaview convertPoint:[ [ this->cocoaview window ] convertScreenToBase:[ event locationInWindow ] ] fromView:nil ];
+		}
+	} else {
+		pt = [ event locationInWindow ];
+	}
 
 	pt.y = this->window_height - pt.y;
 
@@ -546,8 +569,8 @@ bool WindowQuartzSubdriver::WindowResized()
 
 	NSRect newframe = [ this->cocoaview frame ];
 
-	this->window_width = newframe.size.width;
-	this->window_height = newframe.size.height;
+	this->window_width = (int)newframe.size.width;
+	this->window_height = (int)newframe.size.height;
 
 	/* Create Core Graphics Context */
 	free(this->window_buffer);
@@ -599,9 +622,9 @@ CocoaSubdriver *QZ_CreateWindowQuartzSubdriver(int width, int height, int bpp)
 		return NULL;
 	}
 
-	WindowQuartzSubdriver *ret = new WindowQuartzSubdriver(bpp);
+	WindowQuartzSubdriver *ret = new WindowQuartzSubdriver();
 
-	if (!ret->ChangeResolution(width, height)) {
+	if (!ret->ChangeResolution(width, height, bpp)) {
 		delete ret;
 		return NULL;
 	}
