@@ -1,4 +1,4 @@
-/* $Id: dedicated_v.cpp 19603 2010-04-11 10:57:21Z rubidium $ */
+/* $Id: dedicated_v.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -17,13 +17,13 @@
 #include "../network/network.h"
 #include "../network/network_internal.h"
 #include "../console_func.h"
-#include "../variables.h"
 #include "../genworld.h"
 #include "../fileio_type.h"
 #include "../fios.h"
 #include "../blitter/factory.hpp"
 #include "../company_func.h"
 #include "../core/random_func.hpp"
+#include "../saveload/saveload.h"
 #include "dedicated_v.h"
 
 #ifdef BEOS_NET_SERVER
@@ -43,7 +43,8 @@
 
 /**
  * Switches OpenTTD to a console app at run-time, instead of a PM app
- * Necessary to see stdout, etc. */
+ * Necessary to see stdout, etc.
+ */
 static void OS2_SwitchToConsoleMode()
 {
 	PPIB pib;
@@ -70,6 +71,7 @@ static void OS2_SwitchToConsoleMode()
 /* Signal handlers */
 static void DedicatedSignalHandler(int sig)
 {
+	if (_game_mode == GM_NORMAL && _settings_client.gui.autosave_on_exit) DoExitSave();
 	_exit_game = true;
 	signal(sig, DedicatedSignalHandler);
 }
@@ -95,7 +97,7 @@ static void WINAPI CheckForConsoleInput()
 #else
 	DWORD nb;
 	HANDLE hStdin = GetStdHandle(STD_INPUT_HANDLE);
-	while (true) {
+	for (;;) {
 		ReadFile(hStdin, _win_console_thread_buffer, lengthof(_win_console_thread_buffer), &nb, NULL);
 		/* Signal input waiting that input is read and wait for it being handled
 		 * SignalObjectAndWait() should be used here, but it's unsupported in Win98< */
@@ -132,8 +134,10 @@ static void CloseWindowsConsoleThread()
 
 static void *_dedicated_video_mem;
 
-extern bool SafeSaveOrLoad(const char *filename, int mode, GameMode newgm, Subdirectory subdir);
-extern void SwitchToMode(SwitchMode new_mode);
+/* Whether a fork has been done. */
+bool _dedicated_forks;
+
+extern bool SafeLoad(const char *filename, int mode, GameMode newgm, Subdirectory subdir, struct LoadFilter *lf = NULL);
 
 static FVideoDriver_Dedicated iFVideoDriver_Dedicated;
 
@@ -158,6 +162,11 @@ const char *VideoDriver_Dedicated::Start(const char * const *parm)
 	SetConsoleTitle(_T("OpenTTD Dedicated Server"));
 #endif
 
+#ifdef _MSC_VER
+	/* Disable the MSVC assertion message box. */
+	_set_error_mode(_OUT_TO_STDERR);
+#endif
+
 #ifdef __OS2__
 	/* For OS/2 we also need to switch to console mode instead of PM mode */
 	OS2_SwitchToConsoleMode();
@@ -228,7 +237,7 @@ static void DedicatedHandleKeyInput()
 #if defined(UNIX) || defined(__OS2__) || defined(PSP)
 	if (fgets(input_line, lengthof(input_line), stdin) == NULL) return;
 #else
-	/* Handle console input, and singal console thread, it can accept input again */
+	/* Handle console input, and signal console thread, it can accept input again */
 	assert_compile(lengthof(_win_console_thread_buffer) <= lengthof(input_line));
 	strecpy(input_line, _win_console_thread_buffer, lastof(input_line));
 	SetEvent(_hWaitForInputHandling);
@@ -251,7 +260,7 @@ static void DedicatedHandleKeyInput()
 void VideoDriver_Dedicated::MainLoop()
 {
 	uint32 cur_ticks = GetTime();
-	uint32 next_tick = cur_ticks + 30;
+	uint32 next_tick = cur_ticks + MILLISECONDS_PER_TICK;
 
 	/* Signal handlers */
 #if defined(UNIX) || defined(PSP)
@@ -263,10 +272,10 @@ void VideoDriver_Dedicated::MainLoop()
 	/* Load the dedicated server stuff */
 	_is_network_server = true;
 	_network_dedicated = true;
-	_local_company = COMPANY_SPECTATOR;
+	_current_company = _local_company = COMPANY_SPECTATOR;
 
-	/* If SwitchMode is SM_LOAD, it means that the user used the '-g' options */
-	if (_switch_mode != SM_LOAD) {
+	/* If SwitchMode is SM_LOAD_GAME, it means that the user used the '-g' options */
+	if (_switch_mode != SM_LOAD_GAME) {
 		StartNewGameWithoutGUI(GENERATE_NEW_SEED);
 		SwitchToMode(_switch_mode);
 		_switch_mode = SM_NONE;
@@ -274,13 +283,13 @@ void VideoDriver_Dedicated::MainLoop()
 		_switch_mode = SM_NONE;
 		/* First we need to test if the savegame can be loaded, else we will end up playing the
 		 *  intro game... */
-		if (!SafeSaveOrLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_NORMAL, BASE_DIR)) {
+		if (!SafeLoad(_file_to_saveload.name, _file_to_saveload.mode, GM_NORMAL, BASE_DIR)) {
 			/* Loading failed, pop out.. */
 			DEBUG(net, 0, "Loading requested map failed, aborting");
 			_networking = false;
 		} else {
 			/* We can load this game, so go ahead */
-			SwitchToMode(SM_LOAD);
+			SwitchToMode(SM_LOAD_GAME);
 		}
 	}
 
@@ -295,13 +304,12 @@ void VideoDriver_Dedicated::MainLoop()
 		uint32 prev_cur_ticks = cur_ticks; // to check for wrapping
 		InteractiveRandom(); // randomness
 
-		if (!_dedicated_forks)
-			DedicatedHandleKeyInput();
+		if (!_dedicated_forks) DedicatedHandleKeyInput();
 
 		cur_ticks = GetTime();
 		_realtime_tick += cur_ticks - prev_cur_ticks;
 		if (cur_ticks >= next_tick || cur_ticks < prev_cur_ticks || _ddc_fastforward) {
-			next_tick = cur_ticks + 30;
+			next_tick = cur_ticks + MILLISECONDS_PER_TICK;
 
 			GameLoop();
 			UpdateWindows();
