@@ -1,4 +1,4 @@
-/* $Id: dropdown.cpp 19927 2010-06-04 21:10:18Z rubidium $ */
+/* $Id: dropdown.cpp 24900 2013-01-08 22:46:42Z planetmaker $ */
 
 /*
  * This file is part of OpenTTD.
@@ -11,10 +11,13 @@
 
 #include "../stdafx.h"
 #include "../window_gui.h"
+#include "../string_func.h"
 #include "../strings_func.h"
 #include "../window_func.h"
 #include "dropdown_type.h"
 
+#include "dropdown_widget.h"
+
 
 void DropDownListItem::Draw(int left, int right, int top, int bottom, bool sel, int bg_colour) const
 {
@@ -38,20 +41,31 @@ void DropDownListStringItem::Draw(int left, int right, int top, int bottom, bool
 	DrawString(left + WD_FRAMERECT_LEFT, right - WD_FRAMERECT_RIGHT, top, this->String(), sel ? TC_WHITE : TC_BLACK);
 }
 
+/**
+ * Natural sorting comparator function for DropDownList::sort().
+ * @param first Left side of comparison.
+ * @param second Right side of comparison.
+ * @return true if \a first precedes \a second.
+ * @warning All items in the list need to be derivates of DropDownListStringItem.
+ */
+/* static */ bool DropDownListStringItem::NatSortFunc(const DropDownListItem *first, const DropDownListItem *second)
+{
+	char buffer1[512], buffer2[512];
+	GetString(buffer1, static_cast<const DropDownListStringItem*>(first)->String(), lastof(buffer1));
+	GetString(buffer2, static_cast<const DropDownListStringItem*>(second)->String(), lastof(buffer2));
+	return strnatcmp(buffer1, buffer2) < 0;
+}
+
 StringID DropDownListParamStringItem::String() const
 {
 	for (uint i = 0; i < lengthof(this->decode_params); i++) SetDParam(i, this->decode_params[i]);
 	return this->string;
 }
 
-uint DropDownListCharStringItem::Width() const
+StringID DropDownListCharStringItem::String() const
 {
-	return GetStringBoundingBox(this->string).width;
-}
-
-void DropDownListCharStringItem::Draw(int left, int right, int top, int bottom, bool sel, int bg_colour) const
-{
-	DrawString(left + WD_FRAMERECT_LEFT, right - WD_FRAMERECT_RIGHT, top, this->string, sel ? TC_WHITE : TC_BLACK);
+	SetDParamStr(0, this->raw_string);
+	return this->string;
 }
 
 /**
@@ -67,16 +81,12 @@ static void DeleteDropDownList(DropDownList *list)
 	delete list;
 }
 
-/** Widget numbers of the dropdown menu. */
-enum DropdownMenuWidgets {
-	DDM_ITEMS,  ///< Panel showing the dropdown items.
-	DDM_SCROLL, ///< Scrollbar.
-};
-
 static const NWidgetPart _nested_dropdown_menu_widgets[] = {
 	NWidget(NWID_HORIZONTAL),
-		NWidget(WWT_PANEL, COLOUR_END, DDM_ITEMS), SetMinimalSize(1, 1), EndContainer(),
-		NWidget(WWT_SCROLLBAR, COLOUR_END, DDM_SCROLL),
+		NWidget(WWT_PANEL, COLOUR_END, WID_DM_ITEMS), SetMinimalSize(1, 1), SetScrollbar(WID_DM_SCROLL), EndContainer(),
+		NWidget(NWID_SELECTION, INVALID_COLOUR, WID_DM_SHOW_SCROLL),
+			NWidget(NWID_VSCROLLBAR, COLOUR_END, WID_DM_SCROLL),
+		EndContainer(),
 	EndContainer(),
 };
 
@@ -91,7 +101,7 @@ const WindowDesc _dropdown_desc(
 struct DropdownWindow : Window {
 	WindowClass parent_wnd_class; ///< Parent window class.
 	WindowNumber parent_wnd_num;  ///< Parent window number.
-	byte parent_button;           ///< Parent widget number where the window is dropped from.
+	int parent_button;            ///< Parent widget number where the window is dropped from.
 	DropDownList *list;           ///< List with dropdown menu items.
 	int selected_index;           ///< Index of the selected item in the list.
 	byte click_delay;             ///< Timer to delay selection.
@@ -99,8 +109,10 @@ struct DropdownWindow : Window {
 	bool instant_close;           ///< Close the window when the mouse button is raised.
 	int scrolling;                ///< If non-zero, auto-scroll the item list (one time).
 	Point position;               ///< Position of the topleft corner of the window.
+	Scrollbar *vscroll;
 
-	/** Create a dropdown menu.
+	/**
+	 * Create a dropdown menu.
 	 * @param parent        Parent window.
 	 * @param list          Dropdown item list.
 	 * @param selected      Index of the selected item in the list.
@@ -118,20 +130,20 @@ struct DropdownWindow : Window {
 
 		this->CreateNestedTree(&_dropdown_desc);
 
-		uint items_width = size.width - (scroll ? WD_VSCROLLBAR_WIDTH : 0);
-		NWidgetCore *nwi = this->GetWidget<NWidgetCore>(DDM_ITEMS);
+		this->vscroll = this->GetScrollbar(WID_DM_SCROLL);
+
+		uint items_width = size.width - (scroll ? NWidgetScrollbar::GetVerticalDimension().width : 0);
+		NWidgetCore *nwi = this->GetWidget<NWidgetCore>(WID_DM_ITEMS);
 		nwi->SetMinimalSize(items_width, size.height + 4);
 		nwi->colour = wi_colour;
 
-		nwi = this->GetWidget<NWidgetCore>(DDM_SCROLL);
-		if (scroll) {
-			nwi->colour = wi_colour;
-		} else {
-			nwi->min_x = 0; // Make scrollbar invisible.
-		}
+		nwi = this->GetWidget<NWidgetCore>(WID_DM_SCROLL);
+		nwi->colour = wi_colour;
+
+		this->GetWidget<NWidgetStacked>(WID_DM_SHOW_SCROLL)->SetDisplayedPlane(scroll ? 0 : SZSP_NONE);
 
 		this->FinishInitNested(&_dropdown_desc, 0);
-		this->flags4 &= ~WF_WHITE_BORDER_MASK;
+		CLRBITS(this->flags, WF_WHITE_BORDER);
 
 		/* Total length of list */
 		int list_height = 0;
@@ -141,8 +153,8 @@ struct DropdownWindow : Window {
 		}
 
 		/* Capacity is the average number of items visible */
-		this->vscroll.SetCapacity(size.height * (uint16)list->size() / list_height);
-		this->vscroll.SetCount((uint16)list->size());
+		this->vscroll->SetCapacity(size.height * (uint16)list->size() / list_height);
+		this->vscroll->SetCount((uint16)list->size());
 
 		this->parent_wnd_class = parent->window_class;
 		this->parent_wnd_num   = parent->window_number;
@@ -156,21 +168,18 @@ struct DropdownWindow : Window {
 
 	~DropdownWindow()
 	{
+		/* Make the dropdown "invisible", so it doesn't affect new window placement.
+		 * Also mark it dirty in case the callback deals with the screen. (e.g. screenshots). */
+		this->window_class = WC_INVALID;
+		this->SetDirty();
+
 		Window *w2 = FindWindowById(this->parent_wnd_class, this->parent_wnd_num);
 		if (w2 != NULL) {
-			if (w2->nested_array != NULL) {
-				NWidgetCore *nwi2 = w2->GetWidget<NWidgetCore>(this->parent_button);
-				if (nwi2->type == NWID_BUTTON_DROPDOWN) {
-					nwi2->disp_flags &= ~ND_DROPDOWN_ACTIVE;
-				} else {
-					w2->RaiseWidget(this->parent_button);
-				}
-			} else {
-				w2->RaiseWidget(this->parent_button);
-			}
-			w2->SetWidgetDirty(this->parent_button);
+			Point pt = _cursor.pos;
+			pt.x -= w2->left;
+			pt.y -= w2->top;
+			w2->OnDropdownClose(pt, this->parent_button, this->selected_index, this->instant_close);
 		}
-
 		DeleteDropDownList(this->list);
 	}
 
@@ -179,7 +188,8 @@ struct DropdownWindow : Window {
 		return this->position;
 	}
 
-	/** Find the dropdown item under the cursor.
+	/**
+	 * Find the dropdown item under the cursor.
 	 * @param value [out] Selected item, if function returns \c true.
 	 * @return Cursor points to a dropdown item.
 	 */
@@ -187,10 +197,10 @@ struct DropdownWindow : Window {
 	{
 		if (GetWidgetFromPos(this, _cursor.pos.x - this->left, _cursor.pos.y - this->top) < 0) return false;
 
-		NWidgetBase *nwi = this->GetWidget<NWidgetBase>(DDM_ITEMS);
+		NWidgetBase *nwi = this->GetWidget<NWidgetBase>(WID_DM_ITEMS);
 		int y     = _cursor.pos.y - this->top - nwi->pos_y - 2;
 		int width = nwi->current_x - 4;
-		int pos   = this->vscroll.GetPosition();
+		int pos   = this->vscroll->GetPosition();
 
 		const DropDownList *list = this->list;
 
@@ -213,19 +223,14 @@ struct DropdownWindow : Window {
 		return false;
 	}
 
-	virtual void OnPaint()
-	{
-		this->DrawWidgets();
-	}
-
 	virtual void DrawWidget(const Rect &r, int widget) const
 	{
-		if (widget != DDM_ITEMS) return;
+		if (widget != WID_DM_ITEMS) return;
 
 		TextColour colour = (TextColour)this->GetWidget<NWidgetCore>(widget)->colour;
 
 		int y = r.top + 2;
-		int pos = this->vscroll.GetPosition();
+		int pos = this->vscroll->GetPosition();
 		for (DropDownList::const_iterator it = this->list->begin(); it != this->list->end(); ++it) {
 			const DropDownListItem *item = *it;
 			int item_height = item->Height(r.right - r.left + 1);
@@ -235,9 +240,9 @@ struct DropdownWindow : Window {
 
 			if (y + item_height < r.bottom) {
 				bool selected = (this->selected_index == item->result);
-				if (selected) GfxFillRect(r.left + 2, y, r.right - 1, y + item_height - 1, 0);
+				if (selected) GfxFillRect(r.left + 2, y, r.right - 1, y + item_height - 1, PC_BLACK);
 
-				item->Draw(r.left, r.right, y, r.bottom, selected, colour);
+				item->Draw(r.left, r.right, y, y + item_height, selected, colour);
 
 				if (item->masked) {
 					GfxFillRect(r.left + 1, y, r.right - 1, y + item_height - 1, _colour_gradient[colour][5], FILLRECT_CHECKER);
@@ -249,7 +254,7 @@ struct DropdownWindow : Window {
 
 	virtual void OnClick(Point pt, int widget, int click_count)
 	{
-		if (widget != DDM_ITEMS) return;
+		if (widget != WID_DM_ITEMS) return;
 		int item;
 		if (this->GetDropDownItem(item)) {
 			this->click_delay = 4;
@@ -261,12 +266,12 @@ struct DropdownWindow : Window {
 	virtual void OnTick()
 	{
 		if (this->scrolling != 0) {
-			int pos = this->vscroll.GetPosition();
+			int pos = this->vscroll->GetPosition();
 
-			this->vscroll.UpdatePosition(this->scrolling);
+			this->vscroll->UpdatePosition(this->scrolling);
 			this->scrolling = 0;
 
-			if (pos != this->vscroll.GetPosition()) {
+			if (pos != this->vscroll->GetPosition()) {
 				this->SetDirty();
 			}
 		}
@@ -281,8 +286,10 @@ struct DropdownWindow : Window {
 		}
 
 		if (this->click_delay != 0 && --this->click_delay == 0) {
-			/* Make the dropdown "invisible", so it doesn't affect new window placement. */
+			/* Make the dropdown "invisible", so it doesn't affect new window placement.
+			 * Also mark it dirty in case the callback deals with the screen. (e.g. screenshots). */
 			this->window_class = WC_INVALID;
+			this->SetDirty();
 
 			w2->OnDropdownSelect(this->parent_button, this->selected_index);
 			delete this;
@@ -295,17 +302,7 @@ struct DropdownWindow : Window {
 			if (!_left_button_clicked) {
 				this->drag_mode = false;
 				if (!this->GetDropDownItem(item)) {
-					if (this->instant_close) {
-						/* Make the dropdown "invisible", so it doesn't affect new window placement. */
-						this->window_class = WC_INVALID;
-
-						if (GetWidgetFromPos(w2, _cursor.pos.x - w2->left, _cursor.pos.y - w2->top) == this->parent_button) {
-							/* Send event for selected option if we're still
-							 * on the parent button of the list. */
-							w2->OnDropdownSelect(this->parent_button, this->selected_index);
-						}
-						delete this;
-					}
+					if (this->instant_close) delete this;
 					return;
 				}
 				this->click_delay = 2;
@@ -331,32 +328,29 @@ struct DropdownWindow : Window {
 	}
 };
 
-void ShowDropDownList(Window *w, DropDownList *list, int selected, int button, uint width, bool auto_width, bool instant_close)
+/**
+ * Show a drop down list.
+ * @param w        Parent window for the list.
+ * @param list     Prepopulated DropDownList. Will be deleted when the list is
+ *                 closed.
+ * @param selected The initially selected list item.
+ * @param button   The widget which is passed to Window::OnDropdownSelect and OnDropdownClose.
+ *                 Unless you override those functions, this should be then widget index of the dropdown button.
+ * @param wi_rect  Coord of the parent drop down button, used to position the dropdown menu.
+ * @param auto_width The width is determined by the widest item in the list,
+ *                   in this case only one of \a left or \a right is used (depending on text direction).
+ * @param instant_close Set to true if releasing mouse button should close the
+ *                      list regardless of where the cursor is.
+ */
+void ShowDropDownListAt(Window *w, DropDownList *list, int selected, int button, Rect wi_rect, Colours wi_colour, bool auto_width, bool instant_close)
 {
 	DeleteWindowById(WC_DROPDOWN_MENU, 0);
 
-	/* Our parent's button widget is used to determine where to place the drop
-	 * down list window. */
-	Rect wi_rect;
-	Colours wi_colour;
-	NWidgetCore *nwi = w->GetWidget<NWidgetCore>(button);
-	wi_rect.left   = nwi->pos_x;
-	wi_rect.right  = nwi->pos_x + nwi->current_x - 1;
-	wi_rect.top    = nwi->pos_y;
-	wi_rect.bottom = nwi->pos_y + nwi->current_y - 1;
-	wi_colour = nwi->colour;
-
-	if (nwi->type == NWID_BUTTON_DROPDOWN) {
-		nwi->disp_flags |= ND_DROPDOWN_ACTIVE;
-	} else {
-		w->LowerWidget(button);
-	}
-	w->SetWidgetDirty(button);
-
 	/* The preferred position is just below the dropdown calling widget */
 	int top = w->top + wi_rect.bottom + 1;
 
-	if (width == 0) width = wi_rect.right - wi_rect.left + 1;
+	/* The preferred width equals the calling widget */
+	uint width = wi_rect.right - wi_rect.left + 1;
 
 	uint max_item_width = 0;
 
@@ -397,26 +391,70 @@ void ShowDropDownList(Window *w, DropDownList *list, int selected, int button, u
 			scroll = true;
 			/* Add space for the scroll bar if we automatically determined
 			 * the width of the list. */
-			max_item_width += WD_VSCROLLBAR_WIDTH;
+			max_item_width += NWidgetScrollbar::GetVerticalDimension().width;
 		}
 	}
 
 	if (auto_width) width = max(width, max_item_width);
 
-	Point dw_pos = { w->left + (_dynlang.text_dir == TD_RTL ? wi_rect.right + 1 - width : wi_rect.left), top};
+	Point dw_pos = { w->left + (_current_text_dir == TD_RTL ? wi_rect.right + 1 - width : wi_rect.left), top};
 	Dimension dw_size = {width, height};
 	new DropdownWindow(w, list, selected, button, instant_close, dw_pos, dw_size, wi_colour, scroll);
 }
 
-/** Show a dropdown menu window near a widget of the parent window.
- * The result code of the items is their index in the #strings list.
+/**
+ * Show a drop down list.
+ * @param w        Parent window for the list.
+ * @param list     Prepopulated DropDownList. Will be deleted when the list is
+ *                 closed.
+ * @param selected The initially selected list item.
+ * @param button   The widget within the parent window that is used to determine
+ *                 the list's location.
+ * @param width    Override the width determined by the selected widget.
+ * @param auto_width Maximum width is determined by the widest item in the list.
+ * @param instant_close Set to true if releasing mouse button should close the
+ *                      list regardless of where the cursor is.
+ */
+void ShowDropDownList(Window *w, DropDownList *list, int selected, int button, uint width, bool auto_width, bool instant_close)
+{
+	/* Our parent's button widget is used to determine where to place the drop
+	 * down list window. */
+	Rect wi_rect;
+	NWidgetCore *nwi = w->GetWidget<NWidgetCore>(button);
+	wi_rect.left   = nwi->pos_x;
+	wi_rect.right  = nwi->pos_x + nwi->current_x - 1;
+	wi_rect.top    = nwi->pos_y;
+	wi_rect.bottom = nwi->pos_y + nwi->current_y - 1;
+	Colours wi_colour = nwi->colour;
+
+	if ((nwi->type & WWT_MASK) == NWID_BUTTON_DROPDOWN) {
+		nwi->disp_flags |= ND_DROPDOWN_ACTIVE;
+	} else {
+		w->LowerWidget(button);
+	}
+	w->SetWidgetDirty(button);
+
+	if (width != 0) {
+		if (_current_text_dir == TD_RTL) {
+			wi_rect.left = wi_rect.right + 1 - width;
+		} else {
+			wi_rect.right = wi_rect.left + width - 1;
+		}
+	}
+
+	ShowDropDownListAt(w, list, selected, button, wi_rect, wi_colour, auto_width, instant_close);
+}
+
+/**
+ * Show a dropdown menu window near a widget of the parent window.
+ * The result code of the items is their index in the \a strings list.
  * @param w             Parent window that wants the dropdown menu.
  * @param strings       Menu list, end with #INVALID_STRING_ID
  * @param selected      Index of initial selected item.
- * @param button        Button widget number of the parent window #w that wants the dropdown menu.
- * @param disabled_mask Bitmask for diabled items (items with their bit set are not copied to the dropdown list).
+ * @param button        Button widget number of the parent window \a w that wants the dropdown menu.
+ * @param disabled_mask Bitmask for disabled items (items with their bit set are not copied to the dropdown list).
  * @param hidden_mask   Bitmask for hidden items (items with their bit set are displayed, but not selectable in the dropdown list).
- * @param width         Width of the dropdown menu. If \c 0, use the width of parent widget #button.
+ * @param width         Width of the dropdown menu. If \c 0, use the width of parent widget \a button.
  */
 void ShowDropDownMenu(Window *w, const StringID *strings, int selected, int button, uint32 disabled_mask, uint32 hidden_mask, uint width)
 {
