@@ -1,4 +1,4 @@
-/* $Id: window_func.h 22252 2011-03-14 19:03:17Z rubidium $ */
+/* $Id: window_func.h 26024 2013-11-17 13:35:48Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -7,19 +7,20 @@
  * See the GNU General Public License for more details. You should have received a copy of the GNU General Public License along with OpenTTD. If not, see <http://www.gnu.org/licenses/>.
  */
 
-/** @file window_func.h Window functions not directly related to making/drawing windows. */
+/** @file window_func.h %Window functions not directly related to making/drawing windows. */
 
 #ifndef WINDOW_FUNC_H
 #define WINDOW_FUNC_H
 
 #include "window_type.h"
 #include "company_type.h"
+#include "core/geometry_type.hpp"
 
 Window *FindWindowById(WindowClass cls, WindowNumber number);
 Window *FindWindowByClass(WindowClass cls);
 void ChangeWindowOwner(Owner old_owner, Owner new_owner);
 
-void ResizeWindow(Window *w, int x, int y);
+void ResizeWindow(Window *w, int x, int y, bool clamp_to_screen = true);
 int PositionMainToolbar(Window *w);
 int PositionStatusbar(Window *w);
 int PositionNewsMessage(Window *w);
@@ -52,4 +53,7 @@ void SetWindowClassesDirty(WindowClass cls);
 void DeleteWindowById(WindowClass cls, WindowNumber number, bool force = true);
 void DeleteWindowByClass(WindowClass cls);
 
+bool EditBoxInGlobalFocus();
+Point GetCaretPosition();
+
 #endif /* WINDOW_FUNC_H */
