@@ -1,4 +1,4 @@
-/* $Id: window_gui.h 22252 2011-03-14 19:03:17Z rubidium $ */
+/* $Id: window_gui.h 26024 2013-11-17 13:35:48Z rubidium $ */
 
 /*
  * This file is part of OpenTTD.
@@ -18,6 +18,8 @@
 #include "tile_type.h"
 #include "widget_type.h"
 #include "core/smallvec_type.hpp"
+#include "core/smallmap_type.hpp"
+#include "string_type.h"
 
 /** State of handling an event. */
 enum EventState {
@@ -51,9 +53,16 @@ enum WidgetDrawDistances {
 	WD_INSET_RIGHT = 2,         ///< Right offset of string.
 	WD_INSET_TOP   = 1,         ///< Top offset of string.
 
-	WD_VSCROLLBAR_WIDTH  = 12,  ///< Width of a vertical scrollbar.
+	WD_SCROLLBAR_LEFT   = 2,    ///< Left offset of scrollbar.
+	WD_SCROLLBAR_RIGHT  = 2,    ///< Right offset of scrollbar.
+	WD_SCROLLBAR_TOP    = 2,    ///< Top offset of scrollbar.
+	WD_SCROLLBAR_BOTTOM = 2,    ///< Bottom offset of scrollbar.
 
-	WD_HSCROLLBAR_HEIGHT = 12,  ///< Height of a horizontal scrollbar.
+	/* Size of the pure frame bevel without any padding. */
+	WD_BEVEL_LEFT       = 1,    ///< Width of left bevel border.
+	WD_BEVEL_RIGHT      = 1,    ///< Width of right bevel border.
+	WD_BEVEL_TOP        = 1,    ///< Height of top bevel border.
+	WD_BEVEL_BOTTOM     = 1,    ///< Height of bottom bevel border.
 
 	/* FrameRect widgets, all text buttons, panel, editbox */
 	WD_FRAMERECT_LEFT   = 2,    ///< Offset at left to draw the frame rectangular area
@@ -122,7 +131,7 @@ enum WidgetDrawDistances {
 	/* Dropdown widget. */
 	WD_DROPDOWN_HEIGHT     = 12, ///< Height of a drop down widget.
 	WD_DROPDOWNTEXT_LEFT   = 2,  ///< Left offset of the dropdown widget string.
-	WD_DROPDOWNTEXT_RIGHT  = 14, ///< Right offset of the dropdown widget string.
+	WD_DROPDOWNTEXT_RIGHT  = 2,  ///< Right offset of the dropdown widget string.
 	WD_DROPDOWNTEXT_TOP    = 1,  ///< Top offset of the dropdown widget string.
 	WD_DROPDOWNTEXT_BOTTOM = 1,  ///< Bottom offset of the dropdown widget string.
 
@@ -163,12 +172,12 @@ struct WindowDesc : ZeroedMemoryAllocator {
 
 	~WindowDesc();
 
-	WindowPosition default_pos;    ///< Prefered position of the window. @see WindowPosition()
-	int16 default_width;           ///< Prefered initial width of the window.
-	int16 default_height;          ///< Prefered initial height of the window.
+	WindowPosition default_pos;    ///< Preferred position of the window. @see WindowPosition()
+	int16 default_width;           ///< Preferred initial width of the window.
+	int16 default_height;          ///< Preferred initial height of the window.
 	WindowClass cls;               ///< Class of the window, @see WindowClass.
 	WindowClass parent_cls;        ///< Class of the parent window. @see WindowClass
-	uint32 flags;                  ///< Flags. @see WindowDefaultFlags
+	uint32 flags;                  ///< Flags. @see WindowDefaultFlag
 	const NWidgetPart *nwid_parts; ///< Nested widget parts describing the window.
 	int16 nwid_length;             ///< Length of the #nwid_parts array.
 };
@@ -178,9 +187,8 @@ struct WindowDesc : ZeroedMemoryAllocator {
  */
 enum WindowDefaultFlag {
 	WDF_CONSTRUCTION    =   1 << 0, ///< This window is used for construction; close it whenever changing company.
-	WDF_UNCLICK_BUTTONS =   1 << 1, ///< Unclick buttons when the window event times out
-	WDF_MODAL           =   1 << 2, ///< The window is a modal child of some other window, meaning the parent is 'inactive'
-	WDF_NO_FOCUS        =   1 << 3, ///< This window won't get focus/make any other window lose focus when click
+	WDF_MODAL           =   1 << 1, ///< The window is a modal child of some other window, meaning the parent is 'inactive'
+	WDF_NO_FOCUS        =   1 << 2, ///< This window won't get focus/make any other window lose focus when click
 };
 
 /**
@@ -198,6 +206,27 @@ enum SortButtonState {
 	SBS_UP,   ///< Sort descending.
 };
 
+/**
+ * Window flags.
+ */
+enum WindowFlags {
+	WF_TIMEOUT           = 1 <<  0, ///< Window timeout counter.
+
+	WF_DRAGGING          = 1 <<  3, ///< Window is being dragged.
+	WF_SIZING_RIGHT      = 1 <<  4, ///< Window is being resized towards the right.
+	WF_SIZING_LEFT       = 1 <<  5, ///< Window is being resized towards the left.
+	WF_SIZING            = WF_SIZING_RIGHT | WF_SIZING_LEFT, ///< Window is being resized.
+	WF_STICKY            = 1 <<  6, ///< Window is made sticky by user
+	WF_DISABLE_VP_SCROLL = 1 <<  7, ///< Window does not do autoscroll, @see HandleAutoscroll().
+	WF_WHITE_BORDER      = 1 <<  8, ///< Window white border counter bit mask.
+	WF_HIGHLIGHTED       = 1 <<  9, ///< Window has a widget that has a highlight.
+	WF_CENTERED          = 1 << 10, ///< Window is centered and shall stay centered after ReInit.
+};
+DECLARE_ENUM_AS_BIT_SET(WindowFlags)
+
+static const int TIMEOUT_DURATION = 7; ///< The initial timeout value for WF_TIMEOUT.
+static const int WHITE_BORDER_DURATION = 3; ///< The initial timeout value for WF_WHITE_BORDER.
+
 /**
  * Data structure for a window viewport.
  * A viewport is either following a vehicle (its id in then in #follow_vehicle), or it aims to display a specific
@@ -213,6 +242,8 @@ struct ViewportData : ViewPort {
 	int32 dest_scrollpos_y;   ///< Current destination y coordinate to display (virtual screen coordinate of topleft corner of the viewport).
 };
 
+struct QueryString;
+
 /**
  * Data structure for an opened window
  */
@@ -235,7 +266,7 @@ public:
 	 * to destruct them all at the same time too, which is kinda hard.
 	 * @param size the amount of space not to allocate
 	 */
-	FORCEINLINE void *operator new[](size_t size)
+	inline void *operator new[](size_t size)
 	{
 		NOT_REACHED();
 	}
@@ -245,14 +276,17 @@ public:
 	 * Don't free the window directly; it corrupts the linked list when iterating
 	 * @param ptr the pointer not to free
 	 */
-	FORCEINLINE void operator delete(void *ptr)
+	inline void operator delete(void *ptr)
 	{
 	}
 
-	uint16 flags4;              ///< Window flags, @see WindowFlags
+	WindowFlags flags;          ///< Window flags
 	WindowClass window_class;   ///< Window class
 	WindowNumber window_number; ///< Window number within the window class
 
+	uint8 timeout_timer;      ///< Timer value of the WF_TIMEOUT for flags.
+	uint8 white_border_timer; ///< Timer value of the WF_WHITE_BORDER for flags.
+
 	int left;   ///< x position of left edge of the window
 	int top;    ///< y position of top edge of the window
 	int width;  ///< width of the window (number of pixels to the right in x direction)
@@ -265,13 +299,14 @@ public:
 	ViewportData *viewport;          ///< Pointer to viewport data, if present.
 	uint32 desc_flags;               ///< Window/widgets default flags setting. @see WindowDefaultFlag
 	const NWidgetCore *nested_focus; ///< Currently focused nested widget, or \c NULL if no nested widget has focus.
+	SmallMap<int, QueryString*> querystrings; ///< QueryString associated to WWT_EDITBOX widgets.
 	NWidgetBase *nested_root;        ///< Root of the nested tree.
 	NWidgetBase **nested_array;      ///< Array of pointers into the tree. Do not access directly, use #Window::GetWidget() instead.
 	uint nested_array_size;          ///< Size of the nested array.
 	NWidgetStacked *shade_select;    ///< Selection widget (#NWID_SELECTION) to use for shading the window. If \c NULL, window cannot shade.
 	Dimension unshaded_size;         ///< Last known unshaded size (only valid while shaded).
 
-	int scrolling_scrollbar;         ///< Widgetindex of just being dragged scrollbar. -1 of none is active.
+	int scrolling_scrollbar;         ///< Widgetindex of just being dragged scrollbar. -1 if none is active.
 
 	Window *parent;                  ///< Parent window.
 	Window *z_front;                 ///< The window in front of us in z-order.
@@ -285,10 +320,42 @@ public:
 	const Scrollbar *GetScrollbar(uint widnum) const;
 	Scrollbar *GetScrollbar(uint widnum);
 
+	const QueryString *GetQueryString(uint widnum) const;
+	QueryString *GetQueryString(uint widnum);
+
+	virtual const char *GetFocusedText() const;
+	virtual const char *GetCaret() const;
+	virtual const char *GetMarkedText(size_t *length) const;
+	virtual Point GetCaretPosition() const;
+	virtual Rect GetTextBoundingRect(const char *from, const char *to) const;
+	virtual const char *GetTextCharacterAtPosition(const Point &pt) const;
+
 	void InitNested(const WindowDesc *desc, WindowNumber number = 0);
 	void CreateNestedTree(const WindowDesc *desc, bool fill_nested = true);
 	void FinishInitNested(const WindowDesc *desc, WindowNumber window_number = 0);
 
+	/**
+	 * Set the timeout flag of the window and initiate the timer.
+	 */
+	inline void SetTimeout()
+	{
+		this->flags |= WF_TIMEOUT;
+		this->timeout_timer = TIMEOUT_DURATION;
+	}
+
+	/**
+	 * Set the timeout flag of the window and initiate the timer.
+	 */
+	inline void SetWhiteBorder()
+	{
+		this->flags |= WF_WHITE_BORDER;
+		this->white_border_timer = WHITE_BORDER_DURATION;
+	}
+
+	void DisableAllWidgetHighlight();
+	void SetWidgetHighlight(byte widget_index, TextColour highlighted_colour);
+	bool IsWidgetHighlighted(byte widget_index) const;
+
 	/**
 	 * Sets the enabled/disabled status of a widget.
 	 * By default, widgets are enabled.
@@ -404,7 +471,10 @@ public:
 	}
 
 	void UnfocusFocusedWidget();
-	bool SetFocusedWidget(byte widget_index);
+	bool SetFocusedWidget(int widget_index);
+
+	EventState HandleEditBoxKey(int wid, WChar key, uint16 keycode);
+	virtual void InsertTextString(int wid, const char *str, bool marked, const char *caret, const char *insert_location, const char *replacement_end);
 
 	void HandleButtonClick(byte widget);
 	int GetRowFromWidget(int clickpos, int widget, int padding, int line_height = -1) const;
@@ -431,31 +501,9 @@ public:
 
 	void SetShaded(bool make_shaded);
 
-	/**
-	 * Mark this window's data as invalid (in need of re-computing)
-	 * @param data The data to invalidate with
-	 * @param gui_scope Whether the funtion is called from GUI scope.
-	 */
-	void InvalidateData(int data = 0, bool gui_scope = true)
-	{
-		this->SetDirty();
-		if (!gui_scope) {
-			/* Schedule GUI-scope invalidation for next redraw. */
-			*this->scheduled_invalidation_data.Append() = data;
-		}
-		this->OnInvalidateData(data, gui_scope);
-	}
-
-	/**
-	 * Process all scheduled invalidations.
-	 */
-	void ProcessScheduledInvalidations()
-	{
-		for (int *data = this->scheduled_invalidation_data.Begin(); this->window_class != WC_INVALID && data != this->scheduled_invalidation_data.End(); data++) {
-			this->OnInvalidateData(*data, true);
-		}
-		this->scheduled_invalidation_data.Clear();
-	}
+	void InvalidateData(int data = 0, bool gui_scope = true);
+	void ProcessScheduledInvalidations();
+	void ProcessHighlightedInvalidations();
 
 	/*** Event handling ***/
 
@@ -519,10 +567,7 @@ public:
 	 */
 	virtual void OnFocus() {}
 
-	/**
-	 * Called when window looses focus
-	 */
-	virtual void OnFocusLost() {}
+	virtual void OnFocusLost();
 
 	/**
 	 * A key has been pressed.
@@ -531,7 +576,7 @@ public:
 	 * @return #ES_HANDLED if the key press has been handled and no other
 	 *         window should receive the event.
 	 */
-	virtual EventState OnKeyPress(uint16 key, uint16 keycode) { return ES_NOT_HANDLED; }
+	virtual EventState OnKeyPress(WChar key, uint16 keycode) { return ES_NOT_HANDLED; }
 
 	/**
 	 * The state of the control key has changed
@@ -634,6 +679,14 @@ public:
 	 */
 	virtual void OnDropdownSelect(int widget, int index) {}
 
+	virtual void OnDropdownClose(Point pt, int widget, int index, bool instant_close);
+
+	/**
+	 * The text in an editbox has been edited.
+	 * @param widget The widget of the editbox.
+	 */
+	virtual void OnEditboxChanged(int widget) {}
+
 	/**
 	 * The query window opened from this window has closed.
 	 * @param str the new value of the string, NULL if the window
@@ -660,8 +713,9 @@ public:
 	/**
 	 * The user clicked on a vehicle while HT_VEHICLE has been set.
 	 * @param v clicked vehicle. It is guaranteed to be v->IsPrimaryVehicle() == true
+	 * @return True if the click is handled, false if it is ignored.
 	 */
-	virtual void OnVehicleSelect(const struct Vehicle *v) {}
+	virtual bool OnVehicleSelect(const struct Vehicle *v) { return false; }
 
 	/**
 	 * The user cancelled a tile highlight mode that has been set.
@@ -765,27 +819,6 @@ public:
 	virtual ~PickerWindowBase();
 };
 
-/**
- * Window flags
- */
-enum WindowFlags {
-	WF_TIMEOUT_TRIGGER   = 1,       ///< When the timeout should start triggering
-	WF_TIMEOUT_BEGIN     = 7,       ///< The initial value for the timeout
-	WF_TIMEOUT_MASK      = 7,       ///< Window timeout counter bit mask (3 bits)
-	WF_DRAGGING          = 1 <<  3, ///< Window is being dragged
-	WF_SIZING_RIGHT      = 1 <<  4, ///< Window is being resized towards the right.
-	WF_SIZING_LEFT       = 1 <<  5, ///< Window is being resized towards the left.
-	WF_SIZING            = WF_SIZING_RIGHT | WF_SIZING_LEFT, ///< Window is being resized.
-	WF_STICKY            = 1 <<  6, ///< Window is made sticky by user
-
-	WF_DISABLE_VP_SCROLL = 1 <<  7, ///< Window does not do autoscroll, @see HandleAutoscroll()
-
-	WF_WHITE_BORDER_ONE  = 1 <<  8,
-	WF_WHITE_BORDER_MASK = 1 <<  9 | WF_WHITE_BORDER_ONE,
-
-	WF_CENTERED          = 1 << 10, ///< Window is centered and shall stay centered after ReInit
-};
-
 Window *BringWindowToFrontById(WindowClass cls, WindowNumber number);
 Window *FindWindowFromPt(int x, int y);
 
